@@ -75,10 +75,8 @@
 #include <QtGui/QItemDelegate>
 #include <QtGui/QStandardItemModel>
 #include <QtGui/QSpinBox>
-#ifndef __MINGW32__
-// #include <QtGui/QX11Info>
-#endif
 #include <QtGui/QFontDatabase>
+#include <QtGui/QFontComboBox>
 #include <QtGui/QApplication>
 #include <QtGui/QToolTip>
 #include <QtCore/QProcess>
@@ -101,11 +99,22 @@
 #include <QtGui/QRadioButton>
 #include <QtGui/QDirModel>
 #include <QtGui/QHeaderView>
+#include <QtGui/QUndoGroup>
+#include <QtGui/QUndoStack>
+#include <QtGui/QProgressBar>
+#include <QtGui/QDialogButtonBox>
+
 #include <QtSvg/QSvgRenderer>
 #include <QtSvg/QSvgGenerator>
 
 #include <QtScript/QScriptEngine>
 #include <QtScript/QScriptValueIterator>
 #include <QtScript/QScriptable>
+#include <QtScript/QScriptClass>
+#include <QtScript/QScriptClassPropertyIterator>
+#if QT_VERSION >= 0x040500
+#include <QtScriptTools/QScriptEngineDebugger>
+#endif
+
 #endif
 
