@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: element.cpp,v 1.79 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: accidental.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -82,7 +82,7 @@ void Accidental::setSubtype(int i)
                   {
                   s->setSym(leftparenSym);
                   addElement(s, 0.0, 0.0);
-                  double x = symbols[leftparenSym].width(mag());
+                  double x = symbols[leftparenSym].width(magS());
 
                   s = new Symbol(score());
                   switch(i) {
