@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: actions.cpp 1985 2009-08-02 10:33:47Z wschweer $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2008 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -19,6 +19,7 @@
 //=============================================================================
 
 #include "mscore.h"
+#include "score.h"      // states
 #include "icons.h"
 
 //---------------------------------------------------------
@@ -30,9 +31,9 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
          "file-open",
          QT_TR_NOOP("file open"),
-         Qt::CTRL+Qt::Key_O,
+         QKeySequence::Open,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Open"),
+         QT_TR_NOOP("Open..."),
          QT_TR_NOOP("Load Score from File"),
          &fileOpenIcon
          ),
@@ -40,7 +41,7 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
          "file-save",
          QT_TR_NOOP("file save"),
-         Qt::CTRL+Qt::Key_S,
+         QKeySequence::Save,
          Qt::WindowShortcut,
          QT_TR_NOOP("Save"),
          QT_TR_NOOP("Save Score to File"),
@@ -50,17 +51,35 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
          "file-save-as",
          QT_TR_NOOP("file save as"),
-         0,
+         QKeySequence::SaveAs,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Save As"),
+         QT_TR_NOOP("Save As..."),
          QT_TR_NOOP("Save Score to named File"),
          &fileSaveIcon
          ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
+         "file-save-a-copy",
+         QT_TR_NOOP("file save a copy"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Save a Copy..."),
+         QT_TR_NOOP("Save Score to named File, but keep current name")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
+         "file-reload",
+         QT_TR_NOOP("file reload"),
+         0,                         // no shortcut, its an destructive non undoable operation
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Reload"),
+         QT_TR_NOOP("Reload Score from File")
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
          "file-close",
          QT_TR_NOOP("file close"),
-         Qt::CTRL+Qt::Key_W,
+         QKeySequence::Close,
          Qt::WindowShortcut,
          QT_TR_NOOP("Close"),
          QT_TR_NOOP("Close Current Score"),
@@ -72,7 +91,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("file new"),
          Qt::CTRL+Qt::Key_N,
          Qt::WindowShortcut,
-         QT_TR_NOOP("New"),
+         QT_TR_NOOP("New..."),
          QT_TR_NOOP("Create new score"),
          &fileNewIcon
          ),
@@ -82,7 +101,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("print"),
          Qt::CTRL+Qt::Key_P,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Print"),
+         QT_TR_NOOP("Print..."),
          QT_TR_NOOP("Print Score"),
          &printIcon
          ),
@@ -90,7 +109,7 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT,
          "undo",
          QT_TR_NOOP("undo"),
-         Qt::CTRL+Qt::Key_Z,        // QKeySequence::Undo,
+         QKeySequence::Undo,      // Qt::CTRL+Qt::Key_Z, // QKeySequence::Undo,
          Qt::WindowShortcut,
          QT_TR_NOOP("Undo"),
          QT_TR_NOOP("undo last change"),
@@ -100,7 +119,7 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT,
          "redo",
          QT_TR_NOOP("redo"),
-         Qt::CTRL+Qt::SHIFT+Qt::Key_Z,    // QKeySequence::Redo,
+         QKeySequence::Redo,      // Qt::CTRL+Qt::SHIFT+Qt::Key_Z,    // QKeySequence::Redo,
          Qt::WindowShortcut,
          QT_TR_NOOP("Redo"),
          QT_TR_NOOP("redo last undo"),
@@ -130,8 +149,8 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT,
          "paste",
          QT_TR_NOOP("paste"),
-         Qt::CTRL+Qt::Key_V,        // QKeySequence::Paste,
-         Qt::WindowShortcut,
+         Qt::CTRL+Qt::Key_V,        //  QKeySequence::Paste,
+         Qt::ApplicationShortcut,
          QT_TR_NOOP("Paste"),
          QT_TR_NOOP(""),
          &pasteIcon
@@ -198,8 +217,9 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("note input"),
          Qt::Key_N,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Input"),
-         QT_TR_NOOP("Note Input Mode")
+         QT_TR_NOOP("Note Input"),
+         QT_TR_NOOP("toggle note input mode N"),
+         &noteEntryIcon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -207,89 +227,89 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("pitch spell"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Pitch spell"),
-         QT_TR_NOOP("Pitch spell")
+         QT_TR_NOOP("Pitch Spell"),
+         QT_TR_NOOP("Pitch Spell")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval1",
          QT_TR_NOOP("enter unison above"),
-         Qt::Key_1,
+         Qt::ALT + Qt::Key_1,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Unison above"),
-         QT_TR_NOOP("Enter Unison above")
+         QT_TR_NOOP("Unison Above"),
+         QT_TR_NOOP("Enter unison above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval2",
          QT_TR_NOOP("enter second above"),
-         Qt::Key_2,
+         Qt::ALT + Qt::Key_2,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Second above"),
-         QT_TR_NOOP("Enter Second above")
+         QT_TR_NOOP("Second Above"),
+         QT_TR_NOOP("Enter second above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval3",
          QT_TR_NOOP("enter third above"),
-         Qt::Key_3,
+         Qt::ALT + Qt::Key_3,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Third above"),
-         QT_TR_NOOP("Enter Third above")
+         QT_TR_NOOP("Third Above"),
+         QT_TR_NOOP("Enter third above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval4",
          QT_TR_NOOP("enter fourth above"),
-         Qt::Key_4,
+         Qt::ALT + Qt::Key_4,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Fourth above"),
-         QT_TR_NOOP("Enter Fourth above")
+         QT_TR_NOOP("Fourth Above"),
+         QT_TR_NOOP("Enter fourth above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval5",
          QT_TR_NOOP("enter fifth above"),
-         Qt::Key_5,
+         Qt::ALT + Qt::Key_5,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Fifth above"),
-         QT_TR_NOOP("Enter Fifth above")
+         QT_TR_NOOP("Fifth Above"),
+         QT_TR_NOOP("Enter fifth above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval6",
          QT_TR_NOOP("enter sixth above"),
-         Qt::Key_6,
+         Qt::ALT + Qt::Key_6,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Sixth above"),
-         QT_TR_NOOP("Enter Sixth above")
+         QT_TR_NOOP("Sixth Above"),
+         QT_TR_NOOP("Enter sixth above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval7",
          QT_TR_NOOP("enter seventh above"),
-         Qt::Key_7,
+         Qt::ALT + Qt::Key_7,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Seventh above"),
-         QT_TR_NOOP("Enter Seventh above")
+         QT_TR_NOOP("Seventh Above"),
+         QT_TR_NOOP("Enter seventh above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval8",
          QT_TR_NOOP("enter octave above"),
-         Qt::Key_8,
+         Qt::ALT + Qt::Key_8,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Octave above"),
-         QT_TR_NOOP("Enter Octave above")
+         QT_TR_NOOP("Octave Above"),
+         QT_TR_NOOP("Enter octave above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "interval9",
          QT_TR_NOOP("enter ninth above"),
-         Qt::Key_9,
+         Qt::ALT + Qt::Key_9,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Ninth above"),
-         QT_TR_NOOP("Enter Ninth above")
+         QT_TR_NOOP("Ninth Above"),
+         QT_TR_NOOP("Enter ninth above")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -297,8 +317,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter second below"),
          Qt::SHIFT + Qt::Key_2,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Second below"),
-         QT_TR_NOOP("Enter Second below")
+         QT_TR_NOOP("Second Below"),
+         QT_TR_NOOP("Enter second below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -306,8 +326,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter third below"),
          Qt::SHIFT + Qt::Key_3,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Third below"),
-         QT_TR_NOOP("Enter Third below")
+         QT_TR_NOOP("Third Below"),
+         QT_TR_NOOP("Enter third below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -315,8 +335,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter fourth below"),
          Qt::SHIFT + Qt::Key_4,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Fourth below"),
-         QT_TR_NOOP("Enter Fourth below")
+         QT_TR_NOOP("Fourth Below"),
+         QT_TR_NOOP("Enter fourth below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -324,8 +344,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter fifth below"),
          Qt::SHIFT + Qt::Key_5,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Fifth below"),
-         QT_TR_NOOP("Enter Fifth below")
+         QT_TR_NOOP("Fifth Below"),
+         QT_TR_NOOP("Enter fifth below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -333,8 +353,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter sixth below"),
          Qt::SHIFT + Qt::Key_6,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Sixth below"),
-         QT_TR_NOOP("Enter Sixth below")
+         QT_TR_NOOP("Sixth Below"),
+         QT_TR_NOOP("Enter sixth below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -342,8 +362,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter seventh below"),
          Qt::SHIFT + Qt::Key_7,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Seventh below"),
-         QT_TR_NOOP("Enter Seventh below")
+         QT_TR_NOOP("Seventh Below"),
+         QT_TR_NOOP("Enter seventh below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -351,8 +371,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter octave below"),
          Qt::SHIFT + Qt::Key_8,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Octave below"),
-         QT_TR_NOOP("Enter Octave below")
+         QT_TR_NOOP("Octave Below"),
+         QT_TR_NOOP("Enter octave below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -360,8 +380,8 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("enter ninth below"),
          Qt::SHIFT + Qt::Key_9,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Ninth below"),
-         QT_TR_NOOP("Enter Ninth below")
+         QT_TR_NOOP("Ninth Below"),
+         QT_TR_NOOP("Enter ninth below")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
@@ -498,13 +518,41 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("rest"),
          QT_TR_NOOP("enter rest")
          ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "add-staccato",
+         QT_TR_NOOP("add staccato"),
+         Qt::SHIFT+Qt::Key_Period,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("staccato"),
+         QT_TR_NOOP("staccato")
+         ),
+       Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "pad-flat",
+         QT_TR_NOOP("flat"),
+         Qt::Key_Minus,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("flat"),
+         QT_TR_NOOP("flat"),
+         &flatIcon
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "add-trill",
+         QT_TR_NOOP("add trill"),
+	   0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("trill"),
+         QT_TR_NOOP("trill")
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "stretch+",
          QT_TR_NOOP("more stretch"),
          Qt::SHIFT + Qt::Key_Plus,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Add more stretch"),
+         QT_TR_NOOP("Add More Stretch"),
          QT_TR_NOOP("Add more stretch to selected measure")
          ),
       Shortcut(
@@ -513,7 +561,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("less stretch"),
          Qt::SHIFT + Qt::Key_Minus,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Add less stretch"),
+         QT_TR_NOOP("Add Less Stretch"),
          QT_TR_NOOP("Add less stretch to selected measure")
          ),
       Shortcut(
@@ -661,6 +709,96 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("right+ctrl"),
          QT_TR_NOOP("right+ctrl")
          ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-prev-chord",
+         QT_TR_NOOP("add previous chord to selection"),
+         Qt::SHIFT+Qt::Key_Left,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("left+shift"),
+         QT_TR_NOOP("left+shift")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-prev-measure",
+         QT_TR_NOOP("select to beginning of measure"),
+         Qt::CTRL+Qt::SHIFT+Qt::Key_Left,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("left+ctrl"),
+         QT_TR_NOOP("left+ctrl")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-next-chord",
+         QT_TR_NOOP("add next chord to selection"),
+         Qt::SHIFT+Qt::Key_Right,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("right+shift"),
+         QT_TR_NOOP("right+shift")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-next-measure",
+         QT_TR_NOOP("select to end of measure"),
+         Qt::CTRL+Qt::SHIFT+Qt::Key_Right,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("right+ctrl+shift"),
+         QT_TR_NOOP("right+ctrl+shift")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-begin-line",
+         QT_TR_NOOP("select to beginning of line"),
+         Qt::SHIFT+Qt::Key_Home,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("shift+home"),
+         QT_TR_NOOP("shift+home")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-end-line",
+         QT_TR_NOOP("select to end of line"),
+         Qt::SHIFT+Qt::Key_End,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("shift+end"),
+         QT_TR_NOOP("shift+end")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-begin-score",
+         QT_TR_NOOP("select to beginning of score"),
+         Qt::CTRL+Qt::SHIFT+Qt::Key_Home,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("ctrl+shift+home"),
+         QT_TR_NOOP("ctrl+shift+home")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "select-end-score",
+         QT_TR_NOOP("select to end of score"),
+         Qt::CTRL+Qt::SHIFT+Qt::Key_End,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("ctrl+shift+end"),
+         QT_TR_NOOP("ctrl+shift+end")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "select-staff-above",
+         QT_TR_NOOP("add staff above to selection"),
+         Qt::SHIFT+Qt::Key_Up,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("add staff above to selection"),
+         QT_TR_NOOP("add staff above to selection")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "select-staff-below",
+         QT_TR_NOOP("add staff below to selection"),
+         Qt::SHIFT+Qt::Key_Down,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("add staff below to selection"),
+         QT_TR_NOOP("add staff below to selection")
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "page-prev",
@@ -697,15 +835,6 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP(""),
          QT_TR_NOOP("")
          ),
-      Shortcut(
-         STATE_NORMAL | STATE_NOTE_ENTRY,
-         "add-tie",
-         QT_TR_NOOP("add tie"),
-         Qt::SHIFT+Qt::Key_S,
-         Qt::WindowShortcut,
-         QT_TR_NOOP(""),
-         QT_TR_NOOP("")
-         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "add-slur",
@@ -734,7 +863,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("")
          ),
       Shortcut(
-         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY | STATE_SEARCH,
          "escape",
          QT_TR_NOOP("ESCAPE"),
          Qt::Key_Escape,
@@ -751,6 +880,15 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP(""),
          QT_TR_NOOP("")
          ),
+      Shortcut(
+         STATE_NORMAL,
+         "delete-measures",
+         QT_TR_NOOP("delete selected measures"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Delete Selected Measures"),
+         QT_TR_NOOP("Delete Selected Measures")
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "append-measure",
@@ -763,7 +901,7 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "append-measures",
          QT_TR_NOOP("append measures"),
-         0,
+         Qt::CTRL+Qt::SHIFT+Qt::Key_B,
          Qt::WindowShortcut,
          QT_TR_NOOP("Append Measures...")
          ),
@@ -786,39 +924,39 @@ Shortcut MuseScore::sc[] = {
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "insert-hbox",
-         QT_TR_NOOP("Insert horizontal Frame"),
+         QT_TR_NOOP("Insert Horizontal Frame"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Insert horizontal Frame")
+         QT_TR_NOOP("Insert Horizontal Frame")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "insert-vbox",
-         QT_TR_NOOP("Insert vertical Frame"),
+         QT_TR_NOOP("Insert Vertical Frame"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Insert vertical Frame")
+         QT_TR_NOOP("Insert Vertical Frame")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "append-hbox",
-         QT_TR_NOOP("Append horizontal Frame"),
+         QT_TR_NOOP("Append Horizontal Frame"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Append horizontal Frame")
+         QT_TR_NOOP("Append Horizontal Frame")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "append-vbox",
-         QT_TR_NOOP("Append vertical Frame"),
+         QT_TR_NOOP("Append Vertical Frame"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Append vertical Frame")
+         QT_TR_NOOP("Append Vertical Frame")
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "duplet",
-         QT_TR_NOOP("duplet"),
+         QT_TR_NOOP("Duplet"),
          Qt::CTRL+Qt::Key_2,
          Qt::WindowShortcut,
          QT_TR_NOOP("Duplet")
@@ -826,11 +964,19 @@ Shortcut MuseScore::sc[] = {
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "triplet",
-         QT_TR_NOOP("triplet"),
+         QT_TR_NOOP("Triplet"),
          Qt::CTRL+Qt::Key_3,
          Qt::WindowShortcut,
          QT_TR_NOOP("Triplet")
          ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "quadruplet",
+         QT_TR_NOOP("Quadruplet"),
+         Qt::CTRL+Qt::Key_4,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Quadruplet")
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "quintuplet",
@@ -839,74 +985,134 @@ Shortcut MuseScore::sc[] = {
          Qt::WindowShortcut,
          QT_TR_NOOP("Quintuplet")
          ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "sextuplet",
+         QT_TR_NOOP("Sextuplet"),
+         Qt::CTRL+Qt::Key_6,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Sextuplet")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "septuplet",
+         QT_TR_NOOP("Septuplet"),
+         Qt::CTRL+Qt::Key_7,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Septuplet")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "octuplet",
+         QT_TR_NOOP("Octuplet"),
+         Qt::CTRL+Qt::Key_8,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Octuplet")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "nonuplet",
+         QT_TR_NOOP("Nonuplet"),
+         Qt::CTRL+Qt::Key_9,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Nonuplet")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "tuplet-dialog",
+         QT_TR_NOOP("Other Tuplets"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Other...")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "note-longa",
+         QT_TR_NOOP("note longa"),
+         Qt::Key_9,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Longa"),
+         QT_TR_NOOP("Longa"),
+         &longaUpIcon
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "note-breve",
+         QT_TR_NOOP("note breve"),
+         Qt::Key_8,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Double whole note (breve)"),
+         QT_TR_NOOP("Double whole note (breve)"),
+         &brevisIcon
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-1",
          QT_TR_NOOP("pad note 1/1"),
-         Qt::ALT + Qt::Key_6,
+         Qt::Key_7,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/1"),
-         QT_TR_NOOP("1/1"),
+         QT_TR_NOOP("Whole note (semibreve)"),
+         QT_TR_NOOP("Whole note (semibreve)"),
          &noteIcon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-2",
          QT_TR_NOOP("pad note 1/2"),
-         Qt::ALT + Qt::Key_7,
+         Qt::Key_6,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/2"),
-         QT_TR_NOOP("1/2"),
+         QT_TR_NOOP("Half note (minim)"),
+         QT_TR_NOOP("Half note (minim)"),
          &note2Icon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-4",
          QT_TR_NOOP("pad note 1/4"),
-         Qt::ALT + Qt::Key_1,
+         Qt::Key_5,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/4"),
-         QT_TR_NOOP("1/4"),
+         QT_TR_NOOP("Quarter note (crochet)"),
+         QT_TR_NOOP("Quarter note (crochet)"),
          &note4Icon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-8",
          QT_TR_NOOP("pad note 1/8"),
-         Qt::ALT + Qt::Key_2,
+         Qt::Key_4,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/8"),
-         QT_TR_NOOP("1/8"),
+         QT_TR_NOOP("Eighth note (quaver)"),
+         QT_TR_NOOP("Eighth note (quaver)"),
          &note8Icon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-16",
          QT_TR_NOOP("pad note 1/16"),
-         Qt::ALT + Qt::Key_3,
+         Qt::Key_3,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/16"),
-         QT_TR_NOOP("1/16"),
+         QT_TR_NOOP("16th note (semiquaver)"),
+         QT_TR_NOOP("16th note (semiquaver)"),
          &note16Icon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-32",
          QT_TR_NOOP("pad note 1/32"),
-         Qt::ALT + Qt::Key_4,
+         Qt::Key_2,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/32"),
-         QT_TR_NOOP("1/32"),
+         QT_TR_NOOP("32nd note (demi-semiquaver)"),
+         QT_TR_NOOP("32nd note (demi-semiquaver)"),
          &note32Icon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-note-64",
          QT_TR_NOOP("pad note 1/64"),
-         Qt::ALT + Qt::Key_5,
+         Qt::Key_1,
          Qt::WindowShortcut,
-         QT_TR_NOOP("1/64"),
-         QT_TR_NOOP("1/64"),
+         QT_TR_NOOP("64th note (semi-demi-semiquaver)"),
+         QT_TR_NOOP("64th note (semi-demi-semiquaver)"),
          &note64Icon
          ),
       Shortcut(
@@ -915,24 +1121,24 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("pad dot"),
          Qt::Key_Period,
          Qt::WindowShortcut,
-         QT_TR_NOOP("dot"),
-         QT_TR_NOOP("dot"),
+         QT_TR_NOOP("Dot"),
+         QT_TR_NOOP("Dot"),
          &dotIcon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-dotdot",
-         QT_TR_NOOP("pad dot dot"),
+         QT_TR_NOOP("pad double dot"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("dot dot"),
-         QT_TR_NOOP("dot dot"),
+         QT_TR_NOOP("Double dot"),
+         QT_TR_NOOP("Double dot"),
          &dotdotIcon
          ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
-         "pad-tie",
-         QT_TR_NOOP("pad tie"),
+         "tie",
+         QT_TR_NOOP("tie"),
          Qt::Key_Plus,
          Qt::WindowShortcut,
          QT_TR_NOOP("tie"),
@@ -979,16 +1185,6 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("natural"),
          &naturalIcon
          ),
-      Shortcut(
-         STATE_NORMAL | STATE_NOTE_ENTRY,
-         "pad-flat",
-         QT_TR_NOOP("flat"),
-         0,
-         Qt::WindowShortcut,
-         QT_TR_NOOP("flat"),
-         QT_TR_NOOP("flat"),
-         &flatIcon
-         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "pad-flat2",
@@ -1019,6 +1215,39 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("appoggiatura"),
          &appoggiaturaIcon
          ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+	 /* no stroke: 4th*/
+        "pad-grace4",
+        QT_TR_NOOP("grace-4"),
+        0,
+        Qt::WindowShortcut,
+        QT_TR_NOOP("grace-4"),
+        QT_TR_NOOP("grace-4"),
+        &grace4Icon
+        ),
+     Shortcut(
+        STATE_NORMAL | STATE_NOTE_ENTRY,
+      /* no stroke: 16th*/
+        "pad-grace16",
+        QT_TR_NOOP("grace-16"),
+        0,
+        Qt::WindowShortcut,
+        QT_TR_NOOP("grace-16"),
+        QT_TR_NOOP("grace-16"),
+        &grace16Icon
+        ),
+     Shortcut(
+        STATE_NORMAL | STATE_NOTE_ENTRY,
+      /* no stroke: 32th*/
+        "pad-grace32",
+        QT_TR_NOOP("grace-32"),
+        0,
+        Qt::WindowShortcut,
+        QT_TR_NOOP("grace-32"),
+        QT_TR_NOOP("grace-32"),
+        &grace32Icon
+        ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "voice-1",
@@ -1194,7 +1423,7 @@ Shortcut MuseScore::sc[] = {
          ),
 
       Shortcut(
-         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT,
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT | STATE_PLAY,
          "quit",
          QT_TR_NOOP("Quit"),
          Qt::CTRL + Qt::Key_Q,
@@ -1332,6 +1561,15 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("Chord Name"),
          QT_TR_NOOP("Add Chord Text")
          ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "harmony-properties",
+         QT_TR_NOOP("show harmony properties for chord"),
+         Qt::SHIFT+Qt::Key_K,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Harmony Properties"),
+         QT_TR_NOOP("Harmony Properties")
+         ),
       Shortcut(
          STATE_NORMAL | STATE_NOTE_ENTRY,
          "rehearsalmark-text",
@@ -1429,7 +1667,7 @@ Shortcut MuseScore::sc[] = {
          STATE_NORMAL,
          "repeat",
          QT_TR_NOOP("play repeats on"),
-         Qt::CTRL+Qt::Key_R,
+         0,
          Qt::WindowShortcut,
          QT_TR_NOOP("Repeat"),
          QT_TR_NOOP("play repeats on/off"),
@@ -1441,7 +1679,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("load style"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Load Style"),
+         QT_TR_NOOP("Load Style..."),
          QT_TR_NOOP("Load Style"),
          &fileOpenIcon
          ),
@@ -1451,7 +1689,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("save style"),
          0,
          Qt::WindowShortcut,
-         QT_TR_NOOP("Save Style"),
+         QT_TR_NOOP("Save Style..."),
          QT_TR_NOOP("Save Style"),
          &fileSaveIcon
          ),
@@ -1474,7 +1712,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("Transpose")
          ),
       Shortcut (
-         STATE_NOTE_ENTRY,
+         STATE_NORMAL | STATE_NOTE_ENTRY,
          "clef-violin",
          QT_TR_NOOP("violin clef"),
          QKeySequence(Qt::CTRL+Qt::Key_Y, Qt::CTRL+Qt::Key_1),
@@ -1483,7 +1721,7 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("violin clef")
          ),
       Shortcut (
-         STATE_NOTE_ENTRY,
+         STATE_NORMAL | STATE_NOTE_ENTRY,
          "clef-bass",
          QT_TR_NOOP("bass clef"),
          QKeySequence(Qt::CTRL+Qt::Key_Y, Qt::CTRL+Qt::Key_2),
@@ -1491,6 +1729,281 @@ Shortcut MuseScore::sc[] = {
          QT_TR_NOOP("bass clef"),
          QT_TR_NOOP("bass clef")
          ),
-      Shortcut(0, 0, 0),
+      Shortcut (
+         STATE_NORMAL,
+         "voice-x12",
+         QT_TR_NOOP("Exchange Voice 1-2"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Exchange Voice 1-2"),
+         QT_TR_NOOP("Exchange Voice 1-2")
+         ),
+      Shortcut (
+         STATE_NORMAL,
+         "voice-x13",
+         QT_TR_NOOP("Exchange Voice 1-3"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Exchange Voice 1-3"),
+         QT_TR_NOOP("Exchange Voice 1-3")
+         ),
+      Shortcut (
+         STATE_NORMAL,
+         "voice-x14",
+         QT_TR_NOOP("Exchange Voice 1-4"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Exchange Voice 1-4"),
+         QT_TR_NOOP("Exchange Voice 1-4")
+         ),
+      Shortcut (
+         STATE_NORMAL,
+         "voice-x23",
+         QT_TR_NOOP("Exchange Voice 2-3"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Exchange Voice 2-3"),
+         QT_TR_NOOP("Exchange Voice 2-3")
+         ),
+      Shortcut (
+         STATE_NORMAL,
+         "voice-x24",
+         QT_TR_NOOP("Exchange Voice 2-4"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Exchange Voice 2-4"),
+         QT_TR_NOOP("Exchange Voice 2-4")
+         ),
+      Shortcut (
+         STATE_NORMAL,
+         "voice-x34",
+         QT_TR_NOOP("Exchange Voice 3-4"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Exchange Voice 3-4"),
+         QT_TR_NOOP("Exchange Voice 3-4")
+         ),
+      Shortcut (
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "concert-pitch",
+         QT_TR_NOOP("display in concert pitch"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Concert Pitch"),
+         QT_TR_NOOP("Display in Concert Pitch")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "repeat-cmd",
+         QT_TR_NOOP("repeat last command"),
+         Qt::Key_R,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Repeat last command"),
+         QT_TR_NOOP("Repeat last command"),
+         &fileOpenIcon
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "edit-meta",
+         QT_TR_NOOP("edit score meta data"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Meta Data..."),
+         QT_TR_NOOP("Meta Data")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "system-break",
+         QT_TR_NOOP("toggle system break"),
+         Qt::Key_Return,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Toggle System Break"),
+         QT_TR_NOOP("Toggle System Break")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "page-break",
+         QT_TR_NOOP("toggle page break"),
+         Qt::CTRL+Qt::Key_Return,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Toggle Page Break"),
+         QT_TR_NOOP("Toggle Page Break")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "edit-element",
+         QT_TR_NOOP("edit element"),
+         Qt::CTRL+Qt::Key_E,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Edit Element"),
+         QT_TR_NOOP("Edit Element")
+         ),
+      Shortcut(
+         STATE_NORMAL,
+         "reset-positions",
+         QT_TR_NOOP("reset positions"),
+         Qt::CTRL+Qt::Key_R,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Reset Positions"),
+         QT_TR_NOOP("Reset Positions")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_PLAY,
+         "inspector",
+         QT_TR_NOOP("show inspector"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Inspector"),
+         QT_TR_NOOP("Inspector")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "reset-stretch",
+         QT_TR_NOOP("reset measure stretch"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Reset Stretch"),
+         QT_TR_NOOP("Reset Stretch")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "show-invisible",
+         QT_TR_NOOP("show invisible"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Show Invisible"),
+         QT_TR_NOOP("Show Invisible")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "show-frames",
+         QT_TR_NOOP("show frames"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Show Frames"),
+         QT_TR_NOOP("Show Frames")
+         ),
+      Shortcut(
+         STATE_EDIT,
+         "show-keys",
+         QT_TR_NOOP("show keyboard"),
+         Qt::Key_F2,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Show Keyboard"),
+         QT_TR_NOOP("Show Keyboard"),
+         &keysIcon
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_PLAY,
+         "script-debug",
+         QT_TR_NOOP("enable script debugger"),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Enable Script Debugger"),
+         QT_TR_NOOP("Enable Script Debugger")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "rest-1",
+         QT_TR_NOOP("enter 1/1 rest"),
+         QKeySequence(Qt::SHIFT+Qt::Key_R, Qt::SHIFT+Qt::Key_S),
+         Qt::WindowShortcut,
+         QT_TR_NOOP("1/1 rest"),
+         QT_TR_NOOP("enter 1/1 rest")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "rest-2",
+         QT_TR_NOOP("enter 1/2 rest"),
+         QKeySequence(Qt::SHIFT+Qt::Key_R, Qt::SHIFT+Qt::Key_M),
+         Qt::WindowShortcut,
+         QT_TR_NOOP("1/2 rest"),
+         QT_TR_NOOP("enter 1/2 rest")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "rest-4",
+         QT_TR_NOOP("enter 1/4 rest"),
+         QKeySequence(Qt::SHIFT+Qt::Key_R, Qt::SHIFT+Qt::Key_R),
+         Qt::WindowShortcut,
+         QT_TR_NOOP("1/4 rest"),
+         QT_TR_NOOP("enter 1/4 rest")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "rest-8",
+         QT_TR_NOOP("enter 1/8 rest"),
+         QKeySequence(Qt::SHIFT+Qt::Key_R, Qt::SHIFT+Qt::Key_Q),
+         Qt::WindowShortcut,
+         QT_TR_NOOP("1/8 rest"),
+         QT_TR_NOOP("enter 1/8 rest")
+         ),
+      Shortcut(                     // mapped to undo in note entry mode
+         STATE_NOTE_ENTRY,
+         "backspace",
+         QT_TR_NOOP("backspace"),
+		 QKeySequence(Qt::Key_Backspace),    // QKeySequence::Back,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("backspace"),
+         QT_TR_NOOP("backspace")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "find",
+         QT_TR_NOOP("search"),
+         QKeySequence::Find,        // Qt::CTRL + Qt::Key_F,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Search"),
+         QT_TR_NOOP("Search")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT,
+         "zoomin",
+         QT_TR_NOOP("Zoom In"),
+         QKeySequence::ZoomIn,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Zoom In")
+         ),
+      Shortcut(
+         // conflicts with Ctrl+- in edit mode to enter lyrics hyphen
+         // STATE_NORMAL | STATE_NOTE_ENTRY | STATE_EDIT,
+
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "zoomout",
+         QT_TR_NOOP("Zoom Out"),
+         QKeySequence::ZoomOut,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Zoom Out")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "mirror-note",
+         QT_TR_NOOP("mirror note head"),
+         Qt::SHIFT + Qt::Key_X,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("mirror note head"),
+         QT_TR_NOOP("mirror note head"),
+         &flipIcon
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "edit-style",
+         QT_TR_NOOP("Edit Style..."),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Edit Style..."),
+         QT_TR_NOOP("Edit Style...")
+         ),
+      Shortcut(
+         STATE_NORMAL | STATE_NOTE_ENTRY,
+         "edit-text-style",
+         QT_TR_NOOP("Edit Text Style..."),
+         0,
+         Qt::WindowShortcut,
+         QT_TR_NOOP("Edit Text Style..."),
+         QT_TR_NOOP("Edit Text Style...")
+         ),
+      // xml==0  marks end of list
+      Shortcut(0, 0, 0, QKeySequence::UnknownKey),
       };
 
