@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: alsa.cpp,v 1.14 2006/03/02 17:08:30 wschweer Exp $
+//  $Id: alsa.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  AlsaDriver based on code from Fons Adriaensen (clalsadr.cc)
 //    Copyright (C) 2003 Fons Adriaensen
@@ -22,17 +22,18 @@
 //  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //=============================================================================
 
-#include <sys/time.h>
-
 #include "config.h"
+
+#ifdef USE_ALSA
+#include <sys/time.h>
 #include "alsa.h"
 #include "score.h"
 #include "mscore.h"
 #include "preferences.h"
 #include "seq.h"
 #include "alsamidi.h"
+#include "utils.h"
 
-#ifdef USE_ALSA
 //---------------------------------------------------------
 //   AlsaDriver
 //---------------------------------------------------------
@@ -59,7 +60,7 @@ AlsaDriver::AlsaDriver(QString s, unsigned rate,
 
 bool AlsaDriver::init()
       {
-      if (snd_pcm_open (&_play_handle, _name.toLatin1().data(), SND_PCM_STREAM_PLAYBACK, 0) < 0) {
+      if (snd_pcm_open(&_play_handle, _name.toLatin1().data(), SND_PCM_STREAM_PLAYBACK, 0) < 0) {
             _play_handle = 0;
             fprintf (stderr, "Alsa_driver: Cannot open PCM device %s for playback.\n",
                _name.toLatin1().data());
@@ -542,17 +543,6 @@ void AlsaDriver::write(int n, float* l, float* r)
       snd_pcm_mmap_commit(_play_handle, _play_offs, n);
       }
 
-//---------------------------------------------------------
-//   curTime
-//---------------------------------------------------------
-
-static double curTime()
-      {
-      struct timeval t;
-      gettimeofday(&t, 0);
-      return (double)((double)t.tv_sec + (t.tv_usec / 1000000.0));
-      }
-
 //---------------------------------------------------------
 //   AlsaAudio
 //---------------------------------------------------------
@@ -586,10 +576,8 @@ int AlsaAudio::sampleRate() const
 
 AlsaAudio::~AlsaAudio()
       {
-      if (synth)
-            delete synth;
-      if (alsa)
-            delete alsa;
+      delete synth;
+      delete alsa;
       }
 
 //---------------------------------------------------------
@@ -610,16 +598,9 @@ bool AlsaAudio::init()
             fprintf(stderr, "init ALSA audio driver failed\n");
             return false;
             }
-      synth = new ISynth();
-      int sr = alsa->sampleRate();
-      if (!synth->init(sr)) {
-            fprintf(stderr, "Synti init failed\n");
-            delete synth;
-            delete alsa;
-            alsa  = 0;
-            synth = 0;
-            return false;
-            }
+      synth = new FluidS::Fluid();
+      synth->init(alsa->sampleRate());
+
       midiDriver = new AlsaMidiDriver(seq);
       if (!midiDriver->init()) {
             delete midiDriver;
@@ -764,7 +745,7 @@ int AlsaAudio::getState()
 //   putEvent
 //---------------------------------------------------------
 
-void AlsaAudio::putEvent(const MidiOutEvent& e)
+void AlsaAudio::putEvent(const Event& e)
       {
       synth->play(e);
       }
@@ -791,10 +772,10 @@ void AlsaAudio::midiRead()
 //   getPatchInfo
 //---------------------------------------------------------
 
-const MidiPatch* AlsaAudio::getPatchInfo(int port, int ch, const MidiPatch* p)
+const MidiPatch* AlsaAudio::getPatchInfo(bool onlyDrums, const MidiPatch* p)
       {
       if (synth)
-            return synth->getPatchInfo(port, ch, p);
+            return synth->getPatchInfo(onlyDrums, p);
       return 0;
       }
 
