@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: alsa.h,v 1.8 2006/03/03 21:47:11 wschweer Exp $
+//  $Id: alsa.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  AlsaDriver based on code from Fons Adriaensen (clalsadr.cc)
 //  Copyright (C) 2003 Fons Adriaensen
@@ -109,7 +109,6 @@ class AlsaAudio : public Driver {
       int state;
       bool seekflag;
       double startTime;
-      ISynth* synth;
 
       Port midiInPort;
       MidiDriver* midiDriver;
@@ -137,7 +136,7 @@ class AlsaAudio : public Driver {
       void alsaLoop();
       void write(int n, void* l, void* r);
 
-      virtual void putEvent(const MidiOutEvent&);
+      virtual void putEvent(const Event&);
       virtual void process(int, float*, float*, int);
       virtual void midiRead();
       virtual const MidiPatch* getPatchInfo(bool onlyDrums, const MidiPatch*);
