@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: barline.cpp,v 1.3 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: barline.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -28,6 +28,8 @@
 #include "viewer.h"
 #include "staff.h"
 #include "system.h"
+#include "measure.h"
+#include "segment.h"
 
 //---------------------------------------------------------
 //   BarLine
@@ -39,7 +41,7 @@ BarLine::BarLine(Score* s)
       setSubtype(NORMAL_BAR);
       _span = 1;
       yoff  = 0.0;
-      setHeight(point(Spatium(4))); // for use in palettes
+      setHeight(4.0 * spatium()); // for use in palettes
       }
 
 //---------------------------------------------------------
@@ -79,7 +81,7 @@ void BarLine::getY(double* y1, double* y2) const
             *y2 = l2->y2() - yp;
             }
       else {
-            *y2 = 4.0 * _spatium;    // for use in palette
+            *y2 = 4.0 * spatium();    // for use in palette
             *y1 = 0.0;
             }
 
@@ -92,7 +94,7 @@ void BarLine::getY(double* y1, double* y2) const
 
 void BarLine::draw(QPainter& p) const
       {
-      qreal lw = point(score()->style()->barWidth) * mag();
+      qreal lw = point(score()->styleS(ST_barWidth));
       qreal y1, y2;
       getY(&y1, &y2);
 
@@ -101,7 +103,8 @@ void BarLine::draw(QPainter& p) const
       pen.setCapStyle(Qt::FlatCap);
       p.setPen(pen);
 
-      double ld = _spatium * mag();       // actual line distance
+      double mags = magS();
+      double ld   = spatium();       // actual line distance
 
       switch(subtype()) {
             case BROKEN_BAR:
@@ -121,8 +124,8 @@ void BarLine::draw(QPainter& p) const
 
             case END_BAR:
                   {
-                  qreal lw2 = point(score()->style()->endBarWidth) * mag();
-                  qreal d   = point(score()->style()->endBarDistance) * mag();
+                  qreal lw2 = point(score()->styleS(ST_endBarWidth));
+                  qreal d   = point(score()->styleS(ST_endBarDistance));
 
                   p.drawLine(QLineF(lw * .5, y1, lw * .5, y2));
                   pen.setWidthF(lw2);
@@ -135,8 +138,8 @@ void BarLine::draw(QPainter& p) const
 
             case DOUBLE_BAR:
                   {
-                  lw      = point(score()->style()->doubleBarWidth) * mag();
-                  qreal d = point(score()->style()->doubleBarDistance) * mag();
+                  lw      = point(score()->styleS(ST_doubleBarWidth));
+                  qreal d = point(score()->styleS(ST_doubleBarDistance));
 
                   pen.setWidthF(lw);
                   p.setPen(pen);
@@ -150,19 +153,19 @@ void BarLine::draw(QPainter& p) const
 
             case START_REPEAT:
                   {
-                  qreal lw2  = point(score()->style()->endBarWidth) * mag();
-                  qreal lw22 = point(score()->style()->endBarWidth) * .5 * mag();
-                  qreal d1   = point(score()->style()->endBarDistance) * mag();
-                  qreal dotw = symbols[dotSym].width(mag());
+                  qreal lw2  = point(score()->styleS(ST_endBarWidth));
+                  qreal lw22 = point(score()->styleS(ST_endBarWidth));
+                  qreal d1   = point(score()->styleS(ST_endBarDistance));
+                  qreal dotw = symbols[dotSym].width(mags);
                   qreal x2   =  dotw/2;
                   qreal x1   =  dotw + d1 + lw/2;
                   qreal x0   =  dotw + d1 + lw + d1 + lw22;
 
-                  symbols[dotSym].draw(p, mag(), x0, 1.5 * ld);
-                  symbols[dotSym].draw(p, mag(), x0, 2.5 * ld);
+                  symbols[dotSym].draw(p, mags, x0, 1.5 * ld);
+                  symbols[dotSym].draw(p, mags, x0, 2.5 * ld);
                   if (_span == 2) {
-                        symbols[dotSym].draw(p, mag(), x0, y2 - 1.5 * ld);
-                        symbols[dotSym].draw(p, mag(), x0, y2 - 2.5 * ld);
+                        symbols[dotSym].draw(p, mags, x0, y2 - 1.5 * ld);
+                        symbols[dotSym].draw(p, mags, x0, y2 - 2.5 * ld);
                         }
 
                   p.drawLine(QLineF(x1, y1, x1, y2));
@@ -175,20 +178,20 @@ void BarLine::draw(QPainter& p) const
 
             case END_REPEAT:
                   {
-                  qreal lw2  = point(score()->style()->endBarWidth) * mag();
-                  qreal lw22 = point(score()->style()->endBarWidth) * .5 * mag();
-                  qreal d1   = point(score()->style()->endBarDistance) * mag();
+                  qreal lw2  = point(score()->styleS(ST_endBarWidth));
+                  qreal lw22 = point(score()->styleS(ST_endBarWidth));
+                  qreal d1   = point(score()->styleS(ST_endBarDistance));
 
-                  qreal dotw = symbols[dotSym].width(mag());
+                  qreal dotw = symbols[dotSym].width(mags);
                   qreal x0   =  dotw/2;
                   qreal x1   =  dotw + d1 + lw/2;
                   qreal x2   =  dotw + d1 + lw + d1 + lw22;
 
-                  symbols[dotSym].draw(p, mag(), x0, 1.5 * ld);
-                  symbols[dotSym].draw(p, mag(), x0, 2.5 * ld);
+                  symbols[dotSym].draw(p, mags, x0, 1.5 * ld);
+                  symbols[dotSym].draw(p, mags, x0, 2.5 * ld);
                   if (_span == 2) {
-                        symbols[dotSym].draw(p, mag(), x0, y2 - 1.5 * ld);
-                        symbols[dotSym].draw(p, mag(), x0, y2 - 2.5 * ld);
+                        symbols[dotSym].draw(p, mags, x0, y2 - 1.5 * ld);
+                        symbols[dotSym].draw(p, mags, x0, y2 - 2.5 * ld);
                         }
 
                   p.drawLine(QLineF(x1, y1, x1, y2));
@@ -202,26 +205,26 @@ void BarLine::draw(QPainter& p) const
 
             case END_START_REPEAT:
                   {
-                  qreal lw2  = point(score()->style()->endBarWidth) * mag();
-                  qreal lw22 = point(score()->style()->endBarWidth) * .5 * mag();
-                  qreal d1   = point(score()->style()->endBarDistance) * mag();
+                  qreal lw2  = point(score()->styleS(ST_endBarWidth));
+                  qreal lw22 = point(score()->styleS(ST_endBarWidth));
+                  qreal d1   = point(score()->styleS(ST_endBarDistance));
 
-                  qreal dotw = symbols[dotSym].width(mag());
+                  qreal dotw = symbols[dotSym].width(mags);
                   qreal x0   =  dotw/2;
                   qreal x1   =  dotw + d1 + lw/2;
                   qreal x2   =  dotw + d1 + lw + d1 + lw22;
                   qreal x3   =  dotw + d1 + lw + d1 + lw2 + d1 + lw/2;
                   qreal x4   =  dotw + d1 + lw + d1 + lw2 + d1 + lw + d1 - dotw/2;
 
-                  symbols[dotSym].draw(p, mag(), x0, 1.5 * ld);
-                  symbols[dotSym].draw(p, mag(), x0, 2.5 * ld);
-                  symbols[dotSym].draw(p, mag(), x4, 1.5 * ld);
-                  symbols[dotSym].draw(p, mag(), x4, 2.5 * ld);
+                  symbols[dotSym].draw(p, mags, x0, 1.5 * ld);
+                  symbols[dotSym].draw(p, mags, x0, 2.5 * ld);
+                  symbols[dotSym].draw(p, mags, x4, 1.5 * ld);
+                  symbols[dotSym].draw(p, mags, x4, 2.5 * ld);
                   if (_span == 2) {
-                        symbols[dotSym].draw(p, mag(), x0, y2 - ld);
-                        symbols[dotSym].draw(p, mag(), x0, y2 - ld);
-                        symbols[dotSym].draw(p, mag(), x4, y2 - ld);
-                        symbols[dotSym].draw(p, mag(), x4, y2 - ld);
+                        symbols[dotSym].draw(p, mags, x0, y2 - ld);
+                        symbols[dotSym].draw(p, mags, x0, y2 - ld);
+                        symbols[dotSym].draw(p, mags, x4, y2 - ld);
+                        symbols[dotSym].draw(p, mags, x4, y2 - ld);
                         }
 
                   p.drawLine(QLineF(x1, y1, x1, y2));
@@ -272,26 +275,26 @@ QRectF BarLine::bbox() const
       {
       qreal y1, y2;
       getY(&y1, &y2);
-      Spatium w = score()->style()->barWidth * mag();
+      Spatium w = score()->styleS(ST_barWidth);
       qreal dw  = 0.0;
 
       switch(subtype()) {
             case DOUBLE_BAR:
-                  w  = (score()->style()->doubleBarWidth * 2 + score()->style()->doubleBarDistance) * mag();
+                  w  = score()->styleS(ST_doubleBarWidth) * 2 + score()->styleS(ST_doubleBarDistance);
                   dw = point(w);
                   break;
             case START_REPEAT:
             case END_REPEAT:
-                  w  += (score()->style()->endBarWidth + 2 * score()->style()->endBarDistance) * mag();
-                  dw = point(w) + symbols[dotSym].width(mag());
+                  w  += score()->styleS(ST_endBarWidth) + 2 * score()->styleS(ST_endBarDistance);
+                  dw = point(w) + symbols[dotSym].width(magS());
                   break;
             case END_BAR:
-                  w += (score()->style()->endBarWidth + score()->style()->endBarDistance) * mag();
+                  w += score()->styleS(ST_endBarWidth) + score()->styleS(ST_endBarDistance);
                   dw = point(w);
                   break;
             case  END_START_REPEAT:
-                  w  += (score()->style()->endBarWidth + 3 * score()->style()->endBarDistance) * mag();
-                  dw = point(w) + 2 * symbols[dotSym].width(mag());
+                  w  += score()->styleS(ST_endBarWidth) + 3 * score()->styleS(ST_endBarDistance);
+                  dw = point(w) + 2 * symbols[dotSym].width(magS());
                   break;
             case BROKEN_BAR:
             case NORMAL_BAR:
@@ -361,7 +364,7 @@ bool BarLine::startEdit(Viewer*, const QPointF&)
 void BarLine::updateGrips(int* grips, QRectF* grip) const
       {
       *grips   = 1;
-      qreal lw = point(score()->style()->barWidth);
+      qreal lw = point(score()->styleS(ST_barWidth));
       qreal y1, y2;
       getY(&y1, &y2);
       grip[0].translate(QPointF(lw * .5, y2) + canvasPos());
@@ -426,7 +429,7 @@ void BarLine::endEditDrag()
       else {
             qreal ay = s->canvasPos().y();
             qreal y  = s->staff(staffIdx1)->y() + ay;
-            qreal h1 = staff()->lines() * _spatium * staff()->mag();
+            qreal h1 = staff()->lines() * spatium();
 
             for (staffIdx2 = staffIdx1 + 1; staffIdx2 < n; ++staffIdx2) {
                   qreal h = s->staff(staffIdx2)->y() + ay - y;
