@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: bb.cpp 1924 2009-07-11 10:00:40Z wschweer $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -22,7 +22,6 @@
 #include "score.h"
 #include "part.h"
 #include "staff.h"
-#include "layout.h"
 #include "text.h"
 #include "box.h"
 #include "slur.h"
@@ -35,6 +34,7 @@
 #include "layoutbreak.h"
 #include "key.h"
 #include "pitchspelling.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   BBTrack
@@ -57,10 +57,10 @@ BBTrack::~BBTrack()
 //---------------------------------------------------------
 
 struct MNote {
-	ChordEvent* mc;
+	Event* mc;
       QList<Tie*> ties;
 
-      MNote(ChordEvent* _mc) : mc(_mc) {
+      MNote(Event* _mc) : mc(_mc) {
             for (int i = 0; i < mc->notes().size(); ++i)
                   ties.append(0);
             }
@@ -337,7 +337,7 @@ bool BBFile::read(const QString& name)
                               printf("event tick %d > %d\n", tick, endTick);
                               continue;
                               }
-                        NoteEvent* note = new NoteEvent();
+                        Event* note = new Event(ME_NOTE);
                         note->setOntime((tick * division) / bbDivision);
                         note->setPitch(a[idx + 5]);
                         note->setVelo(a[idx + 6]);
@@ -403,7 +403,7 @@ bool Score::importBB(const QString& name)
             Measure* measure  = new Measure(this);
             int tick = sigmap->bar2tick(i, 0, 0);
             measure->setTick(tick);
-      	_layout->add(measure);
+      	add(measure);
             }
 
       //---------------------------------------------------
@@ -414,7 +414,7 @@ bool Score::importBB(const QString& name)
             track->cleanup();
 
       if (tracks->isEmpty()) {
-            for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
+            for (MeasureBase* mb = first(); mb; mb = mb->next()) {
                   if (mb->type() != MEASURE)
                         continue;
                   Measure* measure = (Measure*)mb;
@@ -438,14 +438,15 @@ bool Score::importBB(const QString& name)
 
       Text* text = new Text(this);
       text->setSubtype(TEXT_TITLE);
+      text->setTextStyle(TEXT_STYLE_TITLE);
       text->setText(bb.title());
 
-      MeasureBase* measure = _layout->first();
+      MeasureBase* measure = first();
       if (measure->type() != VBOX) {
             measure = new VBox(this);
             measure->setTick(0);
-            measure->setNext(_layout->first());
-            _layout->add(measure);
+            measure->setNext(first());
+            add(measure);
             }
       measure->add(text);
 
@@ -472,8 +473,8 @@ bool Score::importBB(const QString& name)
                   h->setBaseTpc(table[c.bass-1]);
             else
                   h->setBaseTpc(INVALID_TPC);
-            h->setChordId(c.extension);
-            h->buildText();
+            h->setId(c.extension);
+            h->render();
             m->add(h);
             }
 
@@ -486,7 +487,7 @@ bool Score::importBB(const QString& name)
       int endChorus   = bb.endChorus() - 1;
 
       int n = 0;
-      for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
+      for (MeasureBase* mb = first(); mb; mb = mb->next()) {
             if (mb->type() != MEASURE)
                   continue;
             Measure* measure = (Measure*)mb;
@@ -553,9 +554,9 @@ int Score::processPendingNotes(QList<MNote*>* notes, int len, int track)
       s->add(chord);
 
       foreach (MNote* n, *notes) {
-            QList<NoteEvent*>& nl = n->mc->notes();
+            QList<Event*>& nl = n->mc->notes();
             for (int i = 0; i < nl.size(); ++i) {
-                  NoteEvent* mn = nl[i];
+                  Event* mn = nl[i];
       		Note* note = new Note(this);
                   note->setPitch(mn->pitch());
                   note->setTpc(mn->tpc());
@@ -582,7 +583,7 @@ int Score::processPendingNotes(QList<MNote*>* notes, int len, int track)
                   continue;
                   }
             for (int i = 0; i < nl.size(); ++i) {
-                  NoteEvent* mn = nl[i];
+                  Event* mn = nl[i];
                   Note* note = chord->noteList()->find(mn->pitch());
       		n->ties[i] = new Tie(this);
                   n->ties[i]->setStartNote(note);
@@ -604,7 +605,7 @@ static ciEvent collectNotes(int tick, int voice, ciEvent i, const EventList* el,
             Event* e = *i;
             if (e->type() != ME_CHORD)
                   continue;
-            ChordEvent* mc = (ChordEvent*)e;
+            Event* mc = e;
             if (mc->voice() != voice)
                   continue;
             if ((*i)->ontime() > tick)
@@ -636,7 +637,7 @@ void Score::convertTrack(BBTrack* track, int staffIdx)
 
             for (; i != el.end();) {
                   Event* e = *i;
-                  if (e->type() != ME_CHORD || ((ChordEvent*)e)->voice() != voice) {
+                  if (e->type() != ME_CHORD || e->voice() != voice) {
                         ++i;
                         continue;
                         }
@@ -729,8 +730,8 @@ void BBTrack::quantize(int startTick, int endTick, EventList* dst)
             Event* e = *i;
             if (e->ontime() >= endTick)
                   break;
-            if (e->type() == ME_NOTE && (((NoteEvent*)e)->duration() < mintick))
-                  mintick = ((NoteEvent*)e)->duration();
+            if (e->type() == ME_NOTE && (e->duration() < mintick))
+                  mintick = e->duration();
             }
       if (mintick <= division / 16)        // minimum duration is 1/64
             mintick = division / 16;
@@ -762,7 +763,7 @@ void BBTrack::quantize(int startTick, int endTick, EventList* dst)
             if (e->ontime() >= endTick)
                   break;
             if (e->type() == ME_NOTE) {
-                  NoteEvent* note = (NoteEvent*)e;
+                  Event* note = e;
                   // prefer moving note to the right
       	      int tick = ((note->ontime() + raster/2) / raster) * raster;
                   int diff = tick - note->ontime();
@@ -779,7 +780,7 @@ void BBTrack::quantize(int startTick, int endTick, EventList* dst)
             Event* e = *i;
             if (e->type() != ME_NOTE)
                   continue;
-            NoteEvent* note = (NoteEvent*)e;
+            Event* note = e;
             int tick   = note->ontime();
             int len    = note->duration();
             int ntick  = tick + len;
@@ -832,7 +833,7 @@ void BBTrack::cleanup()
       foreach (Event* e, _events) {
             if (e->type() != ME_NOTE)
                   continue;
-            int offtime  = ((NoteEvent*)e)->offtime();
+            int offtime  = e->offtime();
             if (offtime > lastTick)
                   lastTick = offtime;
             }
@@ -857,14 +858,14 @@ void BBTrack::cleanup()
                   ++ii;
                   for (; ii != dl.end(); ++ii) {
                         Event* ee = *ii;
-                        if (ee->type() != ME_NOTE || ((NoteEvent*)ee)->pitch() != ((NoteEvent*)e)->pitch())
+                        if (ee->type() != ME_NOTE || ee->pitch() != e->pitch())
                               continue;
-                        if (ee->ontime() >= e->ontime() + ((NoteEvent*)e)->duration())
+                        if (ee->ontime() >= e->ontime() + e->duration())
                               break;
-                        ((NoteEvent*)e)->setDuration(ee->ontime() - e->ontime());
+                        e->setDuration(ee->ontime() - e->ontime());
                         break;
                         }
-                  if (((NoteEvent*)e)->duration() <= 0)
+                  if (e->duration() <= 0)
                         continue;
                   }
 		_events.insert(e);
@@ -896,10 +897,10 @@ void BBTrack::findChords()
                   continue;
                   }
 
-            NoteEvent* note  = (NoteEvent*)e;
+            Event* note      = e;
             int ontime       = note->ontime();
             int offtime      = note->offtime();
-            ChordEvent* chord = new ChordEvent();
+            Event* chord = new Event(ME_CHORD);
             chord->setOntime(ontime);
             chord->setDuration(note->duration());
             chord->notes().append(note);
@@ -920,7 +921,7 @@ void BBTrack::findChords()
             for (int k = i + 1; k < n; ++k) {
                   if (_events[k] == 0 || _events[k]->type() != ME_NOTE)
                         continue;
-                  NoteEvent* nn = (NoteEvent*)_events[k];
+                  Event* nn = _events[k];
                   if (nn->ontime() - jitter > ontime)
                         break;
                   if (qAbs(nn->ontime() - ontime) > jitter || qAbs(nn->offtime() - offtime) > jitter)
