@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: beam.cpp,v 1.41 2006/09/15 09:34:57 wschweer Exp $
+//  $Id: beam.cpp 1978 2009-08-01 11:53:57Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -30,17 +30,19 @@
 #include "style.h"
 #include "note.h"
 #include "tuplet.h"
-#include "layout.h"
 #include "system.h"
 #include "tremolo.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   startBeam
 //---------------------------------------------------------
 
+#if 0
 static BeamHint startBeamList[] = {
       BeamHint(0, 0, 0, 0, 0, 0),
       };
+#endif
 
 //---------------------------------------------------------
 //   endBeam
@@ -200,7 +202,7 @@ Beam::Beam(Score* s)
    : Element(s)
       {
       _direction    = AUTO;
-      _up           = true;
+      _up           = -1;
       _userModified = false;
       }
 
@@ -233,10 +235,8 @@ Beam::~Beam()
       //
       foreach(ChordRest* cr, _elements)
             cr->setBeam(0);
-      for (iBeamSegment i = beamSegments.begin();
-         i != beamSegments.end(); ++i) {
+      for (iBeamSegment i = beamSegments.begin(); i != beamSegments.end(); ++i)
             delete *i;
-            }
       }
 
 //---------------------------------------------------------
@@ -287,7 +287,7 @@ void Beam::draw(QPainter& p) const
 
             QPointF ip1 = bs->p1;
             QPointF ip2 = bs->p2;
-            qreal lw2   = point(score()->style()->beamWidth) * .5 * mag();
+            qreal lw2   = point(score()->styleS(ST_beamWidth)) * .5;
 
             QPolygonF a(4);
             a[0] = QPointF(ip1.x(), ip1.y()-lw2);
@@ -318,7 +318,7 @@ void Beam::move(double x, double y)
 //    called before layout spacing of notes
 //---------------------------------------------------------
 
-void Measure::layoutBeams1(ScoreLayout* layout)
+void Measure::layoutBeams1()
       {
       foreach(Beam* beam, _beams)
             beam->clear();
@@ -354,12 +354,12 @@ void Measure::layoutBeams1(ScoreLayout* layout)
                                     if (!s || (s->subtype() != Segment::SegGrace) || !s->element(track))
                                           break;
                                     }
-                              b->layout1(layout);
+                              b->layout1();
                               segment = nseg;
                               }
                         else {
                               cr->setBeam(0);
-                              cr->layoutStem1(layout);
+                              cr->layoutStem1();
                               }
                         continue;
                         }
@@ -368,16 +368,16 @@ void Measure::layoutBeams1(ScoreLayout* layout)
 
                   if ((len >= division) || (bm == BEAM_NO)) {
                         if (beam) {
-                              beam->layout1(layout);
+                              beam->layout1();
                               beam = 0;
                               }
                         if (a1) {
                               a1->setBeam(0);
-                              a1->layoutStem1(layout);
+                              a1->layoutStem1();
                               a1 = 0;
                               }
                         cr->setBeam(0);
-                        cr->layoutStem1(layout);
+                        cr->layoutStem1();
                         continue;
                         }
                   bool beamEnd = false;
@@ -400,7 +400,7 @@ void Measure::layoutBeams1(ScoreLayout* layout)
                                     beamEnd = true;
                               }
                         if (beamEnd) {
-                              beam->layout1(layout);
+                              beam->layout1();
                               beam = 0;
                               a1   = 0;
                               }
@@ -410,18 +410,18 @@ void Measure::layoutBeams1(ScoreLayout* layout)
 
                               // is this the last beam element?
                               if (bm == BEAM_END) {
-                                    beam->layout1(layout);
+                                    beam->layout1();
                                     beam = 0;
                                     }
                               }
                         }
                   if (cr && cr->tuplet() && (cr->tuplet()->elements().back() == cr)) {
                         if (beam) {
-                              beam->layout1(layout);
+                              beam->layout1();
                               beam = 0;
 
                               cr->setBeam(0);
-                              cr->layoutStem1(layout);
+                              cr->layoutStem1();
                               }
                         else if (a1) {
                               beam = a1->beam();
@@ -435,12 +435,12 @@ void Measure::layoutBeams1(ScoreLayout* layout)
                               beam->add(a1);
                               beam->add(cr);
                               a1 = 0;
-                              beam->layout1(layout);
+                              beam->layout1();
                               beam = 0;
                               }
                         else {
                               cr->setBeam(0);
-                              cr->layoutStem1(layout);
+                              cr->layoutStem1();
                               }
                         }
                   else if (cr) {
@@ -457,7 +457,7 @@ void Measure::layoutBeams1(ScoreLayout* layout)
                                    )
                                  ) {
                                     a1->setBeam(0);
-                                    a1->layoutStem1(layout);      //?
+                                    a1->layoutStem1();      //?
                                     a1 = cr;
                                     }
                               else {
@@ -477,10 +477,10 @@ void Measure::layoutBeams1(ScoreLayout* layout)
                         }
                   }
             if (beam)
-                  beam->layout1(layout);
+                  beam->layout1();
             else if (a1) {
                   a1->setBeam(0);
-                  a1->layoutStem1(layout);
+                  a1->layoutStem1();
                   }
             }
       foreach(Beam* beam, _beams) {
@@ -497,13 +497,13 @@ void Measure::layoutBeams1(ScoreLayout* layout)
 //    called after layout spacing of notes
 //---------------------------------------------------------
 
-void Measure::layoutBeams(ScoreLayout* layout)
+void Measure::layoutBeams()
       {
       int nstaves = _score->nstaves();
       int tracks = nstaves * VOICES;
 
       foreach(Beam* beam, _beams)
-            beam->layout(layout);
+            beam->layout();
 
       for (int track = 0; track < tracks; ++track) {
             for (Segment* segment = first(); segment; segment = segment->next()) {
@@ -512,7 +512,7 @@ void Measure::layoutBeams(ScoreLayout* layout)
                         ChordRest* cr = static_cast<ChordRest*>(e);
                         if (cr->beam())
                               continue;
-                        cr->layoutStem(layout);
+                        cr->layoutStem();
                         }
                   }
             }
@@ -538,7 +538,7 @@ QString Beam::xmlType(ChordRest* cr) const
 //   layout1
 //---------------------------------------------------------
 
-void Beam::layout1(ScoreLayout* layout)
+void Beam::layout1()
       {
       //delete old segments
       for (iBeamSegment i = beamSegments.begin(); i != beamSegments.end(); ++i)
@@ -566,17 +566,19 @@ void Beam::layout1(ScoreLayout* layout)
             _up = _direction == UP;
 
       // done twice for beamed chords:
+#if 0 // CS1
       foreach(ChordRest* cr, _elements)
-            cr->layoutAttributes(layout);
+            cr->layoutArticulations();
+#endif
       }
 
 //---------------------------------------------------------
 //   layout
 //---------------------------------------------------------
 
-void Beam::layout(ScoreLayout* layout)
+void Beam::layout()
       {
-      double _spatium = layout->spatium();
+      double _spatium = spatium();
 
       //---------------------------------------------------
       //   calculate direction of beam
@@ -620,45 +622,64 @@ void Beam::layout(ScoreLayout* layout)
       //    - the slope is set to zero on "concave" chord sequences
       //------------------------------------------------------------
 
-      bool concave = false;
-      for (int i = 0; i < _elements.size() - 2; ++i) {
-            int l1 = _elements[i]->line(_up);
-            int l  = _elements[i+1]->line(_up);
-            int l2 = _elements[i+2]->line(_up);
-
-            concave = ((l1 < l2) && ((l < l1) || (l > l2)))
-                    || ((l1 > l2) && ((l > l1) || (l < l2)));
-            if (concave)
-                  break;
-            }
-
-      int l1 = _elements.front()->line(_up);
-      int l2 = _elements.back()->line(_up);
-
-      int cut     = 0;
       qreal slope = 0.0;
+      int cut = 0;
+      if (_userModified) {
+            double p1x = c1->upNote()->canvasPos().x();
+            double p2x = c2->upNote()->canvasPos().x();
+            slope      = (_p2.y() - _p1.y()) / (p2x - p1x);
+            double y2  = _p1.y() + c1->upNote()->chord()->canvasPos().y();
 
-      if (!concave) {
-            double dx = (a2->pos().x() + a2->segment()->pos().x())
-                          - (a1->pos().x() + a1->segment()->pos().x());
-            if (dx) {
-                  slope = (l2 - l1) * _spatium * .5 / dx;
-                  if (fabs(slope) < score()->style()->beamMinSlope) {
-                        cut = slope > 0.0 ? 0 : -1;
-                        slope = 0;
-                        }
-                  else if (slope > score()->style()->beamMaxSlope) {
-                        slope = score()->style()->beamMaxSlope;
-                        cut = 1;
-                        }
-                  else if (-slope > score()->style()->beamMaxSlope) {
-                        slope = -score()->style()->beamMaxSlope;
-                        cut = -1;
-                        }
+            foreach(ChordRest* cr, _elements) {
+                  if (cr->type() != CHORD)
+                        continue;
+                  Chord* c  = static_cast<Chord*>(cr);
+                  QPointF p = c->upNote()->canvasPos();
+                  double y1 = y2 + (p.x() - p1x) * slope;
+                  cr->setUp(y1 < p.y());
                   }
             }
+      else {
+            foreach(ChordRest* cr, _elements)
+                  cr->setUp(_up);
+            bool concave = false;
+            for (int i = 0; i < _elements.size() - 2; ++i) {
+                  int l1 = _elements[i]->line(_up);
+                  int l  = _elements[i+1]->line(_up);
+                  int l2 = _elements[i+2]->line(_up);
 
-      cut *= (_up ? 1 : -1);
+                  concave = ((l1 < l2) && ((l < l1) || (l > l2)))
+                          || ((l1 > l2) && ((l > l1) || (l < l2)));
+                  if (concave)
+                        break;
+                  }
+
+            int l1 = _elements.front()->line(_up);
+            int l2 = _elements.back()->line(_up);
+
+            if (!concave) {
+                  double dx = (a2->pos().x() + a2->segment()->pos().x())
+                                - (a1->pos().x() + a1->segment()->pos().x());
+                  double maxSlope = score()->style(ST_beamMaxSlope).toDouble();
+                  if (dx) {
+                        slope = (l2 - l1) * _spatium * .5 / dx;
+                        if (fabs(slope) < score()->style(ST_beamMinSlope).toDouble()) {
+                              cut = slope > 0.0 ? 0 : -1;
+                              slope = 0;
+                              }
+                        else if (slope > maxSlope) {
+                              slope = maxSlope;
+                              cut = 1;
+                              }
+                        else if (-slope > maxSlope) {
+                              slope = -maxSlope;
+                              cut = -1;
+                              }
+                        }
+                  }
+
+            cut *= (_up ? 1 : -1);
+            }
 
       //---------------------------------------------------
       //    create beam segments
@@ -668,16 +689,18 @@ void Beam::layout(ScoreLayout* layout)
             //   create top beam segment
             //---------------------------------------------
 
-      double xoffLeft  = point(score()->style()->stemWidth)/2;
+      double xoffLeft  = point(score()->styleS(ST_stemWidth)) * .5;
       double xoffRight = xoffLeft;
 
-      QPointF p1s(a1->stemPos(_up, false));
-      QPointF p2s(a2->stemPos(_up, false));
+      QPointF p1s(c1->stemPos(c1->up(), false));
+      QPointF p2s(c2->stemPos(c2->up(), false));
+
       double x1 = p1s.x() - xoffLeft;
       double x2 = p2s.x() + xoffRight;
 
-      QPointF p1, p2;
       double ys = (x2 - x1) * slope;
+
+      QPointF p1, p2;
       if (cut >= 0) {
             // left dot is reference
             p1 = QPointF(x1, p1s.y());
@@ -701,18 +724,21 @@ void Beam::layout(ScoreLayout* layout)
             isGrace = chord->noteType() != NOTE_NORMAL;
             break;
             }
-      Spatium bw        = score()->style()->beamWidth;
-      double bd         = score()->style()->beamDistance;
-      double beamMinLen = point(score()->style()->beamMinLen);
-      double graceMag   = score()->style()->graceNoteMag;
+
+      Spatium bw        = score()->styleS(ST_beamWidth);
+      double bd         = score()->styleD(ST_beamDistance);
+      double beamMinLen = point(score()->styleS(ST_beamMinLen));
+      double graceMag   = score()->styleD(ST_graceNoteMag);
       if (isGrace) {
             setMag(graceMag);
             bw *= graceMag;
             beamMinLen *= graceMag;
             }
-      setMag(isGrace ? graceMag : 1.0);
+      else
+            setMag(1.0);
 
       double beamDist = point(bd * bw + bw) * (_up ? 1.0 : -1.0);
+
       double min      = 1000.0;
       double max      = -1000.0;
 
@@ -721,7 +747,8 @@ void Beam::layout(ScoreLayout* layout)
                   continue;
             Chord* chord  = static_cast<Chord*>(cr);
             QPointF npos(chord->stemPos(_up, true));
-            double bd      = (chord->beams() - 1) * beamDist * (_up ? 1.0 : -1.0);
+            // grow beams with factor 0.5
+            double bd      = (chord->beams() - 1) * beamDist * .5 * (_up ? 1.0 : -1.0);
             double y1      = npos.y();
             double y2      = p1.y() + (npos.x() - x1) * slope;
             double stemLen = _up ? (y1 - y2) : (y2 - y1);
@@ -732,18 +759,20 @@ void Beam::layout(ScoreLayout* layout)
                   max = stemLen;
             }
 
-      // adjst beam position
-      double n = 3.0;   // minimum stem len (should be a style parameter)
-      if (fabs(max-min) > _spatium * 2)
+      // adjust beam position
+      double n = 3.0;
+      if (fabs(max-min) > (_spatium * 2.0))
             n = 2.0;    // reduce minimum stem len (heuristic)
+
       if (isGrace)
             n *= graceMag;
-      double diff = _spatium * n - min;
+      double diff = n * _spatium - min;
       if (_up)
             diff = -diff;
       p1.ry() += diff;
       p2.ry() += diff;
 
+
       //---------------------------------------------------
       if (_userModified) {
             p1.setY(_p1.y());
@@ -878,6 +907,7 @@ void Beam::layout(ScoreLayout* layout)
             if (cr->type() != CHORD)
                   continue;
             Chord* chord = static_cast<Chord*>(cr);
+            bool _up = cr->up();
 
             Stem* stem = chord->stem();
             if (!stem) {
@@ -893,7 +923,7 @@ void Beam::layout(ScoreLayout* layout)
             double y2 = p1.y() + (x2 - x1) * slope;
 
             double stemLen = _up ? (y1 - y2) : (y2 - y1);
-            stem->setLen(spatium(stemLen));
+            stem->setLen(stemLen);
 
             if (_up)
                   npos += QPointF(0, -stemLen);
@@ -902,7 +932,7 @@ void Beam::layout(ScoreLayout* layout)
 
             Tremolo* tremolo = chord->tremolo();
             if (tremolo)
-                  tremolo->layout(layout);
+                  tremolo->layout();
             }
       }
 
@@ -918,7 +948,7 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
             double p1x = c1->upNote()->canvasPos().x();
             double p2x = c2->upNote()->canvasPos().x();
             slope      = (_p2.y() - _p1.y()) / (p2x - p1x);
-            double y2  = _p1.y() + (c1->upNote()->canvasPos() - c1->upNote()->pos()).y();
+            double y2  = _p1.y() + c1->upNote()->chord()->canvasPos().y();
 
             foreach(ChordRest* cr, _elements) {
                   if (cr->type() != CHORD)
@@ -953,7 +983,7 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
             //   create top beam segment
             //---------------------------------------------
 
-      double xoffLeft  = point(score()->style()->stemWidth)/2;
+      double xoffLeft  = point(score()->styleS(ST_stemWidth)) * .5;
       double xoffRight = xoffLeft;
 
       QPointF p1s(c1->stemPos(c1->up(), false));
@@ -971,7 +1001,7 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
       if (_userModified) {
             p1.setY(_p1.y());
             p2.setY(_p2.y());
-            slope = (p2.y() - p1.y()) / (p2.x() - p1.x());
+            slope = (p2.y() - p1.y()) / (p2.x() - p1.x());  // CHECK: already calculated?
             }
       else {
             double yu2, yo1;
@@ -982,9 +1012,6 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
                   if (cr->type() != CHORD)
                         continue;
                   _up = cr->up();
-// printf("  up %d %f %f\n", _up,
-//   cr->stemPos(true, false).y(),
-//   cr->stemPos(false, false).y());
 
                   double y = cr->stemPos(!_up, false).y();
                   if (cr->staffMove() == 0) {
@@ -1005,12 +1032,14 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
 // printf(" = %f   %f %f\n", mY, yu2, yo1);
             p1.ry() = mY;
             p2.ry() = mY;
-            _p1 = p1;
-            _p2 = p2;
             }
+      _p1 = p1;
+      _p2 = p2;
 
-      double beamDist = point(score()->style()->beamDistance * score()->style()->beamWidth
-                        + score()->style()->beamWidth) * (_up ? 1.0 : -1.0);
+      const Style s(score()->style());
+      double bd(s[ST_beamDistance].toDouble());
+      Spatium bw(s[ST_beamWidth].toSpatium());
+      double beamDist = point(bd * bw + bw) * (_up ? 1.0 : -1.0);
       BeamSegment* bs = new BeamSegment;
       beamSegments.push_back(bs);
       bs->p1  = p1;
@@ -1079,7 +1108,7 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
                               bs = new BeamSegment;
                               beamSegments.push_back(bs);
                               double x2 = nn1->stemPos(_up, false).x();
-                              double x3 = x2 + point(score()->style()->beamMinLen);
+                              double x3 = x2 + point(score()->styleS(ST_beamMinLen));
 
                               if (!nn1r) {
                                     double tmp = x3;
@@ -1115,7 +1144,7 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
                   bs = new BeamSegment;
                   beamSegments.push_back(bs);
                   double x3 = nn1->stemPos(_up, false).x();
-                  double x2 = x3 - point(score()->style()->beamMinLen);
+                  double x2 = x3 - point(score()->styleS(ST_beamMinLen));
                   bs->p1 = QPointF(x2, (x2 - x1) * slope + y1);
                   bs->p2 = QPointF(x3, (x3 - x1) * slope + y1);
                   }
@@ -1129,8 +1158,8 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
       foreach(ChordRest* cr, _elements) {
             if (cr->type() != CHORD)
                   continue;
-            Chord* chord = (Chord*)(cr);
-            _up = cr->up();
+            Chord* chord = static_cast<Chord*>(cr);
+            _up = chord->up();
 
             Stem* stem = chord->stem();
             if (!stem) {
@@ -1142,10 +1171,11 @@ void Beam::layoutCrossStaff(int maxTickLen, int move, Chord* c1, Chord* c2)
 
             double x2 = npos.x();
             double y1 = npos.y();
+
             double y2 = p1.y() + (x2 - x1) * slope;
 
             double stemLen = _up ? (y1 - y2) : (y2 - y1);
-            stem->setLen(spatium(stemLen));
+            stem->setLen(stemLen);
 
             if (_up)
                   npos += QPointF(0, -stemLen);
@@ -1181,6 +1211,11 @@ void Beam::write(Xml& xml) const
             xml.tag("y1", _p1.y());
             xml.tag("y2", _p2.y());
             }
+      switch(_direction) {
+            case UP:   xml.tag("StemDirection", QVariant("up")); break;
+            case DOWN: xml.tag("StemDirection", QVariant("down")); break;
+            case AUTO: break;
+            }
       xml.etag();
       }
 
@@ -1202,6 +1237,18 @@ void Beam::read(QDomElement e)
                   _userModified = true;
                   _p2 = QPointF(0.0, val.toDouble());
                   }
+            else if (tag == "StemDirection") {
+                  if (val == "up") {
+                        _direction = UP;
+                        _up = 1;
+                        }
+                  else if (val == "down") {
+                        _direction = DOWN;
+                        _up = 0;
+                        }
+                  else
+                        domError(e);
+                  }
             else if (!Element::readProperties(e))
                   domError(e);
             }
@@ -1239,18 +1286,24 @@ void Beam::updateGrips(int* grips, QRectF* grip) const
 
 bool Beam::isUp()
       {
-      if (_direction == AUTO)
-            return _up;
-      return _direction == UP;
+      return _up;
+      }
+
+void Beam::setBeamDirection(Direction d)
+      {
+      _direction = d;
+      if (d != AUTO)
+            _up = d == UP;
       }
 
 //---------------------------------------------------------
-//   resetUserOffsets
+//   toDefault
 //---------------------------------------------------------
 
-void Beam::resetUserOffsets()
+void Beam::toDefault()
       {
       _direction = AUTO;
+      _up        = -1;
       _userModified = false;
       setGenerated(true);
       }
