@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: beam.h,v 1.6 2006/03/02 17:08:32 wschweer Exp $
+//  $Id: beam.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -84,7 +84,7 @@ class Beam : public Element {
       QList<ChordRest*> _elements;
       BeamSegmentList beamSegments;
       Direction _direction;
-      bool _up;
+      int _up;                // -1: unknown  0: down   1: up
 
       bool _userModified;
       QPointF _p1, _p2;
@@ -108,12 +108,12 @@ class Beam : public Element {
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement);
 
-      virtual void resetUserOffsets();
+      virtual void toDefault();
 
       Measure* measure() const            { return (Measure*)parent(); }
 
-      void layout1(ScoreLayout*);
-      void layout(ScoreLayout*);
+      void layout1();
+      void layout();
 
       const QList<ChordRest*>& elements() { return _elements;  }
       void clear()                        { _elements.clear(); }
@@ -123,12 +123,12 @@ class Beam : public Element {
       virtual void move(double, double);
       virtual QRectF bbox() const;
       virtual void draw(QPainter&) const;
-      bool up() const                     { return _up; }
-      void setUp(bool v)                  { _up = v; }
+      int up() const                      { return _up; }
+      void setUp(int v)                   { _up = v; }
       void setId(int i) const             { _id = i; }
       int id() const                      { return _id; }
 
-      void setBeamDirection(Direction d)  { _direction = d; }
+      void setBeamDirection(Direction d);
       bool isUp();
       };
 
