@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: measure.h,v 1.40 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: box.cpp 1847 2009-05-24 13:05:54Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -51,10 +51,10 @@ Box::Box(Score* score)
 //   layout
 //---------------------------------------------------------
 
-void Box::layout(ScoreLayout* layout)
+void Box::layout()
       {
       foreach (Element* el, _el)
-            el->layout(layout);
+            el->layout();
       }
 
 //---------------------------------------------------------
@@ -74,6 +74,7 @@ void Box::draw(QPainter& p) const
             double w = 2.0 / p.matrix().m11();
             pen.setWidthF(w);
             p.setPen(pen);
+            p.setBrush(QBrush());
             p.drawRect(bbox().adjusted(w, w, -w, -w));
             }
       }
@@ -104,9 +105,9 @@ bool Box::edit(Viewer*, int /*grip*/, int /*key*/, Qt::KeyboardModifiers, const
 void Box::editDrag(int, const QPointF& d)
       {
       if (type() == VBOX)
-            _boxHeight += d.y();
+            _boxHeight += Spatium(d.y() / spatium());
       else {
-            _boxWidth += d.x();
+            _boxWidth += Spatium(d.x() / spatium());
             foreach(Element* e, _el) {
                   if (e->type() == TEXT) {
                         static_cast<Text*>(e)->setModified(true);  // force relayout
@@ -178,6 +179,7 @@ void Box::write(Xml& xml) const
 
 void Box::read(QDomElement e)
       {
+      double _spatium = spatium();
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
             QString val(e.text());
@@ -204,6 +206,16 @@ void Box::read(QDomElement e)
             else if (tag == "Text") {
                   Text* t = new Text(score());
                   t->read(e);
+                  // for backward compatibility
+                  int st = t->subtype();
+                  if (st == TEXT_TITLE)
+                        t->setTextStyle(TEXT_STYLE_TITLE);
+                  else if (st == TEXT_SUBTITLE)
+                        t->setTextStyle(TEXT_STYLE_SUBTITLE);
+                  else if (st == TEXT_COMPOSER)
+                        t->setTextStyle(TEXT_STYLE_COMPOSER);
+                  else if (st == TEXT_POET)
+                        t->setTextStyle(TEXT_STYLE_POET);
                   add(t);
                   }
             else if (tag == "Symbol") {
@@ -218,12 +230,13 @@ void Box::read(QDomElement e)
                   if (!ee.isNull())
                         path = ee.text();
                   Image* image = 0;
-                  if (path.endsWith(".svg"))
+                  QString s(path.toLower());
+                  if (s.endsWith(".svg"))
                         image = new SvgImage(score());
-                  else if (path.endsWith(".jpg")
-                     || path.endsWith(".png")
-                     || path.endsWith(".gif")
-                     || path.endsWith(".xpm")
+                  else if (s.endsWith(".jpg")
+                     || s.endsWith(".png")
+                     || s.endsWith(".gif")
+                     || s.endsWith(".xpm")
                         ) {
                         image = new RasterImage(score());
                         }
@@ -268,21 +281,21 @@ HBox::~HBox()
 //   layout
 //---------------------------------------------------------
 
-void HBox::layout(ScoreLayout* layout)
+void HBox::layout()
       {
       if (parent() && parent()->type() == VBOX) {
             VBox* vb = static_cast<VBox*>(parent());
             double x = vb->leftMargin() * DPMM;
             double y = vb->topMargin() * DPMM;
-            double w = boxWidth().point();
+            double w = point(boxWidth());
             double h = vb->height() - vb->topMargin() * DPMM - vb->bottomMargin() * DPMM;
             setPos(x, y);
             setbbox(QRectF(0.0, 0.0, w, h));
             }
       else {
-            setbbox(QRectF(0.0, 0.0, boxWidth().point(), system()->height()));
+            setbbox(QRectF(0.0, 0.0, point(boxWidth()), system()->height()));
             }
-      Box::layout(layout);
+      Box::layout();
       }
 
 //---------------------------------------------------------
@@ -357,7 +370,7 @@ bool HBox::isMovable() const
 QRectF HBox::drag(const QPointF& pos)
       {
       QRectF r(abbox());
-      setUserOff(QPointF(pos.x() / _spatium, 0.0));
+      setUserOff(QPointF(pos.x(), 0.0));
       return abbox() | r;
       }
 
@@ -407,25 +420,33 @@ void VBox::propertyAction(const QString& cmd)
             }
       Element* s = 0;
       if (cmd == "title-text") {
-            s = new Text(score());
-            s->setSubtype(TEXT_TITLE);
+            Text* t = new Text(score());
+            t->setSubtype(TEXT_TITLE);
+            t->setTextStyle(TEXT_STYLE_TITLE);
+            s = t;
             }
       else if (cmd == "subtitle-text") {
-            s = new Text(score());
-            s->setSubtype(TEXT_SUBTITLE);
+            Text* t = new Text(score());
+            t->setSubtype(TEXT_SUBTITLE);
+            t->setTextStyle(TEXT_STYLE_SUBTITLE);
+            s = t;
             }
       else if (cmd == "composer-text") {
-            s = new Text(score());
-            s->setSubtype(TEXT_COMPOSER);
+            Text* t = new Text(score());
+            t->setSubtype(TEXT_COMPOSER);
+            t->setTextStyle(TEXT_STYLE_COMPOSER);
+            s = t;
             }
       else if (cmd == "poet-text") {
-            s = new Text(score());
-            s->setSubtype(TEXT_POET);
+            Text* t = new Text(score());
+            t->setSubtype(TEXT_POET);
+            t->setTextStyle(TEXT_STYLE_POET);
+            s = t;
             }
       else if (cmd == "insert-hbox") {
             s = new HBox(score());
             double w = width() - leftMargin() * DPMM - rightMargin() * DPMM;
-            static_cast<HBox*>(s)->setBoxWidth(Spatium(w / _spatium));
+            static_cast<HBox*>(s)->setBoxWidth(Spatium(w / spatium()));
             }
       else {
             printf("VBox::propertyAction: %s unknown\n", qPrintable(cmd));
@@ -450,11 +471,11 @@ void VBox::propertyAction(const QString& cmd)
 //   layout
 //---------------------------------------------------------
 
-void VBox::layout(ScoreLayout* layout)
+void VBox::layout()
       {
       setPos(QPointF());      // !?
-      setbbox(QRectF(0.0, 0.0, system()->width(), boxHeight().point()));
-      Box::layout(layout);
+      setbbox(QRectF(0.0, 0.0, system()->width(), point(boxHeight())));
+      Box::layout();
       }
 
 //---------------------------------------------------------
