@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: bracket.cpp,v 1.10 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: bracket.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -25,7 +25,6 @@
 #include "utils.h"
 #include "staff.h"
 #include "score.h"
-#include "layout.h"
 #include "viewer.h"
 #include "system.h"
 #include "sym.h"
@@ -37,7 +36,7 @@
 Bracket::Bracket(Score* s)
    : Element(s)
       {
-      h2       = 3.5 * _spatium;
+      h2       = 3.5 * spatium();
       _span    = 1;
       _column   = 0;
       yoff     = 0.0;
@@ -60,9 +59,9 @@ double Bracket::width() const
       {
       double w;
       if (subtype() == BRACKET_AKKOLADE)
-            w = point(score()->style()->akkoladeWidth);
+            w = point(score()->styleS(ST_akkoladeWidth));
       else
-            w = point(score()->style()->bracketWidth + score()->style()->bracketDistance);
+            w = point(score()->styleS(ST_bracketWidth) + score()->styleS(ST_bracketDistance));
       return w;
       }
 
@@ -70,13 +69,9 @@ double Bracket::width() const
 //   layout
 //---------------------------------------------------------
 
-void Bracket::layout(ScoreLayout* layout)
+void Bracket::layout()
       {
-      double _spatium;
-      if (layout)
-            _spatium = layout->spatium();
-      else
-            _spatium = ::_spatium;
+      double _spatium = spatium();
       path = QPainterPath();
       if (h2 == 0.0)
             return;
@@ -84,7 +79,7 @@ void Bracket::layout(ScoreLayout* layout)
       qreal h = h2 + yoff * .5;
 
       if (subtype() == BRACKET_AKKOLADE) {
-            qreal w         = point(score()->style()->akkoladeWidth);
+            qreal w         = point(score()->styleS(ST_akkoladeWidth));
             const double X1 =  2.0 * w;
             const double X2 = -0.7096 * w;
             const double X3 = -1.234 * w;
@@ -97,7 +92,7 @@ void Bracket::layout(ScoreLayout* layout)
             path.cubicTo(X3,  h - h * .5025, X4,  h - h * .2413, 0, h);
             }
       else if (subtype() == BRACKET_NORMAL) {
-            qreal w = point(score()->style()->bracketWidth);
+            qreal w = point(score()->styleS(ST_bracketWidth));
 
             TextStyle* s = score()->textStyle(TEXT_STYLE_SYMBOL1);
             QChar up = symbols[brackettipsUp].code();
@@ -106,7 +101,7 @@ void Bracket::layout(ScoreLayout* layout)
             QFont f(s->family, lrint(2.0 * _spatium));
 
             qreal o   = _spatium * .17;
-            qreal slw = point(score()->style()->staffLineWidth);
+            qreal slw = point(score()->styleS(ST_staffLineWidth));
 
             path.setFillRule(Qt::WindingFill);
 
@@ -227,7 +222,7 @@ void Bracket::editDrag(int, const QPointF& delta)
       qreal dy = delta.y();
       yoff += dy;
 
-      layout(0);
+      layout();
       }
 
 //---------------------------------------------------------
