@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: bracket.h,v 1.6 2006/03/02 17:08:33 wschweer Exp $
+//  $Id: bracket.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -59,7 +59,7 @@ class Bracket : public Element {
       virtual void draw(QPainter&) const;
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
 
       virtual bool startEdit(Viewer*, const QPointF&);
       virtual bool edit(Viewer*, int, int, Qt::KeyboardModifiers, const QString&);
