@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: canvas.cpp,v 1.80 2006/09/15 09:34:57 wschweer Exp $
+//  $Id: canvas.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -39,7 +39,6 @@
 #include "xml.h"
 #include "text.h"
 #include "note.h"
-#include "layout.h"
 #include "dynamics.h"
 #include "pedal.h"
 #include "volta.h"
@@ -55,6 +54,7 @@
 #include "barline.h"
 #include "system.h"
 #include "magbox.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   Canvas
@@ -87,14 +87,9 @@ Canvas::Canvas(QWidget* parent)
       state            = NORMAL;
       cursor           = 0;
       shadowNote       = 0;
-      cursorTimer      = new QTimer(this);
       mousePressed     = false;
       grips            = 0;
 
-      connect(cursorTimer, SIGNAL(timeout()), SLOT(cursorBlink()));
-      if (preferences.cursorBlink)
-            cursorTimer->start(500);
-
       if (debugMode)
             setMouseTracking(true);
 
@@ -128,7 +123,7 @@ bool Canvas::navigatorVisible() const
 //   setScore
 //---------------------------------------------------------
 
-void Canvas::setScore(Score* s, ScoreLayout* l)
+void Canvas::setScore(Score* s)
       {
       _score = s;
       if (cursor == 0) {
@@ -141,7 +136,6 @@ void Canvas::setScore(Score* s, ScoreLayout* l)
             cursor->setScore(_score);
             shadowNote->setScore(_score);
             }
-      _layout = l;
       if (navigator) {
             navigator->setScore(_score);
             updateNavigator(false);
@@ -175,20 +169,6 @@ bool Canvas::event(QEvent* ev)
       return QWidget::event(ev);
       }
 
-//---------------------------------------------------------
-//   cursorBlink
-//    cursor timer slot
-//---------------------------------------------------------
-
-void Canvas::cursorBlink()
-      {
-      if (!preferences.cursorBlink) {
-            cursor->noBlinking();
-            cursorTimer->stop();
-            }
-      cursor->blink();
-      }
-
 //---------------------------------------------------------
 //   Canvas
 //---------------------------------------------------------
@@ -196,10 +176,8 @@ void Canvas::cursorBlink()
 Canvas::~Canvas()
       {
       delete lasso;
-      if (cursor)
-            delete cursor;
-      if (shadowNote)
-            delete shadowNote;
+      delete cursor;
+      delete shadowNote;
       }
 
 //---------------------------------------------------------
@@ -366,6 +344,7 @@ void Canvas::resizeEvent(QResizeEvent*)
 void Canvas::updateNavigator(bool layoutChanged) const
       {
       if (navigator) {
+            navigator->setScore(_score);  // adapt to new paper size
             if (layoutChanged)
                   navigator->layoutChanged();
             QRectF r(0.0, 0.0, width(), height());
@@ -410,11 +389,11 @@ void Canvas::mousePressEvent(QMouseEvent* ev)
       dragObject = elementNear(startMove);
 
       if (mscore->playEnabled() && dragObject && dragObject->type() == NOTE) {
-            Note* note = (Note*)dragObject;
+            Note* note = static_cast<Note*>(dragObject);
             Part* part = note->staff()->part();
-            int pitch  = note->pitch() + part->pitchOffset();
+            int pitch  = note->ppitch();
             Instrument* i = part->instrument();
-            seq->startNote(i->channel[note->subchannel()], pitch, 60, 1000);
+            seq->startNote(i->channel[note->subchannel()], pitch, 60, 1000, note->tuning());
             }
 
       //-----------------------------------------
@@ -457,7 +436,6 @@ void Canvas::mousePressEvent(QMouseEvent* ev)
                   }
             return;
             }
-
       if (state != EDIT)
             _score->startCmd();
       switch (state) {
@@ -592,12 +570,12 @@ void Canvas::mouseDoubleClickEvent(QMouseEvent* ev)
 void Canvas::mouseMoveEvent(QMouseEvent* ev)
       {
       if (QApplication::mouseButtons() == Qt::MidButton && dragObject) {
-            QString mimeType = _score->sel->mimeType();
+            QString mimeType = _score->selection()->mimeType();
             if (!mimeType.isEmpty()) {
                   QDrag* drag = new QDrag(this);
                   drag->setPixmap(QPixmap());
                   QMimeData* mimeData = new QMimeData;
-                  mimeData->setData(mimeType, _score->sel->mimeData());
+                  mimeData->setData(mimeType, _score->selection()->mimeData());
                   drag->setMimeData(mimeData);
                   _score->endCmd();
                   drag->start(Qt::CopyAction);
@@ -655,18 +633,18 @@ void Canvas::mouseMoveEvent1(QMouseEvent* ev)
                   {
                   if (dragObject && (QApplication::keyboardModifiers() == Qt::ShiftModifier)) {
                         // drag selection
-                        QString mimeType = _score->sel->mimeType();
+                        QString mimeType = _score->selection()->mimeType();
                         if (!mimeType.isEmpty()) {
                               QDrag* drag = new QDrag(this);
                               QMimeData* mimeData = new QMimeData;
-                              mimeData->setData(mimeType, score()->sel->mimeData());
+                              mimeData->setData(mimeType, score()->selection()->mimeData());
                               drag->setMimeData(mimeData);
                               _score->endCmd();
                               //
                               //  also set into the clipboard
                               //
                               QMimeData* clip = new QMimeData();
-                              clip->setData(mimeType, score()->sel->mimeData());
+                              clip->setData(mimeType, score()->selection()->mimeData());
                               QApplication::clipboard()->setMimeData(clip);
                               drag->start(Qt::CopyAction);
                               }
@@ -674,9 +652,9 @@ void Canvas::mouseMoveEvent1(QMouseEvent* ev)
                         }
                   if (dragObject && dragObject->isMovable()) {
                         QPointF o;
-                        if (_score->sel->state() != SEL_STAFF && _score->sel->state() != SEL_SYSTEM) {
+                        if (_score->selection()->state() != SEL_STAFF && _score->selection()->state() != SEL_SYSTEM) {
                               _score->startDrag(dragObject);
-                              o = QPointF(dragObject->userOff() * _spatium);
+                              o = QPointF(dragObject->userOff());
                               setState(DRAG_OBJ);
                               }
                         startMove -= o;
@@ -789,6 +767,11 @@ void Canvas::updateGrips()
 
 void Canvas::mouseReleaseEvent(QMouseEvent* ev)
       {
+      if (ev->button() == Qt::RightButton && ev->buttons() == Qt::LeftButton) {
+            _score->endCmd();
+            return;
+            }
+
       if (ev->buttons() == 0)
             level = 0;
       if (dragCanvasState) {
@@ -863,18 +846,15 @@ void Canvas::mouseReleaseEvent(QMouseEvent* ev)
 
 void Canvas::setBackground(QPixmap* pm)
       {
-      if (bgPixmap)
-            delete bgPixmap;
+      delete bgPixmap;
       bgPixmap = pm;
       update();
       }
 
 void Canvas::setBackground(const QColor& color)
       {
-      if (bgPixmap) {
-            delete bgPixmap;
-            bgPixmap = 0;
-            }
+      delete bgPixmap;
+      bgPixmap = 0;
       _bgColor = color;
       update();
       }
@@ -885,18 +865,15 @@ void Canvas::setBackground(const QColor& color)
 
 void Canvas::setForeground(QPixmap* pm)
       {
-      if (fgPixmap)
-            delete fgPixmap;
+      delete fgPixmap;
       fgPixmap = pm;
       update();
       }
 
 void Canvas::setForeground(const QColor& color)
       {
-      if (fgPixmap) {
-            delete fgPixmap;
-            fgPixmap = 0;
-            }
+      delete fgPixmap;
+      fgPixmap = 0;
       _fgColor = color;
       update();
       }
@@ -1105,12 +1082,10 @@ void Canvas::moveCursor()
       if (track == -1)
             track = 0;
 
-      double d = _spatium * .5;
-
+      double d = cursor->spatium() * .5;
       if (track == cursor->track() && cursor->tick() == _score->inputPos()) {
             _score->addRefresh(cursor->abbox().adjusted(-d, -d, 2*d, 2*d));
             return;
-//            return;
             }
 
       cursor->setTrack(track);
@@ -1118,37 +1093,36 @@ void Canvas::moveCursor()
 
       Segment* segment = _score->tick2segment(cursor->tick());
       if (segment) {
-            System* system = segment->measure()->system();
-            if (system == 0) {
-                  // a new measure was appended but no layout took place
-printf("zero SYSTEM\n");
-                  return;
-                  }
-            cursor->setSegment(segment);
-            double x = segment->canvasPos().x();
-            double y = system->bboxStaff(cursor->staffIdx()).y() + system->canvasPos().y();
-
-            _score->addRefresh(cursor->abbox().adjusted(-d, -d, 2*d, 2*d));
-            cursor->setPos(x - _spatium, y - _spatium);
-            cursor->setHeight(6.0 * _spatium);
-            _score->addRefresh(cursor->abbox().adjusted(-d, -d, 2*d, 2*d));
+            moveCursor(segment, track / VOICES);
             return;
             }
-//      printf("cursor position not found for tick %d, append new measure\n", cursor->tick());
+// printf("cursor position not found for tick %d, append new measure\n", cursor->tick());
       }
 
-void Canvas::moveCursor(Segment* segment)
+void Canvas::moveCursor(Segment* segment, int staffIdx)
       {
       System* system = segment->measure()->system();
-      double x       = segment->canvasPos().x();
-      double y       = system->bboxStaff(0).y() + system->canvasPos().y();
-      double y2      = system->bboxStaff(_score->nstaves()-1).y() + system->canvasPos().y();
+      if (system == 0) {
+            // a new measure was appended but no layout took place
+            printf("zero SYSTEM\n");
+            return;
+            }
+      cursor->setSegment(segment);
+      int idx   = staffIdx == -1 ? 0 : staffIdx;
+      double x  = segment->canvasPos().x();
+      double y  = system->bboxStaff(idx).y() + system->canvasPos().y();
+      double y2 = system->bboxStaff(_score->nstaves()-1).y() + system->canvasPos().y();
 
-      _score->addRefresh(cursor->abbox());
+      double _spatium = cursor->spatium();
+      double d = _spatium * .5;
+      _score->addRefresh(cursor->abbox().adjusted(-d, -d, 2*d, 2*d));
       cursor->setPos(x - _spatium, y - _spatium);
-      cursor->setHeight(y2 - y + 6.0 * _spatium);
+      if (staffIdx == -1)
+            cursor->setHeight(y2 - y + 6.0 * _spatium);
+      else
+            cursor->setHeight(6.0 * _spatium);
+      _score->addRefresh(cursor->abbox().adjusted(-d, -d, 2*d, 2*d));
       cursor->setTick(segment->tick());
-      _score->addRefresh(cursor->abbox());
       }
 
 //---------------------------------------------------------
@@ -1168,19 +1142,21 @@ void Canvas::setCursorOn(bool val)
 void Canvas::setShadowNote(const QPointF& p)
       {
       Position pos;
-      bool divideSegment = score()->padState()->tickLen >= (division/2);
-      if (!score()->getPosition(&pos, p, divideSegment))
+      if (!score()->getPosition(&pos, p, score()->inputState().voice())) {
+            shadowNote->setVisible(false);
             return;
+            }
 
+      shadowNote->setVisible(true);
       Staff* staff      = score()->staff(pos.staffIdx);
+      shadowNote->setMag(staff->mag());
       Instrument* instr = staff->part()->instrument();
       int notehead      = 0;
       int line          = pos.line;
 
       if (instr->useDrumset) {
             Drumset* ds  = instr->drumset;
-            PadState* ps = score()->padState();
-            int pitch    = ps->drumNote;
+            int pitch    = score()->inputState().drumNote;
             if (pitch >= 0 && ds->isValid(pitch)) {
                   line     = ds->line(pitch);
                   notehead = ds->noteHead(pitch);
@@ -1188,7 +1164,7 @@ void Canvas::setShadowNote(const QPointF& p)
             }
       shadowNote->setLine(line);
       shadowNote->setHeadGroup(notehead);
-      shadowNote->setPos(pos.pos.x(), pos.pos.y());
+      shadowNote->setPos(pos.pos);
       }
 
 //---------------------------------------------------------
@@ -1218,7 +1194,7 @@ void Canvas::paintEvent(QPaintEvent* ev)
 
 //            unsigned long long a = cycles();
 // printf("paint event layout\n");
-            _score->layout()->doLayout();
+            _score->doLayout();
 //            unsigned long long b = (cycles() - a) / 1000000LL;
 //            printf("layout %lld\n", b);
 
@@ -1238,8 +1214,8 @@ void Canvas::paintEvent(QPaintEvent* ev)
             paint(r, p);
 
 #if 0
-      foreach (const Page* page, _layout->pages()) {
-            QRect pr = _matrix.mapRect(page->abbox()).toRect();
+      foreach (const Page* page, _score->pages()) {
+            QRect pr = _matrix.mapRect(page->abbox()).toAlignedRect();
             int o = pr.width() / 100;
             int x = pr.x();
             int y = pr.y();
@@ -1263,9 +1239,24 @@ void Canvas::paintEvent(QPaintEvent* ev)
             p.drawLine(dropAnchor);
             }
       if (dragElement) {
-            p.translate(dragElement->canvasPos());
+            QList<const Element*> el;
+            dragElement->collectElements(el);
             p.setPen(preferences.defaultColor);
-            dragElement->draw(p);
+            foreach(const Element* e, el) {
+                  p.save();
+                  p.translate(e->canvasPos());
+                  e->draw(p);
+                  p.restore();
+#if 0
+                  if (debugMode) {
+                        //
+                        //  draw bounding box rectangle
+                        //
+                        p.setPen(QPen(Qt::red, 0, Qt::SolidLine));
+                        p.drawRect(e->abbox());
+                        }
+#endif
+                  }
             }
       }
 
@@ -1288,11 +1279,11 @@ void Canvas::paint(const QRect& rr, QPainter& p)
       QRectF fr = imatrix.mapRect(QRectF(rr));
 
       QRegion r1(rr);
-      foreach (const Page* page, _layout->pages())
-            r1 -= _matrix.mapRect(page->abbox()).toRect();
+      foreach (const Page* page, _score->pages())
+            r1 -= _matrix.mapRect(page->abbox()).toAlignedRect();
 //      p.setClipRect(fr);
 
-      QList<const Element*> ell = _layout->items(fr);
+      QList<const Element*> ell = _score->items(fr);
       drawElements(p, ell);
 
       if (dropRectangle.isValid())
@@ -1330,12 +1321,22 @@ void Canvas::paint(const QRect& rr, QPainter& p)
             QPen pen(QColor(Qt::blue));
             pen.setWidthF(2.0 / p.matrix().m11());
 
-            if (sel->state() == SEL_SYSTEM)
+            if (sel->state() == SEL_SYSTEM){
                   pen.setStyle(Qt::DotLine);
+				  #ifdef Q_WS_MAC
+				  //TODO: remove if qt fix. This is a workaround for a qt bug on mac apparenlty
+				  //For dotline the spaces are not proportional to the line width except for custom dash
+				  QVector<qreal> dashes;
+				  qreal space = 2;
+				  dashes << 2 << space << 2;
+				  pen.setDashPattern(dashes);
+				  #endif
+				  }
             else
                   pen.setStyle(Qt::SolidLine);
 
             p.setPen(pen);
+            double _spatium = score()->spatium();
             double x2      = ss->canvasPos().x() - _spatium;
             int staffStart = sel->staffStart;
             int staffEnd   = sel->staffEnd;
@@ -1365,7 +1366,7 @@ void Canvas::paint(const QRect& rr, QPainter& p)
                   // HACK for whole measure rest:
                   if (ns == 0 || ns == es) {    // last segment?
                         Element* e = s->element(staffStart * VOICES);
-                        if (e && e->type() == REST && e->tickLen() == 0)
+                        if (e && e->type() == REST && static_cast<Rest*>(e)->tickLen() == 0)
                               x2 = s->measure()->abbox().right() - _spatium;
                         }
 
@@ -1528,10 +1529,9 @@ bool Canvas::dragAboveSystem(const QPointF& pos)
 
 void Canvas::dragEnterEvent(QDragEnterEvent* event)
       {
-      if (dragElement) {
-            delete dragElement;
-            dragElement = 0;
-            }
+      double _spatium = score()->spatium();
+      delete dragElement;
+      dragElement = 0;
 
       const QMimeData* data = event->mimeData();
 
@@ -1547,7 +1547,9 @@ void Canvas::dragEnterEvent(QDragEnterEvent* event)
 
             QByteArray a = data->data(mimeSymbolFormat);
 
-// printf("DRAG<%s>\n", a.data());
+            if (debugMode)
+                  printf("DRAG<%s>\n", a.data());
+
             QDomDocument doc;
             int line, column;
             QString err;
@@ -1560,18 +1562,20 @@ void Canvas::dragEnterEvent(QDragEnterEvent* event)
             QDomElement e = doc.documentElement();
 
             dragOffset = QPoint();
-            int type = Element::readType(e, &dragOffset);
+            ElementType type = Element::readType(e, &dragOffset);
 
             Element* el = 0;
             switch(type) {
+                  case SLUR:
+                        el = Element::create(type, gscore);
+                        break;
                   case VOLTA:
                   case OTTAVA:
                   case TRILL:
                   case PEDAL:
                   case HAIRPIN:
                   case TEXTLINE:
-                        el = Element::create(type, score());
-                        ((SLine*)el)->setLen(_spatium * 7);
+                        el = Element::create(type, gscore);
                         break;
                   case IMAGE:
                         {
@@ -1585,12 +1589,14 @@ void Canvas::dragEnterEvent(QDragEnterEvent* event)
                                     }
                               }
                         Image* image = 0;
-                        if (path.endsWith(".svg"))
+                        QString lp(path.toLower());
+
+                        if (lp.endsWith(".svg"))
                               image = new SvgImage(score());
-                        else if (path.endsWith(".jpg")
-                           || path.endsWith(".png")
-                           || path.endsWith(".gif")
-                           || path.endsWith(".xpm")
+                        else if (lp.endsWith(".jpg")
+                           || lp.endsWith(".png")
+                           || lp.endsWith(".gif")
+                           || lp.endsWith(".xpm")
                               )
                               image = new RasterImage(score());
                         else {
@@ -1604,7 +1610,7 @@ void Canvas::dragEnterEvent(QDragEnterEvent* event)
                   case TIMESIG:
                   case BREATH:
                   case GLISSANDO:
-                  case ATTRIBUTE:
+                  case ARTICULATION:
                   case ACCIDENTAL:
                   case DYNAMIC:
                   case TEXT:
@@ -1637,7 +1643,7 @@ void Canvas::dragEnterEvent(QDragEnterEvent* event)
                   dragElement = el;
                   dragElement->setParent(0);
                   dragElement->read(e);
-                  dragElement->layout(score()->layout());
+                  dragElement->layout();
                   }
             }
 
@@ -1648,11 +1654,12 @@ void Canvas::dragEnterEvent(QDragEnterEvent* event)
                         printf("drag Url: %s\n", qPrintable(u.toString()));
                   if (u.scheme() == "file") {
                         QFileInfo fi(u.path());
-                        if (fi.suffix() == "svg"
-                           || fi.suffix() == "jpg"
-                           || fi.suffix() == "png"
-                           || fi.suffix() == "gif"
-                           || fi.suffix() == "xpm"
+                        QString suffix = fi.suffix().toLower();
+                        if (suffix == "svg"
+                           || suffix == "jpg"
+                           || suffix == "png"
+                           || suffix == "gif"
+                           || suffix == "xpm"
                            ) {
                               event->acceptProposedAction();
                               break;
@@ -1728,7 +1735,7 @@ void Canvas::dragMoveEvent(QDragMoveEvent* event)
                   case BREATH:
                   case GLISSANDO:
                   case BRACKET:
-                  case ATTRIBUTE:
+                  case ARTICULATION:
                   case ACCIDENTAL:
                   case TEXT:
                   case TEMPO_TEXT:
@@ -1742,6 +1749,7 @@ void Canvas::dragMoveEvent(QDragMoveEvent* event)
                   case ICON:
                   case CHORD:
                   case SPACER:
+                  case SLUR:
                         {
                         Element* el = elementAt(pos);
                         if (el && el->acceptDrop(this, pos, dragElement->type(), dragElement->subtype()))
@@ -1753,6 +1761,7 @@ void Canvas::dragMoveEvent(QDragMoveEvent* event)
                         break;
                   }
             score()->addRefresh(dragElement->abbox());
+            QRectF r(dragElement->abbox());
             dragElement->setPos(pos - dragOffset);
             score()->addRefresh(dragElement->abbox());
             _score->end();
@@ -1764,11 +1773,12 @@ void Canvas::dragMoveEvent(QDragMoveEvent* event)
             QUrl u = ul.front();
             if (u.scheme() == "file") {
                   QFileInfo fi(u.path());
-                  if (fi.suffix() != "svg"
-                     && fi.suffix() != "jpg"
-                     && fi.suffix() != "png"
-                     && fi.suffix() != "gif"
-                     && fi.suffix() != "xpm"
+                  QString suffix(fi.suffix().toLower());
+                  if (suffix != "svg"
+                     && suffix != "jpg"
+                     && suffix != "png"
+                     && suffix != "gif"
+                     && suffix != "xpm"
                      )
                         return;
                   //
@@ -1826,6 +1836,7 @@ void Canvas::dropEvent(QDropEvent* event)
 
       if (dragElement) {
             _score->startCmd();
+            dragElement->setScore(_score);
             _score->addRefresh(dragElement->abbox());
             switch(dragElement->type()) {
                   case VOLTA:
@@ -1835,6 +1846,7 @@ void Canvas::dropEvent(QDropEvent* event)
                   case DYNAMIC:
                   case HAIRPIN:
                   case TEXTLINE:
+                        dragElement->setScore(score());
                         score()->cmdAdd1(dragElement, pos, dragOffset);
                         event->acceptProposedAction();
                         break;
@@ -1872,7 +1884,7 @@ void Canvas::dropEvent(QDropEvent* event)
                   case BREATH:
                   case GLISSANDO:
                   case BRACKET:
-                  case ATTRIBUTE:
+                  case ARTICULATION:
                   case ACCIDENTAL:
                   case TEXT:
                   case TEMPO_TEXT:
@@ -1887,6 +1899,7 @@ void Canvas::dropEvent(QDropEvent* event)
                   case NOTE:
                   case CHORD:
                   case SPACER:
+                  case SLUR:
                         {
                         Element* el = elementAt(pos);
                         if (!el) {
@@ -1922,12 +1935,13 @@ void Canvas::dropEvent(QDropEvent* event)
             if (u.scheme() == "file") {
                   QFileInfo fi(u.path());
                   Image* s = 0;
-                  if (fi.suffix() == "svg")
+                  QString suffix = fi.suffix().toLower();
+                  if (suffix == "svg")
                         s = new SvgImage(score());
-                  else if (fi.suffix() == "jpg"
-                     || fi.suffix() == "png"
-                     || fi.suffix() == "gif"
-                     || fi.suffix() == "xpm"
+                  else if (suffix == "jpg"
+                     || suffix == "png"
+                     || suffix == "gif"
+                     || suffix == "xpm"
                         )
                         s = new RasterImage(score());
                   else
@@ -1985,10 +1999,15 @@ if (debugMode)
             }
       else {
             printf("cannot drop this object: unknown mime type\n");
+            QStringList sl = md->formats();
+            foreach(QString s, sl)
+                  printf("  %s\n", qPrintable(s));
             _score->end();
             return;
             }
 
+// printf("drop <%s>\n", data.data());
+
       QDomDocument doc;
       int line, column;
       QString err;
@@ -2146,7 +2165,7 @@ static bool elementLower(const Element* e1, const Element* e2)
 
 const QList<const Element*> Canvas::elementsAt(const QPointF& p)
       {
-      QList<const Element*> el = _layout->items(p);
+      QList<const Element*> el = _score->items(p);
       qSort(el.begin(), el.end(), elementLower);
       return el;
       }
@@ -2177,7 +2196,7 @@ Element* Canvas::elementNear(const QPointF& p)
       double w  = (preferences.proximity * .5) / matrix().m11();
       QRectF r(p.x() - w, p.y() - w, 3.0 * w, 3.0 * w);
 
-      QList<const Element*> el = _layout->items(r);
+      QList<const Element*> el = _score->items(r);
       QList<const Element*> ll;
       for (int i = 0; i < el.size(); ++i) {
             const Element* e = el.at(i);
@@ -2205,7 +2224,8 @@ Element* Canvas::elementNear(const QPointF& p)
       foreach(const Element* e, el)
             printf("  %s %d\n", e->name(), e->selected());
 #endif
-      return const_cast<Element*>(ll.at(level % ll.size()));
+      Element* e = const_cast<Element*>(ll.at(level % ll.size()));
+      return e;
       }
 
 //---------------------------------------------------------
@@ -2214,8 +2234,7 @@ Element* Canvas::elementNear(const QPointF& p)
 
 void Canvas::drawElements(QPainter& p,const QList<const Element*>& el)
       {
-      for (int i = 0; i < el.size(); ++i) {
-            const Element* e = el.at(i);
+      foreach(const Element* e, el) {
             e->itemDiscovered = 0;
             if (!e->visible()) {
                   if (score()->printing() || !score()->showInvisible())
@@ -2294,7 +2313,7 @@ void Canvas::paintLasso(QPainter& p, double mag)
       p.setRenderHint(QPainter::Antialiasing, true);
       p.setRenderHint(QPainter::TextAntialiasing, true);
 
-      QList<const Element*> el = _layout->items(QRectF(0.0, 0.0, 100000.0, 1000000.0));
+      QList<const Element*> el = _score->items(QRectF(0.0, 0.0, 100000.0, 1000000.0));
       drawElements(p, el);
       cursor->draw(p);
 
