@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: canvas.h,v 1.35 2006/09/15 09:34:57 wschweer Exp $
+//  $Id: canvas.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -51,7 +51,6 @@ class Canvas : public QFrame, public Viewer {
 
    private:
       Navigator* navigator;
-      ScoreLayout* _layout;
       int level;
 
       State state;
@@ -72,7 +71,6 @@ class Canvas : public QFrame, public Viewer {
 
       //--input state:
       Cursor* cursor;
-      QTimer* cursorTimer;    // blink timer
       ShadowNote* shadowNote;
 
       Lasso* lasso;           ///< temporarily drawn lasso selection
@@ -126,9 +124,6 @@ class Canvas : public QFrame, public Viewer {
       void updateGrips();
       const QList<const Element*> elementsAt(const QPointF&);
 
-   private slots:
-      void cursorBlink();
-
    public slots:
       void cmdCut();
       void cmdCopy();
@@ -149,7 +144,7 @@ class Canvas : public QFrame, public Viewer {
       void modifyElement(Element* obj);
 
       virtual void moveCursor();
-      virtual void moveCursor(Segment*);
+      virtual void moveCursor(Segment*, int staffIdx);
       virtual void setCursorOn(bool);
       void clearScore();
 
@@ -157,7 +152,7 @@ class Canvas : public QFrame, public Viewer {
       void setState(State);
       State getState() const { return state; }
       bool startEdit(Element*, int startGrip = -1);
-      void setScore(Score* s, ScoreLayout*);
+      void setScore(Score* s);
 
       qreal mag() const;
       qreal xoffset() const;
