@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: chord.h,v 1.3 2006/03/02 17:08:33 wschweer Exp $
+//  $Id: chord.h 1917 2009-07-06 17:07:19Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -46,27 +46,29 @@ class Glissando;
 */
 
 class Stem : public Element {
-      Spatium _len;
+      double _len;
       Spatium _userLen;
 
    public:
       Stem(Score*);
       Stem &operator=(const Stem&);
 
-      virtual Stem* clone() const { return new Stem(*this); }
+      virtual Stem* clone() const      { return new Stem(*this); }
       virtual ElementType type() const { return STEM; }
       virtual void draw(QPainter& p) const;
-      void setLen(const Spatium&);
-      Spatium stemLen() const { return _len + _userLen; }
+      void setLen(double v)            { _len = v; }
+      double stemLen() const           { return _len + point(_userLen); }
       virtual QRectF bbox() const;
-      virtual bool isMovable() const  { return true; }
-      virtual bool startEdit(Viewer*, const QPointF&);
+      virtual bool isMovable() const   { return true; }
+      virtual bool startEdit(Viewer*, const QPointF&) { return true; }
+
       virtual void editDrag(int, const QPointF&);
       virtual void updateGrips(int*, QRectF*) const;
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement e);
-      virtual void resetUserOffsets();
+      virtual void toDefault();
       Spatium userLen() const         { return _userLen; }
+      virtual void setVisible(bool f);
       };
 
 //---------------------------------------------------------
@@ -102,7 +104,7 @@ class LedgerLine : public Line {
       LedgerLine(Score*);
       LedgerLine &operator=(const LedgerLine&);
       virtual LedgerLine* clone() const { return new LedgerLine(*this); }
-      virtual ElementType type() const { return LEDGER_LINE; }
+      virtual ElementType type() const  { return LEDGER_LINE; }
       virtual QPointF canvasPos() const;      ///< position in canvas coordinates
       Chord* chord() const { return (Chord*)parent(); }
       };
@@ -150,14 +152,16 @@ class Chord : public ChordRest {
       Arpeggio* _arpeggio;
       Tremolo* _tremolo;
       Glissando* _glissando;
-      NoteType _noteType;     ///< mark grace notes: acciaccatura and appoggiatura
+      NoteType _noteType;           ///< mark grace notes: acciaccatura and appoggiatura
       bool _noStem;
       double _dotPosX;
+      double minSpace, extraSpace;  ///< cached values
 
       virtual qreal upPos()   const;
       virtual qreal downPos() const;
       virtual qreal centerX() const;
-      void addLedgerLine(double x, int staffIdx, int line);
+      void addLedgerLine(double x, int staffIdx, int line, int extend);
+      void addLedgerLines(double x, int move);
 
    public:
       Chord(Score*);
@@ -183,8 +187,8 @@ class Chord : public ChordRest {
 
       QList<LedgerLine*>* ledgerLines()      { return &_ledgerLines; }
 
-      virtual void layoutStem1(ScoreLayout*);
-      virtual void layoutStem(ScoreLayout*);
+      virtual void layoutStem1();
+      virtual void layoutStem();
       NoteList* noteList()                   { return &notes; }
       const NoteList* noteList() const       { return &notes; }
       const Note* upNote() const             { return notes.back(); }
@@ -207,7 +211,8 @@ class Chord : public ChordRest {
       virtual void remove(Element*);
 
       Note* selectedNote() const;
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
+      void layout2();
 
       virtual int upLine() const;
       virtual int downLine() const;
@@ -227,6 +232,7 @@ class Chord : public ChordRest {
       void setDotPosX(double val)         { _dotPosX = val;  }
       bool noStem() const                 { return _noStem;  }
       void setNoStem(bool val)            { _noStem = val;   }
+      virtual void setMag(double val);
       };
 
 #endif
