@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: chordedit.cpp 1933 2009-07-15 16:44:28Z wschweer $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -80,13 +80,11 @@ ChordEdit::ChordEdit(Score* s, QWidget* parent)
       extensionGroup->addButton(extOther,  0);
 
       extOtherCombo->clear();
-      const ChordDescription* cd = Harmony::chords();
-      unsigned int n = Harmony::chordListSize();
-
-      for (unsigned int i = 0; i < n; ++i) {           // HACK
-            const char* p = cd[i].name;
-            if (p)
-            	extOtherCombo->addItem(p, cd[i].id);
+      ChordList* cl = score->style().chordList();
+      foreach (const ChordDescription* cd, *cl) {
+            QString p(cd->name);
+            if (!p.isEmpty())
+            	extOtherCombo->addItem(p, cd->id);
             }
       connect(rootGroup, SIGNAL(buttonClicked(int)), SLOT(chordChanged()));
       connect(extensionGroup, SIGNAL(buttonClicked(int)), SLOT(chordChanged()));
@@ -121,8 +119,7 @@ ChordEdit::ChordEdit(Score* s, QWidget* parent)
 
 ChordEdit::~ChordEdit()
       {
-      if (_harmony)
-            delete _harmony;
+      delete _harmony;
       }
 
 //---------------------------------------------------------
@@ -134,7 +131,7 @@ void ChordEdit::setHarmony(const Harmony* h)
       _harmony = h->clone();
       setRoot(h->rootTpc());
       setBase(h->baseTpc());
-      setExtension(h->chordId());
+      setExtension(h->id());
 
       for (int i = 0; i < h->numberOfDegrees(); ++i)
             addDegree(h->degree(i));
@@ -228,10 +225,10 @@ const ChordDescription* ChordEdit::extension()
             return 0;
       else if (id == 0) {
             int idx = extOtherCombo->currentIndex();
-            return Harmony::chordDescription(extOtherCombo->itemData(idx).toInt());
+            return score->style().chordDescription(extOtherCombo->itemData(idx).toInt());
             }
       else
-            return Harmony::chordDescription(id);
+            return score->style().chordDescription(id);
       }
 
 //---------------------------------------------------------
@@ -301,7 +298,7 @@ void ChordEdit::chordChanged()
             _harmony->addDegree(degree(i));
       _harmony->setRootTpc(root());
       _harmony->setBaseTpc(base());
-      _harmony->setDescr(extension());
+      _harmony->setId(extension() ? extension()->id : 0);
       QString s = _harmony->harmonyName();
       chordLabel->setText(s);
       }
