@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: chordproperties.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -19,15 +19,25 @@
 //=============================================================================
 
 #include "chordproperties.h"
+#include "chord.h"
+#include "note.h"
 
 //---------------------------------------------------------
 //   ChordProperties
 //---------------------------------------------------------
 
-ChordProperties::ChordProperties(QWidget* parent)
+ChordProperties::ChordProperties(const Note* note, QWidget* parent)
    : QDialog(parent)
       {
       setupUi(this);
+      const Chord* chord = note->chord();
+      smallCheckBox->setChecked(chord->small());
+      noStemCheckBox->setChecked(chord->noStem());
+      extraLeadingSpace->setValue(chord->extraLeadingSpace().val());
+      extraTrailingSpace->setValue(chord->extraTrailingSpace().val());
+      tuningSpinBox->setValue(note->tuning());
+      userMirror->setCurrentIndex(int(note->userMirror()));
+      stemDirection->setCurrentIndex(int(chord->stemDirection()));
       }
 
 //---------------------------------------------------------
@@ -40,12 +50,12 @@ bool ChordProperties::small() const
       }
 
 //---------------------------------------------------------
-//   setSmall
+//   getUserMirror
 //---------------------------------------------------------
 
-void ChordProperties::setSmall(bool val)
+int ChordProperties::getUserMirror() const
       {
-      smallCheckBox->setChecked(val);
+      return userMirror->currentIndex();
       }
 
 //---------------------------------------------------------
@@ -58,11 +68,38 @@ bool ChordProperties::noStem() const
       }
 
 //---------------------------------------------------------
-//   setNoStem
+//   getStemDirection
 //---------------------------------------------------------
 
-void ChordProperties::setNoStem(bool val)
+int ChordProperties::getStemDirection() const
       {
-      noStemCheckBox->setChecked(val);
+      return stemDirection->currentIndex();
+      }
+
+//---------------------------------------------------------
+//   leadingSpace
+//---------------------------------------------------------
+
+double ChordProperties::leadingSpace() const
+      {
+      return extraLeadingSpace->value();
+      }
+
+//---------------------------------------------------------
+//   trailingSpace
+//---------------------------------------------------------
+
+double ChordProperties::trailingSpace() const
+      {
+      return extraTrailingSpace->value();
+      }
+
+//---------------------------------------------------------
+//   tuning
+//---------------------------------------------------------
+
+double ChordProperties::tuning() const
+      {
+      return tuningSpinBox->value();
       }
 
