@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: chordproperties.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -23,19 +23,24 @@
 
 #include "ui_chordproperties.h"
 
+class Note;
+
 //---------------------------------------------------------
 //   ChordProperties
 //---------------------------------------------------------
 
-class ChordProperties : public QDialog, public Ui::ChordPropertyBase {
+class ChordProperties : public QDialog, Ui::ChordPropertyBase {
       Q_OBJECT
 
    public:
-      ChordProperties(QWidget* parent = 0);
+      ChordProperties(const Note* c, QWidget* parent = 0);
       bool small() const;
-      void setSmall(bool val);
       bool noStem() const;
-      void setNoStem(bool val);
+      double leadingSpace() const;
+      double trailingSpace() const;
+      double tuning() const;
+      int getUserMirror() const;
+      int getStemDirection() const;
       };
 
 #endif
