@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: chordrest.cpp,v 1.7 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: chordrest.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -26,7 +26,6 @@
 #include "measure.h"
 #include "staff.h"
 #include "tuplet.h"
-#include "layout.h"
 #include "chordlist.h"
 #include "score.h"
 #include "sym.h"
@@ -35,6 +34,7 @@
 #include "breath.h"
 #include "barline.h"
 #include "articulation.h"
+#include "tempo.h"
 #include "tempotext.h"
 
 //---------------------------------------------------------
@@ -58,6 +58,15 @@ DurationElement::DurationElement(const DurationElement& e)
       _tuplet   = e._tuplet;
       }
 
+//---------------------------------------------------------
+//   ticks
+//---------------------------------------------------------
+
+int DurationElement::ticks() const
+      {
+      return _tickLen == 0 ? measure()->tickLen() : _tickLen;
+      }
+
 //---------------------------------------------------------
 //   hasArticulation
 //---------------------------------------------------------
@@ -89,11 +98,13 @@ ChordRest::ChordRest(Score* s)
 ChordRest::ChordRest(const ChordRest& cr)
    : DurationElement(cr)
       {
-      _beam     = 0;
-      _up       = cr._up;
-      _small    = cr._small;
-      _beamMode = cr._beamMode;
-      _dots     = cr._dots;
+      _beam               = 0;
+      _up                 = cr._up;
+      _small              = cr._small;
+      _beamMode           = cr._beamMode;
+      _dots               = cr._dots;
+      _extraLeadingSpace  = cr._extraLeadingSpace;
+      _extraTrailingSpace = cr._extraTrailingSpace;
 
       foreach(Articulation* a, cr.articulations) {            // make deep copy
             Articulation* na = new Articulation(*a);
@@ -109,7 +120,7 @@ ChordRest::ChordRest(const ChordRest& cr)
 
 bool ChordRest::isUp() const
       {
-      return _beam ? _beam->up() : _up;
+      return _beam ? (_beam->up() != -1 ? _beam->up() : _up) : _up;
       }
 
 //---------------------------------------------------------
@@ -156,10 +167,16 @@ QList<Prop> ChordRest::properties(Xml& xml, bool clipboardmode) const
                   }
             pl.append(Prop("BeamMode", s));
             }
+      if (_tickLen)
+            pl.append(Prop("ticklen", _tickLen));
       if (tuplet())
             pl.append(Prop("Tuplet", tuplet()->id()));
       if (_small)
             pl.append(Prop("small", _small));
+      if (_extraLeadingSpace.val() != 0.0)
+            pl.append(Prop("leadingSpace", _extraLeadingSpace.val()));
+      if (_extraTrailingSpace.val() != 0.0)
+            pl.append(Prop("trailingSpace", _extraTrailingSpace.val()));
       if (!clipboardmode) {
             if (tickLen() != duration().ticks(_dots)) {
                   if (_dots)
@@ -188,7 +205,10 @@ void ChordRest::writeProperties(Xml& xml) const
             }
       if (!xml.clipboardmode && _beam)
             xml.tag("Beam", _beam->id());
-      xml.curTick = tick() + tickLen();
+      int len = tickLen();
+      if (type() == REST && len == 0)
+            len = measure()->tickLen();
+      xml.curTick = tick() + len;
       }
 
 //---------------------------------------------------------
@@ -222,7 +242,7 @@ bool ChordRest::readProperties(QDomElement e, const QList<Tuplet*>& tuplets,
                   bm = i;
             _beamMode = BeamMode(bm);
             }
-      else if (tag == "Attribute") {
+      else if (tag == "Attribute" || tag == "Articulation") {     // obsolete: "Attribute"
             Articulation* atr = new Articulation(score());
             atr->read(e);
             add(atr);
@@ -240,6 +260,10 @@ bool ChordRest::readProperties(QDomElement e, const QList<Tuplet*>& tuplets,
             else
                   setTickLen(tickLen());  // set right symbol + dots
             }
+      else if (tag == "leadingSpace")
+            _extraLeadingSpace = Spatium(val.toDouble());
+      else if (tag == "trailingSpace")
+            _extraTrailingSpace = Spatium(val.toDouble());
       else if (tag == "Beam") {
             foreach(Beam* b, beams) {
                   if (b->id() == i) {
@@ -284,6 +308,10 @@ bool ChordRest::readProperties(QDomElement e, const QList<Tuplet*>& tuplets,
             d.setVal(val);
             setDuration(d);
             }
+      else if (tag == "ticklen")
+            setTickLen(i);
+      else if (tag == "tickLen")    // debug
+            setTickLen(i);
       else if (tag == "dots")
             _dots = i;
       else
@@ -297,103 +325,108 @@ bool ChordRest::readProperties(QDomElement e, const QList<Tuplet*>& tuplets,
 
 void ChordRest::setSmall(bool val)
       {
-      _small = val;
-      double m   = _small ? .7 : 1.0;
+      _small     = val;
+      double m = 1.0;
+      if (_small)
+            m = score()->styleD(ST_smallNoteMag);
       if (staff()->small())
-            m *= .7;
+            m *= score()->styleD(ST_smallStaffMag);
       setMag(m);
       }
 
 //---------------------------------------------------------
-//   layoutAttributes
+//   layoutArticulations
+//    if ((a->subtype() == TenutoSym) || (a->subtype() == StaccatoSym))
 //---------------------------------------------------------
 
-void ChordRest::layoutAttributes(ScoreLayout* layout)
+void ChordRest::layoutArticulations()
       {
-      double _spatium = layout->spatium();
+      double _spatium  = spatium();
+      Measure* m       = measure();
+      System* s        = m->system();
+      int idx          = staff()->rstaff();
+      qreal x          = centerX();
 
-      Measure* m = measure();
-      System* s  = m->system();
-      int idx    = staff()->rstaff();
-      qreal x    = centerX();
-      qreal sy   = _spatium;      // TODO: style parameter: distance to top/bottom line
-      qreal sy2  = _spatium * .5; // TODO: style parameter: distance to top/bottom note
+//      double distance  = point(score()->styleS(ST_propertyDistance));
+      double distance1 = point(score()->styleS(ST_propertyDistanceHead));
+      double distance2 = point(score()->styleS(ST_propertyDistanceStem));
 
-      qreal chordTopY = upPos()   - point(score()->style()->propertyDistanceStem) - sy2;
-      qreal chordBotY = downPos() + point(score()->style()->propertyDistanceHead) + sy2;
-      qreal staffTopY = s->bboxStaff(idx).y() - pos().y()      - sy;
-      qreal staffBotY = staffTopY + s->bboxStaff(idx).height() + sy;
+      qreal chordTopY = upPos();
+      qreal chordBotY = downPos();
 
-      qreal dyTop = 0;
-      qreal dyBot = 0;
+      qreal staffTopY = s->bboxStaff(idx).y() - pos().y();
+      qreal staffBotY = staffTopY + 4.0 * _spatium + distance2;
+      staffTopY      -= distance2;
+
+      qreal dy = 0.0;
 
       //
       //    pass 1
       //    place all articulations with anchor at chord/rest
       //
-      for (iArticulation ia = articulations.begin(); ia != articulations.end(); ++ia) {
-            Articulation* a = *ia;
-            qreal y = 0;
-            ArticulationAnchor aa = Articulation::articulationList[a->subtype()].anchor;
-            if (aa == A_CHORD)
-                  aa = isUp() ? A_BOTTOM_CHORD : A_TOP_CHORD;
-            if (aa == A_TOP_CHORD) {
-                  y = chordTopY - dyTop;
+
+      foreach (Articulation* a, articulations) {
+            ArticulationAnchor aa = a->anchor();
+            if (aa != A_CHORD && aa != A_TOP_CHORD && aa != A_BOTTOM_CHORD)
+                  continue;
+
+            qreal sh = a->bbox().height() * mag();
+
+            dy += distance1;
+            if (sh > (_spatium * .5))   // hack
+                  dy += sh * .5;
+            qreal y;
+            if (isUp()) {
+                  y = chordBotY + dy;
+                  //
+                  // check for collision with staff line
+                  //
+                  if (y <= staffBotY-.1) {
+                        qreal l = y / _spatium;
+                        qreal delta = fabs(l - round(l));
+                        if (delta < 0.4) {
+                              y  += _spatium * .5;
+                              dy += _spatium * .5;
+                              }
+                        }
+                  }
+            else {
+                  y = chordTopY - dy;
                   //
                   // check for collision with staff line
                   //
                   if (y >= staffTopY+.1) {
                         qreal l = y / _spatium;
                         qreal delta = fabs(l - round(l));
-                        if (delta < 0.4)
-                              y -= _spatium * .5;
+                        if (delta < 0.4) {
+                              y  -= _spatium * .5;
+                              dy += _spatium * .5;
+                              }
                         }
-                  a->setPos(x, y);
-                  dyTop += (point(score()->style()->propertyDistance) + a->bbox().height());
-                  }
-            else if (aa == A_BOTTOM_CHORD) {
-                  y = chordBotY + dyBot;
-                  //
-                  // check for collision with staff line
-                  //
-                  if (y <= staffBotY+.1) {
-                        qreal l = y / _spatium;
-                        qreal delta = fabs(l - round(l));
-                        if (delta < 0.4)
-                              y += _spatium * .5;
-                        }
-                  a->setPos(x, y);
-                  dyBot += (point(score()->style()->propertyDistance) + a->bbox().height());
                   }
+            a->setPos(x, y);
             }
 
       //
-      //    pass 1
+      //    pass 2
       //    now place all articulations with staff top or bottom anchor
       //
-      if (chordTopY - dyTop > staffTopY)
-            dyTop = 0;
-      else
-            dyTop = staffTopY - (chordTopY - dyTop);
-
-      if (chordBotY + dyBot < staffBotY)
-            dyBot = 0;
-      else
-            dyBot = (chordBotY + dyBot) - staffBotY;
+      qreal dyTop = 0.0;
+      qreal dyBot = 0.0;
 
       for (iArticulation ia = articulations.begin(); ia != articulations.end(); ++ia) {
             Articulation* a = *ia;
             qreal y = 0;
-            ArticulationAnchor aa = Articulation::articulationList[a->subtype()].anchor;
+            ArticulationAnchor aa = a->anchor();
             if (aa == A_TOP_STAFF) {
                   y = staffTopY - dyTop;
                   a->setPos(x, y);
-                  dyTop += (point(score()->style()->propertyDistance) + a->bbox().height());
+                  dyTop += point(score()->styleS(ST_propertyDistance)) + a->bbox().height();
                   }
             else if (aa == A_BOTTOM_STAFF) {
                   y = staffBotY + dyBot;
                   a->setPos(x, y);
-                  dyBot += (point(score()->style()->propertyDistance) + a->bbox().height());
+                  dyBot += point(score()->styleS(ST_propertyDistance)) + a->bbox().height();
                   }
             }
       }
@@ -537,6 +570,15 @@ Element* ChordRest::drop(const QPointF& p1, const QPointF& p2, Element* e)
                   }
                   break;
 
+            case DYNAMIC:
+                  {
+                  e->setTick(tick());
+                  e->setTrack(staffIdx() * VOICES);
+                  e->setParent(m);
+                  score()->undoAddElement(e);
+                  }
+                  break;
+
             default:
                   printf("cannot drop %s\n", e->name());
                   return 0;
@@ -553,3 +595,13 @@ void ChordRest::setBeam(Beam* b)
       _beam = b;
       }
 
+//---------------------------------------------------------
+//   toDefault
+//---------------------------------------------------------
+
+void ChordRest::toDefault()
+      {
+      score()->undoChangeChordRestSpace(this, Spatium(0.0), Spatium(0.0));
+      score()->undoChangeUserOffset(this, QPointF());
+      }
+
