@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: chordrest.h,v 1.4 2006/03/03 21:47:11 wschweer Exp $
+//  $Id: chordrest.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -26,7 +26,6 @@
 #include "duration.h"
 
 class Score;
-class ScoreLayout;
 class Measure;
 class Beam;
 class Tuplet;
@@ -49,6 +48,8 @@ class ChordRest : public DurationElement {
       BeamMode _beamMode;
       bool _up;
       bool _small;
+      Spatium _extraLeadingSpace;
+      Spatium _extraTrailingSpace;
 
       int _dots;
 
@@ -60,15 +61,15 @@ class ChordRest : public DurationElement {
       virtual QPointF canvasPos() const;      ///< position in canvas coordinates
       virtual Element* drop(const QPointF&, const QPointF&, Element*);
 
+      Segment* segment() const                  { return (Segment*)parent(); }
+      virtual Measure* measure() const          { return (Measure*)(parent()->parent()); }
+
       virtual void read(QDomElement, const QList<Tuplet*>&, const QList<Beam*>&) = 0;
       void writeProperties(Xml& xml) const;
       bool readProperties(QDomElement e, const QList<Tuplet*>&, const QList<Beam*>&);
       QList<Prop> properties(Xml&, bool clipboardmode) const;
       virtual QList<Prop> properties(Xml& xml) const { return properties(xml, false); }
 
-      Segment* segment() const                  { return (Segment*)parent(); }
-      Measure* measure() const                  { return (Measure*)(parent()->parent()); }
-
       void setBeamMode(BeamMode m)              { _beamMode = m; }
       BeamMode beamMode() const                 { return _beamMode; }
       void setBeam(Beam* b);
@@ -78,8 +79,8 @@ class ChordRest : public DurationElement {
       virtual qreal downPos() const = 0;
       virtual qreal centerX() const = 0;
 
-      virtual void layoutStem1(ScoreLayout*)    {}
-      virtual void layoutStem(ScoreLayout*)     {}
+      virtual void layoutStem1()                {}
+      virtual void layoutStem()                 {}
       virtual int upLine() const                { return 0;}
       virtual int downLine() const              { return 8;}
       virtual int line(bool up) const           { return up ? upLine() : downLine(); }
@@ -103,7 +104,12 @@ class ChordRest : public DurationElement {
       void setDots(int n)                       { _dots = n; }
       int dots() const                          { return _dots; }
       void setLen(int ticks);
-      void layoutAttributes(ScoreLayout*);
+      void layoutArticulations();
+      Spatium extraLeadingSpace() const         { return _extraLeadingSpace;  }
+      void setExtraLeadingSpace(Spatium v)      { _extraLeadingSpace = v;     }
+      Spatium extraTrailingSpace() const        { return _extraTrailingSpace; }
+      void setExtraTrailingSpace(Spatium v)     { _extraTrailingSpace = v;    }
+      virtual void toDefault();
       };
 
 #endif
