@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: clef.cpp,v 1.11 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: clef.cpp 1847 2009-05-24 13:05:54Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -33,6 +33,8 @@
 #include "viewer.h"
 #include "system.h"
 #include "style.h"
+#include "segment.h"
+#include "measure.h"
 
 // FIXME!
 // only values for CLEF_G..CLEF_G3 CLEF_F and CLEF_C3 are
@@ -43,60 +45,75 @@ const ClefInfo clefTable[] = {
 //             name line octave  yoffset pitchoffset  name
       { "G",   2,  0,   0, 45,
             { 0, 3, -1, 2, 5, 1, 4 , 4, 1,  5, 2, 6, 3, 7 },
-            QT_TR_NOOP("Treble Clef (G Clef)")
+            QT_TRANSLATE_NOOP("clefTable", "Treble clef")
             },
       { "G",   2,  1,   7, 52,
             { 0, 3, -1, 2, 5, 1, 4, 4, 1,  5, 2, 6, 3, 7 },
-            QT_TR_NOOP("Treble Clef 8va") },
+            QT_TRANSLATE_NOOP("clefTable", "Treble clef 8va") },
       { "G",   2,  2,  14, 59,
             { 0, 3, -1, 2, 5, 1, 4, 4, 1,  5, 2, 6, 3, 7 },
-            QT_TR_NOOP("Treble Clef 15ma") },
+            QT_TRANSLATE_NOOP("clefTable", "Treble clef 15ma") },
       { "G",   2, -1,  -7, 38,
             { 0, 3, -1, 2, 5, 1, 4, 4, 1,  5, 2, 6, 3, 7 },
-            QT_TR_NOOP("Treble Clef 8va bassa") },
+            QT_TRANSLATE_NOOP("clefTable", "Treble clef 8vb") },
       { "F",   4,  0, -12, 33,
             { 2, 5, 1, 4, 7, 3, 6, 6, 3, 7, 4, 8, 5, 9 },
-            QT_TR_NOOP("Bass Clef (F Clef)") },
+            QT_TRANSLATE_NOOP("clefTable", "Bass clef") },
       { "F",   4, -1, -19, 26,
             { 2, 5, 1, 4, 7, 3, 6, 6, 3, 7, 4, 8, 5, 9 },
-            QT_TR_NOOP("Bass Clef 8va bassa") },
+            QT_TRANSLATE_NOOP("clefTable", "Bass clef 8vb") },
       { "F",   4, -2, -26, 19,
             { 2, 5, 1, 4, 7, 3, 6, 6, 3, 7, 4, 8, 5, 9 },
-            QT_TR_NOOP("Bass Clef 15mb bassa") },
+            QT_TRANSLATE_NOOP("clefTable", "Bass clef 15mb") },
 
       { "F",   4,  0, -10, 35,
             { 4, 0, 3, -1, 2, 5, 1, 1, 5, 2, 6, 3, 7, 4 },
-            QT_TR_NOOP("Baritone Clef") },
+            QT_TRANSLATE_NOOP("clefTable", "Baritone clef (F clef)") },
       { "F",   2,  0, -14, 31,
             { 1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
-            QT_TR_NOOP("Subbass Clef") },
+            QT_TRANSLATE_NOOP("clefTable", "Subbass clef") },
 
       { "C",   1,  0,  -2, 43,
             { 5, 1, 4, 0, 3, -1, 2, 2, -1, 3, 0, 4, 1, 5 },
-            QT_TR_NOOP("Soprano Clef") },        // CLEF_C1
+            QT_TRANSLATE_NOOP("clefTable", "Soprano clef") },        // CLEF_C1
       { "C",   2,  0,  -4, 41,
             { 3, 6, 2, 5, 1, 4, 0, 0, 4, 1, 5, 2, 6, 3 },
-            QT_TR_NOOP("Mezzo-soprano Clef") },  // CLEF_C2
+            QT_TRANSLATE_NOOP("clefTable", "Mezzo-soprano clef") },  // CLEF_C2
       { "C",   3,  0,  -6, 39,
             { 1, 4, 0, 3, 6, 2, 5, 5, 2, 6, 3, 7, 4, 8 },
-            QT_TR_NOOP("Alto Clef") },           // CLEF_C3
+            QT_TRANSLATE_NOOP("clefTable", "Alto clef") },           // CLEF_C3
       { "C",   4,  0,  -8, 37,
             { 6, 2, 5, 1, 4, 0, 3, 3, 0, 4, 1, 5, 2, 6 },
-            QT_TR_NOOP("Tenor Clef")  },          // CLEF_C4
+            QT_TRANSLATE_NOOP("clefTable", "Tenor clef")  },          // CLEF_C4
 
       { "TAB", 5,  0,   0,  0,
             { 0, 3, -1, 2, 5, 1, 4, 4, 1, 5, 2, 6, 3, 7 },
-            QT_TR_NOOP("Tablature") },
-      { "PERC",2,  0,   0, 45,
+            QT_TRANSLATE_NOOP("clefTable", "Tablature") },
+
+      { "percussion", 2,  0,   0, 45,
             { 0, 3, -1, 2, 5, 1, 4, 4, 1, 5, 2, 6, 3, 7 },
-            QT_TR_NOOP("Percussion") },
+            QT_TRANSLATE_NOOP("clefTable", "Percussion") },
 
       { "C",   5,  0, -10, 35,
             { 4, 0, 3, -1, 2, 5, 1, 1, 5, 2, 6, 3, 7, 4 },
-            QT_TR_NOOP("Baritone Clef") },            // CLEF_C5
+            QT_TRANSLATE_NOOP("clefTable", "Baritone clef (C clef)") },            // CLEF_C5
+
       { "G",   1,  0,   2, 47,
             { 2, 5, 1, 4, 0, 3, -1, 6, 3, 7, 4, 1, 5, 2 },
-            QT_TR_NOOP("French Violin Clef") },       // CLEF_G4
+            QT_TRANSLATE_NOOP("clefTable", "French violin clef") },       // CLEF_G4
+
+      { "F",   4,  1, -5, 40,                                          // CLEF_F_8VA
+            { 2, 5, 1, 4, 7, 3, 6, 6, 3, 7, 4, 8, 5, 9 },
+            QT_TRANSLATE_NOOP("clefTable", "Bass clef 8va") },
+
+      { "F",   4,  2,  2, 47,                                          // CLEF_F_15MA
+            { 2, 5, 1, 4, 7, 3, 6, 6, 3, 7, 4, 8, 5, 9 },
+            QT_TRANSLATE_NOOP("clefTable", "Bass clef 15ma") },
+
+      { "percussion", 2,  0,   0, 45,                                   // CLEF_PERC2 placeholder
+            { 0, 3, -1, 2, 5, 1, 4, 4, 1, 5, 2, 6, 3, 7 },
+            QT_TRANSLATE_NOOP("clefTable", "Percussion") },
+
       };
 
 //---------------------------------------------------------
@@ -142,9 +159,10 @@ QPointF Clef::canvasPos() const
 //   layout
 //---------------------------------------------------------
 
-void Clef::layout(ScoreLayout*)
+void Clef::layout()
       {
-      double smag = _small ? score()->style()->smallClefMag : 1.0;
+      double smag = _small ? score()->style(ST_smallClefMag).toDouble() : 1.0;
+      double _spatium = spatium();
       double msp  = _spatium * smag;
       int val     = subtype();
       double yoff = 0.0;
@@ -249,6 +267,7 @@ void Clef::layout(ScoreLayout*)
                   yoff = 2.0; //(staff()->lines() - 1) * 0.5;
                   break;
             case CLEF_PERC:
+            case CLEF_PERC2:
                   symbol->setSym(percussionclefSym);
                   yoff = 2.0;   //(staff()->lines() - 1) * 0.5;
                   break;
@@ -256,10 +275,33 @@ void Clef::layout(ScoreLayout*)
                   symbol->setSym(trebleclefSym);
                   yoff = 4.0;
                   break;
+            case CLEF_F_8VA:
+                  {
+                  symbol->setSym(bassclefSym);
+                  yoff = 1.0;
+                  Symbol* number = new Symbol(score());
+                  number->setMag(smag);
+                  number->setSym(clefEightSym);
+                  addElement(number, .5 * msp, -1.5 * msp);
+                  }
+                  break;
+            case CLEF_F_15MA:
+                  {
+                  symbol->setSym(bassclefSym);
+                  yoff = 1.0;
+                  Symbol* number = new Symbol(score());
+                  symbol->setMag(smag);
+                  number->setSym(clefOneSym);
+                  addElement(number, .0 * msp, -1.5 * msp);
+                  number = new Symbol(score());
+                  number->setSym(clefFiveSym);
+                  addElement(number, .8 * msp, -1.5 * msp);
+                  }
+                  break;
             }
       addElement(symbol, .0, .0);
       symbol->setMag(smag * mag());
-      setUserOff(QPointF(0.0, yoff * mag()));
+      setUserOff(QPointF(0.0, yoff * _spatium));
       }
 
 //---------------------------------------------------------
@@ -422,6 +464,6 @@ void Clef::setSmall(bool val)
                   setMag(mag() / smallMag);
             */
             }
-      layout(0);
+      layout();
       }
 
