@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: clef.h,v 1.7 2006/03/02 17:08:33 wschweer Exp $
+//  $Id: clef.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -56,7 +56,7 @@ class Clef : public Compound {
 
       virtual bool acceptDrop(Viewer*, const QPointF&, int, int) const;
       virtual Element* drop(const QPointF&, const QPointF&, Element*);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       virtual void space(double& min, double& extra) const;
 
       bool small() const                        { return _small; }
@@ -86,7 +86,10 @@ enum {
       CLEF_F, CLEF_F8, CLEF_F15, CLEF_F_B, CLEF_F_C,
       CLEF_C1, CLEF_C2, CLEF_C3, CLEF_C4,
       CLEF_TAB, CLEF_PERC,
-      CLEF_C5, CLEF_G4, CLEF_MAX
+      CLEF_C5, CLEF_G4,
+      CLEF_F_8VA, CLEF_F_15MA,
+      CLEF_PERC2,
+      CLEF_MAX
       };
 
 
