@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: dynamics.h,v 1.11 2006/03/13 21:35:58 wschweer Exp $
+//  $Id: dynamics.h 1992 2009-08-04 16:30:54Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -22,6 +22,7 @@
 #define __DYNAMICS_H__
 
 #include "text.h"
+#include "ui_dynamicproperties.h"
 
 //---------------------------------------------------------
 //   Dyn
@@ -35,12 +36,18 @@ struct Dyn {
          : velocity(velo), tag(t) {}
       };
 
+enum DynamicType { DYNAMIC_STAFF, DYNAMIC_PART, DYNAMIC_SYSTEM };
+
 //---------------------------------------------------------
 //   Dynamic
 //---------------------------------------------------------
 
 class Dynamic : public Text {
+      Q_DECLARE_TR_FUNCTIONS(Dynamic)
+
       mutable QPointF dragOffset;
+      int _velocity;          // associated midi velocity 0-127
+      DynamicType _dynType;
 
    public:
       Dynamic(Score*);
@@ -49,17 +56,44 @@ class Dynamic : public Text {
       virtual ElementType type() const { return DYNAMIC; }
       Measure* measure() const         { return (Measure*)parent(); }
 
+      virtual void layout();
+
       virtual void setSubtype(int val);
       virtual void setSubtype(const QString&);
       virtual const QString subtypeName() const;
 
-      virtual bool isMovable() const  { return true; }
+      virtual bool isMovable() const   { return true; }
 
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement);
 
+      virtual bool startEdit(Viewer*, const QPointF&);
       virtual void endEdit();
-      virtual void resetUserOffsets();
+      virtual void toDefault();
+
+      void resetType();
+      void setVelocity(int v);
+      int velocity() const;
+      DynamicType dynType() const    { return _dynType; }
+      void setDynType(DynamicType t) { _dynType = t;    }
+
+      virtual bool genPropertyMenu(QMenu* popup) const;
+      virtual void propertyAction(const QString& s);
+      };
+
+//---------------------------------------------------------
+//   DynamicProperties
+//---------------------------------------------------------
+
+class DynamicProperties : public QDialog, public Ui::DynamicProperties {
+      Q_OBJECT
+      Dynamic* dynamic;
+
+   private slots:
+      virtual void accept();
+
+   public:
+      DynamicProperties(Dynamic*, QWidget* parent = 0);
       };
 
 extern Dyn dynList[];
