@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: editstaff.cpp 1910 2009-07-04 10:51:31Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -23,7 +23,9 @@
 #include "part.h"
 #include "editdrumset.h"
 #include "score.h"
-#include "layout.h"
+#include "measure.h"
+#include "undo.h"
+#include "text.h"
 
 //---------------------------------------------------------
 //   EditStaff
@@ -45,6 +47,7 @@ EditStaff::EditStaff(Staff* s, QWidget* parent)
       shortName->setHtml(part->shortNameHtml());
       longName->setHtml(part->longNameHtml());
       slashStyle->setChecked(staff->slashStyle());
+      invisible->setChecked(staff->invisible());
 
       connect(buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(bboxClicked(QAbstractButton*)));
       connect(editDrumset, SIGNAL(clicked()), SLOT(editDrumsetClicked()));
@@ -82,26 +85,26 @@ void EditStaff::bboxClicked(QAbstractButton* button)
 
 void EditStaff::apply()
       {
-      Part* part = staff->part();
-
-      part->setUseDrumset(useDrumset->isChecked());
-      part->setPitchOffset(transposition->value());
-      staff->setLines(lines->value());
-      staff->setSmall(small->isChecked());
-      staff->setSlashStyle(slashStyle->isChecked());
-      part->setShortName(*shortName->document());
-      part->setLongName(*longName->document());
       Score* score = staff->score();
-      score->layout()->setInstrumentNames();
-      MeasureBaseList* ml = score->measures();
-      int staffIdx = score->staffIdx(staff);
-      for (MeasureBase* mb = ml->first(); mb; mb = mb->next()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            MStaff* mstaff = static_cast<Measure*>(mb)->mstaff(staffIdx);
-            mstaff->lines->setLines(staff->lines());
-            }
+      Part* part   = staff->part();
 
+      bool ud = useDrumset->isChecked();
+      int  po = transposition->value();
+      const QTextDocument* ln = longName->document();
+      const QTextDocument* sn = shortName->document();
+
+      bool snd = sn->toHtml() != part->shortName()->doc()->toHtml();
+      bool lnd = ln->toHtml() != part->longName()->doc()->toHtml();
+      if ((ud != part->useDrumset()) || (po != part->pitchOffset()) || snd || lnd)
+            score->undo()->push(new ChangePart(part, ud, po, ln, sn));
+
+      int l        = lines->value();
+      bool s       = small->isChecked();
+      bool noStems = slashStyle->isChecked();
+      bool inv     = invisible->isChecked();
+      if (l != staff->lines() || s != staff->small() || noStems != staff->slashStyle()
+         || inv != staff->invisible())
+            score->undo()->push(new ChangeStaff(staff, l, s, noStems, inv));
       }
 
 //---------------------------------------------------------
