@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: element.cpp,v 1.79 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: element.cpp 1906 2009-07-03 15:55:58Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -33,7 +33,6 @@
 #include "sym.h"
 #include "symbol.h"
 #include "clef.h"
-#include "layout.h"
 #include "viewer.h"
 #include "volta.h"
 #include "ottava.h"
@@ -68,41 +67,140 @@
 #include "harmony.h"
 #include "lyrics.h"
 #include "rest.h"
+#include "slur.h"
+#include "measure.h"
 
 extern bool debugMode;
 extern bool showInvisible;
 
-const char* elementNames[] = {
-      "Symbol", "Text", "SlurSegment", "BarLine",
-      "StemSlash", "Line", "Bracket",
-      "Arpeggio",
-      "Accidental",
-      "Note",
-      "Stem",
-      "Clef", "KeySig", "TimeSig", "Rest",
-      "Breath", "Glissando",
-      "RepeatMeasure",
-      "Image",
-      "Tie",
-      "Attribute", "Dynamic", "Page", "Beam", "Hook", "Lyrics", "Marker", "Jump",
-      "Tuplet",
-      "Tempo", "StaffText",
-      "Harmony",
-      "Volta",
-      "HairpinSegment", "OttavaSegment", "PedalSegment", "TrillSegment", "TextLineSegment",
-      "VoltaSegment",
-      "LayoutBreak", "Spacer",
-      "LedgerLine",
-      "NoteHead", "Tremolo",
-      "Measure", "StaffLines",
-      "Cursor", "Selection", "Lasso", "ShadowNote", "RubberBand",
-      "HairPin", "Ottava", "Pedal", "Trill", "TextLine",
-      "Segment", "System", "Compound", "Chord", "Slur",
-      "Element", "ElementList", "StaffList", "MeasureList", "Layout",
-      "HBox", "VBox",
-      "Icon"
+static const char* elementNames[] = {
+      QT_TRANSLATE_NOOP("elementName", "Symbol"),
+      QT_TRANSLATE_NOOP("elementName", "Text"),
+      QT_TRANSLATE_NOOP("elementName", "SlurSegment"),
+      QT_TRANSLATE_NOOP("elementName", "BarLine"),
+      QT_TRANSLATE_NOOP("elementName", "StemSlash"),
+      QT_TRANSLATE_NOOP("elementName", "Line"),
+      QT_TRANSLATE_NOOP("elementName", "Bracket"),
+      QT_TRANSLATE_NOOP("elementName", "Arpeggio"),
+      QT_TRANSLATE_NOOP("elementName", "Accidental"),
+      QT_TRANSLATE_NOOP("elementName", "Note"),
+      QT_TRANSLATE_NOOP("elementName", "Stem"),
+      QT_TRANSLATE_NOOP("elementName", "Clef"),
+      QT_TRANSLATE_NOOP("elementName", "KeySig"),
+      QT_TRANSLATE_NOOP("elementName", "TimeSig"),
+      QT_TRANSLATE_NOOP("elementName", "Rest"),
+      QT_TRANSLATE_NOOP("elementName", "Breath"),
+      QT_TRANSLATE_NOOP("elementName", "Glissando"),
+      QT_TRANSLATE_NOOP("elementName", "RepeatMeasure"),
+      QT_TRANSLATE_NOOP("elementName", "Image"),
+      QT_TRANSLATE_NOOP("elementName", "Tie"),
+      QT_TRANSLATE_NOOP("elementName", "Articulation"),
+      QT_TRANSLATE_NOOP("elementName", "Dynamic"),
+      QT_TRANSLATE_NOOP("elementName", "Page"),
+      QT_TRANSLATE_NOOP("elementName", "Beam"),
+      QT_TRANSLATE_NOOP("elementName", "Hook"),
+      QT_TRANSLATE_NOOP("elementName", "Lyrics"),
+      QT_TRANSLATE_NOOP("elementName", "Marker"),
+      QT_TRANSLATE_NOOP("elementName", "Jump"),
+      QT_TRANSLATE_NOOP("elementName", "Tuplet"),
+      QT_TRANSLATE_NOOP("elementName", "Tempo"),
+      QT_TRANSLATE_NOOP("elementName", "StaffText"),
+      QT_TRANSLATE_NOOP("elementName", "Harmony"),
+      QT_TRANSLATE_NOOP("elementName", "Volta"),
+      QT_TRANSLATE_NOOP("elementName", "HairpinSegment"),
+      QT_TRANSLATE_NOOP("elementName", "OttavaSegment"),
+      QT_TRANSLATE_NOOP("elementName", "PedalSegment"),
+      QT_TRANSLATE_NOOP("elementName", "TrillSegment"),
+      QT_TRANSLATE_NOOP("elementName", "TextLineSegment"),
+      QT_TRANSLATE_NOOP("elementName", "VoltaSegment"),
+      QT_TRANSLATE_NOOP("elementName", "LayoutBreak"),
+      QT_TRANSLATE_NOOP("elementName", "Spacer"),
+      QT_TRANSLATE_NOOP("elementName", "LedgerLine"),
+      QT_TRANSLATE_NOOP("elementName", "NoteHead"),
+      QT_TRANSLATE_NOOP("elementName", "Tremolo"),
+      QT_TRANSLATE_NOOP("elementName", "Measure"),
+      QT_TRANSLATE_NOOP("elementName", "StaffLines"),
+      QT_TRANSLATE_NOOP("elementName", "Cursor"),
+      QT_TRANSLATE_NOOP("elementName", "Selection"),
+      QT_TRANSLATE_NOOP("elementName", "Lasso"),
+      QT_TRANSLATE_NOOP("elementName", "ShadowNote"),
+      QT_TRANSLATE_NOOP("elementName", "RubberBand"),
+      QT_TRANSLATE_NOOP("elementName", "HairPin"),
+      QT_TRANSLATE_NOOP("elementName", "Ottava"),
+      QT_TRANSLATE_NOOP("elementName", "Pedal"),
+      QT_TRANSLATE_NOOP("elementName", "Trill"),
+      QT_TRANSLATE_NOOP("elementName", "TextLine"),
+      QT_TRANSLATE_NOOP("elementName", "Segment"),
+      QT_TRANSLATE_NOOP("elementName", "System"),
+      QT_TRANSLATE_NOOP("elementName", "Compound"),
+      QT_TRANSLATE_NOOP("elementName", "Chord"),
+      QT_TRANSLATE_NOOP("elementName", "Slur"),
+      QT_TRANSLATE_NOOP("elementName", "Element"),
+      QT_TRANSLATE_NOOP("elementName", "ElementList"),
+      QT_TRANSLATE_NOOP("elementName", "StaffList"),
+      QT_TRANSLATE_NOOP("elementName", "MeasureList"),
+      QT_TRANSLATE_NOOP("elementName", "Layout"),
+      QT_TRANSLATE_NOOP("elementName", "HBox"),
+      QT_TRANSLATE_NOOP("elementName", "VBox"),
+      QT_TRANSLATE_NOOP("elementName", "Icon")
       };
 
+//---------------------------------------------------------
+//   spatiumChanged
+//---------------------------------------------------------
+
+void Element::spatiumChanged(double oldValue, double newValue)
+      {
+      _userOff *= (newValue / oldValue);
+      }
+
+//---------------------------------------------------------
+//   spatium
+//---------------------------------------------------------
+
+double Element::spatium() const
+      {
+      Staff* s = staff();
+      double v = _score->spatium();
+      return s ? v * s->mag() : v;
+      }
+
+//---------------------------------------------------------
+//   pos
+//---------------------------------------------------------
+
+QPointF Element::pos() const
+      {
+      return _pos + _userOff;
+      }
+
+//---------------------------------------------------------
+//   name
+//---------------------------------------------------------
+
+const char* Element::name() const
+      {
+      return name(type());
+      }
+
+//---------------------------------------------------------
+//   userName
+//---------------------------------------------------------
+
+QString Element::userName() const
+      {
+      return qApp->translate("elementName", name(type()));
+      }
+
+//---------------------------------------------------------
+//   abbox
+//---------------------------------------------------------
+
+QRectF Element::abbox() const
+      {
+      return bbox().translated(canvasPos());
+      }
+
 //---------------------------------------------------------
 //   operator >
 //---------------------------------------------------------
@@ -126,63 +224,33 @@ Element::~Element()
                         el->removeAt(el->indexOf(this));
                         }
                   }
-            UndoList* ul = score()->getUndoList();
-            if (!ul->isEmpty()) {
-                  Undo* undo = ul->back();
-                  s = &undo->selection;
-                  QList<Element*>* el = s->elements();
-                  foreach(Element* e, *el) {
-                        if (e == this) {
-                              if (debugMode)
-                                    printf("======~Element: %p still in undo!\n", this);
-                              el->removeAt(el->indexOf(this));
-                              }
-                        }
-                  }
             }
       }
 
-//---------------------------------------------------------
-//   init
-//---------------------------------------------------------
-
-void Element::init()
-      {
-      _tick       = -1;
-      _duration   = -1;
-      _parent     = 0;
-      _selected   = false;
-      _selectable = true;
-      _dropTarget = false;
-      _visible    = true;
-      _generated  = false;
-      _track      = -1;
-      _color      = preferences.defaultColor;
-      _mxmlOff    = 0;
-      _pos.setX(0.0);
-      _pos.setY(0.0);
-      _userOff.setX(0.0);
-      _userOff.setY(0.0);
-      itemDiscovered = 0;
-      _align       = ALIGN_LEFT | ALIGN_TOP;
-      _xoff        = 0;
-      _yoff        = 0;
-      _rxoff       = 0;
-      _ryoff       = 0;
-      _offsetType  = OFFSET_SPATIUM;
-      _systemFlag  = false;
-      }
-
 //---------------------------------------------------------
 //   Element
 //---------------------------------------------------------
 
-Element::Element(Score* s)
+Element::Element(Score* s) :
+   _parent(0),
+   _selected(false),
+   _selectable(true),
+   _dropTarget(false),
+   _generated(false),
+   _visible(true),
+   _systemFlag(false),
+   _subtype(0),
+   _track(-1),
+   _tick(-1),
+   _color(preferences.defaultColor),
+   _mag(1.0),
+   _score(s),
+   _align(ALIGN_LEFT | ALIGN_TOP),
+   _xoff(0), _yoff(0), _rxoff(0), _ryoff(0),
+   _offsetType(OFFSET_SPATIUM),
+   _mxmlOff(0),
+   itemDiscovered(0)
       {
-      _score = s;
-      init();
-      setSubtype(0);
-      _mag = 1.0;
       }
 
 Element::Element(const Element& e)
@@ -210,7 +278,6 @@ Element::Element(const Element& e)
       _userOff    = e._userOff;
       _mxmlOff    = e._mxmlOff;
       _bbox       = e._bbox;
-      _duration   = e._duration;
       itemDiscovered = 0;
       }
 
@@ -260,7 +327,7 @@ QColor Element::curColor() const
 QRectF Element::drag(const QPointF& pos)
       {
       QRectF r(abbox());
-      setUserOff(pos / _spatium);
+      setUserOff(pos);
       return abbox() | r;
       }
 
@@ -271,7 +338,7 @@ QRectF Element::drag(const QPointF& pos)
 
 QPointF Element::canvasPos() const
       {
-      QPointF p(pos());
+      QPointF p(_pos + _userOff);
       if (parent())
             p += parent()->canvasPos();
       return p;
@@ -336,14 +403,11 @@ bool Element::intersects(const QRectF& rr) const
 //    values when calling this method
 //---------------------------------------------------------
 
-void Element::layout(ScoreLayout* layout)
+void Element::layout()
       {
       QPointF o(QPointF(_xoff, _yoff));
       if (_offsetType == OFFSET_SPATIUM)
-            if (layout)
-                  o *= layout->spatium();
-            else
-                  o *= _spatium;
+            o *= spatium();
       else
             o *= DPI;
       if (parent())
@@ -368,7 +432,7 @@ void Element::layout(ScoreLayout* layout)
 //   properties
 //---------------------------------------------------------
 
-QList<Prop> Element::properties(Xml& xml) const
+QList<Prop> Element::properties(Xml& xml, const Element* proto) const
       {
       QList<Prop> pl;
       if (_subtype) {
@@ -377,14 +441,10 @@ QList<Prop> Element::properties(Xml& xml) const
                   pl.append(Prop("subtype", subtypeName()));
             }
       if (!_userOff.isNull())
-            pl.append(Prop("offset", _userOff));
+            pl.append(Prop("offset", _userOff / spatium()));
       if ((track() != xml.curTrack) && (track() != -1)) {
             int t;
-/*            if (track() == -1)
-                  t = -1;
-            else
-                  */
-                  t = track() + xml.trackDiff;
+            t = track() + xml.trackDiff;
             pl.append(Prop("track", t));
             }
       if (selected())
@@ -393,11 +453,9 @@ QList<Prop> Element::properties(Xml& xml) const
             pl.append(Prop("visible", visible()));
       if (_tick != -1 && (_tick != xml.curTick))
             pl.append(Prop("tick", _tick));
-      if (_duration != -1)
-            pl.append(Prop("ticklen", _duration));
       if (_color != preferences.defaultColor)
             pl.append(Prop("color", _color));
-      if (_systemFlag)
+      if (_systemFlag && (proto == 0 || proto->systemFlag() != _systemFlag))
             pl.append(Prop("systemFlag", _systemFlag));
       return pl;
       }
@@ -406,9 +464,9 @@ QList<Prop> Element::properties(Xml& xml) const
 //   writeProperties
 //---------------------------------------------------------
 
-void Element::writeProperties(Xml& xml) const
+void Element::writeProperties(Xml& xml, const Element* proto) const
       {
-      xml.prop(properties(xml));
+      xml.prop(properties(xml, proto));
       if ((_tick != -1) && (_tick != xml.curTick || debugMode))
             xml.curTick = _tick;
       }
@@ -431,10 +489,8 @@ bool Element::readProperties(QDomElement e)
             // do not always call Element::setSubtype():
             this->setSubtype(val);
             }
-      else if (tag == "ticklen")
-            setTickLen(score()->fileDivision(i));
       else if (tag == "offset")
-            setUserOff(readPoint(e));
+            setUserOff(readPoint(e) * spatium());
       else if (tag == "visible")
             setVisible(i);
       else if (tag == "voice")
@@ -613,8 +669,9 @@ QPointF StaffLines::canvasPos() const
 
 QRectF StaffLines::bbox() const
       {
+      double _spatium = spatium();
       int l    = lines() - 1;
-      qreal lw = point(score()->style()->staffLineWidth);
+      qreal lw = point(score()->styleS(ST_staffLineWidth));
 
       switch(l) {
             case 0:
@@ -635,13 +692,14 @@ QRectF StaffLines::bbox() const
 void StaffLines::draw(QPainter& p) const
       {
       QPointF _pos(0.0, 0.0);
+      double _spatium = spatium();
 
 //      p.save();
 //      p.setRenderHint(QPainter::Antialiasing, false);
 //      p.setRenderHint(QPainter::NonCosmeticDefaultPen, true);
 
       QPen pen(p.pen());
-      pen.setWidthF(point(score()->style()->staffLineWidth) * mag());
+      pen.setWidthF(point(score()->styleS(ST_staffLineWidth)));
       if (pen.widthF() * p.worldMatrix().m11() < 1.0)
             pen.setWidth(0);
       pen.setCapStyle(Qt::FlatCap);
@@ -653,13 +711,13 @@ void StaffLines::draw(QPainter& p) const
       switch(lines()) {
             case 1:
                   {
-                  qreal y = _pos.y() + 2 * _spatium * mag();
+                  qreal y = _pos.y() + 2 * _spatium;
                   p.drawLine(QLineF(x1, y, x2, y));
                   }
                   break;
             case 2:
                   {
-                  qreal y = _pos.y() + 1 * _spatium * mag();
+                  qreal y = _pos.y() + 1 * _spatium;
                   p.drawLine(QLineF(x1, y, x2, y));
                   y += 2 * _spatium * mag();
                   p.drawLine(QLineF(x1, y, x2, y));
@@ -667,14 +725,14 @@ void StaffLines::draw(QPainter& p) const
                   break;
             case 3:
                   for (int i = 0; i < lines(); ++i) {
-                        qreal y = _pos.y() + i * _spatium * mag() * 2.0;
+                        qreal y = _pos.y() + i * _spatium * 2.0;
                         p.drawLine(QLineF(x1, y, x2, y));
                         }
                   break;
 
             default:
                   for (int i = 0; i < lines(); ++i) {
-                        qreal y = _pos.y() + i * _spatium * mag();
+                        qreal y = _pos.y() + i * _spatium;
                         p.drawLine(QLineF(x1, y, x2, y));
                         }
                   break;
@@ -688,12 +746,14 @@ void StaffLines::draw(QPainter& p) const
 
 double StaffLines::y1() const
       {
+      double _spatium = spatium();
+
       double y = measure()->system()->staff(staffIdx())->y();
       switch(lines()) {
             case 1:
-                  return y + _pos.y() + 1 * _spatium * mag();
+                  return y + _pos.y() + 1 * _spatium;
             case 2:
-                  return y + _pos.y() + 1 * _spatium * mag();
+                  return y + _pos.y() + 1 * _spatium;
             case 3:
             default:
                   return y + _pos.y();
@@ -706,15 +766,17 @@ double StaffLines::y1() const
 
 double StaffLines::y2() const
       {
+      double _spatium = spatium();
+
       double y = measure()->system()->staff(staffIdx())->y();
       switch(lines()) {
             case 1:
-                  return y + _pos.y() + 3 * _spatium * mag();
+                  return y + _pos.y() + 3 * _spatium;
             case 2:
-                  return y + _pos.y() + 3 * _spatium * mag();
+                  return y + _pos.y() + 3 * _spatium;
             case 3:
             default:
-                  return y + _pos.y() + 4 * _spatium * mag();
+                  return y + _pos.y() + 4 * _spatium;
             }
       }
 
@@ -763,8 +825,7 @@ Line::Line(Score* s, bool v)
 
 void Line::dump() const
       {
-      printf("  width:%g height:%g vert:%d\n",
-         point(_width), point(_len), vertical);
+      printf("  width:%g height:%g vert:%d\n", point(_width), point(_len), vertical);
       }
 
 //---------------------------------------------------------
@@ -789,11 +850,11 @@ void Line::setLineWidth(Spatium w)
 //   layout
 //---------------------------------------------------------
 
-void Line::layout(ScoreLayout* layout)
+void Line::layout()
       {
-      double spatium = layout->spatium();
-      double w = _width.val() * spatium;
-      double l = _len.val() * spatium;
+      double sp = spatium();
+      double w  = _width.val() * sp;
+      double l  = _len.val() * sp;
       double w2 = w * .5;
       if (vertical)
             setbbox(QRectF(-w2, -w2, w, l + w));
@@ -807,13 +868,15 @@ void Line::layout(ScoreLayout* layout)
 
 void Line::draw(QPainter& p) const
       {
+      double sp = spatium();
       QPen pen(p.pen());
-      pen.setWidthF(point(_width));
+      pen.setWidthF(_width.val() * sp);
       p.setPen(pen);
+      double l  = _len.val() * sp;
       if (vertical)
-            p.drawLine(QLineF(0.0, 0.0, 0.0, point(_len)));
+            p.drawLine(QLineF(0.0, 0.0, 0.0, l));
       else
-            p.drawLine(QLineF(0.0, 0.0, point(_len), 0.0));
+            p.drawLine(QLineF(0.0, 0.0, l, 0.0));
       }
 
 //---------------------------------------------------------
@@ -943,7 +1006,7 @@ Cursor::Cursor(Score* s, Viewer* v)
       viewer    = v;
       _on       = false;
       _blink    = true;
-      _h        = 6 * _spatium;
+      _h        = 6 * spatium();
       _seg      = 0;
       }
 
@@ -968,7 +1031,7 @@ void Cursor::draw(QPainter& p) const
             return;
       double x = pos().x();
       if (_seg)
-            x = _seg->canvasPos().x() - _spatium;
+            x = _seg->canvasPos().x() - spatium();
 
       int v = track() == -1 ? 0 : voice();
       p.fillRect(QRectF(x, pos().y(), _bbox.width(), _bbox.height()), QBrush(preferences.selectColor[v]));
@@ -1045,173 +1108,28 @@ QByteArray Element::mimeData(const QPointF& dragOffset) const
       return buffer.buffer();
       }
 
-//---------------------------------------------------------
-//   name2type
-//---------------------------------------------------------
-
-int Element::name2type(const QString& s)
-      {
-      int n = sizeof(elementNames)/sizeof(*elementNames);
-      for (int i = 0; i < n; ++i) {
-            if (s == elementNames[i])
-                  return i;
-            }
-      return -1;
-      }
-
-//---------------------------------------------------------
-//   name2Element
-//---------------------------------------------------------
-
-Element* Element::name2Element(const QString& s, Score* sc)
-      {
-      int type = Element::name2type(s);
-      if (type == -1)
-            return 0;
-      return Element::create(type, sc);
-      }
-
 //---------------------------------------------------------
 //   readType
 //    return -1 if no valid type found
 //---------------------------------------------------------
 
-int Element::readType(QDomElement& e, QPointF* dragOffset)
+ElementType Element::readType(QDomElement& e, QPointF* dragOffset)
       {
-      int type = -1;
+      ElementType type = INVALID;
 
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             if (e.tagName() == "dragOffset")
                   *dragOffset = readPoint(e);
-            else if ((type = name2type(e.tagName())) == -1) {
+            else if ((type = name2type(e.tagName())) == INVALID) {
                   domError(e);
                   break;
                   }
-            if (type >= 0)
+            if (type != INVALID)
                   break;
             }
       return type;
       }
 
-//---------------------------------------------------------
-//   create
-//    Element factory
-//---------------------------------------------------------
-
-Element* Element::create(int type, Score* score)
-      {
-      Element* el = 0;
-      switch(type) {
-            case VOLTA:
-                  el = new Volta(score);
-                  break;
-            case OTTAVA:
-                  el = new Ottava(score);
-                  break;
-            case TEXTLINE:
-                  el = new TextLine(score);
-                  break;
-            case TRILL:
-                  el = new Trill(score);
-                  break;
-            case PEDAL:
-                  el = new Pedal(score);
-                  break;
-            case HAIRPIN:
-                  el = new Hairpin(score);
-                  break;
-            case CLEF:
-                  el = new Clef(score);
-                  break;
-            case KEYSIG:
-                  el = new KeySig(score);
-                  break;
-            case TIMESIG:
-                  el = new TimeSig(score);
-                  break;
-            case BAR_LINE:
-                  el = new BarLine(score);
-                  break;
-            case ARPEGGIO:
-                  el = new Arpeggio(score);
-                  break;
-            case BREATH:
-                  el = new Breath(score);
-                  break;
-            case GLISSANDO:
-                  el = new Glissando(score);
-                  break;
-            case BRACKET:
-                  el = new Bracket(score);
-                  break;
-            case ATTRIBUTE:
-                  el = new Articulation(score);
-                  break;
-            case ACCIDENTAL:
-                  el = new Accidental(score);
-                  break;
-            case DYNAMIC:
-                  el = new Dynamic(score);
-                  break;
-            case TEXT:
-                  el = new Text(score);
-                  break;
-            case STAFF_TEXT:
-                  el = new StaffText(score);
-                  break;
-            case NOTEHEAD:
-                  el = new NoteHead(score);
-                  break;
-            case TREMOLO:
-                  el = new Tremolo(score);
-                  break;
-            case LAYOUT_BREAK:
-                  el = new LayoutBreak(score);
-                  break;
-            case MARKER:
-                  el = new Marker(score);
-                  break;
-            case JUMP:
-                  el = new Jump(score);
-                  break;
-            case REPEAT_MEASURE:
-                  el = new RepeatMeasure(score);
-                  break;
-            case ICON:
-                  el = new Icon(score);
-                  break;
-            case NOTE:
-                  el = new Note(score);
-                  break;
-            case SYMBOL:
-                  el = new Symbol(score);
-                  break;
-            case CHORD:
-                  el = new Chord(score);
-                  break;
-            case REST:
-                  el = new Rest(score);
-                  break;
-            case SPACER:
-                  el = new Spacer(score);
-                  break;
-            case TEMPO_TEXT:
-                  el = new TempoText(score);
-                  break;
-            case HARMONY:
-                  el = new Harmony(score);
-                  break;
-            case LYRICS:
-                  el = new Lyrics(score);
-                  break;
-            default:
-                  printf("Element::create(): cannot create element type %s\n",
-                     elementNames[type]);
-                  break;
-            }
-      return el;
-      }
-
 //---------------------------------------------------------
 //   startEdit
 //---------------------------------------------------------
@@ -1227,7 +1145,7 @@ bool Element::startEdit(Viewer*, const QPointF&)
 
 void Element::editDrag(int, const QPointF& delta)
       {
-      setUserOff(userOff() + delta / _spatium);
+      setUserOff(userOff() + delta);
       }
 
 //---------------------------------------------------------
@@ -1290,4 +1208,195 @@ void Icon::read(QDomElement e)
             }
       }
 
+//---------------------------------------------------------
+//   create
+//    Element factory
+//---------------------------------------------------------
 
+Element* Element::create(ElementType type, Score* score)
+      {
+      switch(type) {
+            case VOLTA:             return new Volta(score);
+            case OTTAVA:            return new Ottava(score);
+            case TEXTLINE:          return new TextLine(score);
+            case TRILL:             return new Trill(score);
+            case PEDAL:             return new Pedal(score);
+            case HAIRPIN:           return new Hairpin(score);
+            case CLEF:              return new Clef(score);
+            case KEYSIG:            return new KeySig(score);
+            case TIMESIG:           return new TimeSig(score);
+            case BAR_LINE:          return new BarLine(score);
+            case ARPEGGIO:          return new Arpeggio(score);
+            case BREATH:            return new Breath(score);
+            case GLISSANDO:         return new Glissando(score);
+            case BRACKET:           return new Bracket(score);
+            case ARTICULATION:      return new Articulation(score);
+            case ACCIDENTAL:        return new Accidental(score);
+            case DYNAMIC:           return new Dynamic(score);
+            case TEXT:              return new Text(score);
+            case STAFF_TEXT:        return new StaffText(score);
+            case NOTEHEAD:          return new NoteHead(score);
+            case TREMOLO:           return new Tremolo(score);
+            case LAYOUT_BREAK:      return new LayoutBreak(score);
+            case MARKER:            return new Marker(score);
+            case JUMP:              return new Jump(score);
+            case REPEAT_MEASURE:    return new RepeatMeasure(score);
+            case ICON:              return new Icon(score);
+            case NOTE:              return new Note(score);
+            case SYMBOL:            return new Symbol(score);
+            case CHORD:             return new Chord(score);
+            case REST:              return new Rest(score);
+            case SPACER:            return new Spacer(score);
+            case TEMPO_TEXT:        return new TempoText(score);
+            case HARMONY:           return new Harmony(score);
+            case LYRICS:            return new Lyrics(score);
+            case STEM:              return new Stem(score);
+            case SLUR:              return new Slur(score);
+
+            case SLUR_SEGMENT:
+            case STEM_SLASH:
+            case LINE:
+            case IMAGE:
+            case TIE:
+            case PAGE:
+            case BEAM:
+            case HOOK:
+            case TUPLET:
+            case HAIRPIN_SEGMENT:
+            case OTTAVA_SEGMENT:
+            case PEDAL_SEGMENT:
+            case TRILL_SEGMENT:
+            case TEXTLINE_SEGMENT:
+            case VOLTA_SEGMENT:
+            case LEDGER_LINE:
+            case MEASURE:
+            case STAFF_LINES:
+            case CURSOR:
+            case SELECTION:
+            case LASSO:
+            case SHADOW_NOTE:
+            case RUBBERBAND:
+            case SEGMENT:
+            case SYSTEM:
+            case COMPOUND:
+            case ELEMENT:
+            case ELEMENT_LIST:
+            case STAFF_LIST:
+            case MEASURE_LIST:
+            case LAYOUT:
+            case HBOX:
+            case VBOX:
+            case MAXTYPE:
+            case INVALID:  break;
+            }
+      printf("cannot create type <%s>\n", Element::name(type));
+      return 0;
+      }
+
+//---------------------------------------------------------
+//   name
+//---------------------------------------------------------
+
+const char* Element::name(ElementType type)
+      {
+      switch(type) {
+            case SYMBOL:            return "Symbol";
+            case TEXT:              return "Text";
+            case SLUR_SEGMENT:      return "SlurSegment";
+            case BAR_LINE:          return "BarLine";
+            case STEM_SLASH:        return "StemSlash";
+            case LINE:              return "Line";
+            case BRACKET:           return "Bracket";
+            case ARPEGGIO:          return "Arpeggio";
+            case ACCIDENTAL:        return "Accidental";
+            case NOTE:              return "Note";
+            case STEM:              return "Stem";
+            case CLEF:              return "Clef";
+            case KEYSIG:            return "KeySig";
+            case TIMESIG:           return "TimeSig";
+            case REST:              return "Rest";
+            case BREATH:            return "Breath";
+            case GLISSANDO:         return "Glissando";
+            case REPEAT_MEASURE:    return "RepeatMeasure";
+            case IMAGE:             return "Image";
+            case TIE:               return "Tie";
+            case ARTICULATION:      return "Articulation";
+            case DYNAMIC:           return "Dynamic";
+            case PAGE:              return "Page";
+            case BEAM:              return "Beam";
+            case HOOK:              return "Hook";
+            case LYRICS:            return "Lyrics";
+            case MARKER:            return "Marker";
+            case JUMP:              return "Jump";
+            case TUPLET:            return "Tuplet";
+            case TEMPO_TEXT:        return "Tempo";
+            case STAFF_TEXT:        return "StaffText";
+            case HARMONY:           return "Harmony";
+            case VOLTA:             return "Volta";
+            case HAIRPIN_SEGMENT:   return "HairpinSegment";
+            case OTTAVA_SEGMENT:    return "OttavaSegment";
+            case PEDAL_SEGMENT:     return "PedalSegment";
+            case TRILL_SEGMENT:     return "TrillSegment";
+            case TEXTLINE_SEGMENT:  return "TextLineSegment";
+            case VOLTA_SEGMENT:     return "VoltaSegment";
+            case LAYOUT_BREAK:      return "LayoutBreak";
+            case SPACER:            return "Spacer";
+            case LEDGER_LINE:       return "LedgerLine";
+            case NOTEHEAD:          return "NoteHead";
+            case TREMOLO:           return "Tremolo";
+            case MEASURE:           return "Measure";
+            case STAFF_LINES:       return "StaffLines";
+            case CURSOR:            return "Cursor";
+            case SELECTION:         return "Selection";
+            case LASSO:             return "Lasso";
+            case SHADOW_NOTE:       return "ShadowNote";
+            case RUBBERBAND:        return "RubberBand";
+            case HAIRPIN:           return "HairPin";
+            case OTTAVA:            return "Ottava";
+            case PEDAL:             return "Pedal";
+            case TRILL:             return "Trill";
+            case TEXTLINE:          return "TextLine";
+            case SEGMENT:           return "Segment";
+            case SYSTEM:            return "System";
+            case COMPOUND:          return "Compound";
+            case CHORD:             return "Chord";
+            case SLUR:              return "Slur";
+            case ELEMENT:           return "Element";
+            case ELEMENT_LIST:      return "ElementList";
+            case STAFF_LIST:        return "StaffList";
+            case MEASURE_LIST:      return "MeasureList";
+            case LAYOUT:            return "Layout";
+            case HBOX:              return "HBox";
+            case VBOX:              return "VBox";
+            case ICON:              return "Icon";
+            case INVALID:
+            case MAXTYPE:
+                  break;
+            }
+      return "??";
+      }
+
+//---------------------------------------------------------
+//   name2type
+//---------------------------------------------------------
+
+ElementType Element::name2type(const QString& s)
+      {
+      for (int i = 0; i < MAXTYPE; ++i) {
+            if (s == elementNames[i])
+                  return ElementType(i);
+            }
+      return INVALID;
+      }
+
+//---------------------------------------------------------
+//   name2Element
+//---------------------------------------------------------
+
+Element* Element::name2Element(const QString& s, Score* sc)
+      {
+      ElementType type = Element::name2type(s);
+      if (type == INVALID)
+            return 0;
+      return Element::create(type, sc);
+      }
