@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: excerpt.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2009 Werner Schweer and others
 //
@@ -23,13 +23,14 @@
 #include "part.h"
 #include "xml.h"
 #include "staff.h"
-#include "layout.h"
 #include "box.h"
 #include "canvas.h"
 #include "style.h"
 #include "page.h"
 #include "text.h"
 #include "slur.h"
+#include "sig.h"
+#include "tempo.h"
 
 //---------------------------------------------------------
 //   read
@@ -92,7 +93,7 @@ bool Excerpt::operator!=(const Excerpt& e) const
 
 Score* Score::createExcerpt(Excerpt* excerpt)
       {
-      Score* s      = new Score();
+      Score* s      = new Score(_style);
       QFileInfo* fi = s->fileInfo();
       QString name  = fileInfo()->path() + "/" + excerpt->name() + ".msc";
       fi->setFile(name);
@@ -123,7 +124,7 @@ Score* Score::createExcerpt(Excerpt* excerpt)
       QDomElement e = doc.documentElement();
       s->read(e);
       if (!excerpt->title().isEmpty()) {
-            MeasureBase* measure = s->layout()->first();
+            MeasureBase* measure = s->first();
             if (measure->type() != VBOX) {
                   measure = new VBox(s);
                   measure->setTick(0);
@@ -131,12 +132,18 @@ Score* Score::createExcerpt(Excerpt* excerpt)
                   }
             Text* txt = new Text(s);
             txt->setSubtype(TEXT_INSTRUMENT_EXCERPT);
+            txt->setTextStyle(TEXT_STYLE_INSTRUMENT_EXCERPT);
             txt->setText(excerpt->title());
             measure->add(txt);
             }
 
       s->renumberMeasures();
       s->setCreated(true);
+      s->rebuildMidiMapping();
+      s->updateChannel();
+      s->fixPpitch();
+//      s->setLayoutAll(true);
+//      s->end();
       return s;
       }
 
@@ -155,7 +162,7 @@ void Score::writeExcerpt(Excerpt* excerpt, Xml& xml)
             endCmd();
             canvas()->setState(Canvas::NORMAL);  //calls endEdit()
             }
-      _style->save(xml);
+      _style.save(xml, true);
       for (int i = 0; i < TEXT_STYLES; ++i) {
             if (*_textStyles[i] != defaultTextStyleArray[i])
                   _textStyles[i]->write(xml);
@@ -165,7 +172,7 @@ void Score::writeExcerpt(Excerpt* excerpt, Xml& xml)
       pageFormat()->write(xml);
       if (rights) {
             xml.stag("rights");
-            xml.writeHtml(rights->toHtml("UTF-8"));
+            xml.writeHtml(rights->getHtml());
             xml.etag();
             }
       if (!_movementNumber.isEmpty())
@@ -197,32 +204,34 @@ void Score::writeExcerpt(Excerpt* excerpt, Xml& xml)
             ++didx;
             }
 
-      // to serialize slurs, the get an id; this id is referenced
+      // to serialize slurs, they get an id; this id is referenced
       // in begin-end elements
       int slurId = 0;
 
       xml.trackDiff = 0;
       foreach(Element* el, _gel) {
+            int staffIdx = el->staffIdx();
             if (el->type() == SLUR) {
                   Slur* slur = static_cast<Slur*>(el);
                   int staffIdx1 = slur->startElement()->staffIdx();
                   int staffIdx2 = slur->endElement()->staffIdx();
                   if (trackOffset[staffIdx1] == HIDDEN || trackOffset[staffIdx2] == HIDDEN)
                         continue;
-                  xml.trackDiff = trackOffset[staffIdx1];
+                  xml.trackDiff = staffIdx; // slur staffIdx is always zero
                   slur->setId(slurId++);
-                  slur->write(xml);
                   }
             else {
                   if (el->track() != -1) {
-                        int staffIdx = el->staffIdx();
-                        if (trackOffset[staffIdx] == HIDDEN)
-                              continue;
-                        xml.trackDiff = trackOffset[staffIdx];
+//                        if (el->type() != VOLTA) {                      // HACK
+                              if (trackOffset[staffIdx] == HIDDEN)
+                                    continue;
+                              xml.trackDiff = trackOffset[staffIdx];
+//                              }
                         }
-                  el->write(xml);
                   }
+            el->write(xml);
             }
+      xml.trackDiff = 0;
       bool isFirstStaff = true;
       for (int staffIdx = 0; staffIdx < _staves.size(); ++staffIdx) {
             Part* part = staff(staffIdx)->part();
@@ -233,7 +242,7 @@ void Score::writeExcerpt(Excerpt* excerpt, Xml& xml)
             xml.curTrack  = staffIdx * VOICES;
             xml.trackDiff = trackOffset[staffIdx];
             xml.stag(QString("Staff id=\"%1\"").arg(staffIdx + 1 + xml.trackDiff/4));
-            for (MeasureBase* m = _layout->first(); m; m = m->next()) {
+            for (MeasureBase* m = first(); m; m = m->next()) {
                   if (isFirstStaff || m->type() == MEASURE)
                         m->write(xml, staffIdx, isFirstStaff);
                   if (m->type() == MEASURE)
