@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: exportly.cpp,v 1.71 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: exportly.cpp 1841 2009-05-20 12:27:19Z wschweer $
 //
 //  Copyright (C) 2007 Werner Schweer and others
 //
@@ -41,7 +41,6 @@
 #include "hairpin.h"
 #include "key.h"
 #include "keysig.h"
-#include "layout.h"
 #include "measure.h"
 #include "note.h"
 #include "ottava.h"
@@ -70,7 +69,6 @@ static  const int BRACKSTAVES=64;
 static  const int MAXPARTGROUPS = 8;
 
 
-
 //---------------------------------------------------------
 //   ExportLy
 //---------------------------------------------------------
@@ -84,11 +82,12 @@ class ExportLy {
   Direction stemDirection;
   int indx;
 
-  int timeabove, timebelow;
   int  n, z1, z2, z3, z4; //timesignatures
   int barlen;
   bool slur;
-  bool pickup; // must be preserved from voice to voice, but deleted before each new measure....
+  bool pickup;
+  bool rehearsalnumbers;
+  bool donefirst; //to prevent doing things in first ordinary bar which are already done in pickupbar
   bool graceswitch, gracebeam;
   int gracecount;
   int prevpitch, staffpitch, chordpitch, oktavdiff;
@@ -105,6 +104,7 @@ class ExportLy {
   int findSlur(const Slur* s) const;
   const char *relativ, *staffrelativ;
   bool voiceActive[VOICES];
+  int prevElTick;
 
   struct lybrackets
   {
@@ -112,7 +112,7 @@ class ExportLy {
     bool bracestart,brakstart, braceend, brakend;
     int braceno, brakno;
   };
-  
+
   struct lybrackets lybracks[BRACKSTAVES];
   void bracktest();
 
@@ -218,9 +218,6 @@ int numval(int num)
 }
 
 
-
-
-
 //---------------------------------------------------------
 // initBrackets -- init array of brackets and braces info
 //---------------------------------------------------------
@@ -235,7 +232,7 @@ void ExportLy::initBrackets()
       lybracks[i].braceend=false;
       lybracks[i].brakend=false;
       lybracks[i].braceno=0;
-      lybracks[i].brakno=0;      
+      lybracks[i].brakno=0;
     }
 }
 
@@ -250,7 +247,7 @@ void ExportLy::brackRegister(int brnumber, int bratype, int staffnr, bool start,
 
 {
   QString br = "";
-  switch(bratype) 
+  switch(bratype)
     {
     case BRACKET_NORMAL:
       if (start) lybracks[staffnr].brakstart=true;
@@ -268,12 +265,11 @@ void ExportLy::brackRegister(int brnumber, int bratype, int staffnr, bool start,
       break;
     case -1: //piano-staff: lilypond makes rigid distance between
 	     //staffs to allow cross-staff beaming.
-      printf("register pianostaff\n");
       lybracks[staffnr].piano=true;
       if (start) lybracks[staffnr].bracestart=true;
-      if (end) {lybracks[staffnr].braceend=true; printf("register pianostaff end\n"); }
+      if (end) lybracks[staffnr].braceend=true;
       lybracks[staffnr].braceno=brnumber;
-      break; 
+      break;
     default:
       printf("bracket subtype %d not understood\n", bratype);
     }
@@ -291,28 +287,21 @@ void ExportLy::findBrackets()
   char groupnumber;
   groupnumber=1;
   const QList<Part*>* il = score->parts();  //list of parts
-  
+
   for (int partnumber = 0; partnumber < il->size(); ++partnumber)  //run thru list of parts
     {
-      printf("partnumber: %d\n", partnumber);
       Part* part = il->at(partnumber);
-      printf("staves in part: %d\n", part->nstaves());
       if (part->nstaves() == 2) pianostaff=true;
       for (int stavno = 0; stavno < part->nstaves(); stavno++) //run thru list of staves in part.
 	{
-	  printf("stavnummer %d\n", stavno);
-	  if (pianostaff) printf("pianostaff\n");
-	  
 	  if (pianostaff)
 	    {
 	      if (stavno==0)
 		{
-		  printf("start of pianostaff\n");
 		  brackRegister(groupnumber, -1, partnumber+stavno, true, false);
 		}
 	      if (stavno==1)
 		{
-		  printf("end of pianostaff\n");
 		  brackRegister(groupnumber, -1, partnumber+stavno, false, true);
 		  pianostaff=false;
 		}
@@ -320,19 +309,17 @@ void ExportLy::findBrackets()
 	  else //not pianostaff
 	    {
 	      Staff* st = part->staff(stavno);
-	      if (st) 
+	      if (st)
 		{
 		  for (int braclev= 0; braclev < st->bracketLevels(); braclev++) //run thru bracketlevels of staff
 		    {
-		      printf("bracketlevel: %d\n", braclev);
 		      if (st->bracket(braclev) != NO_BRACKET) //if bracket
 			{
 			  groupnumber++;
-			  if (groupnumber < MAXPARTGROUPS) 
+			  if (groupnumber < MAXPARTGROUPS)
 			    { //brackRegister(bracketnumber, brackettype, staffnr, start, end)
-			      printf("start groupnumber=%d end=%d\n", groupnumber, partnumber - 1 + st->bracketSpan(braclev));
 			      brackRegister(groupnumber, st->bracket(braclev), partnumber, true, false);
-			      brackRegister(groupnumber,st->bracket(braclev), partnumber-1+st->bracketSpan(braclev), false, true); 
+			      brackRegister(groupnumber,st->bracket(braclev), partnumber-1+st->bracketSpan(braclev), false, true);
 			    }
 			}//end of if bracket.
 		    }//end of bracket-levels of staff
@@ -341,7 +328,7 @@ void ExportLy::findBrackets()
 	}//end of stafflist
     }//end of parts-list
 }//end of findBrackets;
-  
+
 
 
 void ExportLy::bracktest()
@@ -489,9 +476,24 @@ void ExportLy::tempoText(TempoText* text)
 //---------------------------------------------------------
 
 void ExportLy::words(Text* text)
-      {
-	//todo: find mscore-position of text, and position accordingly in lily.
-	out << "^\\markup {" << text->getText() << "}";
+     {
+       char* c;
+       char a;
+       //todo: find exact mscore-position of text and not only anchorpoint, and position accordingly in lily.
+	if (text->subtypeName()== "RehearsalMark")
+	  {
+	    out << "\\mark\\default ";
+	    //There must be an easier way to check if QString is number????
+	    //is there a .toInteger()????
+	    c=text->getText().toAscii().data();
+	    a=(char)c[0];
+	    if ((a>= 48) && (a<= 57))
+	      {
+		rehearsalnumbers=true;
+	      }
+	  }
+	else
+	  out << "^\\markup {" << text->getText() << "}";
       }
 
 
@@ -511,8 +513,9 @@ void ExportLy::hairpin(Hairpin* hp, int tick)
 	    if (art == 1) //crescendo
 	      out << "\\> ";
 	  }
-	else   out << "\\! ";
+	else   out << "\\! "; //end of hairpin
       }
+
 //---------------------------------------------------------
 //   ottava
 // <octave-shift type="down" size="8" relative-y="14"/>
@@ -760,6 +763,7 @@ void ExportLy::handleElement(Element* el, bool start)
 	      dynamic((Dynamic*) instruction);
 	      break;
 	    case HAIRPIN:
+	      printf("funnet HÅRNÅL i ankerliste\n");
 	      hairpin((Hairpin*) instruction, anchors[i].tick);
 	      break;
 	    case OTTAVA:
@@ -773,7 +777,7 @@ void ExportLy::handleElement(Element* el, bool start)
 	      break;
 	    default:
 	      printf("InstructionHandler::handleElement: direction type %s at tick %d not implemented\n",
-		     elementNames[instruction->type()], anchors[i].tick);
+		     Element::name(instruction->type()), anchors[i].tick);
 	      break;
 	    }
 	  removeAnchor(i);
@@ -811,25 +815,21 @@ bool ExportLy::findSpecificMatchInMeasure(int tick, Staff* stf, Measure* m, int
       for (Segment* seg = m->first(); seg; seg = seg->next())
 	{
 	  Element* el = seg->element(st);
-	  if (!el)
-	    continue;
-	  if (!(el->isChordRest()) && ((el->tick() == tick) || ((el->tick() + el->tickLen()) == tick)))
-	    {
-	      printf("funnet uten chordorrest\n");
-	    }
-
-	  if ((el->isChordRest() && el->staff() == stf) && ((el->tick() == tick) || ((el->tick() + el->tickLen()) == tick)))
+	  if (!el) continue;
+	  if ((el->isChordRest()) && (el->staff() == stf) && ((el->tick() >= tick)))
 	    {
+	      if (el->tick() > tick) tick=prevElTick;
 	      anker.anchor=el;
 	      found=true;
 	      anker.tick=tick;
 	      anker.start=true;
-	      continue;
+	      goto fertig;
 	    }
-	}
+	    prevElTick=el->tick();
+	 }
     }
-// if (found=true) return true; else return false;
-      return found;
+ fertig:
+ return found;
 }
 
 
@@ -857,9 +857,10 @@ bool ExportLy::findSpecificMatchInPart(int tick, Staff* st, Score* sc, int strac
       if (mb->type() != MEASURE)
 	continue;
       Measure* m = (Measure*)mb;
-      found= findSpecificMatchInMeasure(tick, st, m, strack, etrack);
-    }
-  return found;
+      found=findSpecificMatchInMeasure(tick, st, m, strack, etrack);
+      if (found) break;
+     }
+return found;
 }
 
 //---------------------------------------------------------
@@ -870,7 +871,6 @@ bool ExportLy::findMatchInPart(int tick, Staff* st, Score* sc, Part*, int strack
       {
 	bool found=false;
 	found = findSpecificMatchInPart(tick, st, sc, strack, etrack);
-	found =	findSpecificMatchInPart(tick, st, sc, strack, etrack);
 	return found;
       }
 
@@ -884,6 +884,7 @@ void ExportLy::buildInstructionList(Part* p, int strack, int etrack)
 {
   // part-level elements stored in the score layout
 
+  prevElTick=0;
   foreach(Element* instruction, *(score->gel()))
     {
       bool found=false;
@@ -891,12 +892,12 @@ void ExportLy::buildInstructionList(Part* p, int strack, int etrack)
 	{
 	case JUMP:
 	  printf("score JUMP found at tick: %d\n", instruction->tick());
-
 	case MARKER:
 	  printf("score MARKER found at tick: %d\n", instruction->tick());
 	case HAIRPIN:
 	case OTTAVA:
 	case PEDAL:
+	case DYNAMIC:
 	case TEXTLINE:
 	  {
 	    SLine* sl = (SLine*) instruction;
@@ -912,16 +913,16 @@ void ExportLy::buildInstructionList(Part* p, int strack, int etrack)
 		anker.instruct=instruction;
 		storeAnchor(anker);
 	      }
-	  }
 	  break;
-	default:
-	  // all others silently ignored
-	  // printf("InstructionHandler::buildInstructionList: direction type %s not implemented\n",
-	  //        elementNames[instruction->type()]);
+	  }
+	  default:
+	  // all others ignored
+	  //printf(" instruction type %s not implemented\n", Element::name(instruction->type()));
 	  break;
 	}
     }// end foreach element....
 
+
   // part-level elements stored in measures
   for (MeasureBase* mb = score->measures()->first(); mb; mb = mb->next())
     {
@@ -959,19 +960,21 @@ void ExportLy::buildInstructionList(Measure* m, int strack, int etrack)
 	case SYMBOL:
 	case TEMPO_TEXT:
 	case TEXT:
+	case HAIRPIN:
+	case OTTAVA:
+	case PEDAL:
 	case STAFF_TEXT:
 	  found = findMatchInMeasure(instruction->tick(), instruction->staff(), m, strack, etrack);
 	  if (found)
 	    {
 	      anker.instruct=instruction;
 	      storeAnchor(anker);
-	      found=false;
+	      printf("put instruction type %d in anchorlist\n", instruction->type());
 	    }
-	  if (!found) printf("element ikke forankret: %d\n", instruction->type());
+	  //else if (!found) printf("element not anchored?!: %d\n", instruction->type());
 	  break;
 	default:
 	  printf("Intet measurerelevant anker\n");
-	  break;
 	}
     }
 }
@@ -1116,6 +1119,14 @@ int ExportLy::voltaCheckBar(Measure* meas, int i)
     default:
       break;
     }//switch
+
+  bool rs = meas->repeatFlags() & RepeatStart;
+  if (rs)
+    {
+      i++;
+      voltarray[i].voltart=startrepeat;
+      voltarray[i].barno=taktnr-1; //set as last element in previous measure.
+    }
   return i;
 }//end voltacheckbarline
 
@@ -1140,7 +1151,7 @@ void  ExportLy::findVolta()
 
   i=0;
 
-  for (MeasureBase * m=score->layout()->first(); m; m=m->next())
+  for (MeasureBase * m=score->first(); m; m=m->next())
     {// for all measures
       if (m->type() !=MEASURE )
 	continue;
@@ -1165,7 +1176,6 @@ void  ExportLy::findVolta()
 		  voltarray[i].voltart = startending;
 		  voltarray[i].barno=taktnr-1; //register as last element i previous measure
 		}
-
 	      if (v->tick2() == m->tick() + m->tickLen()) // if it is at the end of measure
 		{
 		  i++;
@@ -1181,13 +1191,12 @@ void  ExportLy::findVolta()
 		}
 	    }
 	}// for all elements
-
       i=voltaCheckBar((Measure *) m, i);
-
     }//for all measures
-
   lastind=i;
 
+  //  for (i=0; i<lastind; i++)
+  //  cout << "voltatest: " << voltarray[i].barno << "  " << voltarray[i].voltart <<"\n";
 }// end findvolta
 
 
@@ -1210,23 +1219,23 @@ void ExportLy::writeClef(int clef)
 {
   out << "\\clef ";
   switch(clef) {
-  case CLEF_G:      out << "treble";       break;
-  case CLEF_F:      out << "bass";         break;
-  case CLEF_G1:     out << "\"treble^8\""; break;
-  case CLEF_G2:     out << "\"treble^15\"";break;
-  case CLEF_G3:     out << "\"treble_8\""; break;
-  case CLEF_F8:     out << "\"bass_8\"";   break;
-  case CLEF_F15:    out << "\"bass_15\"";  break;
-  case CLEF_F_B:    out << "bass";         break;
-  case CLEF_F_C:    out << "bass";         break;
-  case CLEF_C1:     out <<  "soprano";     break;
-  case CLEF_C2:     out <<  "mezzo-soprano";break;
-  case CLEF_C3:     out <<  "alto";        break;
-  case CLEF_C4:     out <<  "tenor";       break;
-  case CLEF_TAB:    out <<  "tab";         break;
-  case CLEF_PERC:   out <<  "percussion";  break;
+  case CLEF_G:      out << "treble\n";         break;
+  case CLEF_F:      out << "bass\n";           break;
+  case CLEF_G1:     out << "\"treble^8\"\n";   break;
+  case CLEF_G2:     out << "\"treble^15\"\n";  break;
+  case CLEF_G3:     out << "\"treble_8\"\n";   break;
+  case CLEF_F8:     out << "\"bass_8\"\n";     break;
+  case CLEF_F15:    out << "\"bass_15\"\n";    break;
+  case CLEF_F_B:    out << "bass\n";           break;
+  case CLEF_F_C:    out << "bass\n";           break;
+  case CLEF_C1:     out <<  "soprano\n";       break;
+  case CLEF_C2:     out <<  "mezzo-soprano\n"; break;
+  case CLEF_C3:     out <<  "alto\n";          break;
+  case CLEF_C4:     out <<  "tenor\n";         break;
+  case CLEF_TAB:    out <<  "tab\n";           break;
+  case CLEF_PERC:   out <<  "percussion\n";    break;
   }
-  out << " ";
+
 }
 
 //---------------------------------------------------------
@@ -1235,8 +1244,16 @@ void ExportLy::writeClef(int clef)
 
 void ExportLy::writeTimeSig(TimeSig* sig)
 {
+  int st = sig->subtype();
   sig->getSig(&n, &z1, &z2, &z3, &z4);
-  timebelow=n;
+  //lilypond writes 4/4 as C by default, so only check for cut.
+  if (st == TSIG_ALLA_BREVE)
+    {
+      z1=2;
+      n=2;
+      // 2/2 automatically written as alla breve by lily.
+    }
+  indent();
   out << "\\time " << z1 << "/" << n << " ";
 }
 
@@ -1247,12 +1264,12 @@ void ExportLy::writeTimeSig(TimeSig* sig)
 void ExportLy::writeKeySig(int st)
 {
   st = char(st & 0xff);
-  if (st == 0)
-    return;
+  indent();
   out << "\\key ";
   switch(st) {
+  case 7:  out << "cis"; break;
   case 6:  out << "fis"; break;
-  case 5:  out << "h";   break;
+  case 5:  out << "b";   break;
   case 4:  out << "e";   break;
   case 3:  out << "a";   break;
   case 2:  out << "d";   break;
@@ -1534,14 +1551,14 @@ void ExportLy::writeChord(Chord* c)
 	    out << "\\stemDown ";
 	//   else if (d == AUTO)
 // 	    {
-// 	      if (graceswitch == true) 
+// 	      if (graceswitch == true)
 // 		{
 // 		  out << "\\stemNeutral "; // we set this at the end of graces anyway.
 // 		}
 // 	    }
 	}
     }
-  
+
   bool tie=false;
   NoteList* nl = c->noteList();
 
@@ -1605,6 +1622,7 @@ void ExportLy::writeChord(Chord* c)
       oktavdiff=prevpitch - purepitch;
       int oktreit=numval(oktavdiff);
 
+
       while (oktreit > 0)
 	{
 	  if ((oktavdiff < -6) or ((prevnote=="b") and (oktavdiff < -5)))
@@ -1620,6 +1638,7 @@ void ExportLy::writeChord(Chord* c)
 	  oktreit=oktreit-12;
 	}
 
+
       prevpitch=purepitch;
       pitchlist[j]=purepitch;
       j++;
@@ -1629,7 +1648,6 @@ void ExportLy::writeChord(Chord* c)
 	  chordpitch=prevpitch;
 	  chordnote=cleannote;
 	}
-      //^^^^^^remember pitch of chordnote to write next chordnote relative to.
 
       ++i; //number of notes in chord, we progress to next chordnote
       if (i == nl->end())
@@ -1642,10 +1660,9 @@ void ExportLy::writeChord(Chord* c)
     out << ">"; //endofchord sign
     cleannote=chordnote;
     //if this is a chord, use first note of chord as previous note
-    //instead of actually previous note.
+    //instead of actual previous note.
     }
 
-//remember bottom note of chord to write next single, non-chord note relative to:
   j=0;
   prevpitch=pitchlist[0];
    while (pitchlist[j] !=0)
@@ -1656,7 +1673,7 @@ void ExportLy::writeChord(Chord* c)
 
   writeLen(c->tickLen());
 
-  
+
   if (graceswitch)
     {
       out << " ( "; //gracenotes always slurred. Remove by hand in lilyfile otherwise.
@@ -1708,7 +1725,7 @@ int ExportLy::getLen(int l, int* dots)
       len  = 1;
       *dots = 1;
     }
-  else if (l == 5 * division)
+  else if (l == 5 * division) //doubledotted whole
     {
       len = 1;
       *dots = 2;
@@ -1720,6 +1737,11 @@ int ExportLy::getLen(int l, int* dots)
       len = 2;
       *dots = 1;
     }
+  else if (l == ((division/2)*7)) // double-dotted half: 7/8 used for \partial bar.
+    {
+      len = 2;
+      *dots=2;
+    }
   else if (l == 2 * division)
     len = 2;
   else if (l == division)
@@ -1761,7 +1783,7 @@ int ExportLy::getLen(int l, int* dots)
     len = 16;
   else if (l == division /3*4)
     len = 32;
-  else printf("unsupported len %d (%d,%d)\n", l, l/division, l % division);
+  else printf("measure: %d, unsupported len %d (%d,%d)\n", measurenumber, l, l/division, l % division);
   return len;
 }
 
@@ -1797,6 +1819,7 @@ void ExportLy::writeLen(int ticks)
 	  break;
 	}
       curTicks = ticks;
+      if (dots>0) curTicks = 0; //first note after dotted: always explicit length
     }
 }
 
@@ -1916,7 +1939,7 @@ void ExportLy::writeVolta(int measurenumber, int lastind)
 	      out << "\\bar \"|.\"";
 	      break;
           default:
-	    case none: printf("strange voltarraycontents?\n");
+	    // case none: printf("strange voltarraycontents?\n");
 	      break;
 	    }//end switch
 
@@ -1941,11 +1964,18 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 {
   int i=0;
   char cvoicenum, cstaffnum;
-
+  if (m->no() >0) out << " | % " << m->no() << "\n" ; //barchecksign and barnumber for previous measure
   measurenumber=m->no()+1;
 
-  if (measurenumber==1)
+   if (m->irregular())
+     {
+       printf("irregular measure, number: %d\n", measurenumber);
+     }
+
+
+  if ((measurenumber==1) and (donefirst==false))
     {
+      donefirst=true;
       level=0;
       indent();
       cvoicenum=voice+65;
@@ -1973,13 +2003,12 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 	  out << "\\set Staff.shortInstrumentName = #\"" <<staffname[staffInd].partshort << "\"\n";
 	  indent();
 	  writeClef(staff->clef(0));
+	  indent();
+	  out << "%staffkeysig\n";
+	  //done in first measure anyway: ??
 	  writeKeySig(staff->keymap()->key(0));
-	  out << "\n";
-	  indent();
-	  score->sigmap->timesig(0, z1, n);
-	  out << "\\time " << z1<< "/" << n << " \n";
-	  indent();
-	  out << "\n\n";
+// 	  score->sigmap->timesig(0, z1, n);
+// 	  out << "\\time " << z1<< "/" << n << " \n";
 	}
 
       switch(voice)
@@ -2012,17 +2041,17 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 	      repeatactive=true;
 	    }
 	}
-    }// if start of first measure
+    }// END if start of first measure
 
   indent();
   int tick = m->tick();
-  barlen = m->tickLen();
   int measuretick=0;
   Element* e;
 
+
   for(Segment* s = m->first(); s; s = s->next())
     {
-      // for all segments in measure.
+      // for all segments in measure. Get element:
       e = s->element(staffInd * VOICES + voice);
 
       if (!(e == 0 || e->generated()))  voiceActive[voice] = true;
@@ -2035,8 +2064,9 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 	  break;
 	case TIMESIG:
 	  {
+	    out << "%bartimesig: \n";
 	    writeTimeSig((TimeSig*)e);
-	    out << "\n\n";
+	    out << "\n";
 	    barlen=m->tickLen();
 	    int nombarlen=z1*division;
 	    if (n==8) nombarlen=nombarlen/2;
@@ -2064,17 +2094,23 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 		    }
 		  } //end switch (punkt)
 	      }
+	    curTicks=-1; //we always need explicit length after timesig.
 	    indent();
-	    break;	  }
+	    break;
+	  }
 	case KEYSIG:
+	  indent();
+	  out << "%barkeysig: \n";
+	  indent();
 	  writeKeySig(e->subtype());
+	  curTicks=-1; //feels safe to force explicit length after keysig
 	  break;
 	case CHORD:
 	  {
 	    int ntick = e->tick() - tick;
 	    if (ntick > 0)
 	      {
-		writeRest(ntick, 2);
+		writeRest(ntick, 2);//invisible rest: s
 		curTicks=-1;
 	      }
 	    tick += ntick;
@@ -2088,12 +2124,15 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 	  {
 	    findTuplets((Rest *) e);
 	    int l = e->tickLen();
-	    if (l == 0) {
-	      l = ((Rest*)e)->segment()->measure()->tickLen();
-	      writeRest(l, 1);
-	    }
+	    int mlen=((Rest*)e)->segment()->measure()->tickLen();
+	    //	    printf("pauselengde: %d, taktlengde %d \n",l, mlen );
+	    if ((l==mlen) or (l==0))
+	      {
+		l = ((Rest*)e)->segment()->measure()->tickLen();
+		writeRest(l, 1); //wholemeasure rest: R
+	      }
 	    else
-	      writeRest(l, 0);
+	      writeRest(l, 0);//ordinary rest: r
 	    tick += l;
 	    measuretick=measuretick+l;
 
@@ -2114,19 +2153,10 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
 	  printf("ordinary elements: Jump found\n");
 	  break;
 	default:
-	  printf("Export Lilypond: unsupported element <%s>\n", e->name());
+	  //printf("Export Lilypond: unsupported element <%s>\n", e->name());
 	  break;
 	}
       handleElement(e, true); //check for instructions anchored to element e.
-      //check for instructions anchored to start of next element
-//       Element * nextel;
-//       Segment* nextseg;
-//       nextseg=s->next();
-//       if(nextseg)
-// 	{ printf("nextseg \n");
-// 	  nextel=nextseg->element(staffInd * VOICES + voice);
-// 	  handleElement(nextel, staffInd, false);
-// 	}
     } //end for all segments
 
 
@@ -2170,7 +2200,7 @@ void ExportLy::writeVoiceMeasure(Measure* m, Staff* staff, int staffInd, int voi
       }
 
   writeVolta(measurenumber, lastind);
-  out << " | % " << m->no()+1 << "\n" ; //barcheck and barnumber
+
 } //end write VoiceMeasure
 
 
@@ -2187,12 +2217,13 @@ void ExportLy::writeScore()
   chordpitch=41;
   repeatactive=false;
   int staffInd = 0;
-  int np = score->parts()->size();
+  //int np = score->parts()->size();
   graceswitch=false;
   int voice=0;
   cleannote="c";
   prevnote="c";
   gracecount=0;
+  donefirst=false;
 
 
   foreach(Part* part, *score->parts())
@@ -2201,8 +2232,8 @@ void ExportLy::writeScore()
       initAnchors();
       resetAnchor(anker);
       int n = part->staves()->size();
-      staffname[staffInd].partname  = part->longName();
-      staffname[staffInd].partshort = part->shortName();
+      staffname[staffInd].partname  = part->longName()->getText();
+      staffname[staffInd].partshort = part->shortName()->getText();
       curTicks=-1;
       pickup=false;
 
@@ -2274,7 +2305,7 @@ void ExportLy::writeScore()
 	    not have been necessary. But I have not found it, so
 	    please tell me if there is. This means that the voice is
 	    not written to the outputstream (os) which is connected to
-	    the output file immediately, but buffered to an
+	    the output file, immediately, but buffered to an
 	    outputstream (out) which is only connected to a string
 	    ("voicebuffer") in memory. If there is no material in the
 	    voice, this string is set to "". If there is material in
@@ -2285,14 +2316,18 @@ void ExportLy::writeScore()
 	    {
 	      prevpitch=staffpitch;
 	      relativ=staffrelativ;
+	      donefirst=false;
 
 	      //for all measures in this voice:
-	      for (MeasureBase* m = score->layout()->first(); m; m = m->next())
+	      for (MeasureBase* m = score->first(); m; m = m->next())
 		{
 		  if (m->type() != MEASURE)
 		    continue;
 		    writeVoiceMeasure((Measure*)m, staff, staffInd, voice);
 		}
+	      level--;
+	      indent();
+	      out << "\\bar \"|.\" \n"; //thin-thick barline as last.
 	      level=0;
 	      indent();
 	      out << "}% end of last bar in partorvoice\n\n";
@@ -2306,10 +2341,7 @@ void ExportLy::writeScore()
 	  int voiceno=0;
 
 	  for (voice = 0; voice < VOICES; ++voice)
-	    {
 	    if (voiceActive[voice]) voiceno++;
-	    printf("voice: %d, voiceno: %d\n", voice, voiceno);
-	    }
 
 	  if (voiceno == 1) staffname[staffInd].simultaneousvoices=false;
 
@@ -2357,7 +2389,7 @@ void ExportLy::writeScore()
 //-------------------
 void ExportLy::writeScoreBlock()
 {
-  bracktest();
+  //  bracktest();
   level=0;
   os << "\n\\score { \n";
   level++;
@@ -2374,7 +2406,7 @@ void ExportLy::writeScoreBlock()
 	{
 	  ++level;
 	  indentF();
-	  os << "\\context StaffGroup = " << (char)(lybracks[indx].brakno + 64) << "<< \n";  
+	  os << "\\context StaffGroup = " << (char)(lybracks[indx].brakno + 64) << "<< \n";
 	}
 
       if (lybracks[indx].bracestart)
@@ -2389,7 +2421,7 @@ void ExportLy::writeScoreBlock()
 	      pianostaff=true;
 	    }
 	  else
-	  os << "\\context GrandStaff = " << (char)(lybracks[indx].braceno + 64) << "<< \n";  
+	  os << "\\context GrandStaff = " << (char)(lybracks[indx].braceno + 64) << "<< \n";
 	}
 
       ++level;
@@ -2415,7 +2447,7 @@ void ExportLy::writeScoreBlock()
       --level;
       indentF();
       os << ">>\n";
-      
+
       if (((lybracks[indx].brakstart) and (lybracks[indx].brakend)) or ((lybracks[indx].bracestart) and (lybracks[indx].braceend)))
 	{
 	  //if bracket or brace starts and ends on same staff: one-staff brace/bracket.
@@ -2428,16 +2460,16 @@ void ExportLy::writeScoreBlock()
       if (lybracks[indx].brakend)
 	{  --level;
 	  indentF();
-	  os << ">> %end of StaffGroup" << (char)(lybracks[indx].brakno + 64) << "\n";   
+	  os << ">> %end of StaffGroup" << (char)(lybracks[indx].brakno + 64) << "\n";
 	}
       if (lybracks[indx].braceend)
 	{
 	  --level;
 	  indentF();
 	  if (lybracks[indx].piano)
-	  os << ">> %end of PianoStaff" << (char)(lybracks[indx].braceno + 64) << "\n";  
+	  os << ">> %end of PianoStaff" << (char)(lybracks[indx].braceno + 64) << "\n";
 	  else
-	  os << ">> %end of GrandStaff" << (char)(lybracks[indx].braceno + 64) << "\n";  
+	  os << ">> %end of GrandStaff" << (char)(lybracks[indx].braceno + 64) << "\n";
 	}
 
       ++indx;
@@ -2454,13 +2486,24 @@ void ExportLy::writeScoreBlock()
   os << "%% remove previous line to get barnumbers only at beginning of system.\n";
   indentF();
   os << " #(set-accidental-style 'modern-cautionary)\n";
+if (rehearsalnumbers)
+    {
+      indentF();
+      os << "\\set Score.markFormatter = #format-mark-box-numbers %%boxed rehearsal-numbers \n";
+    }
+ else
+   {
+     indentF();
+     os << "\\set Score.markFormatter = #format-mark-box-letters %%boxed rehearsal-marks\n";
+   }
   indentF();
-  os << "\\set Score.markFormatter = #format-mark-box-letters %%boxed rehearsal-marks\n";
-
+  os << "\\override Score.TimeSignature #'style = #'() %%makes timesigs always numerical\n";
+  indentF();
+  os << "%% remove previous line to get cut-time/alla breve or common time \n";
   --level;
   indentF();
   os << ">>\n";
- 
+
   --level;
   --level;
   indentF();
@@ -2468,7 +2511,7 @@ void ExportLy::writeScoreBlock()
 
   if (((pianostaff) and (indx==2)) or (indx < 2))
     os << "#(set-global-staff-size 20)\n";
-  else if (indx > 1)
+  else if (indx > 2)
     os << "#(set-global-staff-size 14)\n";
 }// end scoreblock
 
@@ -2480,10 +2523,12 @@ void ExportLy::writeScoreBlock()
 bool ExportLy::write(const QString& name)
 {
   pianostaff=false;
+  rehearsalnumbers=false;
   f.setFileName(name);
   if (!f.open(QIODevice::WriteOnly))
     return false;
   os.setDevice(&f);
+  os.setCodec("utf8");
   out.setString(&voicebuffer);
   os << "%=============================================\n"
     "%   created by MuseScore Version: " << VERSION << "\n"
@@ -2516,13 +2561,17 @@ bool ExportLy::write(const QString& name)
 
   // TODO/O.G.: better choose between standard formats and specified paper
   // dimensions. We normally don't need both.
-  
+
   double lw = pf->width() - pf->evenLeftMargin - pf->evenRightMargin;
   os << "\\paper {\n";
   os <<  "  line-width    = " << lw * INCH << "\\mm\n";
   os <<  "  left-margin   = " << pf->evenLeftMargin * INCH << "\\mm\n";
   os <<  "  top-margin    = " << pf->evenTopMargin * INCH << "\\mm\n";
   os <<  "  bottom-margin = " << pf->evenBottomMargin * INCH << "\\mm\n";
+  os <<  "  indent = 0 \\mm \n";
+  os <<  "  %%set to ##t if your score is less than one page: \n";
+  os <<  "  ragged-last-bottom = ##f \n";
+  os <<  "  ragged-bottom = ##f  \n";
   os <<  "  }\n\n";
 
 
@@ -2533,7 +2582,7 @@ bool ExportLy::write(const QString& name)
   os << "\\header {\n";
 
   ++level;
-  const MeasureBase* m = score->layout()->first();
+  const MeasureBase* m = score->first();
   foreach(const Element* e, *m->el()) {
     if (e->type() != TEXT)
       continue;
@@ -2563,7 +2612,7 @@ bool ExportLy::write(const QString& name)
   if (score->rights)
     {
       indentF();
-      os << "copyright = \"" << score->rights->toPlainText() << "\"\n";
+      os << "copyright = \"" << score->rights->getText() << "\"\n";
     }
 
   indentF();
@@ -2584,101 +2633,91 @@ bool ExportLy::write(const QString& name)
 
 
 /*----------------------- NEWS and HISTORY:--------------------  */
-/* NEW 5.feb.2009:
-   separated grandstaff (variable distance between staffs) from
+
+/* mar. 2009 always explicit end-bar -> no need to declare last bar as
+   incomplete, but writes to end-bars when last bar is complete. This
+   doesn't show in print.
+
+   12.feb.2009. - Removed bug: double instrument definitions in pieces
+   with pickup-measure (prev'ly defined both in measure 0 and measure
+   1). - Removed bug: nonrecognition of startrepeats. -Improved
+   recognition of whole-measure rests.
+
+   NEW 5.feb.2009: separated grandstaff (variable distance between staffs) from
    pianostaff: constant distance between staffs to prepare for cross-staff
-   beaming (not implemented). Brackets/braces for single staffs.*/
+   beaming (not implemented). Brackets/braces for single staffs.
 
-/* NEW 25.jan.2009: 
-   -- system brackets and braces for simple scores.
+   NEW 25.jan.2009: system brackets and braces for simple scores.
    Unsolved complications for multistaff instruments (piano, organ,
-   harp), and for bracketing single staffs.*/
+   harp), and for bracketing single staffs.
 
-/* NEW 22.jan. 2009 
+   NEW 22.jan. 2009
    -- fixed a problem with beames on grace-notes, and
-   some faults produced by the previous revision of exportly.*/
+   some faults produced by the previous revision of exportly.
 
-/* NEW 18. jan. 2009
-   -- Now avoids export of empty voices. */
+   NEW 18. jan. 2009
+   -- Now avoids export of empty voices.
 
-/* NEW 23.dec.2008
-   -- export of note of lengths longa and brevis, and some rests longer than whole.*/
+   NEW 23.dec.2008
+   -- export of note of lengths longa and brevis, and some rests longer than whole.
 
-/* NEW 9. dec. 2008:
+   NEW 9. dec. 2008:
    -- Some improvements to triplets and finding the right octave for single note after chord.
-   -- started work on codas and segnos.*/
+   -- started work on codas and segnos.
 
-/* NEW 24. nov. 2008:
-   -- added dynamic signs and staff-text, by stealing from exportxml.cpp.*/
+   NEW 24. nov. 2008:
+   -- added dynamic signs and staff-text, by stealing from exportxml.cpp.
 
-/* NEW 1. nov. 2008
+   NEW 1. nov. 2008
    --pickupbar (But not irregular bars in general.)
    --ties
    --management of incomplete bars in voices 2-4.
-*/
 
-/* NEW 26. oct. 2008
+   NEW 26. oct. 2008
   - voice separation and recombination in score-block for easier editing of Lilypondfile.
     todo/unresolved: writes voices 2-4 in Lilypond file even if voice is empty.
   - better finding of correct octave when jumping intervals of fifths or more.
-*/
 
-
-/* NEW 10.oct.2008:
+  NEW 10.oct.2008:
    - rudimentary handling of slurs.
    - voltas and endings
    - dotted 8ths and 4ths.
    - triplets, but not general tuplets.
-   - PianoStaff reactivated.*/
+   - PianoStaff reactivated.
 
-
-/*NEW:
-  1. dotted 16th notes
-  2. Relative pitches
-  3. More grace-note types
-  4. Slurs and beams in grace-notes
-  5. Some adjustments in articulations
-  6. separation of staffs/voices from score-block. Unfinished for pianostaffs/GrandStaffs and voices on one staff.
-  7. completed the clef-secton.
-  Points 2 and 6, and also in a smaller degree 5, enhances human readability of lilypond-code.
+   NEW:
+   1. dotted 16th notes
+   2. Relative pitches
+   3. More grace-note types
+   4. Slurs and beams in grace-notes
+   5. Some adjustments in articulations
+   6. separation of staffs/voices from score-block. Unfinished for pianostaffs/GrandStaffs and voices on one staff.
+   7. completed the clef-secton.
+   Points 2 and 6, and also in a smaller degree 5, enhances human readability of lilypond-code.
 */
 
 
 /*----------------------TODOS------------------------------------*/
 
 /* TODO: PROJECTS
-   3. 
    4. Lyrics (low priority in relation to music itself)
    5. Collisions in crowded multi-voice staffs (e.g. cello-suite).
    6. General tuplets
    7  Use linked list instead of static array for dynamics etcs.
    8. Determine whether text goes above or below staff.
-   10. fingerings, chordname; rehearsal marks as \mark in all voices.
-   11. sort out piano-staff and general multistaff-scores.
+   10. fingerings, chordname; rehearsal marks as \mark\default in all voices.
    12. correct indentation in score-block.
    13. cross-staff beaming in pianostaff
+   14. Real multimeasure rests (for now only one-measure rests are exported).
+   18. lacking fermata above/below rest.
+   -  big problems with hairpins.
+   ExportLy::symbol(): rcomma not supported
    - etc.etc.etc.etc........
 */
 
 /*TODO: BUGS
-
-  - repeated voice/part definition in Brandeburg
-
-  - massive failure on gollywog and Bilder 
-
-  - failure in writing double repeat (left-right) in Berlin21.
-
-   - \stemUp \stemDown : sometimes correct sometimes not??? Maybe I
-  have not understood Lily's rules for the use of these commands?
-  Lily's own stem direction algorithms are good enough. Until a better
-  idea comes along: drop export of stem direction and leave it to
-  LilyPond.
-  
- - in Tarrega study: \stemNeutral failure, second voice, first part, from last bar before repeat.
-
- - metronome marks must be given as \tempo 4 = 60 and not as markups.
-
- - lacking fermata above rest.
-
- - etc. etc. etc. ad nauseam.
+  - massive failure on gollywog and Bilder
+  - in Tarrega study: \stemNeutral failure, second voice, first part, from last bar before repeat.
+  - metronome marks must be given as \tempo 4 = 60 and not as markups.
+  - etc. etc. etc. ad nauseam.
  */
