@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: harmony.h 1933 2009-07-15 16:44:28Z wschweer $
 //
-//  Copyright (C) 2008 Werner Schweer and others
+//  Copyright (C) 2008-2009 Werner Schweer and others
 //
 //  Some Code inspired by "The JAZZ++ Midi Sequencer"
 //  Copyright (C) 1994-2000 Andreas Voss and Per Sigmond, all rights reserved.
@@ -53,7 +53,7 @@ class HDegree {
 //---------------------------------------------------------
 
 class HChord {
-//      static const HChord C0;
+      QString str;
 
    protected:
       int keys;
@@ -63,7 +63,8 @@ class HChord {
       HChord(int k) { keys = k; }
       HChord(int a, int b, int c=-1, int d=-1, int e=-1, int f=-1, int g=-1,
             int h=-1, int i=-1, int k=-1, int l=-1);
-      HChord(const char*);
+      HChord(const QString&);
+
       void rotate(int semiTones);
 
       bool contains(int key) const {       // key in chord?
@@ -87,16 +88,99 @@ class HChord {
       void add(const QList<HDegree>& degreeList);
       };
 
+
+//---------------------------------------------------------
+//   RenderAction
+//---------------------------------------------------------
+
+struct RenderAction {
+      enum RenderActionType {
+            RENDER_SET, RENDER_MOVE, RENDER_PUSH, RENDER_POP,
+            RENDER_NOTE, RENDER_ACCIDENTAL
+            };
+
+      RenderActionType type;
+      double movex, movey;          // RENDER_MOVE
+      QString text;                 // RENDER_SET
+
+      RenderAction() {}
+      RenderAction(RenderActionType t) : type(t) {}
+      };
+
 //---------------------------------------------------------
 //   ChordDescription
 //---------------------------------------------------------
 
 struct ChordDescription {
       int id;                 // Chord id number (Band In A Box Chord Number)
-      const char* name;       // chord name as entered from the keyboard (without root/base)
-      const char* xmlKind;    // MusicXml description: kind
-      const char* xmlDegrees; // MusicXml description: list of degrees (if any)
+      QString name;           // chord name as entered from the keyboard (without root/base)
+      QString xmlKind;        // MusicXml description: kind
+      QStringList xmlDegrees; // MusicXml description: list of degrees (if any)
       HChord chord;           // C based chord
+      QList<RenderAction> renderList;
+
+   public:
+      void read(QDomElement);
+      };
+
+//---------------------------------------------------------
+//   ChordSymbol
+//---------------------------------------------------------
+
+struct ChordSymbol {
+      int fontIdx;
+      QString name;
+      QChar code;
+
+      ChordSymbol() { fontIdx = -1; }
+      bool isValid() const { return fontIdx != -1; }
+      };
+
+//---------------------------------------------------------
+//   ChordFont
+//---------------------------------------------------------
+
+struct ChordFont {
+      QString family;
+      double mag;
+      };
+
+//---------------------------------------------------------
+//   ChordList
+//---------------------------------------------------------
+
+class ChordList : public QMap<int, const ChordDescription*> {
+      QHash<QString, ChordSymbol> symbols;
+
+   public:
+      QList<ChordFont> fonts;
+      QList<RenderAction> renderListRoot;
+      QList<RenderAction> renderListBase;
+
+      ChordList() {}
+      virtual ~ChordList();
+      bool read(const QString& path);
+      void read(QDomElement);
+      ChordSymbol symbol(const QString& s) const { return symbols.value(s); }
+      };
+
+//---------------------------------------------------------
+//   TextSegment
+//---------------------------------------------------------
+
+struct TextSegment {
+      QFont font;
+      QString text;
+      double x, y;
+
+      TextSegment()                { x = y = 0.0; }
+      double width() const;
+      QRectF boundingRect() const;
+
+      TextSegment(const QFont& f, double _x, double _y) : font(f), x(_x), y(_y) {}
+      TextSegment(const QString&, const QFont&, double x, double y);
+      void set(const QString&, const QFont&, double x, double y);
+      void setText(const QString& t)      { text = t; }
       };
 
 //---------------------------------------------------------
@@ -118,14 +202,18 @@ struct ChordDescription {
 class Harmony : public Text {
       Q_DECLARE_TR_FUNCTIONS(Harmony)
 
-      static const ChordDescription chordList[];
-      static QHash<int, const ChordDescription*> chordHash;
+      int _rootTpc;                       // root note for chord
+      int _baseTpc;                       // bass note or chord base; used for "slash" chords
+                                          // or notation of base note in chord
+      int _id;
+      QString _userName;
 
-      int _baseTpc;                   // bass note or chord base; used for "slash" chords
-                                      // or notation of base note in chord
-      int _rootTpc;                   // root note for chord
-      const ChordDescription* _descr; // chord description
       QList<HDegree> _degreeList;
+      QList<QFont> fontList;              // temp values used in render()
+      QList<TextSegment*> textList;       // rendered chord
+
+      virtual void draw(QPainter&) const;
+      void render(const QList<RenderAction>& renderList, double&, double&, int tpc);
 
    public:
       Harmony(Score*);
@@ -135,45 +223,48 @@ class Harmony : public Text {
       virtual ElementType type() const         { return HARMONY; }
       Measure* measure()                       { return (Measure*)parent(); }
 
-      void setDescr(const ChordDescription* d) { _descr = d; }
-      const ChordDescription* descr() const { return _descr; }
+      void setId(int d)                        { _id = d; }
+      int id() const                           { return _id;           }
+
+      const ChordDescription* descr() const;
 
       virtual bool genPropertyMenu(QMenu*) const;
       virtual void propertyAction(const QString&);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
 
+      virtual bool startEdit(Viewer*, const QPointF&);
       virtual void endEdit();
 
-      int baseTpc() const                  { return _baseTpc;      }
-      void setBaseTpc(int val)             { _baseTpc = val;       }
-      int rootTpc() const                  { return _rootTpc;      }
-      void setRootTpc(int val)             { _rootTpc = val;       }
-      int chordId() const                  { return _descr ? _descr->id : 0; }
-      void addDegree(const HDegree& d)     { _degreeList << d;     }
-      int numberOfDegrees() const          { return _degreeList.size();   }
-      HDegree degree(int i) const          { return _degreeList.value(i); }
-      void clearDegrees()                  { _degreeList.clear(); }
+      int baseTpc() const                      { return _baseTpc;      }
+      void setBaseTpc(int val)                 { _baseTpc = val;       }
+      int rootTpc() const                      { return _rootTpc;      }
+      void setRootTpc(int val)                 { _rootTpc = val;       }
+      void addDegree(const HDegree& d)         { _degreeList << d;            }
+      int numberOfDegrees() const              { return _degreeList.size();   }
+      HDegree degree(int i) const              { return _degreeList.value(i); }
+      void clearDegrees()                      { _degreeList.clear();         }
+      const QList<HDegree>& degreeList() const { return _degreeList;          }
 
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement);
       QString harmonyName() const;
-      QString extensionName() const        { return _descr ? _descr->name : QString(); }
-      void buildText();
+      void render(const TextStyle* ts = 0);
 
-      const ChordDescription* parseHarmony(const QString& s, int* root, int* base);
-      const char* xmlKind() const         { return _descr ? _descr->xmlKind : 0; }
-      const char* xmlDegrees() const      { return _descr ? _descr->xmlDegrees : 0; }
+      void parseHarmony(const QString& s, int* root, int* base);
+
+      QString extensionName() const    { return _id != -1 ? descr()->name       : _userName;     }
+      QString xmlKind() const          { return _id != -1 ? descr()->xmlKind    : QString();     }
+      QStringList xmlDegrees() const   { return _id != -1 ? descr()->xmlDegrees : QStringList(); }
 
       void resolveDegreeList();
-      void setChordId(int id);
 
-      static const ChordDescription* fromXml(const QString& s,  const QList<HDegree>&);
-      static const ChordDescription* fromXml(const QString& s);
+      virtual void textStyleChanged(const QVector<TextStyle*>&s);
+      virtual bool isEmpty() const;
+      virtual qreal baseLine() const;
 
-      static void initHarmony();
-      static const ChordDescription* chords()  { return chordList; }
-      static unsigned int chordListSize();
-      static const ChordDescription* chordDescription(int id) { return chordHash[id]; }
+      const ChordDescription* fromXml(const QString& s,  const QList<HDegree>&);
+      const ChordDescription* fromXml(const QString& s);
+      virtual void spatiumChanged(double oldValue, double newValue);
       };
 
 #endif
