@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: icons.cpp,v 1.16 2006/03/02 17:08:34 wschweer Exp $
+//  $Id: icons.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -66,6 +66,7 @@ QIcon windowIcon;
 
 QIcon acciaccaturaIcon, appoggiaturaIcon;
 QIcon grace4Icon, grace16Icon, grace32Icon;
+QIcon keysIcon;
 
 //---------------------------------------------------------
 //   symIcon
@@ -74,14 +75,9 @@ QIcon grace4Icon, grace16Icon, grace32Icon;
 
 QIcon symIcon(const Sym& sc, int size, int width, int height)
       {
-      double mag = (size/20.0) * 0.6 * (_spatium / (SPATIUM20 * DPI)) * PDPI / DPI;
+      double mag  = .6 * (size/20.0) * PDPI / DPI;
+      QRectF bb   = sc.bbox(mag);
 
-      double mag1 = double(preferences.noteEntryIconHeight) / double(ICON_HEIGHT);
-      mag = mag * mag1 * 10;
-      width  *= 10;
-      height *= 10;
-
-      QRectF bb = sc.bbox(mag);
       qreal w   = bb.width();
       qreal h   = bb.height();
       qreal x   = (width  - w) * .5 - bb.x();
@@ -186,5 +182,6 @@ void genIcons()
       grace4Icon     = QIcon(":/data/grace4.svg");
       grace16Icon    = QIcon(":/data/grace16.svg");
       grace32Icon    = QIcon(":/data/grace32.svg");
+      keysIcon       = QIcon(":/data/keyboard.svg");
       }
 
