@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: image.h 1995 2009-08-05 13:52:17Z lasconic $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -32,9 +32,10 @@ class ImagePath;
 //---------------------------------------------------------
 
 class Image : public BSymbol {
+   Q_DECLARE_TR_FUNCTIONS(Image)
    protected:
       ImagePath* _ip;
-      mutable QImage buffer;        ///< cached rendering
+      mutable QPixmap buffer;        ///< cached rendering
       QSizeF sz;
       bool _lockAspectRatio;
       mutable bool _dirty;
