@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: midi.cpp,v 1.38 2006/03/22 12:04:14 wschweer Exp $
+//  $Id: importmidi.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -38,7 +38,6 @@
 #include "measure.h"
 #include "style.h"
 #include "part.h"
-#include "layout.h"
 #include "timesig.h"
 #include "barline.h"
 #include "pedal.h"
@@ -50,6 +49,7 @@
 #include "preferences.h"
 #include "box.h"
 #include "importmidi.h"
+#include "keysig.h"
 
 static unsigned const char gmOnMsg[] = { 0x7e, 0x7f, 0x09, 0x01 };
 static unsigned const char gsOnMsg[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41 };
@@ -636,220 +636,6 @@ static QString instrName(int type, int hbank, int lbank, int program)
       return QString();
       }
 
-//---------------------------------------------------------
-//   exportMidi
-//---------------------------------------------------------
-
-class ExportMidi {
-      QFile f;
-      Score* cs;
-
-      void writeHeader();
-
-   public:
-      MidiFile mf;
-
-      ExportMidi(Score* s) { cs = s; }
-      bool write(const QString& name);
-      };
-
-//---------------------------------------------------------
-//   exportMidi
-//    return false on error
-//---------------------------------------------------------
-
-bool Score::saveMidi(const QString& name)
-      {
-      ExportMidi em(this);
-      return em.write(name);
-      }
-
-//---------------------------------------------------------
-//   writeHeader
-//---------------------------------------------------------
-
-void ExportMidi::writeHeader()
-      {
-      MidiTrack* track  = mf.tracks()->front();
-      MeasureBase* measure  = cs->layout()->first();
-
-      foreach (const Element* e, *measure->el()) {
-            if (e->type() == TEXT) {
-                  const Text* text = (const Text*)(e);
-                  QString str = text->getText();
-                  int len     = str.length() + 1;
-                  unsigned char* data = new unsigned char[len];
-                  strcpy((char*)(data), str.toLatin1().data());
-                  MetaEvent* ev = new MetaEvent();
-                  ev->setOntime(0);
-                  ev->setData(data);
-                  ev->setLen(len);
-
-                  switch (text->subtype()) {
-                        case TEXT_TITLE:
-                              ev->setMetaType(META_TITLE);
-                              track->insert(ev);
-                              break;
-                        case TEXT_SUBTITLE:
-                              ev->setMetaType(META_SUBTITLE);
-                              track->insert(ev);
-                              break;
-                        case TEXT_COMPOSER:
-                              ev->setMetaType(META_COMPOSER);
-                              track->insert(ev);
-                              break;
-                        case TEXT_TRANSLATOR:
-                              ev->setMetaType(META_TRANSLATOR);
-                              track->insert(ev);
-                              break;
-                        case TEXT_POET:
-                              ev->setMetaType(META_POET);
-                              track->insert(ev);
-                              break;
-                        default:
-                              delete ev;
-                              break;
-                        }
-                  }
-            }
-
-      //--------------------------------------------
-      //    write time signature
-      //--------------------------------------------
-
-      SigList* sigmap = cs->sigmap;
-      for (iSigEvent is = sigmap->begin(); is != sigmap->end(); ++is) {
-            SigEvent se   = is->second;
-            unsigned char* data = new unsigned char[4];
-            data[0] = se.nominator;
-            int n;
-            switch (se.denominator) {
-                  case 1:  n = 0; break;
-                  case 2:  n = 1; break;
-                  case 4:  n = 2; break;
-                  case 8:  n = 3; break;
-                  case 16: n = 4; break;
-                  case 32: n = 5; break;
-                  default:
-                        n = 2;
-                        printf("ExportMidi: unknown time signature %d/%d\n", se.nominator, n);
-                        break;
-                  }
-            data[1] = n;
-            data[2] = 24;
-            data[3] = 8;
-
-            MetaEvent* ev = new MetaEvent;
-            ev->setMetaType(META_TIME_SIGNATURE);
-            ev->setData(data);
-            ev->setLen(4);
-            ev->setOntime(is->first);
-            track->insert(ev);
-            }
-
-      //---------------------------------------------------
-      //    write key signatures
-      //    assume every staff corresponds to a midi track
-      //---------------------------------------------------
-
-      QList<MidiTrack*>* tl = mf.tracks();
-      for (int i = 0; i < tl->size(); ++i) {
-            MidiTrack* track  = tl->at(i);
-
-            KeyList* keymap = cs->staff(i)->keymap();
-            for (iKeyEvent ik = keymap->begin(); ik != keymap->end(); ++ik) {
-                  MetaEvent* ev  = new MetaEvent;
-                  ev->setOntime(ik->first);
-                  int key       = ik->second;   // -7 -- +7
-                  ev->setMetaType(META_KEY_SIGNATURE);
-                  ev->setLen(2);
-                  unsigned char* data = new unsigned char[2];
-                  data[0]   = key;
-                  data[1]   = 0;  // major
-                  ev->setData(data);
-                  track->insert(ev);
-                  }
-            }
-
-      //--------------------------------------------
-      //    write tempo changes
-      //--------------------------------------------
-
-      TempoList* tempomap = cs->tempomap;
-      for (iTEvent it = tempomap->begin(); it != tempomap->end(); ++it) {
-            MetaEvent* ev = new MetaEvent;
-            ev->setOntime(it->first);
-            //
-            // compute midi tempo: microseconds / quarter note
-            //
-            int tempo = lrint((1.0 / it->second.tempo) * 1000000.0);
-
-            ev->setMetaType(META_TEMPO);
-            ev->setLen(3);
-            unsigned char* data = new unsigned char[3];
-            data[0]   = tempo >> 16;
-            data[1]   = tempo >> 8;
-            data[2]   = tempo;
-            ev->setData(data);
-            track->insert(ev);
-            }
-      }
-
-//---------------------------------------------------------
-//  write
-//    export midi file
-//    return false on error
-//---------------------------------------------------------
-
-bool ExportMidi::write(const QString& name)
-      {
-      f.setFileName(name);
-      if (!f.open(QIODevice::WriteOnly))
-            return false;
-
-      mf.setDivision(::division);
-      mf.setFormat(1);
-      QList<MidiTrack*>* tracks = mf.tracks();
-      int nstaves = cs->nstaves();
-
-      for (int i = 0; i < nstaves; ++i)
-            tracks->append(new MidiTrack(&mf));
-
-      writeHeader();
-
-      SigList* sigmap = cs->sigmap;
-      int tickOffset  = sigmap->ticksMeasure(0);
-
-      foreach (Staff* staff, cs->staves()) {
-            Part* part       = staff->part();
-            int channel      = part->midiChannel();
-            int staffIdx     = staff->idx();
-            MidiTrack* track = tracks->at(staffIdx);
-            track->setOutPort(0);
-            track->setOutChannel(channel);
-
-            if (staff->isTop()) {
-                  if (part->midiProgram() != -1)
-                        track->insert(new ControllerEvent(0, channel, CTRL_PROGRAM, part->midiProgram()));
-                  track->insert(new ControllerEvent(2, channel, CTRL_VOLUME,      part->volume()));
-                  track->insert(new ControllerEvent(4, channel, CTRL_PANPOT,      part->pan()));
-                  track->insert(new ControllerEvent(6, channel, CTRL_REVERB_SEND, part->reverb()));
-                  track->insert(new ControllerEvent(8, channel, CTRL_CHORUS_SEND, part->chorus()));
-                  }
-            EventMap events;
-            cs->toEList(&events, preferences.midiExpandRepeats, tickOffset, staffIdx);
-            for (EventMap::const_iterator i = events.constBegin(); i != events.constEnd(); ++i) {
-                  if (i.value()->type() == ME_NOTEON) {
-                        NoteOn* n = (NoteOn*)i.value();
-                        track->insert(new NoteOn(i.key(), n->channel(), n->pitch(), n->velo()));
-                        }
-                  else
-                        printf("writeMidi: unknown midi event 0x%02x\n", i.value()->type());
-                  }
-            }
-      return !mf.write(&f);
-      }
-
 //---------------------------------------------------------
 //   addLyrics
 //---------------------------------------------------------
@@ -890,27 +676,27 @@ void Score::addLyrics(int tick, int staffIdx, const QString& txt)
 //   processMeta
 //---------------------------------------------------------
 
-void MidiFile::processMeta(Score* cs, MidiTrack* track, int staffIdx, MetaEvent* mm)
+void MidiFile::processMeta(Score* cs, MidiTrack* track, Event* mm)
       {
       int tick = mm->ontime();
-      Staff* staff = cs->staff(staffIdx);
+      Staff* staff = track->staff();
       unsigned char* data = (unsigned char*)mm->data();
+      int staffIdx = staff ? cs->staffIdx(staff) : -1;
 
       switch (mm->metaType()) {
             case META_TEXT:
             case META_LYRIC:
-                  {
-                  QString s((char*)data);
-                  cs->addLyrics(tick, staffIdx, s);
-                  }
+                  if (staff) {
+                        QString s((char*)data);
+                        cs->addLyrics(tick, staffIdx, s);
+                        }
                   break;
 
             case META_TRACK_NAME:
-                  {
-                  QString txt((char*)data);
-                  staff->part()->setTrackName(txt);
-                  staff->part()->setLongName(txt);
-                  }
+                  if (staff) {
+                        QString txt((char*)data);
+                        track->setName(txt);
+                        }
                   break;
 
             case META_TEMPO:
@@ -922,15 +708,17 @@ void MidiFile::processMeta(Score* cs, MidiTrack* track, int staffIdx, MetaEvent*
                   break;
 
             case META_KEY_SIGNATURE:
-                  {
-                  int key = ((char*)data)[0];
-                  if (key < -7 || key > 7) {
-                        printf("ImportMidi: illegal key %d\n", key);
-                        break;
+                  if (staff) {
+                        int key = ((char*)data)[0];
+                        if (key < -7 || key > 7) {
+                              printf("ImportMidi: illegal key %d\n", key);
+                              break;
+                              }
+                        (*staff->keymap())[mm->ontime()] = key;
+                        track->setHasKey(true);
                         }
-                  (*staff->keymap())[mm->ontime()] = key;
-                  track->setHasKey(true);
-                  }
+                  else
+                        printf("meta key: no staff\n");
                   break;
             case META_COMPOSER:     // mscore extension
             case META_POET:
@@ -941,36 +729,42 @@ void MidiFile::processMeta(Score* cs, MidiTrack* track, int staffIdx, MetaEvent*
                   Text* text = new Text(cs);
                   switch(mm->metaType()) {
                         case META_COMPOSER:
-                              text->setSubtype(TEXT_COMPOSER);
+                        case TEXT_TITLE:
+                              text->setTextStyle(TEXT_STYLE_COMPOSER);
                               break;
                         case META_TRANSLATOR:
-                              text->setSubtype(TEXT_TRANSLATOR);
+                              text->setTextStyle(TEXT_STYLE_TRANSLATOR);
                               break;
                         case META_POET:
-                              text->setSubtype(TEXT_POET);
+                              text->setTextStyle(TEXT_STYLE_POET);
                               break;
                         case META_SUBTITLE:
-                              text->setSubtype(TEXT_SUBTITLE);
+                              text->setTextStyle(TEXT_STYLE_SUBTITLE);
                               break;
                         case META_TITLE:
-                              text->setSubtype(TEXT_TITLE);
+                              text->setTextStyle(TEXT_STYLE_TITLE);
                               break;
                         }
 
                   text->setText((char*)(mm->data()));
 
-                  ScoreLayout* layout = cs->layout();
-                  MeasureBase* measure = layout->first();
+                  MeasureBase* measure = cs->first();
                   if (measure->type() != VBOX) {
                         measure = new VBox(cs);
                         measure->setTick(0);
-                        measure->setNext(layout->first());
-                        layout->add(measure);
+                        measure->setNext(cs->first());
+                        cs->add(measure);
                         }
                   measure->add(text);
                   }
                   break;
 
+            case META_COPYRIGHT:
+                  {
+                  cs->setCopyright(QString((char*)(mm->data())));
+                  }
+                  break;
+
             case META_TIME_SIGNATURE:
                   {
                   int z = data[0];
@@ -1012,27 +806,25 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
             track->maxPitch = 0;
             track->minPitch = 127;
             track->medPitch = 0;
-            track->program  = 0;
+            track->setProgram(0);
 		int events      = 0;
             const EventList el = track->events();
             foreach (Event* e, track->events()) {
                   if (e->type() == ME_NOTE) {
                         ++events;
-                        int pitch = ((NoteEvent*)e)->pitch();
+                        int pitch = e->pitch();
                         if (pitch > track->maxPitch)
                               track->maxPitch = pitch;
                         if (pitch < track->minPitch)
                               track->minPitch = pitch;
                         track->medPitch += pitch;
                         }
-                  else if (e->type() == ME_CONTROLLER && ((ControllerEvent*)e)->controller() == CTRL_PROGRAM) {
-                        track->program = ((ControllerEvent*)e)->value();
+                  else if (e->type() == ME_CONTROLLER && e->controller() == CTRL_PROGRAM) {
+                        track->setProgram(e->value());
                         }
                   }
-            if (events == 0) {
-                  printf("empty track found\n");
+            if (events == 0)
                   track->setStaffIdx(-1);       // dont create staff for this track
-                  }
             else {
                   track->setStaffIdx(staffIdx++);
 	            track->medPitch /= events;
@@ -1049,21 +841,26 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
       for (int i = 0; i < ntracks; ++i) {
             MidiTrack* track = tracks->at(i);
             int staffIdx = track->staffIdx();
-            if (staffIdx == -1)
+            if (staffIdx == -1) {
+                  track->setStaff(0);
                   continue;
+                  }
             int program  = track->getInitProgram();
+            track->setProgram(program);
             Part* part   = new Part(this);
 
             Staff* s = new Staff(this, part, 0);
             part->insertStaff(s);
             _staves.push_back(s);
+            track->setStaff(s);
+
             if (track->isDrumTrack()) {
                   s->clefList()->setClef(0, CLEF_PERC);
                   part->setDrumset(smDrumset);
                   }
             else {
                   if ((i < (ntracks-1)) && (tracks->at(i+1)->outChannel() == track->outChannel()
-                     && ((program & 0xff) == 0))) {
+                     && ((program & 0xff) == 0) && tracks->at(i+1)->staffIdx() != -1)) {
                         // assume that the current track and the next track
                         // form a piano part
                         Staff* ss = new Staff(this, part, 1);
@@ -1075,31 +872,12 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
                         s->setBracketSpan(0, 2);
                         ss->clefList()->setClef(0, CLEF_F);
                         ++i;
+                        tracks->at(i)->setStaff(ss);
                         }
                   else {
                         s->clefList()->setClef(0, track->medPitch < 58 ? CLEF_F : CLEF_G);
                         }
                   }
-            if (track->name().isEmpty()) {
-                  int hbank = -1, lbank = -1;
-                  if (program == -1)
-                        program = 0;
-                  else {
-                        hbank = (program >> 16);
-                        lbank = (program >> 8) & 0xff;
-                        program = program & 0xff;
-                        }
-                  QString t(instrName(mf->midiType(), hbank, lbank, program));
-                  if (!t.isEmpty())
-                        part->setLongName(t);
-                  }
-            else
-
-                  part->setLongName(track->name());
-
-            part->setTrackName(part->longName());
-            part->setMidiChannel(track->outChannel());
-            part->setMidiProgram(track->program & 0x7f);  // only GM
             _parts.push_back(part);
             }
 
@@ -1164,7 +942,7 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
             for (ciEvent ie = el.begin(); ie != el.end(); ++ie) {
                   if ((*ie)->type() != ME_NOTE)
                         continue;
-                  int tick = (*ie)->ontime() + ((NoteEvent*)(*ie))->duration();
+                  int tick = (*ie)->ontime() + (*ie)->duration();
                   if (tick > lastTick)
                         lastTick = tick;
                   }
@@ -1183,7 +961,7 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
             int tick = sigmap->bar2tick(i, 0, 0);
             measure->setTick(tick);
 
-      	_layout->add(measure);
+      	add(measure);
             }
 
 	foreach (MidiTrack* midiTrack, *tracks) {
@@ -1192,25 +970,45 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
             midiTrack->cleanup();
             }
 
-	staffIdx = 0;
-	foreach (MidiTrack* midiTrack, *tracks) {
-            if (midiTrack->staffIdx() == -1)
-                  continue;
-            convertTrack(midiTrack);
-            }
-
       //---------------------------------------------------
       //  process meta events
       //---------------------------------------------------
 
-      staffIdx = 0;
       foreach (MidiTrack* track, *tracks) {
-            if (track->staffIdx() != -1)
-                  staffIdx = track->staffIdx();
             foreach (Event* e, track->events()) {
                   if (e->type() == ME_META)
-                        mf->processMeta(this, track, staffIdx, (MetaEvent*)e);
+                        mf->processMeta(this, track, e);
                   }
+            if (debugMode) {
+                  printf("Track %2d:%2d key %d <%s><%s>\n", track->outChannel(),
+                     track->outPort(), track->hasKey(), qPrintable(track->name()),
+                     qPrintable(track->comment()));
+                  }
+
+            Part* part = track->staff() ? track->staff()->part() : 0;
+            if (part) {
+                  int program = track->program();
+                  if (track->name().isEmpty()) {
+                        int hbank = -1, lbank = -1;
+                        if (program == -1)
+                              program = 0;
+                        else {
+                              hbank = (program >> 16);
+                              lbank = (program >> 8) & 0xff;
+                              program = program & 0xff;
+                              }
+                        QString t(instrName(mf->midiType(), hbank, lbank, program));
+                        if (!t.isEmpty())
+                              part->setLongName(t);
+                        }
+                  else
+                        part->setLongName(track->name());
+                  part->setTrackName(part->longName()->getText());
+                  part->setMidiChannel(track->outChannel());
+                  part->setMidiProgram(track->program() & 0x7f);  // only GM
+                  }
+            if (track->staffIdx() != -1)
+                  convertTrack(track);
             }
 
       for (iSigEvent is = sigmap->begin(); is != sigmap->end(); ++is) {
@@ -1236,10 +1034,10 @@ void Score::convertMidi(MidiFile* mf, int /*shortestNote*/)
 //---------------------------------------------------------
 
 struct MNote {
-	ChordEvent* mc;
+	Event* mc;
       QList<Tie*> ties;
 
-      MNote(ChordEvent* _mc) : mc(_mc) {
+      MNote(Event* _mc) : mc(_mc) {
             for (int i = 0; i < mc->notes().size(); ++i)
                   ties.append(0);
             }
@@ -1254,13 +1052,13 @@ void Score::convertTrack(MidiTrack* midiTrack)
       int key      = findKey(midiTrack, sigmap);
       int staffIdx = midiTrack->staffIdx();
       midiTrack->findChords();
-      int voices = midiTrack->separateVoices(2);
 
-      Staff* cstaff = staff(staffIdx);
+      int voices         = midiTrack->separateVoices(2);
+      Staff* cstaff      = midiTrack->staff();
 	const EventList el = midiTrack->events();
-
-      Drumset* drumset = cstaff->part()->drumset();
-      bool useDrumset  = cstaff->part()->useDrumset();
+      Drumset* drumset   = cstaff->part()->drumset();
+      bool useDrumset    = cstaff->part()->useDrumset();
+      KeyList* km        = cstaff->keymap();
 
       for (int voice = 0; voice < voices; ++voice) {
             QList<MNote*> notes;
@@ -1271,7 +1069,7 @@ void Score::convertTrack(MidiTrack* midiTrack)
                         ++i;
                         continue;
                         }
-                  if (((ChordEvent*)e)->voice() != voice) {
+                  if (e->voice() != voice) {
                         ++i;
                         continue;
                         }
@@ -1300,9 +1098,9 @@ void Score::convertTrack(MidiTrack* midiTrack)
                         s->add(chord);
 
                   	foreach (MNote* n, notes) {
-                              QList<NoteEvent*>& nl = n->mc->notes();
+                              QList<Event*>& nl = n->mc->notes();
                               for (int i = 0; i < nl.size(); ++i) {
-                                    NoteEvent* mn = nl[i];
+                                    Event* mn = nl[i];
                         		Note* note = new Note(this);
                                     note->setPitch(mn->pitch());
                                     note->setTpc(mn->tpc());
@@ -1330,7 +1128,7 @@ printf("unmapped drum note 0x%02x %d\n", mn->pitch(), mn->pitch());
                                     continue;
                                     }
                               for (int i = 0; i < nl.size(); ++i) {
-                                    NoteEvent* mn = nl[i];
+                                    Event* mn = nl[i];
                                     Note* note = chord->noteList()->find(mn->pitch());
             				n->ties[i] = new Tie(this);
                                     n->ties[i]->setStartNote(note);
@@ -1374,10 +1172,9 @@ printf("unmapped drum note 0x%02x %d\n", mn->pitch(), mn->pitch());
                               continue;
                         if ((*i)->ontime() != ctick)
                               break;
-                        ChordEvent* mc = (ChordEvent*)e;
-                        if (mc->voice() != voice)
+                        if (e->voice() != voice)
                               continue;
-                  	MNote* n = new MNote(mc);
+                  	MNote* n = new MNote(e);
             	      notes.append(n);
                         }
                   }
@@ -1400,7 +1197,7 @@ printf("unmapped drum note 0x%02x %d\n", mn->pitch(), mn->pitch());
                         }
                   chord->setLen(len);
             	foreach (MNote* n, notes) {
-                        foreach(NoteEvent* mn, n->mc->notes()) {
+                        foreach(Event* mn, n->mc->notes()) {
                   		Note* note = new Note(this);
                               note->setPitch(mn->pitch());
             	      	note->setTrack(staffIdx * VOICES + voice);
@@ -1432,7 +1229,22 @@ printf("unmapped drum note 0x%02x %d\n", mn->pitch(), mn->pitch());
                   }
             }
       if (!midiTrack->hasKey() && !midiTrack->isDrumTrack()) {
-            (*cstaff->keymap())[0] = key;
+            (*km)[0] = key;
+            }
+      for (ciKeyEvent i = km->begin(); i != km->end(); ++i) {
+            int tick = i->first;
+            int key  = i->second;
+            if (tick == 0)
+                  continue;
+            KeySig* ks = new KeySig(this);
+            ks->setTrack(staffIdx * VOICES);
+            ks->setTick(tick);
+            ks->setGenerated(false);
+            ks->setSubtype(key);
+            ks->setMag(cstaff->mag());
+            Measure* m = tick2measure(tick);
+            Segment* seg = m->getSegment(ks);
+            seg->add(ks);
             }
       }
 
@@ -1503,11 +1315,16 @@ bool Score::importMidi(const QString& name)
             return false;
 
       MidiFile mf;
-      if (!mf.read(&fp)) {
+      try {
+            mf.read(&fp);
+            }
+      catch(QString errorText) {
             QMessageBox::warning(0,
                QWidget::tr("MuseScore: load midi"),
-               QString("Load failed: ") + mf.error(),
+               QString("Load failed: ") + errorText,
                QString::null, QWidget::tr("Quit"), QString::null, 0, 1);
+            fp.close();
+            return false;
             }
       fp.close();
 
