@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: input.h,v 1.4 2006/03/02 17:08:34 wschweer Exp $
+//  $Id: input.h 1936 2009-07-17 16:12:47Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -21,31 +21,41 @@
 #ifndef __INPUT_H__
 #define __INPUT_H__
 
+#include "globals.h"
+#include "durationtype.h"
+
 class Slur;
 class ChordRest;
+struct Drumset;
 
 //---------------------------------------------------------
 //   InputState
 //---------------------------------------------------------
 
 class InputState {
-      int _pos;
-
    public:
+      int dots;
+//      int len;
+      Duration duration;      // currently selected duration
+      bool rest;
+      int pad;
+      int pitch;
+      int prefix;
+      NoteType noteType;
+      BeamMode beamMode;
+      int drumNote;
+      Drumset* drumset;
       int track;
       bool noteEntryMode;
       Slur* slur;
       ChordRest* cr;
 
-      InputState() {
-            track         = 0;
-            _pos          = 0;
-            noteEntryMode = false;
-            slur          = 0;
-            cr            = 0;
-            }
-      int pos() const      { return _pos; }
-      void setPos(int val) { _pos = val; }
+      InputState();
+      int pos() const;
+      void setPos(ChordRest* cr);
+
+      int voice() const { return track % VOICES; }
+      int tickLen() { return duration.ticks(dots); }
       };
 
 #endif
