@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: instrdialog.cpp,v 1.32 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: instrdialog.cpp 2002 2009-08-06 21:21:29Z lasconic $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -30,12 +30,13 @@
 #include "staff.h"
 #include "part.h"
 #include "segment.h"
-#include "layout.h"
 #include "style.h"
 #include "editinstrument.h"
 #include "drumset.h"
 #include "slur.h"
 #include "seq.h"
+#include "measure.h"
+#include "line.h"
 
 //---------------------------------------------------------
 //   StaffListItem
@@ -78,7 +79,7 @@ void StaffListItem::setPartIdx(int val)
 void StaffListItem::setClef(int val)
       {
       _clef = val;
-      setText(1, InstrumentsDialog::tr(clefTable[_clef].name));
+      setText(1, qApp->translate("clefTable", clefTable[_clef].name));
       }
 
 //---------------------------------------------------------
@@ -335,10 +336,10 @@ void InstrumentsDialog::on_upButton_clicked()
       QTreeWidgetItem* item = wi.front();
 
       if (item->type() == PART_LIST_ITEM) {
-            partiturList->selectionModel()->clear();
             bool isExpanded = partiturList->isItemExpanded(item);
             int idx = partiturList->indexOfTopLevelItem(item);
             if (idx) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = partiturList->takeTopLevelItem(idx);
                   partiturList->insertTopLevelItem(idx-1, item);
                   partiturList->setItemExpanded(item, isExpanded);
@@ -346,10 +347,10 @@ void InstrumentsDialog::on_upButton_clicked()
                   }
             }
       else {
-            partiturList->selectionModel()->clear();
             QTreeWidgetItem* parent = item->parent();
             int idx = parent->indexOfChild(item);
             if (idx) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = parent->takeChild(idx);
                   parent->insertChild(idx-1, item);
                   partiturList->setItemSelected(item, true);
@@ -373,6 +374,7 @@ void InstrumentsDialog::on_downButton_clicked()
             int idx = partiturList->indexOfTopLevelItem(item);
             int n = partiturList->topLevelItemCount();
             if (idx < (n-1)) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = partiturList->takeTopLevelItem(idx);
                   partiturList->insertTopLevelItem(idx+1, item);
                   partiturList->setItemExpanded(item, isExpanded);
@@ -384,6 +386,7 @@ void InstrumentsDialog::on_downButton_clicked()
             int idx = parent->indexOfChild(item);
             int n = parent->childCount();
             if (idx < (n-1)) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = parent->takeChild(idx);
                   parent->insertChild(idx+1, item);
                   partiturList->setItemSelected(item, true);
@@ -494,6 +497,8 @@ void MuseScore::editInstrList()
       // process modified partitur list
       //
       cs->startCmd();
+	  //TODO check if current selection is in a removed staff?
+	  cs->selection()->clear();
 
       QTreeWidget* pl = instrList->partiturList;
       Part* part   = 0;
@@ -532,9 +537,9 @@ void MuseScore::editInstrList()
                   QTreeWidgetItem* ci = 0;
                   rstaff = 0;
                   for (int cidx = 0; (ci = pli->child(cidx)); ++cidx) {
-                        StaffListItem* sli = (StaffListItem*)ci;
-                        Staff* staff = new Staff(cs, part, rstaff);
-                        sli->staff = staff;
+                        StaffListItem* sli = static_cast<StaffListItem*>(ci);
+                        Staff* staff       = new Staff(cs, part, rstaff);
+                        sli->staff         = staff;
                         staff->setRstaff(rstaff);
                         // ++rstaff;
                         staff->clefList()->setClef(0, sli->clef());
@@ -547,6 +552,8 @@ void MuseScore::editInstrList()
                         cs->undoInsertStaff(staff, staffIdx + rstaff);
                         ++rstaff;
                         }
+printf("set bar line span %d %p\n", part->nstaves(), part->staves()->front());
+                  part->staves()->front()->setBarLineSpan(part->nstaves());
                   cs->cmdInsertPart(part, staffIdx);
                   staffIdx += rstaff;
                   }
@@ -556,7 +563,7 @@ void MuseScore::editInstrList()
                   for (int cidx = 0; (ci = pli->child(cidx)); ++cidx) {
                         StaffListItem* sli = (StaffListItem*)ci;
                         if (sli->op == ITEM_DELETE) {
-                              cs->layout()->systems()->clear();
+                              cs->systems()->clear();
                               Staff* staff = sli->staff;
                               int sidx = staff->idx();
                               int eidx = sidx + 1;
@@ -628,10 +635,8 @@ void MuseScore::editInstrList()
                   break;
                   }
             }
-      if (sort) {
-            cs->sortStaves(dl);
-            cs->undoOp(dl);
-            }
+      if (sort)
+            cs->undo()->push(new SortStaves(cs, dl));
 
       cs->setLayoutAll(true);
       cs->endCmd();
@@ -695,8 +700,6 @@ void Score::cmdRemovePart(Part* part)
                   }
             }
 #endif
-      adjustBracketsDel(sidx, eidx);
-
       //
       //    adjust measures
       //
@@ -707,10 +710,8 @@ void Score::cmdRemovePart(Part* part)
             m->cmdRemoveStaves(sidx, eidx);
             }
 
-      for (int i = 0; i < n; ++i) {
-            Staff* staff = _staves[sidx];
-            undoRemoveStaff(staff, sidx);
-            }
+      for (int i = 0; i < n; ++i)
+            cmdRemoveStaff(sidx);
       undoRemovePart(part, sidx);
       }
 
@@ -747,6 +748,7 @@ void Score::removePart(Part* part)
 void Score::insertStaff(Staff* staff, int idx)
       {
       _staves.insert(idx, staff);
+
       staff->part()->insertStaff(staff);
 
       int track = idx * VOICES;
@@ -781,7 +783,7 @@ void Score::adjustBracketsDel(int sidx, int eidx)
 //                        }
                   }
             int span = staff->barLineSpan();
-            if ((sidx >= staffIdx) && (eidx < (staffIdx + span)))
+            if ((sidx >= staffIdx) && (eidx <= (staffIdx + span)))
                   undoChangeBarLineSpan(staff, span - (eidx-sidx));
             }
       }
@@ -799,7 +801,7 @@ void Score::adjustBracketsIns(int sidx, int eidx)
                   int span = staff->bracketSpan(i);
                   if ((span == 0) || ((staffIdx + span) < sidx) || (staffIdx > eidx))
                         continue;
-                  if ((sidx >= staffIdx) && (eidx <= (staffIdx + span)))
+                  if ((sidx >= staffIdx) && (eidx < (staffIdx + span)))
                         undoChangeBracketSpan(staff, i, span + (eidx-sidx));
 //                  else {
 //                        printf("TODO: adjust brackets\n");
@@ -818,16 +820,28 @@ void Score::adjustBracketsIns(int sidx, int eidx)
 void Score::cmdRemoveStaff(int staffIdx)
       {
       foreach(Element* e, _gel) {
-            if (e->type() != SLUR)
-                  continue;
-            Slur* slur = (Slur*)e;
-            if ((slur->staffIdx() == staffIdx) || (slur->staffIdx2() == staffIdx)) {
-                  undoRemoveElement(slur);
+            switch(e->type()) {
+                  case VOLTA:
+                  case TRILL:
+                  case PEDAL:
+                        if (e->staffIdx() == staffIdx) {
+                              undoRemoveElement(e);
+                              }
+                        break;
+                  case SLUR:
+                        {
+                        Slur* slur = static_cast<Slur*>(e);
+                        if ((slur->staffIdx() == staffIdx) || (slur->staffIdx2() == staffIdx))
+                              undoRemoveElement(slur);
+                        }
+                        break;
+                  default:
+                        break;
                   }
             }
+      adjustBracketsDel(staffIdx, staffIdx+1);
       Staff* s = staff(staffIdx);
       undoRemoveStaff(s, staffIdx);
-      adjustBracketsDel(staffIdx, staffIdx+1);
       }
 
 //---------------------------------------------------------
@@ -857,7 +871,7 @@ void Score::removeStaff(Staff* staff)
 
 void Score::sortStaves(QList<int>& dst)
       {
-      _layout->systems()->clear();  //??
+      systems()->clear();  //??
       _parts.clear();
       Part* curPart = 0;
       QList<Staff*> dl;
