@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: instrdialog.cpp,v 1.32 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: instrtemplate.cpp 1930 2009-07-14 18:01:50Z wschweer $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -23,6 +23,8 @@
 #include "style.h"
 #include "sym.h"
 #include "drumset.h"
+#include "clef.h"
+#include "bracket.h"
 
 QList<InstrumentTemplate*> instrumentTemplates;
 
@@ -32,16 +34,57 @@ QList<InstrumentTemplate*> instrumentTemplates;
 
 InstrumentTemplate::InstrumentTemplate()
       {
-      QTextOption to = name.defaultTextOption();
-      to.setUseDesignMetrics(true);
-      to.setWrapMode(QTextOption::NoWrap);
-      name.setUseDesignMetrics(true);
-      name.setDefaultTextOption(to);
-      shortName.setUseDesignMetrics(true);
-      shortName.setDefaultTextOption(to);
-      name.setDefaultFont(defaultTextStyleArray[TEXT_STYLE_INSTRUMENT_LONG].font());
-      shortName.setDefaultFont(defaultTextStyleArray[TEXT_STYLE_INSTRUMENT_SHORT].font());
-      drumset = 0;
+      staves        = 1;
+      clefIdx[0]    = CLEF_G;
+      staffLines[0] = 5;
+      smallStaff[0] = false;
+      bracket       = NO_BRACKET;
+      minPitchA     = 0;
+      maxPitchA     = 127;
+      minPitchP     = 0;
+      maxPitchP     = 127;
+      transpose     = 0;
+      useDrumset    = false;
+      drumset       = 0;
+      }
+
+InstrumentTemplate::InstrumentTemplate(const InstrumentTemplate& t)
+      {
+      group      = t.group;
+      trackName  = t.trackName;
+      name       = t.name;
+      shortName  = t.shortName;
+      staves     = t.staves;
+
+      for (int i = 0; i < MAX_STAVES; ++i) {
+            clefIdx[i]    = t.clefIdx[i];
+            staffLines[i] = t.staffLines[i];
+            smallStaff[i] = t.smallStaff[i];
+            }
+      bracket    = t.bracket;
+      minPitchA  =  t.minPitchA;
+      maxPitchA  =  t.maxPitchA;
+      minPitchP  =  t.minPitchP;
+      maxPitchP  =  t.maxPitchP;
+      transpose  =  t.transpose;
+      useDrumset =  t.useDrumset;
+      if (t.drumset)
+            drumset = new Drumset(*t.drumset);
+      else
+            drumset = 0;
+      midiActions = t.midiActions;
+
+      foreach(Channel* c, t.channel) {
+            Channel* ch = new Channel(*c);
+            channel.append(ch);
+            }
+      }
+
+InstrumentTemplate::~InstrumentTemplate()
+      {
+      delete drumset;
+      foreach(Channel* c, channel)
+            delete c;
       }
 
 //---------------------------------------------------------
@@ -51,8 +94,8 @@ InstrumentTemplate::InstrumentTemplate()
 void InstrumentTemplate::write(Xml& xml) const
       {
       xml.stag("Instrument");
-      xml.tag("name", name.toPlainText());            // TODO
-      xml.tag("short-name", shortName.toPlainText()); // TODO
+      xml.tag("name", name);
+      xml.tag("short-name",  shortName);
       xml.tag("description", trackName);
       if (staves == 1) {
             xml.tag("clef", clefIdx[0]);
@@ -72,10 +115,10 @@ void InstrumentTemplate::write(Xml& xml) const
                   }
             }
       xml.tag("bracket", bracket);
-      if (minPitch != 0)
-            xml.tag("minPitch", minPitch);
-      if (maxPitch != 127)
-            xml.tag("maxPitch", maxPitch);
+      if (minPitchA != 0 || maxPitchA != 127)
+            xml.tag("aPitchRange", QString("%1-%2").arg(minPitchA).arg(maxPitchA));
+      if (minPitchP != 0 || maxPitchP != 127)
+            xml.tag("pPitchRange", QString("%1-%2").arg(minPitchP).arg(maxPitchP));
       if (transpose)
             xml.tag("transposition", transpose);
       if (useDrumset) {
@@ -89,6 +132,61 @@ void InstrumentTemplate::write(Xml& xml) const
       xml.etag();
       }
 
+//---------------------------------------------------------
+//   setPitchRange
+//---------------------------------------------------------
+
+void InstrumentTemplate::setPitchRange(const QString& s, char* a, char* b) const
+      {
+      QStringList sl = s.split("-");
+      if (sl.size() != 2) {
+            *a = 0;
+            *b = 127;
+            return;
+            }
+      *a = sl[0].toInt();
+      *b = sl[1].toInt();
+      }
+
+//---------------------------------------------------------
+//   parseInstrName
+//---------------------------------------------------------
+
+static QString parseInstrName(const QString& name)
+      {
+      QString sName;
+      QDomDocument dom;
+      int line, column;
+      QString err;
+      if (!dom.setContent(name, false, &err, &line, &column)) {
+            QString col, ln;
+            col.setNum(column);
+            ln.setNum(line);
+            QString error = err + "\n at line " + ln + " column " + col;
+            printf("error: %s\n", qPrintable(error));
+            printf("   data:<%s>\n", qPrintable(name));
+            return QString();
+            }
+
+      for (QDomNode e = dom.documentElement(); !e.isNull(); e = e.nextSiblingElement()) {
+            for (QDomNode ee = e.firstChild(); !ee.isNull(); ee = ee.nextSibling()) {
+                  QDomElement el = ee.toElement();
+                  QString tag(el.tagName());
+                  if (tag == "symbol") {
+                        QString name = el.attribute(QString("name"));
+                        if (name == "flat")
+                              sName += "b";
+                        else if (name == "sharp")
+                              sName += "#";
+                        }
+                  QDomText t = ee.toText();
+                  if (!t.isNull())
+                        sName += t.data();
+                  }
+            }
+      return sName;
+      }
+
 //---------------------------------------------------------
 //   read
 //---------------------------------------------------------
@@ -104,72 +202,22 @@ void InstrumentTemplate::read(const QString& g, QDomElement e)
             smallStaff[i] = false;
             }
       bracket    = -1;
-      minPitch   = 0;
-      maxPitch   = 127;
+      minPitchA  = 0;
+      maxPitchA  = 127;
+      minPitchP  = 0;
+      maxPitchP  = 127;
       transpose  = 0;
       useDrumset = false;
 
-
-      double extraMag = 1.0;
-      double mag = _spatium * extraMag / (SPATIUM20 * DPI);
-      QFont font("MScore1");
-      font.setPointSizeF(12.0 * mag);     // TODO: get from style
-
-      QString sName;
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
             QString val(e.text());
             int i = val.toInt();
 
-            if (tag == "name") {
-                  QTextCursor cursor(&name);
-                  QTextCharFormat f = cursor.charFormat();
-                  QTextCharFormat sf(f);
-                  sf.setFont(font);
-
-                  for (QDomNode ee = e.firstChild(); !ee.isNull(); ee = ee.nextSibling()) {
-                        QDomElement de1 = ee.toElement();
-                        QString tag(de1.tagName());
-                        if (tag == "symbol") {
-                              QString name = de1.attribute(QString("name"));
-                              if (name == "flat") {
-                                    sName += "b";
-                                    cursor.insertText(QString(0xe10d), sf);
-                                    }
-                              else if (name == "sharp") {
-                                    sName += "#";
-                                    cursor.insertText(QString(0xe10c), sf);
-                                    }
-                              }
-                        QDomText t = ee.toText();
-                        if (!t.isNull()) {
-                              sName += t.data();
-                              cursor.insertText(t.data(), f);
-                              }
-                        }
-                  }
-            else if (tag == "short-name") {
-                  QTextCursor cursor(&shortName);
-                  QTextCharFormat f = cursor.charFormat();
-                  QTextCharFormat sf(f);
-                  sf.setFont(font);
-
-                  for (QDomNode ee = e.firstChild(); !ee.isNull(); ee = ee.nextSibling()) {
-                        QDomElement de1 = ee.toElement();
-                        QString tag(de1.tagName());
-                        if (tag == "symbol") {
-                              QString name = de1.attribute(QString("name"));
-                              if (name == "flat")
-                                    cursor.insertText(QString(0xe10d), sf);
-                              else if (name == "sharp")
-                                    cursor.insertText(QString(0xe10c), sf);
-                              }
-                        QDomText t = ee.toText();
-                        if (!t.isNull()) {
-                              cursor.insertText(t.data(), f);
-                              }
-                        }
-                  }
+            if (tag == "name")
+                  name = Xml::htmlToString(e);
+            else if (tag == "short-name")
+                  shortName = Xml::htmlToString(e);
             else if (tag == "description")
                   trackName = val;
             else if (tag == "staves")
@@ -194,10 +242,10 @@ void InstrumentTemplate::read(const QString& g, QDomElement e)
                   }
             else if (tag == "bracket")
                   bracket = i;
-            else if (tag == "minPitch")
-                  minPitch = i;
-            else if (tag == "maxPitch")
-                  maxPitch = i;
+            else if (tag == "aPitchRange")
+                  setPitchRange(val, &minPitchA, &maxPitchA);
+            else if (tag == "pPitchRange")
+                  setPitchRange(val, &minPitchP, &maxPitchP);
             else if (tag == "transposition")
                   transpose = i;
             else if (tag == "drumset")
@@ -232,12 +280,18 @@ void InstrumentTemplate::read(const QString& g, QDomElement e)
             a->reverb       = 0;
             a->name         = "normal";
             a->program      = 0;
+            a->bank         = 0;
             a->volume       = 100;
             a ->pan         = 60;
             channel.append(a);
             }
+      if (useDrumset) {
+            if (channel[0]->bank == 0)
+                  channel[0]->bank = 128;
+            channel[0]->updateInitList();
+            }
       if (trackName.isEmpty())
-            trackName = sName;
+            trackName = parseInstrName(name);
       }
 
 //---------------------------------------------------------
@@ -255,6 +309,23 @@ static void readInstrumentGroup(QDomElement e)
                   t->read(group, e);
                   instrumentTemplates.push_back(t);
                   }
+            else if (tag == "ref") {
+                  QString name = e.text();
+                  InstrumentTemplate* ttt = 0;
+                  foreach(InstrumentTemplate* tt, instrumentTemplates) {
+                        if (tt->trackName == name) {
+                              ttt = tt;
+                              break;
+                              }
+                        }
+                  if (ttt) {
+                        InstrumentTemplate* t = new InstrumentTemplate(*ttt);
+                        t->group = group;
+                        instrumentTemplates.push_back(t);
+                        }
+                  else
+                        printf("instrument reference not found <%s>\n", qPrintable(name));
+                  }
             else
                   domError(e);
             }
@@ -273,17 +344,22 @@ bool loadInstrumentTemplates(const QString& instrTemplates)
       QDomDocument doc;
       int line, column;
       QString err;
+
       bool rv = doc.setContent(&qf, false, &err, &line, &column);
+
       docName = qf.fileName();
       qf.close();
 
+      foreach(InstrumentTemplate* t, instrumentTemplates)
+            delete t;
       instrumentTemplates.clear();
+
       if (!rv) {
             QString s;
             s.sprintf("error reading file %s at line %d column %d: %s\n",
                instrTemplates.toLatin1().data(), line, column, err.toLatin1().data());
 
-            QMessageBox::critical(0, "MuseScore: Read File", s);
+            QMessageBox::critical(0, "MuseScore: Read Template File", s);
             return true;
             }
 
