@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: instrtemplate.h 1930 2009-07-14 18:01:50Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -35,17 +35,21 @@ class Staff;
 
 struct InstrumentTemplate {
       QString group;
-      QString trackName;            // also used for track name
-      QTextDocument name;           ///< shown on first system
-      QTextDocument shortName;      ///< shown on followup systems
+      QString trackName;     ///< also used for track name
+      QString name;          ///< shown on first system
+      QString shortName;     ///< shown on followup systems
 
       int staves;             // 1 <= MAX_STAVES
       int clefIdx[MAX_STAVES];
       int staffLines[MAX_STAVES];
-      int smallStaff[MAX_STAVES];
+      bool smallStaff[MAX_STAVES];
       int bracket;            // bracket type (NO_BRACKET)
-      char minPitch;
-      char maxPitch;
+
+      char minPitchA;         // pitch range playable by an amateur
+      char maxPitchA;
+      char minPitchP;         // pitch range playable by professional
+      char maxPitchP;
+
       char transpose;          // for transposing instruments
       bool useDrumset;
       Drumset* drumset;
@@ -53,7 +57,10 @@ struct InstrumentTemplate {
       QList<Channel*> channel;
 
       InstrumentTemplate();
+      InstrumentTemplate(const InstrumentTemplate&);
+      ~InstrumentTemplate();
 
+      void setPitchRange(const QString& s, char* a, char* b) const;
       void write(Xml& xml) const;
       void read(const QString& group, QDomElement);
       };
