@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: instrument.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -45,13 +45,13 @@ void NamedEventList::read(QDomElement e)
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
             if (tag == "program") {
-                  ControllerEvent* ev = new ControllerEvent();
+                  Event* ev = new Event(ME_CONTROLLER);
                   ev->setController(CTRL_PROGRAM);
                   ev->setValue(e.attribute("value", "0").toInt());
                   events.append(ev);
                   }
             else if (tag == "controller") {
-                  ControllerEvent* ev = new ControllerEvent();
+                  Event* ev = new Event(ME_CONTROLLER);
                   ev->setController(e.attribute("ctrl", "0").toInt());
                   ev->setValue(e.attribute("value", "0").toInt());
                   events.append(ev);
@@ -71,8 +71,10 @@ Instrument::Instrument()
       a->name         = "normal";
       channel.append(a);
 
-      minPitch        = 0;
-      maxPitch        = 127;
+      minPitchA        = 0;
+      maxPitchA        = 127;
+      minPitchP        = 0;
+      maxPitchP        = 127;
       pitchOffset     = 0;
       drumset         = 0;
       useDrumset      = false;
@@ -85,10 +87,14 @@ Instrument::Instrument()
 void Instrument::write(Xml& xml) const
       {
       xml.stag("Instrument");
-      if (minPitch > 0)
-            xml.tag("minPitch", minPitch);
-      if (maxPitch < 127)
-            xml.tag("maxPitch", maxPitch);
+      if (minPitchP > 0)
+            xml.tag("minPitchP", minPitchP);
+      if (maxPitchP < 127)
+            xml.tag("maxPitchP", maxPitchP);
+      if (minPitchA > 0)
+            xml.tag("minPitchA", minPitchA);
+      if (maxPitchA < 127)
+            xml.tag("maxPitchA", maxPitchA);
       if (pitchOffset)
             xml.tag("transposition", pitchOffset);
       if (useDrumset) {
@@ -109,6 +115,7 @@ void Instrument::write(Xml& xml) const
 void Instrument::read(QDomElement e)
       {
       int program = -1;
+      int bank    = 0;
       int chorus = 30;
       int reverb = 30;
       int volume = 100;
@@ -122,10 +129,23 @@ void Instrument::read(QDomElement e)
             QString tag(e.tagName());
             QString val(e.text());
             int i = val.toInt();
-            if (tag == "minPitch")
-                  minPitch = i;
-            else if (tag == "maxPitch")
-                  maxPitch = i;
+
+            if (tag == "minPitch") {      // obsolete
+                  minPitchP = i;
+                  minPitchA = i;
+                  }
+            else if (tag == "maxPitch") {       // obsolete
+                  maxPitchP = i;
+                  maxPitchA = i;
+                  }
+            else if (tag == "minPitchA")
+                  minPitchA = i;
+            else if (tag == "minPitchP")
+                  minPitchP = i;
+            else if (tag == "maxPitchA")
+                  maxPitchA = i;
+            else if (tag == "maxPitchP")
+                  maxPitchP = i;
             else if (tag == "transposition")
                   pitchOffset = i;
             else if (tag == "useDrumset") {
@@ -175,10 +195,16 @@ void Instrument::read(QDomElement e)
             a->reverb       = reverb;
             a->name         = "normal";
             a->program      = program;
+            a->bank         = bank;
             a->volume       = volume;
             a ->pan         = pan;
             channel.append(a);
             }
+      if (useDrumset) {
+            if (channel[0]->bank == 0)
+                  channel[0]->bank = 128;
+            channel[0]->updateInitList();
+            }
       }
 
 //---------------------------------------------------------
@@ -204,8 +230,7 @@ Channel::Channel()
             init.append(0);
       channel  = -1;
       program  = -1;
-      hbank    = -1;
-      lbank    = -1;
+      bank     = 0;
       volume   = 100;
       pan      = 64;
       chorus   = 30;
@@ -252,44 +277,36 @@ void Channel::read(QDomElement e)
                   program = e.attribute("value", "-1").toInt();
                   if (program == -1)
                         program = val.toInt();
-                  ControllerEvent* e = new ControllerEvent();
-                  e->setController(CTRL_PROGRAM);
-                  e->setValue(program);
-                  init[A_PROGRAM] = e;
                   }
             else if (tag == "controller") {
                   int value = e.attribute("value", "0").toInt();
                   int ctrl  = e.attribute("ctrl", "0").toInt();
-                  ControllerEvent* e = new ControllerEvent();
-                  e->setController(ctrl);
-                  e->setValue(value);
                   switch(ctrl) {
                         case CTRL_HBANK:
-                              init[A_HBANK] = e;
-                              hbank = value;
+                              bank = (value << 7) + (bank & 0x7f);
                               break;
                         case CTRL_LBANK:
-                              init[A_LBANK] = e;
-                              lbank = value;
+                              bank = (bank & ~0x7f) + (value & 0x7f);
                               break;
                         case CTRL_VOLUME:
-                              init[A_VOLUME] = e;
                               volume = value;
                               break;
                         case CTRL_PANPOT:
                               pan = value;
-                              init[A_PAN] = e;
                               break;
                         case CTRL_CHORUS_SEND:
                               chorus = value;
-                              init[A_CHORUS] = e;
                               break;
                         case CTRL_REVERB_SEND:
                               reverb = value;
-                              init[A_REVERB] = e;
                               break;
                         default:
+                              {
+                              Event* e = new Event(ME_CONTROLLER);
+                              e->setController(ctrl);
+                              e->setValue(value);
                               init.append(e);
+                              }
                               break;
                         }
                   }
@@ -306,47 +323,43 @@ void Channel::read(QDomElement e)
 void Channel::updateInitList() const
       {
       for (int i = 0; i < A_INIT_COUNT; ++i) {
-            if (init[i]) {
-                  delete init[i];
-                  init[i] = 0;
-                  }
+            delete init[i];
+            init[i] = 0;
             }
-      ControllerEvent* e;
+      Event* e;
       if (program != -1) {
-            e = new ControllerEvent();
+            e = new Event(ME_CONTROLLER);
             e->setController(CTRL_PROGRAM);
             e->setValue(program);
             init[A_PROGRAM] = e;
             }
 
-      if (hbank != -1) {
-            e = new ControllerEvent();
-            e->setController(CTRL_HBANK);
-            e->setValue(hbank);
-            init[A_HBANK] = e;
-            }
-      if (lbank != -1) {
-            e = new ControllerEvent();
-            e->setController(CTRL_LBANK);
-            e->setValue(lbank);
-            init[A_LBANK] = e;
-            }
-      e = new ControllerEvent();
+      e = new Event(ME_CONTROLLER);
+      e->setController(CTRL_HBANK);
+      e->setValue((bank >> 7) & 0x7f);
+      init[A_HBANK] = e;
+
+      e = new Event(ME_CONTROLLER);
+      e->setController(CTRL_LBANK);
+      e->setValue(bank & 0x7f);
+      init[A_LBANK] = e;
+
+      e = new Event(ME_CONTROLLER);
       e->setController(CTRL_VOLUME);
       e->setValue(volume);
       init[A_VOLUME] = e;
 
-      e = new ControllerEvent();
+      e = new Event(ME_CONTROLLER);
       e->setController(CTRL_PANPOT);
       e->setValue(pan);
       init[A_PAN] = e;
 
-      e = new ControllerEvent();
+      e = new Event(ME_CONTROLLER);
       e->setController(CTRL_CHORUS_SEND);
       e->setValue(chorus);
       init[A_CHORUS] = e;
 
-      e = new ControllerEvent();
+      e = new Event(ME_CONTROLLER);
       e->setController(CTRL_REVERB_SEND);
       e->setValue(reverb);
       init[A_REVERB] = e;
