@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: part.h,v 1.8 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: instrument.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -72,8 +72,7 @@ struct Channel {
       mutable EventList init;
 
       int program;     // current values as shown in mixer
-      int hbank;       // initialized from "init"
-      int lbank;
+      int bank;        // initialized from "init"
       char volume;
       char pan;
       char chorus;
@@ -94,8 +93,7 @@ struct Channel {
 //---------------------------------------------------------
 
 struct Instrument {
-      char minPitch;
-      char maxPitch;
+      char minPitchA, maxPitchA, minPitchP, maxPitchP;
       char pitchOffset;
 
       Drumset* drumset;
