@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: jackaudio.cpp,v 1.8 2006/03/02 17:08:34 wschweer Exp $
+//  $Id: jackaudio.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -52,8 +52,7 @@ JackAudio::~JackAudio()
                      strerror(errno));
                   }
             }
-      if (synth)
-            delete synth;
+      delete synth;
       }
 
 //---------------------------------------------------------
@@ -332,13 +331,10 @@ bool JackAudio::init()
                   fprintf(stderr, "no jack port for right channel found!\n");
                   }
             }
-      synth = new ISynth();
-      if (!synth->init(_sampleRate)) {
-            fprintf(stderr, "Synti init failed\n");
-            delete synth;
-            synth = 0;
-            return false;
-            }
+
+      synth = new FluidS::Fluid();
+      synth->init(_sampleRate);
+
       midiDriver = new AlsaMidiDriver(seq);
       if (!midiDriver->init()) {
             delete midiDriver;
@@ -388,7 +384,7 @@ int JackAudio::getState()
 //   putEvent
 //---------------------------------------------------------
 
-void JackAudio::putEvent(const MidiOutEvent& e)
+void JackAudio::putEvent(const Event& e)
       {
       synth->play(e);
       }
