@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: jackaudio.h,v 1.3 2006/03/02 17:08:34 wschweer Exp $
+//  $Id: jackaudio.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -25,7 +25,7 @@
 #include "driver.h"
 #include <jack/jack.h>
 
-class ISynth;
+class Synth;
 class Seq;
 class MidiDriver;
 
@@ -42,7 +42,6 @@ class JackAudio : public Driver {
       jack_port_t* portR;
       jack_port_t* portL;
 
-      ISynth* synth;
       MidiDriver* midiDriver;
 
       static int processAudio(jack_nframes_t, void*);
@@ -66,7 +65,7 @@ class JackAudio : public Driver {
       virtual void stopTransport();
       virtual int getState();
       virtual int sampleRate() const { return _sampleRate; }
-      virtual void putEvent(const MidiOutEvent&);
+      virtual void putEvent(const Event&);
       virtual void process(int, float*, float*, int);
       virtual const MidiPatch* getPatchInfo(bool onlyDrums, const MidiPatch*);
       virtual void midiRead();
