@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: layout.cpp,v 1.61 2006/09/08 19:37:08 lvinken Exp $
+//  $Id: layout.cpp 1927 2009-07-13 17:37:11Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -29,7 +29,6 @@
 #include "text.h"
 #include "staff.h"
 #include "style.h"
-#include "layout.h"
 #include "timesig.h"
 #include "canvas.h"
 #include "chord.h"
@@ -42,12 +41,14 @@
 #include "system.h"
 #include "part.h"
 #include "utils.h"
+#include "measure.h"
+#include "preferences.h"
 
 //---------------------------------------------------------
 //   rebuildBspTree
 //---------------------------------------------------------
 
-void ScoreLayout::rebuildBspTree()
+void Score::rebuildBspTree()
       {
       QRectF r;
       QList<const Element*> el;
@@ -60,8 +61,13 @@ void ScoreLayout::rebuildBspTree()
             r |= page->abbox();
             page->collectElements(el);
             }
-      foreach (const Element* element, *score()->gel()) {
+      foreach (const Element* element, _gel) {
             if (element->track() != -1) {
+                  if (element->staffIdx() < 0 || element->staffIdx() >= nstaves()) {
+                        printf("element %s bad staffIdx %d(track:%d) >= staves(%d)\n",
+                           element->name(), element->staffIdx(), element->track(), nstaves());
+                        continue;
+                        }
                   if (!element->staff()->show())
                         continue;
                   }
@@ -80,41 +86,18 @@ void ScoreLayout::rebuildBspTree()
 //   first
 //---------------------------------------------------------
 
-MeasureBase* ScoreLayout::first() const
+MeasureBase* Score::first() const
       {
-      return _score->measures()->first();
+      return _measures.first();
       }
 
 //---------------------------------------------------------
 //   last
 //---------------------------------------------------------
 
-MeasureBase* ScoreLayout::last()  const
+MeasureBase* Score::last()  const
       {
-      return _score->measures()->last();
-      }
-
-//---------------------------------------------------------
-//   ScoreLayout
-//---------------------------------------------------------
-
-ScoreLayout::ScoreLayout(Score* score)
-   : Element(score)
-      {
-      _spatium = ::_spatium;
-
-      _systems.clear();
-      _pages.clear();
-      _needLayout  = false;
-      _pageFormat  = new PageFormat;
-      _paintDevice = 0;
-      startLayout  = 0;
-      }
-
-ScoreLayout::~ScoreLayout()
-      {
-      if (_pageFormat)
-            delete _pageFormat;
+      return _measures.last();
       }
 
 //---------------------------------------------------------
@@ -167,10 +150,9 @@ int Score::clefOffset(int tick, Staff* staff) const
 //   already existent systems and pages are reused
 //---------------------------------------------------------
 
-void ScoreLayout::doLayout()
+void Score::doLayout()
       {
-      ::_spatium  = _spatium;        // needed for preview
-      _spatiumMag = _spatium / (DPI * SPATIUM20);
+// printf("doLayout============\n");
       _needLayout = false;
 
       if (startLayout) {
@@ -180,7 +162,6 @@ void ScoreLayout::doLayout()
                   return;
                   }
             }
-
       if (first() == 0) {
             // score is empty
             foreach(Page* page, _pages)
@@ -188,7 +169,7 @@ void ScoreLayout::doLayout()
             _pages.clear();
 
             Page* page = addPage();
-            page->layout(this);
+            page->layout();
             page->setNo(0);
             page->setPos(0.0, 0.0);
 
@@ -203,11 +184,15 @@ void ScoreLayout::doLayout()
       //             on context
       //-----------------------------------------------------------------------
 
-      for (int staffIdx = 0; staffIdx < _score->nstaves(); ++staffIdx) {
+      for (int staffIdx = 0; staffIdx < nstaves(); ++staffIdx) {
             for (MeasureBase* mb = first(); mb; mb = mb->next()) {
                   mb->setDirty();
                   if (mb->type() == MEASURE) {
                         Measure* m = static_cast<Measure*>(mb);
+#if 1 // CS1
+                        if (staffIdx == 0)
+                              m->layoutBeams1();  // find hooks / stem direction
+#endif
                         m->layout0(staffIdx);
                         }
                   }
@@ -236,12 +221,15 @@ void ScoreLayout::doLayout()
       //---------------------------------------------------
 
       for (MeasureBase* mb = first(); mb; mb = mb->next()) {
-            if (mb->type() == MEASURE)
-                  static_cast<Measure*>(mb)->layout2(this);
+            if (mb->type() == MEASURE) {
+                  Measure* m = static_cast<Measure*>(mb);
+                  //if (m->multiMeasure() >= 0)
+                        m->layout2();
+                  }
             }
 
-      foreach(Element* el, *score()->gel())
-            el->layout(this);
+      foreach(Element* el, _gel)
+            el->layout();
 
       //---------------------------------------------------
       //    remove remaining pages and systems
@@ -266,14 +254,15 @@ void ScoreLayout::doLayout()
 //    add generated timesig keysig and clef
 //---------------------------------------------------------
 
-void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
+void Score::processSystemHeader(Measure* m, bool isFirstSystem)
       {
       int tick = m->tick();
-      int nstaves = _score->nstaves();
-      for (int i = 0; i < nstaves; ++i) {
-            Staff* staff   = _score->staff(i);
-            if (!m->system()->staff(i)->show())
+      int i = 0;
+      foreach(Staff* staff, _staves) {
+            if (!m->system()->staff(i)->show()) {
+                  ++i;
                   continue;
+                  }
 
             KeySig* hasKeysig = 0;
             Clef*   hasClef   = 0;
@@ -309,12 +298,12 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
                               break;
                         }
                   }
-            bool needKeysig = keyIdx && (isFirstSystem || _score->style()->genKeysig);
+            bool needKeysig = keyIdx && (isFirstSystem || styleB(ST_genKeysig));
             if (needKeysig && !hasKeysig) {
                   //
                   // create missing key signature
                   //
-                  KeySig* ks = new KeySig(_score);
+                  KeySig* ks = new KeySig(this);
                   ks->setTrack(i * VOICES);
                   ks->setTick(tick);
                   ks->setGenerated(true);
@@ -330,14 +319,14 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
                   seg->setElement(track, 0);    // TODO: delete element
                   m->setDirty();
                   }
-            bool needClef = isFirstSystem || _score->style()->genClef;
+            bool needClef = isFirstSystem || styleB(ST_genClef);
             if (needClef) {
                   int idx = staff->clefList()->clef(tick);
                   if (!hasClef) {
                         //
                         // create missing clef
                         //
-                        hasClef = new Clef(_score);
+                        hasClef = new Clef(this);
                         hasClef->setTrack(i * VOICES);
                         hasClef->setTick(tick);
                         hasClef->setGenerated(true);
@@ -357,6 +346,7 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
                         m->setDirty();
                         }
                   }
+            ++i;
             }
       }
 
@@ -364,11 +354,11 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
 //   getNextSystem
 //---------------------------------------------------------
 
-System* ScoreLayout::getNextSystem(bool isFirstSystem, bool isVbox)
+System* Score::getNextSystem(bool isFirstSystem, bool isVbox)
       {
       System* system;
       if (curSystem >= _systems.size()) {
-            system = new System(_score);
+            system = new System(this);
             _systems.append(system);
             }
       else {
@@ -378,7 +368,7 @@ System* ScoreLayout::getNextSystem(bool isFirstSystem, bool isVbox)
       system->setFirstSystem(isFirstSystem);
       system->setVbox(isVbox);
       if (!isVbox) {
-            int nstaves = _score->nstaves();
+            int nstaves = Score::nstaves();
             for (int i = system->staves()->size(); i < nstaves; ++i)
                   system->insertStaff(i);
             int dn = system->staves()->size() - nstaves;
@@ -392,11 +382,11 @@ System* ScoreLayout::getNextSystem(bool isFirstSystem, bool isVbox)
 //   getCurPage
 //---------------------------------------------------------
 
-void ScoreLayout::getCurPage()
+void Score::getCurPage()
       {
       Page* page = curPage >= _pages.size() ? addPage() : _pages[curPage];
       page->setNo(curPage);
-      page->layout(this);
+      page->layout();
       double x = (curPage == 0) ? 0.0 : _pages[curPage - 1]->pos().x()
          + page->width() + ((curPage & 1) ? 1.0 : 50.0);
       page->setPos(x, 0.0);
@@ -407,11 +397,11 @@ void ScoreLayout::getCurPage()
 //    return true, if next page must be relayouted
 //---------------------------------------------------------
 
-bool ScoreLayout::layoutPage()
+bool Score::layoutPage()
       {
       Page* page = _pages[curPage];
-      const double slb = point(score()->style()->staffLowerBorder);
-      const double sub = point(score()->style()->staffUpperBorder);
+      const double slb = point(styleS(ST_staffLowerBorder));
+      const double sub = point(styleS(ST_staffUpperBorder));
 
       // usable width of page:
       qreal w  = page->loWidth() - page->lm() - page->rm();
@@ -436,7 +426,7 @@ bool ScoreLayout::layoutPage()
                   system->setWidth(w);
                   VBox* vbox = (VBox*) curMeasure;
                   vbox->setParent(system);
-                  vbox->layout(this);
+                  vbox->layout();
                   double bh = vbox->height();
 
                   // put at least one system on page
@@ -451,11 +441,11 @@ bool ScoreLayout::layoutPage()
 
                   curMeasure = curMeasure->next();
                   ++curSystem;
-                  y += bh + score()->style()->boxSystemDistance.point();
+                  y += bh + point(styleS(ST_frameSystemDistance));
                   nettoHeight += bh;
                   if (y > ey)
                         break;
-                  nettoHeight += score()->style()->boxSystemDistance.point();
+                  nettoHeight += point(styleS(ST_frameSystemDistance));
                   }
             else {
                   if (firstSystemOnPage) {
@@ -503,10 +493,10 @@ bool ScoreLayout::layoutPage()
 
       double ph = page->loHeight() - page->bm() - page->tm() - slb - sub;
 
-      if (restHeight > (ph * (1.0 - score()->style()->pageFillLimit)))
+      if (restHeight > (ph * (1.0 - styleD(ST_pageFillLimit))))
             return true;
 
-      double systemDistance = score()->style()->systemDistance.point();
+      double systemDistance = point(styleS(ST_systemDistance));
       double extraDist = (rows > 1) ? ((ph - nettoHeight + systemDistance) / (rows - 1.0)) : 0.0;
       y = 0;
       int n = page->systems()->size();
@@ -530,7 +520,7 @@ bool ScoreLayout::layoutPage()
 //    measures in sequence
 //---------------------------------------------------------
 
-Measure* ScoreLayout::skipEmptyMeasures(Measure* m)
+Measure* Score::skipEmptyMeasures(Measure* m, System* system)
       {
       Measure* sm = m;
       int n       = 0;
@@ -544,12 +534,13 @@ Measure* ScoreLayout::skipEmptyMeasures(Measure* m)
             m = static_cast<Measure*>(mb);
             }
       m = sm;
-      if (n >= score()->style()->minEmptyMeasures) {
+      if (n >= styleI(ST_minEmptyMeasures)) {
             for (int i = 0; i < (n-1); ++i) {
                   m->setMultiMeasure(-1);
+                  m->setSystem(system);
                   m = static_cast<Measure*>(m->next());
                   }
-            m->setMultiMeasure(n);
+            m->setMultiMeasure(n);  // last measure is presented as multi measure rest
             }
       else
             m->setMultiMeasure(0);
@@ -560,30 +551,30 @@ Measure* ScoreLayout::skipEmptyMeasures(Measure* m)
 //   layoutSystem1
 //---------------------------------------------------------
 
-bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
+bool Score::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
       {
       System* system = getNextSystem(isFirstSystem, false);
 
       double xo = 0;
       if (curMeasure->type() == HBOX)
-            xo = curMeasure->width();
+            xo = point(static_cast<Box*>(curMeasure)->boxWidth());
 
       system->setInstrumentNames();
-      system->layout(this, xo);
+      system->layout(xo);
       minWidth            = system->leftMargin();
       double systemWidth  = w;
 
       bool continueFlag   = false;
 
-      int nstaves = _score->nstaves();
+      int nstaves = Score::nstaves();
       bool isFirstMeasure = true;
 //      MeasureBase* firstMeasure = curMeasure;
 
       for (; curMeasure;) {
             if (curMeasure->type() == MEASURE) {
                   Measure* m = static_cast<Measure*>(curMeasure);
-                  if (score()->style()->createMultiMeasureRests)
-                        curMeasure = skipEmptyMeasures(m);
+                  if (styleB(ST_createMultiMeasureRests))
+                        curMeasure = skipEmptyMeasures(m, system);
                   else
                         m->setMultiMeasure(0);
                   }
@@ -594,7 +585,7 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
             double stretch = 0.0;
 
             if (curMeasure->type() == HBOX) {
-                  ww = static_cast<Box*>(curMeasure)->boxWidth().point();
+                  ww = point(static_cast<Box*>(curMeasure)->boxWidth());
                   if (!isFirstMeasure)
                         continueFlag = true;    //try to put another system on current row
                   }
@@ -620,28 +611,29 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
                                     if (!isFirstMeasure || (seg->subtype() == Segment::SegTimeSigAnnounce))
                                           seg->setElement(track, 0);
                                     }
+                              double staffMag = staff(staffIdx)->mag();
                               if (el->type() == CLEF) {
                                     Clef* clef = static_cast<Clef*>(el);
-                                    if (!isFirstMeasure || (seg != m->first()))
-                                          clef->setSmall(true);
-                                    else
-                                          clef->setSmall(false);
-                                    double staffMag = score()->staff(staffIdx)->mag();
+                                    clef->setSmall(!isFirstMeasure || (seg != m->first()));
                                     clef->setMag(staffMag);
                                     }
+                              else if (el->type() == KEYSIG || el->type() == TIMESIG)
+                                    el->setMag(staffMag);
+//TODO                              double staffMag = staff(staffIdx)->mag();
+//                              el->setMag(staffMag);
                               }
                         }
 
                   m->createEndBarLines();
-                  m->layoutBeams1(this);  // find hooks
+                  m->layoutBeams1();  // find hooks
 
-                  m->layoutX(this, 1.0);
+                  m->layoutX(1.0);
                   ww      = m->layoutWidth().stretchable;
-                  stretch = m->userStretch() * score()->style()->measureSpacing;
+                  stretch = m->userStretch() * styleD(ST_measureSpacing);
 
                   ww *= stretch;
-                  if (ww < point(score()->style()->minMeasureWidth))
-                        ww = point(score()->style()->minMeasureWidth);
+                  if (ww < point(styleS(ST_minMeasureWidth)))
+                        ww = point(styleS(ST_minMeasureWidth));
                   isFirstMeasure = false;
                   }
 
@@ -668,11 +660,11 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
       bool showChanged = false;
       int staves = system->staves()->size();
       int staffIdx = 0;
-      foreach (Part* p, *score()->parts()) {
+      foreach (Part* p, _parts) {
             int nstaves   = p->nstaves();
             bool hidePart = false;
 
-            if (score()->style()->hideEmptyStaves && (staves > 1)) {
+            if (styleB(ST_hideEmptyStaves) && (staves > 1)) {
                   hidePart = true;
                   for (int i = staffIdx; i < staffIdx + nstaves; ++i) {
                         foreach(MeasureBase* m, system->measures()) {
@@ -688,7 +680,7 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
 
             for (int i = staffIdx; i < staffIdx + nstaves; ++i) {
                   SysStaff* s  = system->staff(i);
-                  Staff* staff = score()->staff(i);
+                  Staff* staff = Score::staff(i);
                   bool oldShow = s->show();
                   s->setShow(hidePart ? false : staff->show());
                   if (oldShow != s->show()) {
@@ -714,7 +706,7 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
 //    x, y  position of row on page
 //---------------------------------------------------------
 
-QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
+QList<System*> Score::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
    bool isFirstSystem, double* h)
       {
       bool raggedRight = layoutDebug;
@@ -734,7 +726,7 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
       //
       // dont stretch last system row, if minWidth is <= lastSystemFillLimit
       //
-      if (curMeasure == 0 && ((minWidth / rowWidth) <= score()->style()->lastSystemFillLimit))
+      if (curMeasure == 0 && ((minWidth / rowWidth) <= styleD(ST_lastSystemFillLimit)))
             raggedRight = true;
 
       //-------------------------------------------------------
@@ -756,21 +748,22 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
                   }
             MeasureBase* lm = system->measures().back();
             int tick        = lm->tick() + lm->tickLen();
-            SigEvent sig1   = _score->sigmap->timesig(tick - 1);
-            SigEvent sig2   = _score->sigmap->timesig(tick);
+            SigEvent sig1   = sigmap->timesig(tick - 1);
+            SigEvent sig2   = sigmap->timesig(tick);
 
-            if (score()->style()->genCourtesyTimesig && !sig1.nominalEqual(sig2)) {
+            if (styleB(ST_genCourtesyTimesig) && !sig1.nominalEqual(sig2)) {
                   while (lm && lm->type() != MEASURE)
                         lm = lm->prev();
                   if (lm) {
                         Measure* m = (Measure*)lm;
                         Segment* s  = m->getSegment(Segment::SegTimeSigAnnounce, tick);
-                        int nstaves = score()->nstaves();
+                        int nstaves = Score::nstaves();
                         for (int track = 0; track < nstaves * VOICES; track += VOICES) {
                               if (s->element(track) == 0) {
-                                    TimeSig* ts = new TimeSig(score(), sig2.denominator, sig2.nominator);
+                                    TimeSig* ts = new TimeSig(this, sig2.denominator, sig2.nominator);
                                     ts->setTrack(track);
                                     ts->setGenerated(true);
+                                    ts->setMag(ts->staff()->mag());
                                     s->add(ts);
                                     needRelayout = true;
                                     }
@@ -784,18 +777,22 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
             //
             //    compute repeat bar lines
             //
+            bool firstMeasure = true;
             for (int i = 0; i < n; ++i) {
                   MeasureBase* mb = ml[i];
                   if (mb->type() != MEASURE)
                         continue;
                   Measure* m = (Measure*)mb;
 
+                  // first measure repeat?
+                  bool fmr = firstMeasure && (m->repeatFlags() & RepeatStart);
+
                   if (i == (n-1)) {       // last measure in system?
                         if (m->repeatFlags() & RepeatEnd)
                               m->setEndBarLineType(END_REPEAT, true);
                         else if (m->endBarLineGenerated())
                               m->setEndBarLineType(NORMAL_BAR, true);
-                        needRelayout |= m->setStartRepeatBarLine((i == 0) && (m->repeatFlags() & RepeatStart));
+                        needRelayout |= m->setStartRepeatBarLine(fmr);
                         }
                   else {
                         MeasureBase* mb = m->next();
@@ -805,7 +802,7 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
                         if (mb->type() == MEASURE)
                               nm = (Measure*)mb;
 
-                        needRelayout |= m->setStartRepeatBarLine((i == 0) && (m->repeatFlags() & RepeatStart));
+                        needRelayout |= m->setStartRepeatBarLine(fmr);
                         if (m->repeatFlags() & RepeatEnd) {
                               if (nm && (nm->repeatFlags() & RepeatStart))
                                     m->setEndBarLineType(END_START_REPEAT, true);
@@ -818,6 +815,7 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
                               m->setEndBarLineType(NORMAL_BAR, true);
                         }
                   needRelayout |= m->createEndBarLines();
+                  firstMeasure = false;
                   }
             }
 
@@ -827,11 +825,11 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
       foreach(System* system, sl) {
             foreach (MeasureBase* mb, system->measures()) {
                   if (mb->type() == HBOX)
-                        minWidth += ((Box*)mb)->boxWidth().point();
+                        minWidth += point(((Box*)mb)->boxWidth());
                   else {
                         Measure* m = (Measure*)mb;
                         if (needRelayout)
-                              m->layoutX(this, 1.0);
+                              m->layoutX(1.0);
                         minWidth    += m->layoutWidth().stretchable;
                         totalWeight += m->tickLen() * m->userStretch();
                         }
@@ -857,25 +855,25 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
                         Measure* m    = static_cast<Measure*>(mb);
                         double weight = m->tickLen() * m->userStretch();
                         ww            = m->layoutWidth().stretchable + rest * weight;
-                        m->layout(this, ww);
+                        m->layout(ww);
                         }
                   else if (mb->type() == HBOX) {
                         mb->setPos(pos);
-                        ww = static_cast<Box*>(mb)->boxWidth().point();
-                        mb->layout(this);
+                        ww = point(static_cast<Box*>(mb)->boxWidth());
+                        mb->layout();
                         }
                   pos.rx() += ww;
                   }
             system->setPos(xx + x, y);
             double w = pos.x();
             system->setWidth(w);
-            system->layout2(this);
+            system->layout2();
             foreach(MeasureBase* mb, system->measures()) {
                   if (mb->type() == HBOX)
                         mb->setHeight(system->height());
                   }
             xx += w;
-            double hh = system->height() + system->staves()->back()->distance();
+            double hh = system->height() + point(system->staves()->back()->distance());
             if (hh > *h)
                   *h = hh;
             }
@@ -886,7 +884,7 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
 //   addPage
 //---------------------------------------------------------
 
-Page* ScoreLayout::addPage()
+Page* Score::addPage()
       {
       Page* page = new Page(this);
       page->setNo(_pages.size());
@@ -898,7 +896,7 @@ Page* ScoreLayout::addPage()
 //   setPageFormat
 //---------------------------------------------------------
 
-void ScoreLayout::setPageFormat(const PageFormat& pf)
+void Score::setPageFormat(const PageFormat& pf)
       {
       *_pageFormat = pf;
       }
@@ -907,7 +905,7 @@ void ScoreLayout::setPageFormat(const PageFormat& pf)
 //   setInstrumentNames
 //---------------------------------------------------------
 
-void ScoreLayout::setInstrumentNames()
+void Score::setInstrumentNames()
       {
       for (iSystem is = systems()->begin(); is != systems()->end(); ++is)
             (*is)->setInstrumentNames();
@@ -942,9 +940,9 @@ static Note* searchTieNote(Note* note, Segment* segment, int track)
  Rebuild tie connections.
 */
 
-void ScoreLayout::connectTies()
+void Score::connectTies()
       {
-      int tracks = _score->nstaves() * VOICES;
+      int tracks = nstaves() * VOICES;
       for (MeasureBase* mb = first(); mb; mb = mb->next()) {
             if (mb->type() != MEASURE)
                   continue;
@@ -980,14 +978,18 @@ void ScoreLayout::connectTies()
 //   add
 //---------------------------------------------------------
 
-void ScoreLayout::add(Element* el)
+void Score::add(Element* el)
       {
       if (el->type() == MEASURE || el->type() == HBOX || el->type() == VBOX) {
-            _score->measures()->add((MeasureBase*)el);
+            measures()->add((MeasureBase*)el);
             }
       else {
-            el->setParent(this);
-            score()->gel()->append(el);
+            if (el->check())
+                  _gel.append(el);
+            else {
+                  printf("remove invalid element <%s>\n", el->name());
+                  delete el;
+                  }
             }
       }
 
@@ -995,31 +997,14 @@ void ScoreLayout::add(Element* el)
 //   remove
 //---------------------------------------------------------
 
-void ScoreLayout::remove(Element* el)
+void Score::remove(Element* el)
       {
       if (el->type() == MEASURE || el->type() == HBOX || el->type() == VBOX) {
-            _score->measures()->remove((MeasureBase*)el);
+            measures()->remove(static_cast<MeasureBase*>(el));
             }
       else {
-            int idx = score()->gel()->indexOf(el);
-            if (idx == -1)
-                  printf("ScoreLayout::remove(): element not found\n");
-            else
-                  score()->gel()->removeAt(idx);
-            }
-      }
-
-//---------------------------------------------------------
-//   change
-//---------------------------------------------------------
-
-void ScoreLayout::change(Element* o, Element* n)
-      {
-      if (n->type() == MEASURE || n->type() == HBOX || n->type() == VBOX)
-            _score->measures()->change((MeasureBase*)o, (MeasureBase*)n);
-      else {
-            remove(o);
-            add(n);
+            if (!_gel.removeOne(el))
+                  printf("Score::remove(): element not found\n");
             }
       }
 
@@ -1027,7 +1012,7 @@ void ScoreLayout::change(Element* o, Element* n)
 //   reLayout
 //---------------------------------------------------------
 
-void ScoreLayout::reLayout(Measure* m)
+void Score::reLayout(Measure* m)
       {
       _needLayout = true;
       startLayout = m;
@@ -1039,13 +1024,14 @@ void ScoreLayout::reLayout(Measure* m)
 //    a full layout must be done starting at "startLayout"
 //---------------------------------------------------------
 
-bool ScoreLayout::doReLayout()
+bool Score::doReLayout()
       {
+#if 0
       if (startLayout->type() == MEASURE) {
-            for (int staffIdx = 0; staffIdx < _score->nstaves(); ++staffIdx)
+            for (int staffIdx = 0; staffIdx < nstaves(); ++staffIdx)
                   static_cast<Measure*>(startLayout)->layout0(staffIdx);
             }
-
+#endif
       System* system  = startLayout->system();
       qreal sysWidth  = system->width();
       double minWidth = system->leftMargin();
@@ -1057,18 +1043,20 @@ bool ScoreLayout::doReLayout()
       foreach(m, system->measures()) {
             double ww;
             if (m->type() == HBOX) {
-                  ww = static_cast<Box*>(m)->boxWidth().point();
+                  ww = point(static_cast<Box*>(m)->boxWidth());
                   }
             else if (m->type() == MEASURE) {
                   Measure* measure = static_cast<Measure*>(m);
-                  measure->layoutBeams1(this);
-                  measure->layoutX(this, 1.0);
+                  for (int staffIdx = 0; staffIdx < nstaves(); ++staffIdx)
+                        measure->layout0(staffIdx);
+                  measure->layoutBeams1();
+                  measure->layoutX(1.0);
                   ww      = measure->layoutWidth().stretchable;
-                  double stretch = measure->userStretch() * score()->style()->measureSpacing;
+                  double stretch = measure->userStretch() * styleD(ST_measureSpacing);
 
                   ww *= stretch;
-                  if (ww < point(score()->style()->minMeasureWidth))
-                        ww = point(score()->style()->minMeasureWidth);
+                  if (ww < point(styleS(ST_minMeasureWidth)))
+                        ww = point(styleS(ST_minMeasureWidth));
                   }
             minWidth += ww;
             }
@@ -1081,13 +1069,13 @@ bool ScoreLayout::doReLayout()
       m = m->next();
       if (m && m->subtype() == MEASURE) {
             Measure* measure = static_cast<Measure*>(m);
-            measure->layoutX(this, 1.0);
+            measure->layoutX(1.0);
             double ww      = measure->layoutWidth().stretchable;
-            double stretch = measure->userStretch() * score()->style()->measureSpacing;
+            double stretch = measure->userStretch() * styleD(ST_measureSpacing);
 
             ww *= stretch;
-            if (ww < point(score()->style()->minMeasureWidth))
-                  ww = point(score()->style()->minMeasureWidth);
+            if (ww < point(styleS(ST_minMeasureWidth)))
+                  ww = point(styleS(ST_minMeasureWidth));
             if ((minWidth + ww) <= sysWidth)    // if another measure fits, do full layout
                   return false;
             }
@@ -1098,7 +1086,7 @@ bool ScoreLayout::doReLayout()
       double totalWeight = 0.0;
       foreach (MeasureBase* mb, system->measures()) {
             if (mb->type() == HBOX)
-                  minWidth += static_cast<Box*>(mb)->boxWidth().point();
+                  minWidth += point(static_cast<Box*>(mb)->boxWidth());
             else {
                   Measure* m   = static_cast<Measure*>(mb);
                   minWidth    += m->layoutWidth().stretchable;
@@ -1121,19 +1109,19 @@ bool ScoreLayout::doReLayout()
                   Measure* m    = static_cast<Measure*>(mb);
                   double weight = m->tickLen() * m->userStretch();
                   ww            = m->layoutWidth().stretchable + rest * weight;
-                  m->layout(this, ww);
+                  m->layout(ww);
                   }
             else if (mb->type() == HBOX) {
                   mb->setPos(pos);
-                  ww = static_cast<Box*>(mb)->boxWidth().point();
-                  mb->layout(this);
+                  ww = point(static_cast<Box*>(mb)->boxWidth());
+                  mb->layout();
                   }
             pos.rx() += ww;
             }
 
       foreach(MeasureBase* mb, system->measures()) {
             if (mb->type() == MEASURE)
-                  static_cast<Measure*>(mb)->layout2(this);
+                  static_cast<Measure*>(mb)->layout2();
             }
 
       rebuildBspTree();
@@ -1144,9 +1132,10 @@ bool ScoreLayout::doReLayout()
 //   clear
 //---------------------------------------------------------
 
-void ScoreLayout::clear()
+#if 0
+void Score::clear()
       {
       _pages.clear();
       _systems.clear();
       }
-
+#endif
