@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: layoutbreak.cpp,v 1.1 2006/03/27 14:16:24 wschweer Exp $
+//  $Id: layoutbreak.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -22,7 +22,6 @@
 #include "preferences.h"
 #include "score.h"
 #include "canvas.h"
-#include "layout.h"
 
 //---------------------------------------------------------
 //   LayoutBreak
@@ -62,9 +61,9 @@ void LayoutBreak::draw(QPainter& p) const
 //   layout
 //---------------------------------------------------------
 
-void LayoutBreak::layout(ScoreLayout* layout)
+void LayoutBreak::layout()
       {
-      double _spatium = layout->spatium();
+      double _spatium = spatium();
       path      = QPainterPath();
       lw        = _spatium * 0.3;
       double h  = _spatium * 4;
@@ -104,7 +103,7 @@ void LayoutBreak::layout(ScoreLayout* layout)
       QRectF bb(0, 0, w, h);
       bb.adjust(-lw, -lw, lw, lw);
       setbbox(bb);
-      Element::layout(layout);      // alignment & offset
+      Element::layout();      // alignment & offset
       }
 
 //---------------------------------------------------------
