@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: layoutbreak.h,v 1.1 2006/03/27 14:16:24 wschweer Exp $
+//  $Id: layoutbreak.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -37,7 +37,7 @@ class LayoutBreak : public Element {
       QPainterPath path;
 
       virtual void draw(QPainter&) const;
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
 
    public:
       LayoutBreak(Score*);
