@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: lilypond.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2007 Werner Schweer and others
 //
@@ -20,13 +20,13 @@
 
 #include "score.h"
 #include "element.h"
-#include "layout.h"
 #include "part.h"
 #include "staff.h"
 #include "note.h"
 #include "rest.h"
 #include "chord.h"
 #include "barline.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   LNote
@@ -263,8 +263,7 @@ void Lilypond::createMeasure()
       if (tick >= measure->tick() + measure->tickLen()) {
             measure = new Measure(score);
             measure->setTick(tick);
-            measure->setTickLen(division * 4);
-            score->layout()->add(measure);
+            score->add(measure);
             }
       }
 
@@ -294,7 +293,6 @@ void Lilypond::addNote(const LNote& lnote)
       note->setPitch(lnote.pitch);
       note->setParent(chord);
       note->setTrack(staff->idx() * VOICES);
-      note->setTickLen(lnote.len);
       chord->add(note);
       tick += lnote.len;
       }
@@ -390,8 +388,7 @@ void Lilypond::convert()
 
       measure = new Measure(score);
       measure->setTick(tick);
-      measure->setTickLen(division * 4);
-      score->layout()->add(measure);
+      score->add(measure);
 
       QChar c = lookup();
       if (c == '\\') {
