@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: line.cpp,v 1.4 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: line.cpp 1889 2009-06-21 17:00:53Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -24,7 +24,6 @@
 #include "measure.h"
 #include "score.h"
 #include "xml.h"
-#include "layout.h"
 #include "viewer.h"
 #include "system.h"
 
@@ -39,6 +38,17 @@ LineSegment::LineSegment(Score* s)
       _system = 0;
       }
 
+LineSegment::LineSegment(const LineSegment& s)
+   : Element(s)
+      {
+      _p2          = s._p2;
+      _userOff2    = s._userOff2;
+      r1           = s.r1;
+      r2           = s.r2;
+      _segmentType = s._segmentType;
+      _system      = s._system;
+      }
+
 //---------------------------------------------------------
 //   updateGrips
 //---------------------------------------------------------
@@ -46,11 +56,20 @@ LineSegment::LineSegment(Score* s)
 void LineSegment::updateGrips(int* grips, QRectF* grip) const
       {
       *grips = 2;
-      QPointF pp2(_p2 + _userOff2 * _spatium + canvasPos());
+      QPointF pp2(_p2 + _userOff2 + canvasPos());
       grip[1].translate(pp2);
       grip[0].translate(canvasPos());
       }
 
+//---------------------------------------------------------
+//   pos2
+//---------------------------------------------------------
+
+QPointF LineSegment::pos2() const
+      {
+      return _p2 + _userOff2;
+      }
+
 //---------------------------------------------------------
 //   gripAnchor
 //---------------------------------------------------------
@@ -61,29 +80,28 @@ QPointF LineSegment::gripAnchor(int grip) const
       QRectF sbb(_system->staff(staffIdx)->bbox());
       QPointF sp(sbb.topLeft() + _system->canvasPos());
 
+      int tck = _system->measures().front()->tick();
+
       System* s;  // dummy
-      QPointF p1(line()->tick2pos(grip, line()->tick(), staffIdx, &s));
-      QPointF p2(sbb.x() + sbb.width() + _system->canvasPos().x(), sp.y());
-      QPointF p3(line()->tick2pos(grip, line()->tick2(), staffIdx, &s));
 
       if (grip == 0) {
             switch(_segmentType) {
                   case SEGMENT_SINGLE:
                   case SEGMENT_BEGIN:
-                        return p1;
+                        return line()->tick2pos(grip, line()->tick(), staffIdx, &s);
                   case SEGMENT_MIDDLE:
                   case SEGMENT_END:
-                        return sp;
+                        return line()->tick2pos(grip, tck, staffIdx, &s);
                   }
             }
       else {
             switch(_segmentType) {
                   case SEGMENT_SINGLE:
                   case SEGMENT_END:
-                        return p3;
+                        return line()->tick2pos(grip, line()->tick2(), staffIdx, &s);
                   case SEGMENT_BEGIN:
                   case SEGMENT_MIDDLE:
-                        return p2;
+                        return QPointF(sbb.x() + sbb.width() + _system->canvasPos().x(), sp.y());
                   }
             }
       return QPointF();
@@ -114,15 +132,37 @@ bool LineSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modi
             int track = line()->track();
             int tick1 = line()->tick();
             int tick2 = line()->tick2();
+            bool snapSegment = line()->anchor() == ANCHOR_SEGMENT;
 
             if (key == Qt::Key_Left) {
                   if (curGrip == 0) {
-                        int t1 = score()->prevSeg1(tick1, track);
+                        int t1;
+                        if (snapSegment) {
+                              t1 = score()->prevSeg1(tick1, track);
+                              }
+                        else {
+                              Measure* m = score()->tick2measure(tick1)->prevMeasure();
+                              t1 = m ? m->tick() : 0;
+                              }
                         if (t1 >= 0)
                               tick1 = t1;
                         }
                   else if (curGrip == 1) {
-                        int t2 = score()->prevSeg1(tick2, track);
+                        int t2;
+                        if (snapSegment) {
+                              t2 = score()->prevSeg1(tick2, track);
+                              }
+                        else {
+                              Measure* m = score()->tick2measure(tick2);
+                              if (m->tick() == tick2) {
+                                    Measure* mm = m->prevMeasure();
+                                    t2 = mm ? mm->tick() : 0;
+                                    }
+                              else {
+                                    // at end of score?
+                                    t2 = m->tick();
+                                    }
+                              }
                         if (t2 >= 0)
                               tick2 = t2;
                         if (tick1 >= tick2)
@@ -131,56 +171,65 @@ bool LineSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modi
                   }
             else if (key == Qt::Key_Right) {
                   if (curGrip == 0) {
-                        int t1 = score()->nextSeg1(tick1, track);
+                        int t1;
+                        if (snapSegment) {
+                              t1 = score()->nextSeg1(tick1, track);
+                              }
+                        else {
+                              Measure* m  = score()->tick2measure(tick1);
+                              Measure* mb = m->nextMeasure();
+                              t1 = mb ? mb->tick() : m->tick() + m->tickLen();
+                              }
                         if (t1 >= 0)
                               tick1 = t1;
                         if (tick1 >= tick2)
                               return true;
                         }
                   else if (curGrip == 1) {
-                        int t2 = score()->nextSeg1(tick2, track);
-                        if (t2 >= 0)
+                        int t2 = 0;
+                        if (snapSegment) {
+                              t2 = score()->nextSeg1(tick2, track);
+                              }
+                        else {
+                              Measure* m  = score()->tick2measure(tick2);
+                              if (m) {
+                                    Measure* mb = m->nextMeasure();
+                                    t2 = mb ? mb->tick() : m->tick() + m->tickLen();
+                                    }
+                              }
+                        if (t2 > tick1)
                               tick2 = t2;
                         }
                   }
             line()->setTick(tick1);
             line()->setTick2(tick2);
 
-            line()->layout(score()->layout());
+            line()->layout();
             if (line()->lineSegments().size() != segments)
                   score()->changeLineSegment(curGrip == 1);
+            if (line()->type() == OTTAVA)
+                  score()->fixPpitch();
             return true;
             }
       return false;
       }
 
-//---------------------------------------------------------
-//   pos2anchor
-//---------------------------------------------------------
-
-QPointF LineSegment::pos2anchor(const QPointF& pos, int* tick) const
-      {
-      QPointF anchor;
-      score()->pos2TickAnchor(pos, staffIdx(), tick, &anchor);
-      return anchor;
-      }
-
 //---------------------------------------------------------
 //   editDrag
 //---------------------------------------------------------
 
 void LineSegment::editDrag(int curGrip, const QPointF& d)
       {
-      QPointF delta(d.x() / _spatium, 0);
+      QPointF delta(d.x(), 0);
 
       if (line()->type() == TEXTLINE) {
             TextLine* tl = static_cast<TextLine*>(line());
             if (tl->diagonal())
-                  delta.setY(d.y() / _spatium);
+                  delta.setY(d.y());
             }
 
       if (curGrip == 0) {
-            _userOff  += delta;
+            setUserOff(userOff() + delta);
             _userOff2 -= delta;
             }
       else
@@ -188,12 +237,22 @@ void LineSegment::editDrag(int curGrip, const QPointF& d)
       }
 
 //---------------------------------------------------------
-//   resetUserOffsets
+//   spatiumChanged
 //---------------------------------------------------------
 
-void LineSegment::resetUserOffsets()
+void LineSegment::spatiumChanged(double ov, double nv)
       {
-      Element::resetUserOffsets();
+      Element::spatiumChanged(ov, nv);
+      _userOff2 *= nv / ov;
+      }
+
+//---------------------------------------------------------
+//   toDefault
+//---------------------------------------------------------
+
+void LineSegment::toDefault()
+      {
+      Element::toDefault();
       setUserOff2(QPointF());
       }
 
@@ -207,6 +266,17 @@ SLine::SLine(Score* s)
       setTick(0);
       _tick2 = 0;
       _diagonal = false;
+      _anchor = ANCHOR_SEGMENT;
+      }
+
+SLine::SLine(const SLine& s)
+   : Element(s)
+      {
+      _tick2    = s._tick2;
+      _diagonal = s._diagonal;
+      _anchor   = s._anchor;
+      foreach(LineSegment* ls, s.segments)
+            add(ls->clone());
       }
 
 //---------------------------------------------------------
@@ -243,16 +313,37 @@ QPointF SLine::tick2pos(int grip, int tick, int staffIdx, System** system)
             }
       System* sys = seg->measure()->system();
 
-      // do not go into next system when tick2 is start of system
-      if (grip == 1 && sys->firstMeasure()->tick() == seg->tick()) {
-            Measure* m = sys->firstMeasure()->prevMeasure();
-            if (m) {
-                  seg = m->last();
-                  sys = seg->measure()->system();
+      if (_anchor == ANCHOR_SEGMENT) {
+            // do not go into next system when tick2 is start of system
+            if (grip == 1 && sys->firstMeasure()->tick() == seg->tick()) {
+                  Measure* m = sys->firstMeasure()->prevMeasure();
+                  if (m) {
+                        seg = m->last();
+                        sys = seg->measure()->system();
+                        }
                   }
+            *system = sys;
+            return QPointF(seg->canvasPos().x(), sys->staff(staffIdx)->bbox().y() + sys->canvasPos().y());
+            }
+      else {
+            Measure* m = seg->measure();
+            double y   = sys->staff(staffIdx)->bbox().y() + sys->canvasPos().y();
+            double x   = m->canvasPos().x();
+            if (m->tick() < seg->tick()) {      // to end of last measure?
+                  x += m->bbox().width();
+                  }
+            else if (grip == 1 && sys->firstMeasure() == m) {
+                  m = m->prevMeasure();
+                  if (m) {
+                        seg = m->last();
+                        sys = seg->measure()->system();
+                        x   = m->canvasPos().x() + m->bbox().width();
+                        y   = sys->staff(staffIdx)->bbox().y() + sys->canvasPos().y();
+                        }
+                  }
+            *system = sys;
+            return QPointF(x, y);
             }
-      *system     = sys;
-      return QPointF(seg->canvasPos().x(), sys->staff(staffIdx)->bbox().y() + sys->canvasPos().y());
       }
 
 //---------------------------------------------------------
@@ -260,9 +351,9 @@ QPointF SLine::tick2pos(int grip, int tick, int staffIdx, System** system)
 //    compute segments from tick1 tick2
 //---------------------------------------------------------
 
-void SLine::layout(ScoreLayout* layout)
+void SLine::layout()
       {
-      if (!parent()) {
+      if (score() == gscore) {
             //
             // when used in a palette, SLine has no parent and
             // tick and tick2 has no meaning so no layout is
@@ -270,7 +361,7 @@ void SLine::layout(ScoreLayout* layout)
             //
             if (!segments.isEmpty()) {
                   LineSegment* s = segments.front();
-                  s->layout(layout);
+                  s->layout();
                   setbbox(s->bbox());
                   }
             return;
@@ -284,7 +375,7 @@ void SLine::layout(ScoreLayout* layout)
       QPointF p1 = tick2pos(0, tick(), staffI, &s1);
       QPointF p2 = tick2pos(1, _tick2, staffI, &s2);
 
-      QList<System*>* systems = layout->systems();
+      QList<System*>* systems = score()->systems();
       int sysIdx1        = systems->indexOf(s1);
       int sysIdx2        = systems->indexOf(s2);
       int segmentsNeeded = sysIdx2 - sysIdx1 + 1;
@@ -347,31 +438,41 @@ void SLine::layout(ScoreLayout* layout)
                   seg->setPos(seg->gripAnchor(0));
                   seg->setXpos2(p2.x() - seg->ipos().x());
                   }
-            seg->layout(layout);
+            seg->layout();
             }
       }
 
 //---------------------------------------------------------
 //   writeProperties
+//    write properties different from prototype
 //---------------------------------------------------------
 
-void SLine::writeProperties(Xml& xml) const
+void SLine::writeProperties(Xml& xml, const SLine* proto) const
       {
-      Element::writeProperties(xml);
-      if (parent() == 0) {
+      Element::writeProperties(xml, proto);
+      if (_diagonal && (proto == 0 || proto->diagonal() != _diagonal))
+            xml.tag("diagonal", _diagonal);
+      if (_anchor != ANCHOR_SEGMENT && (proto == 0 || proto->anchor() != _anchor))
+            xml.tag("anchor", _anchor);
+      if (score() == gscore) {
             // when used as icon
-            LineSegment* s = segments.front();
-            xml.tag("length", s->pos2().x());
+            if (!segments.isEmpty()) {
+                  LineSegment* s = segments.front();
+                  xml.tag("length", s->pos2().x());
+                  }
+            else
+                  xml.tag("length", spatium() * 4);
             return;
             }
       xml.tag("tick2", _tick2);
-      xml.tag("diagonal", _diagonal);
       //
       // check if user has modified the default layout
       //
       bool modified = false;
       foreach(LineSegment* seg, segments) {
-            if (!seg->userOff().isNull() || !seg->userOff2().isNull()) {
+            if (!seg->userOff().isNull()
+               || !seg->userOff2().isNull()
+               || !seg->visible()) {
                   modified = true;
                   break;
                   }
@@ -384,8 +485,9 @@ void SLine::writeProperties(Xml& xml) const
       //
       foreach(LineSegment* seg, segments) {
             xml.stag("Segment");
-            xml.tag("off1", seg->userOff());
-            xml.tag("off2", seg->userOff2());
+            xml.tag("off1", seg->userOff() / spatium());
+            xml.tag("off2", seg->userOff2() / spatium());
+            seg->Element::writeProperties(xml);
             xml.etag();
             }
       }
@@ -396,10 +498,6 @@ void SLine::writeProperties(Xml& xml) const
 
 bool SLine::readProperties(QDomElement e)
       {
-      foreach(LineSegment* seg, segments)
-            delete seg;
-      segments.clear();
-
       if (Element::readProperties(e))
             return true;
       QString tag(e.tagName());
@@ -412,10 +510,10 @@ bool SLine::readProperties(QDomElement e)
             LineSegment* ls = createLineSegment();
             for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
                   if (e.tagName() == "off1")
-                        ls->setUserOff(readPoint(e));
+                        ls->setUserOff(readPoint(e) * spatium());
                   else if (e.tagName() == "off2")
-                        ls->setUserOff2(readPoint(e));
-                  else
+                        ls->setUserOff2(readPoint(e) * spatium());
+                  else if (!ls->Element::readProperties(e))
                         domError(e);
                   }
             add(ls);
@@ -426,6 +524,8 @@ bool SLine::readProperties(QDomElement e)
             setLen(val.toDouble());
       else if (tag == "diagonal")
             setDiagonal(i);
+      else if (tag == "anchor")
+            setAnchor(Anchor(i));
       else
             return false;
       return true;
@@ -445,22 +545,6 @@ void SLine::setLen(double l)
       s->setPos2(QPointF(l, 0));
       }
 
-//---------------------------------------------------------
-//   draw
-//---------------------------------------------------------
-
-void SLine::draw(QPainter& p) const
-      {
-      QList<const Element*> el;
-      collectElements(el);
-      foreach(const Element* e, el) {
-            p.save();
-            p.translate(e->pos());
-            e->draw(p);
-            p.restore();
-            }
-      }
-
 //---------------------------------------------------------
 //   collectElements
 //---------------------------------------------------------
@@ -536,6 +620,9 @@ void SLine::write(Xml& xml) const
 
 void SLine::read(QDomElement e)
       {
+      foreach(LineSegment* seg, segments)
+            delete seg;
+      segments.clear();
       setTrack(0);  // set default track
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             if (!SLine::readProperties(e))
@@ -543,4 +630,15 @@ void SLine::read(QDomElement e)
             }
       }
 
+//---------------------------------------------------------
+//   check
+//    check for consistency; return false if element is
+//    invalid
+//---------------------------------------------------------
 
+bool SLine::check() const
+      {
+      if (tick2() <= tick())
+            return false;
+      return true;
+      }
