@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: listedit.cpp,v 1.42 2006/09/15 09:34:57 wschweer Exp $
+//  $Id: listedit.cpp 1867 2009-06-08 13:33:29Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -39,7 +39,6 @@
 #include "hook.h"
 #include "dynamics.h"
 #include "slur.h"
-#include "layout.h"
 #include "lyrics.h"
 #include "volta.h"
 #include "line.h"
@@ -62,7 +61,7 @@ class ElementItem : public QTreeWidgetItem
 
    public:
       ElementItem(QTreeWidget* lv, Element* e);
-      ElementItem(ElementItem* ei, Element* e);
+      ElementItem(QTreeWidgetItem* ei, Element* e);
       Element* element() const { return el; }
       void init();
       };
@@ -74,7 +73,7 @@ ElementItem::ElementItem(QTreeWidget* lv, Element* e)
       init();
       }
 
-ElementItem::ElementItem(ElementItem* ei, Element* e)
+ElementItem::ElementItem(QTreeWidgetItem* ei, Element* e)
    : QTreeWidgetItem(ei, e->type())
       {
       el = e;
@@ -283,7 +282,9 @@ void PageListEditor::updateList(Score* s)
       {
       cs = s;
       list->clear();
-      ElementItem* li = new ElementItem(list, cs->layout());
+
+      QTreeWidgetItem* li = new QTreeWidgetItem(list, INVALID);
+      li->setText(0, "Global");
       foreach(Element* el, *cs->gel()) {
             if (el->type() == SLUR) {
                   ElementItem* se = new ElementItem(li, el);
@@ -291,9 +292,25 @@ void PageListEditor::updateList(Score* s)
                   foreach(Element* el1, *slur->slurSegments())
                         new ElementItem(se, el1);
                   }
+            else if (el->type() == VOLTA) {
+                  ElementItem* ei = new ElementItem(li, el);
+                  Volta* volta = static_cast<Volta*>(el);
+                  if (volta->beginText())
+                        new ElementItem(ei, volta->beginText());
+                  if (volta->continueText())
+                        new ElementItem(ei, volta->continueText());
+                  SLine* line = static_cast<SLine*>(el);
+                  foreach(LineSegment* ls, line->lineSegments()) {
+                        ElementItem* sse = new ElementItem(ei, ls);
+                        if (ls->type() == TEXTLINE_SEGMENT) {
+                              if (static_cast<TextLineSegment*>(ls)->text())
+                                    new ElementItem(sse, static_cast<TextLineSegment*>(ls)->text());
+                              }
+                        }
+                  }
             else if (el->isSLine()) {
                   ElementItem* se = new ElementItem(li, el);
-                  SLine* line = (SLine*)el;
+                  SLine* line = static_cast<SLine*>(el);
                   foreach(LineSegment* ls, line->lineSegments()) {
                         ElementItem* sse = new ElementItem(se, ls);
                         if (ls->type() == TEXTLINE_SEGMENT) {
@@ -308,7 +325,7 @@ void PageListEditor::updateList(Score* s)
 
       int staves = cs->nstaves();
       int tracks = staves * VOICES;
-      foreach(Page* page, cs->layout()->pages()) {
+      foreach(Page* page, cs->pages()) {
             ElementItem* pi = new ElementItem(list, page);
 
             if (page->copyright())
@@ -471,7 +488,9 @@ void PageListEditor::itemChanged(QTreeWidgetItem* i, QTreeWidgetItem*)
       {
       if (i == 0)
             return;
-      Element* el = ((ElementItem*)i)->element();
+      if (i->type() == INVALID)
+            return;
+      Element* el = static_cast<ElementItem*>(i)->element();
       setWindowTitle(QString("MuseScore: List Edit: ") + el->name());
       ShowElementBase* ew = 0;
       switch (el->type()) {
@@ -578,9 +597,29 @@ MeasureView::MeasureView()
       layout->addWidget(seg);
       layout->addStretch(10);
       connect(mb.sel, SIGNAL(itemClicked(QTreeWidgetItem*,int)), SLOT(elementClicked(QTreeWidgetItem*)));
+      connect(mb.nextButton, SIGNAL(clicked()), SLOT(nextClicked()));
+      connect(mb.prevButton, SIGNAL(clicked()), SLOT(prevClicked()));
       seg->show();
       }
 
+//---------------------------------------------------------
+//   nextClicked
+//---------------------------------------------------------
+
+void MeasureView::nextClicked()
+      {
+      emit elementChanged(((MeasureBase*)element())->next());
+      }
+
+//---------------------------------------------------------
+//   prevClicked
+//---------------------------------------------------------
+
+void MeasureView::prevClicked()
+      {
+      emit elementChanged(((MeasureBase*)element())->prev());
+      }
+
 //---------------------------------------------------------
 //   setElement
 //---------------------------------------------------------
@@ -611,6 +650,8 @@ void MeasureView::setElement(Element* e)
             item->setData(0, Qt::UserRole, QVariant::fromValue<void*>(p));
             mb.sel->addTopLevelItem(item);
             }
+      mb.prevButton->setEnabled(m->prev());
+      mb.nextButton->setEnabled(m->next());
       }
 
 //---------------------------------------------------------
@@ -848,6 +889,8 @@ void ShowNoteWidget::setElement(Element* e)
       ShowElementBase::setElement(e);
 
       nb.pitch->setValue(note->pitch());
+      nb.ppitch->setValue(note->ppitch());
+      nb.velo->setValue(note->velocity());
       nb.line->setValue(note->line());
       nb.move->setValue(note->staffMove());
       nb.mirror->setChecked(note->mirror());
@@ -1103,7 +1146,6 @@ TextView::TextView()
 
 void TextView::textChanged()
       {
-
       emit scoreChanged();
       }
 
@@ -1114,6 +1156,11 @@ void TextView::textChanged()
 void TextView::setElement(Element* e)
       {
       Text* te = (Text*)e;
+      tb.textStyle->clear();
+      foreach(const TextStyle* s, e->score()->textStyles()) {
+            tb.textStyle->addItem(s->name);
+            }
+
       ShowElementBase::setElement(e);
       tb.text->setDocument(te->doc()->clone());
       tb.xoffset->setValue(te->xoff());
@@ -1121,6 +1168,7 @@ void TextView::setElement(Element* e)
       tb.rxoffset->setValue(te->rxoff());
       tb.ryoffset->setValue(te->ryoff());
       tb.offsetType->setCurrentIndex(int(te->offsetType()));
+      tb.textStyle->setCurrentIndex(te->textStyle());
       }
 
 //---------------------------------------------------------
@@ -1556,6 +1604,24 @@ void VoltaView::segmentClicked(QTreeWidgetItem* item)
       emit elementChanged(e);
       }
 
+//---------------------------------------------------------
+//   beginTextClicked
+//---------------------------------------------------------
+
+void VoltaView::beginTextClicked()
+      {
+      emit elementChanged(static_cast<Volta*>(element())->beginText());
+      }
+
+//---------------------------------------------------------
+//   continueTextClicked
+//---------------------------------------------------------
+
+void VoltaView::continueTextClicked()
+      {
+      emit elementChanged(static_cast<Volta*>(element())->continueText());
+      }
+
 //---------------------------------------------------------
 //   VoltaView
 //---------------------------------------------------------
@@ -1563,11 +1629,20 @@ void VoltaView::segmentClicked(QTreeWidgetItem* item)
 VoltaView::VoltaView()
    : ShowElementBase()
       {
+      // SLineBase
       QWidget* w = new QWidget;
       lb.setupUi(w);
       layout->addWidget(w);
+
+      // TextLineBase
+      w = new QWidget;
+      tlb.setupUi(w);
+      layout->addWidget(w);
+
       layout->addStretch(10);
       connect(lb.segments, SIGNAL(itemClicked(QTreeWidgetItem*,int)), SLOT(segmentClicked(QTreeWidgetItem*)));
+      connect(tlb.beginText, SIGNAL(clicked()), SLOT(beginTextClicked()));
+      connect(tlb.continueText, SIGNAL(clicked()), SLOT(continueTextClicked()));
       }
 
 //---------------------------------------------------------
@@ -1579,7 +1654,11 @@ void VoltaView::setElement(Element* e)
       Volta* volta = (Volta*)e;
       ShowElementBase::setElement(e);
 
+      tlb.lineWidth->setValue(volta->lineWidth().val());
       lb.tick2->setValue(volta->tick2());
+      lb.anchor->setCurrentIndex(int(volta->anchor()));
+      lb.diagonal->setChecked(volta->diagonal());
+
       lb.segments->clear();
       const QList<LineSegment*>& el = volta->lineSegments();
       foreach(const Element* e, el) {
@@ -1588,6 +1667,8 @@ void VoltaView::setElement(Element* e)
             item->setData(0, Qt::UserRole, QVariant::fromValue<void*>((void*)e));
             lb.segments->addTopLevelItem(item);
             }
+      tlb.beginText->setEnabled(volta->beginText());
+      tlb.continueText->setEnabled(volta->continueText());
       }
 
 //---------------------------------------------------------
