@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: listedit.h,v 1.19 2006/03/30 07:32:34 wschweer Exp $
+//  $Id: listedit.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -36,6 +36,7 @@
 #include "ui_slurtie.h"
 #include "ui_slur.h"
 #include "ui_line.h"
+#include "ui_textline.h"
 #include "ui_linesegment.h"
 #include "ui_lyrics.h"
 
@@ -186,6 +187,8 @@ class MeasureView : public ShowElementBase {
 
    private slots:
       void elementClicked(QTreeWidgetItem* item);
+      void nextClicked();
+      void prevClicked();
 
    public:
       MeasureView();
@@ -462,10 +465,13 @@ class TieView : public ShowElementBase {
 class VoltaView : public ShowElementBase {
       Q_OBJECT;
 
+      Ui::TextLineBase tlb;
       Ui::SLineBase lb;
 
    private slots:
       void segmentClicked(QTreeWidgetItem* item);
+      void beginTextClicked();
+      void continueTextClicked();
 
    public:
       VoltaView();
