@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: measure.h,v 1.40 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: measure.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -95,7 +95,7 @@ class Measure : public MeasureBase {
       Text* _noText;          ///< Measure number text object
 
       double _userStretch;
-      bool _irregular;        ///< Irregular measure, do not count
+      bool _irregular;              ///< Irregular measure, do not count
       bool _breakMultiMeasureRest;  ///< set by user
       bool _breakMMRest;            ///< set by layout
 
@@ -109,9 +109,12 @@ class Measure : public MeasureBase {
       bool _endBarLineVisible;
       QColor _endBarLineColor;
 
+      mutable int _tickLen;   // cached value
+      mutable unsigned sigSerial;
+
       void push_back(Segment* e);
       void push_front(Segment* e);
-      void layoutBeams(ScoreLayout*);
+      void layoutBeams();
 
    public:
       Measure(Score*);
@@ -123,7 +126,6 @@ class Measure : public MeasureBase {
       virtual void write(Xml&, int, bool writeSystemElements) const;
       virtual void write(Xml&) const;
       void writeBox(Xml&) const;
-      virtual void read(QDomElement);
       void readBox(QDomElement);
       virtual bool startEdit(Viewer*, const QPointF&) { return false; }
 
@@ -139,19 +141,20 @@ class Measure : public MeasureBase {
       StaffLines* staffLines(int staffIdx) { return staves[staffIdx]->lines; }
       QList<Beam*>* beams()                { return &_beams;      }
       QList<Tuplet*>* tuplets()            { return &_tuplets;    }
-
-      int    no() const                { return _no;          }
-      bool   irregular() const         { return _irregular;   }
-      void   setIrregular(bool val)    { _irregular = val;    }
-      int    noOffset() const          { return _noOffset;    }
-      Text* noText() const             { return _noText;      }
-      void   setNo(int n)              { _no = n;             }
-      void   setNoOffset(int n)        { _noOffset = n;       }
-      virtual double distance(int i) const      { return staves[i]->distance; }
+      int no() const                       { return _no;          }
+      bool irregular() const               { return _irregular;   }
+      void setIrregular(bool val)          { _irregular = val;    }
+      int noOffset() const                 { return _noOffset;    }
+      Text* noText() const                 { return _noText;      }
+      void setNo(int n)                    { _no = n;             }
+      void setNoOffset(int n)              { _noOffset = n;       }
+      virtual double distance(int i) const { return staves[i]->distance; }
       virtual Spatium userDistance(int i) const;
 
       int size() const                 { return _size;       }
       virtual int tickLen() const;
+      virtual void setTick(int t);
+
       Segment* first() const           { return _first;      }
       Segment* last() const            { return _last;       }
       void remove(Segment*);
@@ -159,9 +162,9 @@ class Measure : public MeasureBase {
       double userStretch() const       { return _userStretch; }
       void setUserStretch(double v)    { _userStretch = v;  }
 
-      void layoutX(ScoreLayout*, double stretch);
-      void layout(ScoreLayout*, double width);
-      void layout2(ScoreLayout*);
+      void layoutX(double stretch);
+      void layout(double width);
+      void layout2();
 
       Chord* findChord(int tick, int track, bool grace);
       ChordRest* findChordRest(int tick, int track);
@@ -172,7 +175,7 @@ class Measure : public MeasureBase {
       void insertMStaff(MStaff* staff, int idx);
       void removeMStaff(MStaff* staff, int idx);
 
-      void layoutBeams1(ScoreLayout*);
+      void layoutBeams1();
       void layoutChords(Segment* segment, int startTrack, char* tversatz);
       void layout0(int staff);
 
