@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: measurebase.h 1845 2009-05-23 12:33:40Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -86,6 +86,7 @@ class MeasureBase : public Element {
       Measure* nextMeasure();
       Measure* prevMeasure();
 
+      virtual int tickLen() const            { return 0;       }
       virtual void write(Xml&, int, bool) const = 0;
 
       virtual void collectElements(QList<const Element*>& el) const;
@@ -104,6 +105,8 @@ class MeasureBase : public Element {
       virtual void add(Element*);
       virtual void remove(Element*);
       void setDirty()                        { _dirty = true; }
+      virtual void textStyleChanged(const QVector<TextStyle*>&);
+      virtual void spatiumChanged(double oldValue, double newValue);
       };
 
 #endif
