@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: menus.cpp,v 1.46 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: menus.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2008 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -57,6 +57,7 @@
 #include "chord.h"
 #include "drumset.h"
 #include "spacer.h"
+#include "measure.h"
 
 extern bool useFactorySettings;
 
@@ -70,6 +71,9 @@ void MuseScore::showPalette(bool visible)
       if (paletteBox == 0) {
             paletteBox = new PaletteBox(this);
 
+            connect(paletteBox, SIGNAL(paletteVisible(bool)), a, SLOT(setChecked(bool)));
+            addDockWidget(Qt::LeftDockWidgetArea, paletteBox);
+
             if (!useFactorySettings) {
                   QFile f(dataPath + "/" + "mscore-palette.xml");
                   if (f.exists()) {
@@ -81,9 +85,6 @@ void MuseScore::showPalette(bool visible)
                         }
                   }
 
-            connect(paletteBox, SIGNAL(paletteVisible(bool)), a, SLOT(setChecked(bool)));
-            addDockWidget(Qt::LeftDockWidgetArea, paletteBox);
-
             //-----------------------------------
             //    notes
             //-----------------------------------
@@ -140,12 +141,16 @@ void MuseScore::showPalette(bool visible)
             Palette* sp = new Palette;
             sp->setName(tr("Clefs"));
             sp->setMag(0.8);
-            sp->setGrid(33, 65);
+            sp->setGrid(33, 60);
             sp->setYOffset(1.0);
+            int clefs[20] = {
+                  0, 1, 2, 3, 16, 9, 10, 11, 12, 15, 4, 17, 18, 5, 6, 7, 8, 14, 13, CLEF_PERC2
+                  };
 
-            for (int i = 0; i < CLEF_MAX; ++i) {
-                  Clef* k = new ::Clef(gscore, i);
-                  sp->append(k, tr(clefTable[i].name));
+            for (int i = 0; i < 20; ++i) {
+                  int j = clefs[i];
+                  Clef* k = new ::Clef(gscore, j);
+                  sp->append(k, qApp->translate("clefTable", clefTable[j].name));
                   }
             paletteBox->addPalette(sp);
 
@@ -162,16 +167,16 @@ void MuseScore::showPalette(bool visible)
             for (int i = 0; i < 7; ++i) {
                   KeySig* k = new KeySig(gscore);
                   k->setSubtype(i+1);
-                  sp->append(k, tr(keyNames[i*2]));
+                  sp->append(k, qApp->translate("MuseScore", keyNames[i*2]));
                   }
             for (int i = -7; i < 0; ++i) {
                   KeySig* k = new KeySig(gscore);
                   k->setSubtype(i & 0xff);
-                  sp->append(k, tr(keyNames[(7 + i) * 2 + 1]));
+                  sp->append(k, qApp->translate("MuseScore", keyNames[(7 + i) * 2 + 1]));
                   }
             KeySig* k = new KeySig(gscore);
             k->setSubtype(0);
-            sp->append(k, tr(keyNames[14]));
+            sp->append(k, qApp->translate("MuseScore", keyNames[14]));
             paletteBox->addPalette(sp);
 
             //-----------------------------------
@@ -208,20 +213,20 @@ void MuseScore::showPalette(bool visible)
 
             struct {
                   BarType type;
-                  QString name;
+                  const char* name;
                   } t[] = {
-                  { NORMAL_BAR,       "normal" },
-                  { BROKEN_BAR,       "broken" },
-                  { END_BAR,          "End Bar" },
-                  { DOUBLE_BAR,       "Double Bar" },
-                  { START_REPEAT,     "Start Repeat" },
-                  { END_REPEAT,       "End Repeat" },
-                  { END_START_REPEAT, "End-Start Repeat" },
+                  { NORMAL_BAR,       QT_TR_NOOP("normal") },
+                  { BROKEN_BAR,       QT_TR_NOOP("broken") },
+                  { END_BAR,          QT_TR_NOOP("End Bar") },
+                  { DOUBLE_BAR,       QT_TR_NOOP("Double Bar") },
+                  { START_REPEAT,     QT_TR_NOOP("Start Repeat") },
+                  { END_REPEAT,       QT_TR_NOOP("End Repeat") },
+                  { END_START_REPEAT, QT_TR_NOOP("End-Start Repeat") },
                   };
             for (unsigned i = 0; i < sizeof(t)/sizeof(*t); ++i) {
                   BarLine* b  = new BarLine(gscore);
                   b->setSubtype(t[i].type);
-                  sp->append(b, t[i].name);
+                  sp->append(b, tr(t[i].name));
                   }
             paletteBox->addPalette(sp);
 
@@ -234,6 +239,10 @@ void MuseScore::showPalette(bool visible)
             sp->setMag(.8);
             sp->setGrid(84, 23);
 
+            Slur* slur = new Slur(gscore);
+            slur->setId(0);
+            sp->append(slur, tr("slur"));
+
             Hairpin* gabel0 = new Hairpin(gscore);
             gabel0->setSubtype(0);
             sp->append(gabel0, tr("crescendo"));
@@ -298,7 +307,6 @@ void MuseScore::showPalette(bool visible)
 
             TextLine* textLine = new TextLine(gscore);
             textLine->setBeginText("VII");
-            textLine->setHasBeginText(true);
             textLine->setEndHook(true);
             textLine->setEndHookHeight(Spatium(1.5));
             sp->append(textLine, tr("text line"));
@@ -363,8 +371,8 @@ void MuseScore::showPalette(bool visible)
             // b1->setHeight(_spatium * 7);
             // b2->setHeight(_spatium * 7);
 
-            sp->append(b1, "Bracket");
-            sp->append(b2, "Akkolade");
+            sp->append(b1, tr("Bracket"));
+            sp->append(b2, tr("Akkolade"));
 
             paletteBox->addPalette(sp);
 
@@ -372,7 +380,7 @@ void MuseScore::showPalette(bool visible)
             //    Attributes, Ornaments
             //-----------------------------------
 
-            unsigned nn = NOTE_ATTRIBUTES;
+            unsigned nn = ARTICULATIONS;
             sp = new Palette;
             sp->setName(tr("Articulations, Ornaments"));
             sp->setGrid(42, 25);
@@ -410,9 +418,9 @@ void MuseScore::showPalette(bool visible)
 
             sp = new Palette;
             sp->setName(tr("Dynamics"));
-            sp->setMag(0.9);
+            sp->setMag(.8);
             sp->setGrid(42, 28);
-            sp->setYOffset(-12.0);
+//            sp->setYOffset(-12.0);
 
             static const char* dynS[] = {
                   "ppp", "pp", "p", "mp", "mf", "f", "ff", "fff"
@@ -438,6 +446,7 @@ void MuseScore::showPalette(bool visible)
             for (unsigned i = 0; i < strlen(finger); ++i) {
                   Text* k = new Text(gscore);
                   k->setSubtype(TEXT_FINGERING);
+                  k->setTextStyle(TEXT_STYLE_FINGERING);
                   k->setText(QString(finger[i]));
                   sp->append(k, tr("fingering %1").arg(finger[i]));
                   }
@@ -445,6 +454,7 @@ void MuseScore::showPalette(bool visible)
             for (unsigned i = 0; i < strlen(stringnumber); ++i) {
                   Text* k = new Text(gscore);
                   k->setSubtype(TEXT_STRING_NUMBER);
+                  k->setTextStyle(TEXT_STYLE_STRING_NUMBER);
                   k->setText(QString(stringnumber[i]));
                   sp->append(k, tr("string number %1").arg(stringnumber[i]));
                   }
@@ -463,31 +473,31 @@ void MuseScore::showPalette(bool visible)
 
             NoteHead* nh = new NoteHead(gscore);
             nh->setSym(halfheadSym);
-            sp->append(nh, QString("normal"));
+            sp->append(nh, tr("normal", "note head"));
 
             nh = new NoteHead(gscore);
             nh->setSym(halfcrossedheadSym);
-            sp->append(nh, QString("crossed"));
+            sp->append(nh, tr("crossed", "note head"));
 
             nh = new NoteHead(gscore);
             nh->setSym(halfdiamondheadSym);
-            sp->append(nh, QString("diamond"));
+            sp->append(nh, tr("diamond", "note head"));
 
             nh = new NoteHead(gscore);
             nh->setSym(halftriangleheadSym);
-            sp->append(nh, QString("triangle"));
+            sp->append(nh, tr("triangle", "note head"));
 
             nh = new NoteHead(gscore);
             nh->setSym(halfdiamond2headSym);
-            sp->append(nh, QString("diamond2"));
+            sp->append(nh, tr("diamond2", "note head"));
 
             nh = new NoteHead(gscore);
             nh->setSym(halfslashheadSym);
-            sp->append(nh, QString("slash"));
+            sp->append(nh, tr("slash", "note head"));
 
             nh = new NoteHead(gscore);
             nh->setSym(xcircledheadSym);
-            sp->append(nh, QString("xcircle"));
+            sp->append(nh, tr("xcircle", "note head"));
 
             paletteBox->addPalette(sp);
 
@@ -500,14 +510,18 @@ void MuseScore::showPalette(bool visible)
             sp->setGrid(27, 40);
             sp->setDrawGrid(true);
             const char* tremoloName[] = {
-                  "1 through stem", "2 through stem", "3 through stem",
-                  "1 between notes", "2 between notes", "3 between notes"
+                  QT_TR_NOOP("1 through stem"),
+                  QT_TR_NOOP("2 through stem"),
+                  QT_TR_NOOP("3 through stem"),
+                  QT_TR_NOOP("1 between notes"),
+                  QT_TR_NOOP("2 between notes"),
+                  QT_TR_NOOP("3 between notes")
                   };
 
             for (int i = 0; i < 6; ++i) {
                   Tremolo* tremolo = new Tremolo(gscore);
                   tremolo->setSubtype(i);
-                  sp->append(tremolo, tremoloName[i]);
+                  sp->append(tremolo, tr(tremoloName[i]));
                   }
             paletteBox->addPalette(sp);
 
@@ -733,9 +747,9 @@ void MuseScore::clefMenu()
             clefPalette->setRestrictHeight(false);
             clefPalette->setWindowTitle(tr("MuseScore: Clefs"));
             sp->setGrid(60, 80);
-            for (int i = 0; i < 16; ++i) {
+            for (int i = 0; i < CLEF_MAX; ++i) {
                   Clef* k = new ::Clef(gscore, i);
-                  sp->append(k, tr(clefTable[i].name));
+                  sp->append(k, qApp->translate("clefTable", clefTable[i].name));
                   }
             }
       clefPalette->show();
@@ -758,16 +772,16 @@ void MuseScore::keyMenu()
             for (int i = 0; i < 7; ++i) {
                   KeySig* k = new KeySig(gscore);
                   k->setSubtype(i+1);
-                  sp->append(k, tr(keyNames[i*2]));
+                  sp->append(k, qApp->translate("MuseScore", keyNames[i*2]));
                   }
             for (int i = -7; i < 0; ++i) {
                   KeySig* k = new KeySig(gscore);
                   k->setSubtype(i & 0xff);
-                  sp->append(k, tr(keyNames[(7 + i) * 2 + 1]));
+                  sp->append(k, qApp->translate("MuseScore", keyNames[(7 + i) * 2 + 1]));
                   }
             KeySig* k = new KeySig(gscore);
             k->setSubtype(0);
-            sp->append(k, tr(keyNames[14]));
+            sp->append(k, qApp->translate("MuseScore", keyNames[14]));
             }
       keyPalette->show();
       keyPalette->raise();
@@ -799,7 +813,7 @@ void MuseScore::lineMenu()
             linePalette->setWindowTitle(tr("MuseScore: Lines"));
             sp->setGrid(100, 30);
 
-            double l = _spatium * 8;
+            double l = gscore->spatium() * 8;
 
             Hairpin* gabel0 = new Hairpin(gscore);
             gabel0->setSubtype(0);
@@ -883,7 +897,6 @@ void MuseScore::lineMenu()
 
             TextLine* textLine = new TextLine(gscore);
             textLine->setBeginText("VII");
-            textLine->setHasBeginText(true);
             sp->append(textLine, tr("text line"));
             textLine->setEndHook(true);
             textLine->setEndHookHeight(Spatium(1.5));
@@ -909,6 +922,7 @@ void MuseScore::bracketMenu()
             bracketPalette->setWindowTitle(tr("MuseScore: System Brackets"));
             sp->setGrid(40, 80);
 
+            double _spatium = gscore->spatium();
             Bracket* b1 = new Bracket(gscore);
             b1->setSubtype(BRACKET_NORMAL);
             Bracket* b2 = new Bracket(gscore);
@@ -936,7 +950,7 @@ void MuseScore::noteAttributesMenu()
             noteAttributesPalette = new PaletteScrollArea(sp);
             noteAttributesPalette->setRestrictHeight(false);
             noteAttributesPalette->setWindowTitle(tr("MuseScore: Note Attributes"));
-            unsigned nn = NOTE_ATTRIBUTES;
+            unsigned nn = ARTICULATIONS;
             sp->setGrid(42, 30);
 
             for (unsigned i = 0; i < nn; ++i) {
@@ -985,6 +999,7 @@ void MuseScore::dynamicsMenu()
             dynamicsPalette->setRestrictHeight(false);
             dynamicsPalette->setWindowTitle(tr("MuseScore: Dynamics"));
             sp->setGrid(90, 40);
+            sp->resize(300, 200);
 
             for (int i = 0; i < 27; ++i) {
                   Dynamic* dynamic = new Dynamic(gscore);
@@ -1022,21 +1037,21 @@ void MuseScore::barMenu()
 
             struct {
                   BarType type;
-                  QString name;
+                  const char* name;
                   } t[] = {
-                  { BROKEN_BAR,    "broken" },
-                  { NORMAL_BAR,    "normal" },
-                  { END_BAR,       "End Bar" },
-                  { DOUBLE_BAR,    "Double Bar" },
-                  { START_REPEAT,  "Start Repeat" },
-                  { END_REPEAT,    "End Repeat" },
-                  { END_START_REPEAT, "End-Start Repeat" },
+                  { BROKEN_BAR,       QT_TR_NOOP("broken") },
+                  { NORMAL_BAR,       QT_TR_NOOP("normal") },
+                  { END_BAR,          QT_TR_NOOP("End Bar") },
+                  { DOUBLE_BAR,       QT_TR_NOOP("Double Bar") },
+                  { START_REPEAT,     QT_TR_NOOP("Start Repeat") },
+                  { END_REPEAT,       QT_TR_NOOP("End Repeat") },
+                  { END_START_REPEAT, QT_TR_NOOP("End-Start Repeat") },
                   };
             for (unsigned i = 0; i < sizeof(t)/sizeof(*t); ++i) {
                   BarLine* b  = new BarLine(gscore);
-                  b->setHeight(point(Spatium(4)));
+                  b->setHeight(4 * gscore->spatium());
                   b->setSubtype(t[i].type);
-                  sp->append(b, t[i].name);
+                  sp->append(b, tr(t[i].name));
                   }
             }
       barPalette->show();
@@ -1052,10 +1067,11 @@ void MuseScore::fingeringMenu()
       if (fingeringPalette == 0) {
             Palette* sp = new Palette;
             sp->setMag(1.5);
+            sp->resize(300, 200);
             fingeringPalette = new PaletteScrollArea(sp);
             fingeringPalette->setRestrictHeight(false);
             fingeringPalette->setWindowTitle(tr("MuseScore: Fingering"));
-            sp->setGrid(50, 50);
+            sp->setGrid(28, 30);
 
             const char finger[] = "012345pimac";
             Text* k;
@@ -1066,6 +1082,13 @@ void MuseScore::fingeringMenu()
                   k->setText(QString(finger[i]));
                   sp->append(k, QString("fingering %1").arg(finger[i]));
                   }
+            const char stringnumber[] = "0123456";
+            for (unsigned i = 0; i < strlen(stringnumber); ++i) {
+                  Text* k = new Text(gscore);
+                  k->setSubtype(TEXT_STRING_NUMBER);
+                  k->setText(QString(stringnumber[i]));
+                  sp->append(k, tr("string number %1").arg(stringnumber[i]));
+                  }
             }
       fingeringPalette->show();
       fingeringPalette->raise();
@@ -1140,8 +1163,9 @@ void MuseScore::updateDrumset()
       if (cs == 0 || paletteBox == 0 || drumPalette == 0)
             return;
 
-      PadState* padState = cs->padState();
-      Drumset* ds        = padState->drumset;
+      double _spatium = gscore->spatium();
+      const InputState& padState = cs->inputState();
+      Drumset* ds        = padState.drumset;
       if (ds != drumset) {
             drumset = ds;
             drumPalette->clear();
@@ -1181,7 +1205,7 @@ void MuseScore::updateDrumset()
                         note->setHeadGroup(noteHead);
                         chord->add(note);
                         Stem* stem = new Stem(gscore);
-                        stem->setLen(Spatium(up ? -3.0 : 3.0));
+                        stem->setLen((up ? -3.0 : 3.0) * _spatium);
                         chord->setStem(stem);
                         stem->setPos(note->stemPos(up));
                         drumPalette->append(chord, drumset->name(pitch));
@@ -1194,7 +1218,7 @@ void MuseScore::updateDrumset()
             drumPalette->setSelected(-1);
             for (int pitch = 0; pitch < 128; ++pitch) {
                   if (drumset->isValid(pitch)) {
-                        if (pitch == padState->drumNote) {
+                        if (pitch == padState.drumNote) {
                               drumPalette->setSelected(i);
                               break;
                               }
@@ -1213,8 +1237,8 @@ void MuseScore::drumPaletteSelected(int idx)
       {
       if (cs == 0)
             return;
-      PadState* padState = cs->padState();
-      Drumset* ds        = padState->drumset;
+      InputState& padState = cs->inputState();
+      Drumset* ds        = padState.drumset;
       if (ds == 0)
             return;
       int i = 0;
@@ -1222,8 +1246,8 @@ void MuseScore::drumPaletteSelected(int idx)
             if (!drumset->isValid(pitch))
                   continue;
             if (i == idx) {
-                  padState->drumNote = pitch;
-                  padState->voice    = ds->voice(pitch);
+                  padState.drumNote = pitch;
+                  padState.track    = (padState.track / VOICES) * VOICES + ds->voice(pitch);
                   cs->setPadState();
                   break;
                   }
