@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: musedata.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2007 Werner Schweer and others
 //
@@ -20,7 +20,6 @@
 
 #include "musedata.h"
 #include "score.h"
-#include "layout.h"
 #include "part.h"
 #include "staff.h"
 #include "barline.h"
@@ -36,6 +35,8 @@
 #include "dynamics.h"
 #include "lyrics.h"
 #include "articulation.h"
+#include "sig.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   musicalAttribute
@@ -149,7 +150,7 @@ void MuseData::openSlur(int idx, int tick, Staff* staff, int voice)
       slur[idx] = new Slur(score);
       slur[idx]->setStart(tick, staffIdx * VOICES + voice);
       slur[idx]->setTrack(staffIdx * VOICES + voice);
-      slur[idx]->setParent(score->layout());
+//      slur[idx]->setParent(score->layout());
       score->addElement(slur[idx]);
       }
 
@@ -485,8 +486,7 @@ void MuseData::readBackup(const QString& s)
 
 Measure* MuseData::createMeasure()
       {
-      ScoreLayout* la = score->layout();
-      for (MeasureBase* mb = la->first(); mb; mb = mb->next()) {
+      for (MeasureBase* mb = score->first(); mb; mb = mb->next()) {
             if (mb->type() != MEASURE)
                   continue;
             Measure* m = (Measure*)mb;
@@ -500,7 +500,6 @@ Measure* MuseData::createMeasure()
                   score->sigmap->timesig(st, z, n);
                   score->sigmap->add(st, curTick - st, z, n);
                   score->sigmap->add(curTick, z, n);
-                  m->setTickLen(curTick - st);
                   break;
                   }
             if (curTick < st + l) {
@@ -520,7 +519,7 @@ Measure* MuseData::createMeasure()
       	      }
             }
 #endif
-      la->add(measure);
+      score->add(measure);
       return measure;
       }
 
