@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: musicxml.h,v 1.11 2006/04/05 08:15:12 wschweer Exp $
+//  $Id: musicxml.h 1915 2009-07-06 11:05:11Z lasconic $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -70,6 +70,50 @@ const int MAX_PART_GROUPS = 8;
 const int MAX_SLURS       = 8;
 const int MAX_BRACKETS    = 8;
 
+//---------------------------------------------------------
+//   CreditWords
+//    a single parsed MusicXML credit-words element
+//---------------------------------------------------------
+
+struct CreditWords {
+      double defaultX;
+      double defaultY;
+      QString justify;
+      QString hAlign;
+      QString vAlign;
+      QString words;
+      CreditWords(double a, double b, QString c, QString d, QString e, QString f) {
+            defaultX = a;
+            defaultY = b;
+            justify  = c;
+            hAlign   = d;
+            vAlign   = e;
+            words    = f;
+            }
+      };
+
+typedef  QList<CreditWords*> CreditWordsList;
+typedef  CreditWordsList::iterator iCreditWords;
+typedef  CreditWordsList::const_iterator ciCreditWords;
+
+//---------------------------------------------------------
+//   MusicXml
+//---------------------------------------------------------
+
+/**
+ The MusicXML "creator" meta-data element.
+*/
+
+class MusicXmlCreator {
+      QString _type;
+      QString _text;
+
+   public:
+      MusicXmlCreator(QString& tp, QString& txt) { _type = tp; _text = txt; }
+      QString crType() const                     { return _type; }
+      QString crText() const                     { return _text; }
+      };
+
 //---------------------------------------------------------
 //   MusicXml
 //---------------------------------------------------------
@@ -95,6 +139,8 @@ class MusicXml {
       int tick;         ///< Current position in MusicXML time
       int maxtick;      ///< Maxtick of a measure, used to calculate measure len
       int lastMeasureLen;
+      unsigned int multiMeasureRestCount;       ///< Remaining measures in a multi measure rest
+      bool startMultiMeasureRest;               ///< Multi measure rest started in this measure
 
       int lastLen;      ///< Needed for chords
       int maxLyrics;
@@ -105,6 +151,7 @@ class MusicXml {
       QString composer;
       QString poet;
       QString translator;
+      CreditWordsList credits;
 
       std::vector<MusicXmlWedge> wedgeList;
       std::vector<MusicXmlPartGroup*> partGroupList;
@@ -117,7 +164,7 @@ class MusicXml {
 
       void addWedge(int no, int startPos, qreal rx, qreal ry, bool above, int subType);
       void genWedge(int no, int endPos, Measure*, int staff);
-
+      void doCredits();
       void direction(Measure* measure, int staff, QDomElement node);
       void scorePartwise(QDomElement);
       void xmlPartList(QDomElement);
@@ -144,6 +191,13 @@ struct XmlChordExtension {
       const char* xmlName;
       };
 
+enum {
+      NoSystem          = 0,
+      TopSystem         = 1,
+      NewSystem         = 2,
+      NewPage           = 3
+      };
+
 extern const XmlChordExtension chordExtensions[];
 
 #endif
