@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: navigate.cpp,v 1.34 2006/03/02 17:08:37 wschweer Exp $
+//  $Id: navigate.cpp 1851 2009-05-25 18:03:41Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -33,7 +33,6 @@
 #include "input.h"
 #include "measure.h"
 #include "page.h"
-#include "layout.h"
 
 //---------------------------------------------------------
 //   nextChordRest
@@ -45,6 +44,7 @@ ChordRest* nextChordRest(ChordRest* cr)
       {
       if (!cr)
             return 0;
+      int tick = cr->tick() + cr->ticks();
       Segment* seg = cr->segment();
       int track = cr->track();
 
@@ -55,8 +55,12 @@ ChordRest* nextChordRest(ChordRest* cr)
             if (seg->measure()->multiMeasure() < 0)
                   continue;
             Element* e = seg->element(track);
-            if (e && e->isChordRest())
-                  return static_cast<ChordRest*>(e);
+            if (e && e->isChordRest() && e->tick()) {
+                  if (e->tick() == tick)
+                        return static_cast<ChordRest*>(e);
+                  else if (e->tick() > tick)
+                        return 0;
+                  }
             }
       return 0;
       }
@@ -277,13 +281,13 @@ ChordRest* Score::nextMeasure(ChordRest* element, bool selectBehavior)
       int endTick = element->measure()->last()->nextChordRest(element->track(), true)->tick();
       bool last = false;
 
-      bool range = sel->state() == SEL_STAFF;
+      bool range = selection()->state() == SEL_STAFF;
       if (range) {
-            if (element->tick() != endTick && sel->endSegment()->tick() <= endTick) {
+            if (element->tick() != endTick && selection()->endSegment()->tick() <= endTick) {
                   measure = element->measure();
                   last = true;
                   }
-            else if (element->tick() == endTick && sel->isEndActive())
+            else if (element->tick() == endTick && selection()->isEndActive())
                   last = true;
             }
       else if (element->tick() != endTick && selectBehavior) {
@@ -327,8 +331,8 @@ ChordRest* Score::prevMeasure(ChordRest* element)
       int startTick = element->measure()->first()->nextChordRest(element->track())->tick();
       bool last = false;
 
-      bool range = sel->state() == SEL_STAFF;
-      if (range && sel->isEndActive() && sel->startSegment()->tick() <= startTick)
+      bool range = selection()->state() == SEL_STAFF;
+      if (range && selection()->isEndActive() && selection()->startSegment()->tick() <= startTick)
             last = true;
       else if (element->tick() != startTick) {
             measure = element->measure();
@@ -372,6 +376,8 @@ void Score::adjustCanvasPosition(Element* el, bool playBack)
             m = static_cast<Lyrics*>(el)->measure();
       else if (el->type() == HARMONY)
             m = static_cast<Harmony*>(el)->measure();
+      else if (el->type() == MEASURE)
+            m = static_cast<Measure*>(el);
       else
             return;
 
@@ -476,9 +482,9 @@ void Score::adjustCanvasPosition(Element* el, bool playBack)
 
 void Score::pageNext()
       {
-      if (_layout->pages().empty())
+      if (pages().empty())
             return;
-      Page* page = _layout->pages().back();
+      Page* page = pages().back();
       qreal mag  = canvas()->xMag() * DPI / PDPI;
       qreal x    = canvas()->xoffset() - (page->width() + 25.0) * mag;
       qreal lx   = 10.0 - page->canvasPos().x() * mag;
@@ -495,9 +501,9 @@ void Score::pageNext()
 
 void Score::pagePrev()
       {
-      if (_layout->pages().empty())
+      if (pages().empty())
             return;
-      Page* page = _layout->pages().back();
+      Page* page = pages().back();
       qreal mag  = canvas()->xMag() * DPI / PDPI;
       qreal x = canvas()->xoffset() +( page->width() + 25.0) * mag;
       if (x > 10.0)
@@ -524,9 +530,9 @@ void Score::pageTop()
 
 void Score::pageEnd()
       {
-      if (_layout->pages().empty())
+      if (pages().empty())
             return;
-      Page* lastPage = _layout->pages().back();
+      Page* lastPage = pages().back();
       QPointF p(lastPage->canvasPos());
       qreal mag  = canvas()->xMag() * DPI / PDPI;
       canvas()->setOffset(25.0 - p.x() * mag, 25.0);
