@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: part.cpp,v 1.14 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: newwizard.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -23,12 +23,12 @@
 #include "instrtemplate.h"
 #include "score.h"
 #include "staff.h"
-#include "layout.h"
 #include "clef.h"
 #include "part.h"
 #include "drumset.h"
 #include "palette.h"
 #include "keysig.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   InstrumentWizard
@@ -41,7 +41,7 @@ InstrumentWizard::InstrumentWizard(QWidget* parent)
       instrumentList->setSelectionMode(QAbstractItemView::SingleSelection);
       partiturList->setSelectionMode(QAbstractItemView::SingleSelection);
 
-      instrumentList->setHeaderLabels(QStringList("Instrument List"));
+      instrumentList->setHeaderLabels(QStringList(tr("Instrument List")));
       QStringList header = (QStringList() << tr("Staves") << tr("Clef"));
       partiturList->setHeaderLabels(header);
 
@@ -65,6 +65,16 @@ InstrumentWizard::InstrumentWizard(QWidget* parent)
       belowButton->setEnabled(false);
       }
 
+//---------------------------------------------------------
+//   init
+//---------------------------------------------------------
+
+void InstrumentWizard::init()
+      {
+      partiturList->clear();
+      emit completeChanged(false);
+      }
+
 //---------------------------------------------------------
 //   on_instrumentList_itemSelectionChanged
 //---------------------------------------------------------
@@ -192,6 +202,7 @@ void InstrumentWizard::on_upButton_clicked()
             bool isExpanded = partiturList->isItemExpanded(item);
             int idx = partiturList->indexOfTopLevelItem(item);
             if (idx) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = partiturList->takeTopLevelItem(idx);
                   partiturList->insertTopLevelItem(idx-1, item);
                   partiturList->setItemExpanded(item, isExpanded);
@@ -202,6 +213,7 @@ void InstrumentWizard::on_upButton_clicked()
             QTreeWidgetItem* parent = item->parent();
             int idx = parent->indexOfChild(item);
             if (idx) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = parent->takeChild(idx);
                   parent->insertChild(idx-1, item);
                   partiturList->setItemSelected(item, true);
@@ -225,6 +237,7 @@ void InstrumentWizard::on_downButton_clicked()
             int idx = partiturList->indexOfTopLevelItem(item);
             int n = partiturList->topLevelItemCount();
             if (idx < (n-1)) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = partiturList->takeTopLevelItem(idx);
                   partiturList->insertTopLevelItem(idx+1, item);
                   partiturList->setItemExpanded(item, isExpanded);
@@ -236,6 +249,7 @@ void InstrumentWizard::on_downButton_clicked()
             int idx = parent->indexOfChild(item);
             int n = parent->childCount();
             if (idx < (n-1)) {
+                  partiturList->selectionModel()->clear();
                   QTreeWidgetItem* item = parent->takeChild(idx);
                   parent->insertChild(idx+1, item);
                   partiturList->setItemSelected(item, true);
@@ -338,6 +352,7 @@ void InstrumentWizard::createInstruments(Score* cs)
                   part->staves()->push_back(staff);
                   cs->staves().insert(staffIdx + rstaff, staff);
                   }
+            part->staves()->front()->setBarLineSpan(part->nstaves());
 
             // insert part
             cs->insertPart(part, staffIdx);
@@ -438,6 +453,19 @@ NewWizardPage1::NewWizardPage1(QWidget* parent)
       setLayout(grid);
       }
 
+//---------------------------------------------------------
+//   initializePage
+//---------------------------------------------------------
+
+void NewWizardPage1::initializePage()
+      {
+      w->title->setText("");
+      w->subtitle->setText("");
+      // w->composer->text();
+      // w->poet->text();
+      // w->copyright->text();
+      }
+
 //---------------------------------------------------------
 //   NewWizardPage2
 //---------------------------------------------------------
@@ -459,6 +487,15 @@ NewWizardPage2::NewWizardPage2(QWidget* parent)
       connect(w, SIGNAL(completeChanged(bool)), this, SLOT(setComplete(bool)));
       }
 
+//---------------------------------------------------------
+//   initializePage
+//---------------------------------------------------------
+
+void NewWizardPage2::initializePage()
+      {
+      w->init();
+      }
+
 //---------------------------------------------------------
 //   setComplete
 //---------------------------------------------------------
@@ -496,7 +533,7 @@ NewWizardPage4::NewWizardPage4(QWidget* parent)
 
       QStringList nameFilter;
       nameFilter.append("*.mscz");
-      nameFilter.append("*.msc");
+      nameFilter.append("*.mscx");
 
       model = new QDirModel;
       model->setNameFilters(nameFilter);
@@ -553,16 +590,16 @@ NewWizardPage5::NewWizardPage5(QWidget* parent)
       for (int i = 0; i < 7; ++i) {
             KeySig* k = new KeySig(gscore);
             k->setSubtype(i+1);
-            sp->append(k, tr(keyNames[i*2]));
+            sp->append(k, qApp->translate("MuseScore", keyNames[i*2]));
             }
       for (int i = -7; i < 0; ++i) {
             KeySig* k = new KeySig(gscore);
             k->setSubtype(i & 0xff);
-            sp->append(k, tr(keyNames[(7 + i) * 2 + 1]));
+            sp->append(k, qApp->translate("MuseScore", keyNames[(7 + i) * 2 + 1]));
             }
       KeySig* k = new KeySig(gscore);
       k->setSubtype(0);
-      sp->append(k, keyNames[14]);
+      sp->append(k, qApp->translate("MuseScore", keyNames[14]));
       sp->setSelected(14);
 
       sp->resizeWidth(300);
