@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: note.h,v 1.45 2006/03/03 16:20:42 wschweer Exp $
+//  $Id: note.h 1853 2009-05-26 18:00:27Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -73,33 +73,37 @@ class NoteHead : public Symbol {
 class Note : public Element {
       Q_DECLARE_TR_FUNCTIONS(Note)
 
-      static int smallNoteHeads[HEAD_GROUPS][4];
-
       int _subchannel;        ///< articulation
-
-      int _pitch;             ///< Note pitch as midi value (0 - 127).
-      int _tpc;               ///< tonal pitch class
-
       int _line;              ///< y-Position; 0 - top line.
-      int _staffMove;         ///< -1, 0, +1, used for crossbeaming
 
-      int _userAccidental;    ///< editorial accidental type (0-15)
+      char _pitch;             ///< Note pitch as midi value (0 - 127).
+      char _ppitch;            ///< played pitch (honor voltas etc.); cached value
+      double _tuning;          ///< pitch offset in cent, playable only by internal synthesizer
+      char _velocity;          ///< midi playback velocity (0 - 127); cached value
+      char _tpc;               ///< tonal pitch class
+      char _staffMove;         ///< -1, 0, +1, used for crossbeaming
+
+      char _userAccidental;    ///< editorial accidental type (0-15)
+      char _headGroup;
+
+      bool _mirror;           ///< True if note is mirrored at stem.
+      DirectionH _userMirror; ///< user override of mirror
+
+      bool _hidden;           ///< markes this note as the hidden one if there are
+                              ///< overlapping notes; hidden notes are not played
+                              ///< and heads + accidentals are not shown
+
+      int _accidentalType;
       Accidental* _accidental;
 
-      int _head;              ///< note head symbol number
-      int _headGroup;
-      bool _mirror;           ///< True if note is mirrored at stem.
       ElementList _el;        ///< fingering, other text, symbols or images
-
       Tie* _tieFor;
       Tie* _tieBack;
 
       int _lineOffset;        ///< Used during mouse dragging.
-      bool _hidden;           ///< markes this note as the hidden one if there are
-                              ///< overlapping notes; hidden notes are not played
-                              ///< and heads are not shown
+      int  _head;              ///< note head symbol number
 
-      virtual bool isMovable() const { return true; }
+      virtual bool isMovable() const;
       virtual QRectF drag(const QPointF& s);
       virtual void endDrag();
 
@@ -112,7 +116,7 @@ class Note : public Element {
       virtual ElementType type() const { return NOTE; }
       virtual QRectF bbox() const;
       virtual QPointF canvasPos() const;      ///< position in canvas coordinates
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       virtual void collectElements(QList<const Element*>& el) const;
       virtual void setTrack(int val);
 
@@ -125,18 +129,25 @@ class Note : public Element {
       int headGroup() const           { return _headGroup; }
       void setHeadGroup(int val);
 
-      int pitch() const               { return _pitch; }
+      int pitch() const               { return _pitch;    }
       void setPitch(int val);
-      int tpc() const                 { return _tpc; }
+      int ppitch() const              { return _ppitch;   }
+      void setPpitch(int v)           { _ppitch = v;      }
+      double tuning() const           { return _tuning;   }
+      void setTuning(double v)        { _tuning = v;      }
+      int velocity() const            { return _velocity; }
+      void setVelocity(int v)         { _velocity = v;    }
+
+      int tpc() const                 { return _tpc;      }
       void setTpc(int v);
       void changePitch(int val);
 
       int userAccidental() const      { return _userAccidental; }
       void setUserAccidental(int n)   { _userAccidental = n; }
 
-      int accidentalSubtype() const   { return _accidental ? _accidental->subtype() : ACC_NONE; }
-      Accidental* accidental() const  { return _accidental; }
-      void setAccidentalSubtype(int);
+      int accidentalType() const      { return _accidentalType; }
+      Accidental* accidental() const  { return _accidental;     }
+      void setAccidentalType(int);
       void changeAccidental(int);
 
       int line() const                { return _line + _lineOffset;   }
@@ -160,7 +171,9 @@ class Note : public Element {
       virtual void read(QDomElement);
       void write(Xml& xml, int start, int end) const;
       virtual void write(Xml& xml) const { write(xml, 0, 0); }
+
       QPointF stemPos(bool upFlag) const;    ///< Point to connect stem.
+      double stemYoff(bool upFlag) const;
 
       virtual bool acceptDrop(Viewer*, const QPointF&, int, int) const;
       virtual Element* drop(const QPointF&, const QPointF&, Element*);
@@ -183,7 +196,11 @@ class Note : public Element {
       int subchannel() const        { return _subchannel; }
       void setSubchannel(int val)   { _subchannel = val;  }
 
-      virtual void setTickLen(int n) { printf("Note: setTickLen %d\n", n); }
+      DirectionH userMirror() const    { return _userMirror; }
+      void setUserMirror(DirectionH d) { _userMirror = d; }
+
+      virtual void toDefault();
+      virtual void setMag(double val);
       };
 
 //---------------------------------------------------------
