@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: pa.h 1853 2009-05-26 18:00:27Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -23,7 +23,7 @@
 
 #include "config.h"
 
-class ISynth;
+class Synth;
 class Seq;
 class MidiDriver;
 
@@ -40,7 +40,6 @@ class Portaudio : public Driver {
       unsigned pos;
       double startTime;
 
-      ISynth* synth;
       MidiDriver* midiDriver;
 
    public:
@@ -69,9 +68,10 @@ class Portaudio : public Driver {
       int currentApi() const;
       int currentDevice() const;
 
-      virtual void putEvent(const MidiOutEvent&);
+      virtual void putEvent(const Event&);
       virtual void process(int, float*, float*, int);
       virtual void midiRead();
+      const MidiPatch* getPatchInfo(bool onlyDrums, const MidiPatch* p);
       };
 
 #endif
