@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: page.cpp,v 1.30 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: page.cpp 1980 2009-08-01 14:25:33Z lvinken $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -26,7 +26,6 @@
 #include "preferences.h"
 #include "measure.h"
 #include "style.h"
-#include "layout.h"
 #include "chord.h"
 #include "beam.h"
 #include "tuplet.h"
@@ -44,13 +43,10 @@
 //   Page
 //---------------------------------------------------------
 
-Page::Page(ScoreLayout* s)
-   : Element(s->score())
+Page::Page(Score* s)
+   : Element(s),
+   _no(0), _pageNo(0), _copyright(0)
       {
-      _layout    = s;
-      _no        = 0;
-      _pageNo    = 0;
-      _copyright = 0;
       }
 
 //---------------------------------------------------------
@@ -59,35 +55,35 @@ Page::Page(ScoreLayout* s)
 
 double Page::tm() const
       {
-      PageFormat* pf = _layout->pageFormat();
+      PageFormat* pf = score()->pageFormat();
       return ((!pf->twosided || isOdd()) ? pf->oddTopMargin : pf->evenTopMargin) * DPI;
       }
 
 double Page::bm() const
       {
-      PageFormat* pf = _layout->pageFormat();
+      PageFormat* pf = score()->pageFormat();
       return ((!pf->twosided || isOdd()) ? pf->oddBottomMargin : pf->evenBottomMargin) * DPI;
       }
 
 double Page::lm() const
       {
-      PageFormat* pf = _layout->pageFormat();
+      PageFormat* pf = score()->pageFormat();
       return ((!pf->twosided || isOdd()) ? pf->oddLeftMargin : pf->evenLeftMargin) * DPI;
       }
 
 double Page::rm() const
       {
-      PageFormat* pf = _layout->pageFormat();
+      PageFormat* pf = score()->pageFormat();
       return ((!pf->twosided || isOdd()) ? pf->oddRightMargin : pf->evenRightMargin) * DPI;
       }
 
 double Page::loWidth() const
       {
-      return _layout->pageFormat()->width() * DPI;
+      return score()->pageFormat()->width() * DPI;
       }
 double Page::loHeight() const
       {
-      return _layout->pageFormat()->height() * DPI;
+      return score()->pageFormat()->height() * DPI;
       }
 
 //---------------------------------------------------------
@@ -113,26 +109,27 @@ void Page::setNo(int n)
 //   layout
 //---------------------------------------------------------
 
-void Page::layout(ScoreLayout* layout)
+void Page::layout()
       {
       setbbox(QRectF(0.0, 0.0, loWidth(), loHeight()));
 
       // add page number
       int n = no() + 1 + _score->_pageOffset;
-      if (score()->style()->showPageNumber && ((n > 1) || score()->style()->showPageNumberOne)) {
+      if (score()->styleB(ST_showPageNumber) && ((n > 1) || score()->styleB(ST_showPageNumberOne))) {
             int subtype = (n & 1) ? TEXT_PAGE_NUMBER_ODD : TEXT_PAGE_NUMBER_EVEN;
+            int style   = (n & 1) ? TEXT_STYLE_PAGE_NUMBER_ODD : TEXT_STYLE_PAGE_NUMBER_EVEN;
             if (_pageNo == 0) {
                   _pageNo = new Text(score());
                   _pageNo->setParent(this);
                   }
-            // TODO: always set subtype so that style changes take immediate effect
             if (subtype != _pageNo->subtype()) {
                   _pageNo->setSubtype(subtype);
+                  _pageNo->setTextStyle(style);
                   }
             _pageNo->setText(QString("%1").arg(n));
-            _pageNo->layout(layout);
+            _pageNo->layout();
             }
-      else if (_pageNo) {
+      else {
             delete _pageNo;
             _pageNo = 0;
             }
@@ -140,23 +137,17 @@ void Page::layout(ScoreLayout* layout)
       // add copyright to page
       if (score()->state() == STATE_EDIT) {      // for special case: edit copyright
             if (_copyright)
-                  _copyright->layout(layout);
+                  _copyright->layout();
             }
       else {
             if (_score->rights) {
                   if (_copyright == 0) {
-                        _copyright = new Text(score());
+                        _copyright = new TextC(*_score->rights);
                         _copyright->setParent(this);
                         }
-                  if (_copyright->getText() != _score->rights->toHtml()) {
-                        _copyright->setHtml(_score->rights->toHtml());
-                        }
-                  // always set subtype so that style changes
-                  // take immediate effect:
-                  _copyright->setSubtype(TEXT_COPYRIGHT);
-                  _copyright->layout(layout);
+                  _copyright->layout();
                   }
-            else if (_copyright) {
+            else {
                   delete _copyright;
                   _copyright = 0;
                   }
@@ -268,20 +259,20 @@ void Page::collectElements(QList<const Element*>& el) const
 //---------------------------------------------------------
 
 PageFormat::PageFormat()
+   : size(preferences.paperSize),
+   _width(preferences.paperWidth),
+   _height(preferences.paperHeight),
+   evenLeftMargin(10.0 / INCH),
+   evenRightMargin(10.0 / INCH),
+   evenTopMargin(10.0 / INCH),
+   evenBottomMargin(20.0 / INCH),
+   oddLeftMargin(10.0 / INCH),
+   oddRightMargin(10.0 / INCH),
+   oddTopMargin(10.0 / INCH),
+   oddBottomMargin(20.0 / INCH),
+   landscape(false),
+   twosided(false)
       {
-      size             = preferences.paperSize;
-      _width           = preferences.paperWidth;
-      _height          = preferences.paperHeight;
-      evenLeftMargin   = 10.0 / INCH;
-      evenRightMargin  = 10.0 / INCH;
-      evenTopMargin    = 10.0 / INCH;
-      evenBottomMargin = 20.0 / INCH;
-      oddLeftMargin    = 10.0 / INCH;
-      oddRightMargin   = 10.0 / INCH;
-      oddTopMargin     = 10.0 / INCH;
-      oddBottomMargin  = 20.0 / INCH;
-      landscape        = false;
-      twosided         = false;
       }
 
 //---------------------------------------------------------
@@ -394,10 +385,8 @@ void PageFormat::read(QDomElement e)
 //---------------------------------------------------------
 //   read
 //  <page-layout>
-//	  <page-height>
-//	  <page-width>
-//      <pageFormat>A6</pageFormat>
-//      <landscape>1</landscape>
+//      <page-height>
+//      <page-width>
 //      <page-margins>
 //         <left-margin>28.3465</left-margin>
 //         <right-margin>28.3465</right-margin>
@@ -406,27 +395,22 @@ void PageFormat::read(QDomElement e)
 //         </page-margins>
 //      </page-layout>
 //
-//    sizes are given in units of 1/10 spatium; t
+//    sizes are given in units of 1/10 spatium;
 //---------------------------------------------------------
 
-void PageFormat::readMusicXML(QDomElement e)
+void PageFormat::readMusicXML(QDomElement e, double conversion)
       {
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
             QString val(e.text());
-            int i = val.toInt();
-            if (tag == "pageFormat") {
-                  size = paperSizeNameToIndex(val);
-                  }
-            else if (tag == "landscape")
-                  landscape = i;
-            else if (tag == "page-margins") {
+            if (tag == "page-margins") {
                   QString type = e.attribute("type","both");
                   double lm = 0.0, rm = 0.0, tm = 0.0, bm = 0.0;
                   for (QDomElement ee = e.firstChildElement(); !ee.isNull(); ee = ee.nextSiblingElement()) {
                         QString tag(ee.tagName());
-//                        double val = ee.text().toDouble() * (18/4)/ PPI  * .1;
-                        double val = ee.text().toDouble() * 0.45 / PPI;
+                        //double val = ee.text().toDouble() * (18/4)/ PPI  * .1;
+                        //double val = ee.text().toDouble() * 0.45 / PPI; OLD!!!
+                        double val = ee.text().toDouble() * conversion;
                         if (tag == "left-margin")
                               lm = val;
                         else if (tag == "right-margin")
@@ -454,15 +438,20 @@ void PageFormat::readMusicXML(QDomElement e)
                   }
             else if (tag == "page-height") {
                   size = paperSizeNameToIndex("Custom");
-                  _height = val.toDouble() * 0.45 / PPI;
+                  _height = val.toDouble() * conversion;
                   }
             else if (tag == "page-width") {
                   size = paperSizeNameToIndex("Custom");
-                  _width = val.toDouble() * 0.45 / PPI;
+                  _width = val.toDouble() * conversion;
                   }
             else
                   domError(e);
             }
+      printf("PageFormat::readMusicXML size=%d, height=%g, width=%g\n",
+      size, _height, _width);
+      int match = paperSizeSizeToIndex(_width, _height);
+      printf("PageFormat::readMusicXML match=%d\n", match);
+      if (match >= 0) size = match;
       }
 
 //---------------------------------------------------------
@@ -509,6 +498,39 @@ void PageFormat::write(Xml& xml)
       xml.etag();
       }
 
+//---------------------------------------------------------
+//   writeMusicXML
+//---------------------------------------------------------
+
+void PageFormat::writeMusicXML(Xml& xml, double conversion )
+      {
+      xml.stag("page-layout");
+
+      //double t = 2 * PPI * 10 / 9;
+
+      xml.tag("page-height", height() * conversion);
+      xml.tag("page-width", width() * conversion);
+      QString type("both");
+      if (twosided) {
+            type = "even";
+            xml.stag(QString("page-margins type=\"%1\"").arg(type));
+            xml.tag("left-margin",   evenLeftMargin * conversion);
+            xml.tag("right-margin",  evenRightMargin * conversion);
+            xml.tag("top-margin",    evenTopMargin * conversion);
+            xml.tag("bottom-margin", evenBottomMargin * conversion);
+            xml.etag();
+            type = "odd";
+            }
+      xml.stag(QString("page-margins type=\"%1\"").arg(type));
+      xml.tag("left-margin",   oddLeftMargin * conversion);
+      xml.tag("right-margin",  oddRightMargin * conversion);
+      xml.tag("top-margin",    oddTopMargin * conversion);
+      xml.tag("bottom-margin", oddBottomMargin * conversion);
+      xml.etag();
+
+      xml.etag();
+      }
+
 //---------------------------------------------------------
 //   clear
 //---------------------------------------------------------
@@ -526,7 +548,7 @@ void Page::add(Element* e)
       {
       if (e->type() == TEXT && e->subtype() == TEXT_COPYRIGHT) {
             e->setParent(this);
-            _copyright = (Text*)e;
+            _copyright = static_cast<TextC*>(e);
             }
       else {
             printf("cannot add %s to %s\n", e->name(), name());
@@ -547,3 +569,4 @@ void Page::remove(Element* e)
             printf("cannot remove %s from %s\n", e->name(), name());
             }
       }
+
