@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: palette.cpp,v 1.23 2006/03/06 21:08:55 wschweer Exp $
+//  $Id: palette.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -22,11 +22,9 @@
 #include "mscore.h"
 #include "element.h"
 #include "style.h"
-#include "spatium.h"
 #include "globals.h"
 #include "sym.h"
 #include "symbol.h"
-#include "layout.h"
 #include "score.h"
 #include "image.h"
 #include "xml.h"
@@ -34,6 +32,7 @@
 #include "note.h"
 #include "chord.h"
 #include "clef.h"
+#include "segment.h"
 
 //---------------------------------------------------------
 //   needsStaff
@@ -156,8 +155,10 @@ void Palette::mouseDoubleClickEvent(QMouseEvent* ev)
       int i = idx(ev->pos());
       if (i == -1)
             return;
-      Score* score        = mscore->currentScore();
-      Selection* sel      = score->selection();
+      Score* score   = mscore->currentScore();
+      if (score == 0)
+            return;
+      Selection* sel = score->selection();
 
       if (sel->state() == SEL_NONE)
             return;
@@ -248,7 +249,7 @@ void Palette::mouseMoveEvent(QMouseEvent* ev)
             if (cells[currentIdx]) {
                   Element* el  = cells[currentIdx]->element;
                   QRect ir     = idxRect(currentIdx);
-                  qreal mag    = PALETTE_SPATIUM * extraMag / _spatium;
+                  qreal mag    = PALETTE_SPATIUM * extraMag / gscore->spatium();
                   QPointF spos = QPointF(dragStartPosition) / mag;
                   spos        -= QPointF(cells[currentIdx]->x, cells[currentIdx]->y);
 
@@ -317,9 +318,9 @@ void Palette::append(Element* s, const QString& name)
 
 void Palette::append(int symIdx)
       {
-      Symbol* s = new Symbol(0);
+      Symbol* s = new Symbol(gscore);
       s->setSym(symIdx);
-      append(s, ::symbols[symIdx].name());
+      append(s, qApp->translate("symbol", ::symbols[symIdx].name()));
       }
 
 //---------------------------------------------------------
@@ -331,8 +332,7 @@ void Palette::add(int idx, Element* s, const QString& name)
       PaletteCell* cell = new PaletteCell;
 
       if (idx < cells.size()) {
-            if (cells[idx])
-                  delete cells[idx];
+            delete cells[idx];
             }
       else {
             for (int i = cells.size(); i <= idx; ++i)
@@ -357,10 +357,9 @@ void Palette::add(int idx, Element* s, const QString& name)
 
 void Palette::paintEvent(QPaintEvent*)
       {
-      qreal mag = PALETTE_SPATIUM * extraMag / _spatium;
-      ScoreLayout layout(gscore);
-      layout.setSpatium(_spatium);
-      layout.setPaintDevice(this);
+      qreal mag = PALETTE_SPATIUM * extraMag / gscore->spatium();
+      gscore->setSpatium(SPATIUM20  * DPI);
+      gscore->setPaintDevice(this);
 
       QPainter p(this);
       p.setRenderHint(QPainter::Antialiasing, true);
@@ -385,7 +384,7 @@ void Palette::paintEvent(QPaintEvent*)
       //
 
       QPen pen(palette().brush(QPalette::Normal, QPalette::Text).color());
-      pen.setWidthF(defaultStyle.staffLineWidth.val() * PALETTE_SPATIUM * extraMag);
+      pen.setWidthF(defaultStyle[ST_staffLineWidth].toSpatium().val() * PALETTE_SPATIUM * extraMag);
 
       for (int idx = 0; idx < cells.size(); ++idx) {
             QRect r = idxRect(idx);
@@ -404,7 +403,7 @@ void Palette::paintEvent(QPaintEvent*)
                   int row    = idx / c;
                   int column = idx % c;
 
-                  el->layout(&layout);
+                  el->layout();
                   el->setPos(0.0, 0.0);   // HACK
 
                   if (drawStaff) {
@@ -429,7 +428,7 @@ void Palette::paintEvent(QPaintEvent*)
                   double sy;
 
                   if (drawStaff)
-                        sy = gy + gh * .5 - 2.0 * _spatium;
+                        sy = gy + gh * .5 - 2.0 * gscore->spatium();
                   else
                         sy  = gy + (gh - sh) * .5 - el->bbox().y();
                   double sx  = gx + (gw - sw) * .5 - el->bbox().x();
@@ -512,10 +511,11 @@ void Palette::dragEnterEvent(QDragEnterEvent* event)
                u.path().toLatin1().data());
             if (u.scheme() == "file") {
                   QFileInfo fi(u.path());
-                  if (fi.suffix() == "svg"
-                     || fi.suffix() == "jpg"
-                     || fi.suffix() == "png"
-                     || fi.suffix() == "xpm"
+                  QString suffix(fi.suffix().toLower());
+                  if (suffix == "svg"
+                     || suffix == "jpg"
+                     || suffix == "png"
+                     || suffix == "xpm"
                      ) {
                         event->acceptProposedAction();
                         }
@@ -574,16 +574,17 @@ void Palette::dropEvent(QDropEvent* event)
             if (u.scheme() == "file") {
                   QFileInfo fi(u.path());
                   Image* s = 0;
-                  if (fi.suffix() == "svg")
+                  QString suffix(fi.suffix().toLower());
+                  if (suffix == "svg")
                         s = new SvgImage(0);
-                  else if (fi.suffix() == "jpg"
-                     || fi.suffix() == "png"
-                     || fi.suffix() == "xpm"
+                  else if (suffix == "jpg"
+                     || suffix == "png"
+                     || suffix == "xpm"
                         )
                         s = new RasterImage(0);
                   else
                         return;
-                  qreal mag = PALETTE_SPATIUM * extraMag / _spatium;
+                  qreal mag = PALETTE_SPATIUM * extraMag / gscore->spatium();
                   s->setPath(u.path());
                   s->setSize(QSizeF(hgrid / mag, hgrid / mag));
                   e = s;
@@ -602,7 +603,7 @@ void Palette::dropEvent(QDropEvent* event)
             docName = "--";
             QDomElement el = doc.documentElement();
             QPointF dragOffset;
-            int type = Element::readType(el, &dragOffset);
+            ElementType type = Element::readType(el, &dragOffset);
 
             if (type == IMAGE) {
                   // look ahead for image type
@@ -615,11 +616,12 @@ void Palette::dropEvent(QDropEvent* event)
                               }
                         }
                   Image* image = 0;
-                  if (path.endsWith(".svg"))
+                  QString s(path.toLower());
+                  if (s.endsWith(".svg"))
                         image = new SvgImage(0);
-                  else if (path.endsWith(".jpg")
-                     || path.endsWith(".png")
-                     || path.endsWith(".xpm")
+                  else if (s.endsWith(".jpg")
+                     || s.endsWith(".png")
+                     || s.endsWith(".xpm")
                         )
                         image = new RasterImage(0);
                   else {
@@ -752,11 +754,12 @@ void Palette::read(QDomElement e)
                                                 }
                                           }
                                     Image* image = 0;
-                                    if (path.endsWith(".svg"))
+                                    QString s(path.toLower());
+                                    if (s.endsWith(".svg"))
                                           image = new SvgImage(gscore);
-                                    else if (path.endsWith(".jpg")
-                                       || path.endsWith(".png")
-                                       || path.endsWith(".xpm")
+                                    else if (s.endsWith(".jpg")
+                                       || s.endsWith(".png")
+                                       || s.endsWith(".xpm")
                                           )
                                           image = new RasterImage(gscore);
                                     else {
@@ -1141,9 +1144,11 @@ bool PaletteBox::read(QFile* qf)
       docName = qf->fileName();
       for (QDomElement e = doc.documentElement(); !e.isNull(); e = e.nextSiblingElement()) {
             if (e.tagName() == "museScore") {
-                  // QString version = e.attribute(QString("version"));
-                  // QStringList sl = version.split('.');
-                  // int versionId = sl[0].toInt() * 100 + sl[1].toInt();
+                  QString version = e.attribute(QString("version"));
+                  QStringList sl = version.split('.');
+                  int versionId = sl[0].toInt() * 100 + sl[1].toInt();
+                  gscore->setMscVersion(versionId);
+
                   for (QDomElement ee = e.firstChildElement(); !ee.isNull();  ee = ee.nextSiblingElement()) {
                         QString tag(ee.tagName());
                         if (tag == "Palette") {
