@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: part.h,v 1.8 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: part.h 1848 2009-05-24 15:24:10Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -29,7 +29,7 @@ class Staff;
 class Score;
 class Drumset;
 class InstrumentTemplate;
-class TextBase;
+class TextC;
 
 //---------------------------------------------------------
 //   Part
@@ -39,8 +39,8 @@ class Part {
       Score* _score;
       QString _trackName;           ///< used in tracklist
 
-      TextBase* _longNameBase;
-      TextBase* _shortNameBase;
+      TextC* _longName;
+      TextC* _shortName;
 
       Instrument _instrument;
       QList<Staff*> _staves;
@@ -54,23 +54,23 @@ class Part {
 
       void read(QDomElement);
       void write(Xml& xml) const;
-      int nstaves() const                      { return _staves.size(); }
-      QList<Staff*>* staves()                  { return &_staves; }
-      const QList<Staff*>* staves() const      { return &_staves; }
+      int nstaves() const                       { return _staves.size(); }
+      QList<Staff*>* staves()                   { return &_staves; }
+      const QList<Staff*>* staves() const       { return &_staves; }
       Staff* staff(int idx) const;
-      void setId(const QString& s)             { _id = s; }
-      QString id() const                       { return _id; }
-      QString trackName() const                { return _trackName;  }
-      void setTrackName(const QString& s)      { _trackName = s; }
+      void setId(const QString& s)              { _id = s; }
+      QString id() const                        { return _id; }
+      QString trackName() const                 { return _trackName;  }
+      void setTrackName(const QString& s)       { _trackName = s; }
 
-      QString shortName() const;
-      QString longName()  const;
       QString shortNameHtml() const;
       QString longNameHtml()  const;
 
-      TextBase** longNameBase()                { return &_longNameBase; }
-      TextBase** shortNameBase()               { return &_shortNameBase; }
+      TextC* longName()                         { return _longName; }
+      TextC* shortName()                        { return _shortName; }
       void setLongName(const QString& s);
+      void setLongNameEncoded(const QString& s);
+      void setShortNameEncoded(const QString& s);
       void setShortName(const QString& s);
       void setLongNameHtml(const QString& s);
       void setShortNameHtml(const QString& s);
@@ -79,16 +79,24 @@ class Part {
 
       void setStaves(int);
 
-      void setMinPitch(int val)                { _instrument.minPitch = val;     }
-      void setMaxPitch(int val)                { _instrument.maxPitch = val;     }
+      void setAmateurPitchRange(int a, int b)  {
+            _instrument.minPitchA = a;
+            _instrument.maxPitchA = b;
+            }
+      void setProfessionalPitchRange(int a, int b)  {
+            _instrument.minPitchP = a;
+            _instrument.maxPitchP = b;
+            }
 
       void setDrumset(Drumset* ds)             { _instrument.drumset = ds;       }
       Drumset* drumset() const                 { return _instrument.drumset;     }
       bool useDrumset() const                  { return _instrument.useDrumset;  }
       void setUseDrumset(bool val);
 
-      int minPitch() const                     { return _instrument.minPitch;    }
-      int maxPitch() const                     { return _instrument.maxPitch;    }
+      int minPitchP() const                    { return _instrument.minPitchP;    }
+      int maxPitchP() const                    { return _instrument.maxPitchP;    }
+      int minPitchA() const                    { return _instrument.minPitchA;    }
+      int maxPitchA() const                    { return _instrument.maxPitchA;    }
 
       int volume() const;
       int reverb() const;
