@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: partedit.cpp,v 1.4 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: partedit.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -23,6 +23,8 @@
 #include "part.h"
 #include "partedit.h"
 #include "seq.h"
+#include "undo.h"
+#include "synti.h"
 
 //---------------------------------------------------------
 //   PartEdit
@@ -121,7 +123,6 @@ void InstrumentListEditor::updateAll(Score* score)
       foreach (const MidiMapping& m, *mm) {
             QWidgetItem* wi = (QWidgetItem*)(vb->itemAt(idx));
             PartEdit* pe    = (PartEdit*)(wi->widget());
-            pe->setPart(m.part, m.articulation);
             const MidiPatch* p = 0;
             for (;;) {
                   p = seq->getPatchInfo(m.part->useDrumset(), p);
@@ -130,6 +131,7 @@ void InstrumentListEditor::updateAll(Score* score)
                   pe->patch->addItem(p->name);
                   }
             ++idx;
+            pe->setPart(m.part, m.articulation);
             }
       }
 
@@ -157,29 +159,10 @@ void PartEdit::patchChanged(int n)
             if (p == 0)
                   break;
             if (idx == n) {
-                  channel->program = p->prog;
-                  channel->hbank   = p->hbank;
-                  channel->lbank   = p->lbank;
-
-                  MidiOutEvent event;
-                  int idx    = channel->channel;
-                  event.port = part->score()->midiPort(idx);
-                  event.type = ME_CONTROLLER | part->score()->midiChannel(idx);
-
-                  if (channel->hbank != -1) {
-                        event.a    = CTRL_HBANK;
-                        event.b    = channel->hbank;
-                        seq->sendEvent(event);
-                        }
-                  if (channel->lbank != -1) {
-                        event.a    = CTRL_LBANK;
-                        event.b    = channel->lbank;
-                        seq->sendEvent(event);
-                        }
-                  event.type = ME_PROGRAM | part->score()->midiChannel(idx);
-                  event.a    = channel->program;
-                  event.b    = part->useDrumset();
-                  seq->sendEvent(event);
+                  Score* score = part->score();
+                  score->startCmd();
+                  score->undo()->push(new ChangePatch(part, channel, p->prog, p->bank));
+                  score->endCmd();
                   return;
                   }
             }
