@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: mscore.cpp,v 1.105 2006/09/15 09:34:57 wschweer Exp $
+//  $Id: pitchspelling.cpp 1863 2009-06-04 16:13:37Z wschweer $
 //
 //  Copyright (C) 2007 Werner Schweer and others
 //
@@ -188,6 +188,15 @@ QString tpc2name(int tpc, bool germanNames)
       return s;
       }
 
+void tpc2name(int tpc, bool germanNames, QChar* name, int* acc)
+      {
+      const char names[]  = "FCGDAEB";
+      const char gnames[] = "FCGDAEH";
+      *acc   = ((tpc+1) / 7) - 2;
+      int idx = (tpc + 1) % 7;
+      *name = QChar::fromLatin1(germanNames ? gnames[idx] : names[idx]);
+      }
+
 //---------------------------------------------------------
 //   tpc2stepName
 //---------------------------------------------------------
@@ -404,7 +413,7 @@ static const int ASIZE        = 1024;   // 2 ** WINDOW
 //   computeWindow
 //---------------------------------------------------------
 
-static int computeWindow(const QList<NoteEvent*>& notes, int start, int end, int keyIdx)
+static int computeWindow(const QList<Event*>& notes, int start, int end, int keyIdx)
       {
       int p   = 10000;
       int idx = -1;
@@ -559,7 +568,7 @@ int computeWindow(const QList<Note*>& notes, int start, int end)
 //   spell
 //---------------------------------------------------------
 
-void spell(QList<NoteEvent*>& notes, int key)
+void spell(QList<Event*>& notes, int key)
       {
       key += 7;
 
