@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: plugins.cpp 1931 2009-07-14 18:40:48Z wschweer $
 //
-//  Copyright (C) 2008 Werner Schweer and others
+//  Copyright (C) 2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -20,10 +20,17 @@
 
 #include "mscore.h"
 #include "globals.h"
-#include "score.h"
 #include "script.h"
 #include "config.h"
-#include "qscriptembeddeddebugger.h"
+#include "scscore.h"
+#include "scchord.h"
+#include "screst.h"
+#include "sccursor.h"
+#include "scnote.h"
+#include "scharmony.h"
+#include "sctext.h"
+#include "scbytearray.h"
+#include "scmeasure.h"
 
 //---------------------------------------------------------
 //   registerPlugin
@@ -43,7 +50,7 @@ void MuseScore::registerPlugin(const QString& pluginPath)
       if (debugMode)
             printf("Register Plugin <%s>\n", qPrintable(pluginPath));
 
-      QScriptEngine se(0);
+      ScriptEngine se;
       QScriptValue val  = se.evaluate(f.readAll(), pluginPath);
       if (se.hasUncaughtException()) {
             QScriptValue sv = se.uncaughtException();
@@ -95,25 +102,36 @@ void MuseScore::registerPlugin(const QString& pluginPath)
                   QMenu* menu = qobject_cast<QMenu*>(o);
                   if (!menu)
                         continue;
+                  if (debugMode)
+                        printf("check menu <%s><%s>\n", qPrintable(menu->objectName()), qPrintable(m));
                   if (menu->objectName() == m) {
                         curMenu = menu;
                         found = true;
+                        if (debugMode)
+                              printf("  found\n");
                         break;
                         }
                   }
             if (!found) {
                   if (i == 0) {
                         curMenu = new QMenu(m, menuBar());
-                        curMenu->setObjectName("Plugin");
+                        curMenu->setObjectName(m);
                         menuBar()->insertMenu(menuBar()->actions().back(), (QMenu*)curMenu);
+                        if (debugMode)
+                              printf("add Menu <%s>\n", qPrintable(m));
                         }
                   else if (i + 1 == n) {
                         QAction* a = ((QMenu*)curMenu)->addAction(m);
                         connect(a, SIGNAL(triggered()), pluginMapper, SLOT(map()));
                         pluginMapper->setMapping(a, pluginIdx);
+                        if (debugMode)
+                              printf("add action <%s>\n", qPrintable(m));
                         }
-                  else
+                  else {
                         curMenu = ((QMenu*)curMenu)->addMenu(m);
+                        if (debugMode)
+                              printf("add menu <%s>\n", qPrintable(m));
+                        }
                   }
             }
       }
@@ -128,6 +146,8 @@ void MuseScore::loadPlugins()
       connect(pluginMapper, SIGNAL(mapped(int)), SLOT(pluginTriggered(int)));
 
       QDir pluginDir(mscoreGlobalShare + "plugins");
+      if (debugMode)
+            printf("Plugin Path <%s>\n", qPrintable(mscoreGlobalShare + "plugins"));
       QDirIterator it(pluginDir, QDirIterator::Subdirectories);
       while (it.hasNext()) {
             it.next();
@@ -140,6 +160,74 @@ void MuseScore::loadPlugins()
             }
       }
 
+bool MuseScore::loadPlugin(const QString& filename)
+      {
+      bool result = false;
+
+      QDir pluginDir(mscoreGlobalShare + "plugins");
+      if (debugMode)
+            printf("Plugin Path <%s>\n", qPrintable(mscoreGlobalShare + "plugins"));
+
+      if (filename.endsWith(".js")){
+        QFileInfo fi(pluginDir, filename);
+        if (fi.exists()){
+          QString path(fi.filePath());
+          registerPlugin(path);
+          result = true;
+          }
+        }
+        return result;
+      }
+
+//---------------------------------------------------------
+//   ScriptEngine
+//---------------------------------------------------------
+
+ScriptEngine::ScriptEngine()
+   : QScriptEngine()
+      {
+      static const char* xts[] = {
+            "qt.core", "qt.gui", "qt.xml", "qt.network", "qt.uitools"
+            };
+      for (unsigned i = 0; i < sizeof(xts)/sizeof(*xts); ++i) {
+            importExtension(xts[i]);
+            if (hasUncaughtException()) {
+                  QScriptValue val = uncaughtException();
+                  printf("%s\n", qPrintable(val.toString()));
+                  }
+            }
+
+      baClass = new ByteArrayClass(this);
+      globalObject().setProperty("ByteArray", baClass->constructor());
+
+      scoreClass = new ScScore(this);
+      globalObject().setProperty("Score", scoreClass->constructor());
+
+      cursorClass = new ScSCursor(this);
+      globalObject().setProperty("Cursor", cursorClass->constructor());
+
+      ScChord* chordClass = new ScChord(this);
+      globalObject().setProperty("Chord", chordClass->constructor());
+
+      ScRest* restClass = new ScRest(this);
+      globalObject().setProperty("Rest", restClass->constructor());
+
+      ScNote* noteClass = new ScNote(this);
+      globalObject().setProperty("Note", noteClass->constructor());
+
+      ScHarmony* harmonyClass = new ScHarmony(this);
+      globalObject().setProperty("Harmony", harmonyClass->constructor());
+
+      ScText* textClass = new ScText(this);
+      globalObject().setProperty("Text", textClass->constructor());
+
+      ScMeasure* measureClass = new ScMeasure(this);
+      globalObject().setProperty("Measure", measureClass->constructor());
+
+      QScriptValue v = newVariant(division);
+      globalObject().setProperty("division", v);
+      }
+
 //---------------------------------------------------------
 //   pluginTriggered
 //---------------------------------------------------------
@@ -156,26 +244,9 @@ void MuseScore::pluginTriggered(int idx)
       if (debugMode)
             printf("Run Plugin <%s>\n", qPrintable(pp));
       if (se == 0) {
-            se = new QScriptEngine(0);
-
-#ifdef HAS_SCRIPT_INTERFACE
-            se->importExtension("qt.core");
-            se->importExtension("qt.gui");
-            se->importExtension("qt.xml");
-            se->importExtension("qt.network");
-            se->importExtension("qt.uitools");
-#endif
-
-#ifdef HAS_SCRIPT_DEBUG
-            if (scriptDebug) {
-                  if (debugger == 0)
-                        debugger = new QScriptEmbeddedDebugger();
-                  debugger->attachTo(se);
-                  debugger->breakAtFirstStatement();
-                  }
-#endif
-#if QT_VERSION >= 0x040400
+            se = new ScriptEngine();
             if (debugMode) {
+                  // > qt4.4 required
                   QStringList lp = qApp->libraryPaths();
                   foreach(const QString& s, lp)
                         printf("lib path <%s>\n", qPrintable(s));
@@ -190,12 +261,22 @@ void MuseScore::pluginTriggered(int idx)
                   foreach(const QString& s, sl)
                         printf("  <%s>\n", qPrintable(s));
                   }
-#endif
             }
-      QScriptValue v = se->newQObject(cs);
-      se->globalObject().setProperty("score", v);
-      v = se->newVariant(division);
-      se->globalObject().setProperty("division", v);
+      if (scriptDebug) {
+            if (debugger == 0) {
+                  debugger = new QScriptEngineDebugger();
+                  debugger->attachTo(se);
+                  }
+            debugger->action(QScriptEngineDebugger::InterruptAction)->trigger();
+            }
+
+      QScriptValue v = se->getScoreClass()->newInstance(cs);
+      se->globalObject().setProperty("curScore", v);
+
+      SCursor c;
+      v = se->getCursorClass()->newInstance(c);
+      se->globalObject().setProperty("curCursor", v);
+
       QFileInfo fi(f);
       pluginPath = fi.absolutePath();
       v = se->newVariant(pluginPath);
@@ -208,7 +289,12 @@ void MuseScore::pluginTriggered(int idx)
             printf("Run plugin: no run function found\n");
             return;
             }
+
+      foreach(Score* s, scoreList)
+            s->startCmd();
       run.call();
+      foreach(Score* s, scoreList)
+            s->endCmd();
+      cs->end();
       }
 
-
