@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: preferences.h,v 1.20 2006/04/06 13:03:11 wschweer Exp $
+//  $Id: preferences.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -45,7 +45,6 @@ struct MidiRemote {
 //---------------------------------------------------------
 
 struct Preferences {
-      bool cursorBlink;
       bool bgUseColor;
       bool fgUseColor;
       QString bgWallpaper;
@@ -77,8 +76,6 @@ struct Preferences {
       SessionStart sessionStart;
       QString startScore;
       QString workingDirectory;
-      QString lastSaveDirectory;
-      QString lastSaveCopyDirectory;
       bool showSplashScreen;
       MidiRemote rewind, play, stop;
       MidiRemote len1, len2, len4, len8, len16, len32;
@@ -106,7 +103,9 @@ struct Preferences {
       double paperWidth, paperHeight;     // only valid if paperSize is QPrinter::Custom
       bool landscape;
       bool twosided;
+      double spatium;
       int defaultPlayDuration;      // len of note play during note entry
+      QString importStyleFile;
 
       bool dirty;
 
@@ -153,6 +152,7 @@ class PreferenceDialog : public QDialog, private Ui::PrefsDialogBase {
       void paperSizeChanged(double);
       void pageFormatSelected(int);
       void landscapeToggled(bool);
+      void styleFileButtonClicked();
 
    signals:
       void preferencesChanged();
