@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: score.cpp,v 1.24 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: rendermidi.cpp 1984 2009-08-02 09:59:28Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -27,7 +27,6 @@
 #include "score.h"
 #include "volta.h"
 #include "note.h"
-#include "layout.h"
 #include "instrument.h"
 #include "part.h"
 #include "chord.h"
@@ -39,9 +38,15 @@
 #include "articulation.h"
 #include "arpeggio.h"
 #include "durationtype.h"
+#include "measure.h"
+#include "tempo.h"
+#include "repeatlist.h"
+#include "velo.h"
+#include "dynamics.h"
 
 //---------------------------------------------------------
 //   ARec
+//    articulation record
 //---------------------------------------------------------
 
 struct ARec {
@@ -60,7 +65,7 @@ void Score::updateChannel()
       foreach(Part* part, _parts) {
             alist.clear();
             for (int i = 0; i < part->staves()->size(); ++i) {
-                  for (MeasureBase* mb = layout()->first(); mb; mb = mb->next()) {
+                  for (MeasureBase* mb = first(); mb; mb = mb->next()) {
                         if (mb->type() != MEASURE)
                               continue;
                         Measure* m = static_cast<Measure*>(mb);
@@ -147,8 +152,7 @@ bool Score::isVolta(int tick, int repeat) const
 //   collectChord
 //---------------------------------------------------------
 
-void Score::collectChord(EventMap* events, Instrument* instr,
-   int pitchOffset, Chord* chord, int tick, int len)
+void Score::collectChord(EventMap* events, Instrument* instr, Chord* chord, int tick, int len)
       {
       NoteList* nl = chord->noteList();
       Arpeggio* arpeggio = chord->arpeggio();
@@ -165,23 +169,143 @@ void Score::collectChord(EventMap* events, Instrument* instr,
                   continue;
             if (note->tieBack())
                   continue;
-            int idx = instr->channel[note->subchannel()]->channel;
-            NoteOn* ev = new NoteOn();
-            int pitch = note->pitch() + pitchOffset;
-            if (pitch > 127)
-                  pitch = 127;
-            ev->setPitch(pitch);
-            ev->setVelo(60);
-            ev->setNote(note);
+            int idx    = instr->channel[note->subchannel()]->channel;
+            int pitch  = note->ppitch();
+            Event* ev = new Event(ME_NOTEON);
             ev->setChannel(idx);
+            ev->setPitch(pitch);
+            ev->setVelo(note->velocity());
+            ev->setTuning(note->tuning());
+            ev->setNote(note);
             events->insertMulti(tick + i * arpeggioOffset, ev);
 
-            ev = new NoteOn();
-            ev->setPitch(pitch);
-            ev->setVelo(0);
-            ev->setNote(note);
-            ev->setChannel(idx);
-            events->insertMulti(tick + len, ev);
+            Event* evo = new Event(ME_NOTEON);
+            evo->setChannel(idx);
+            evo->setPitch(pitch);
+            evo->setVelo(0);
+            evo->setNote(note);
+            events->insertMulti(tick + len, evo);
+            }
+      }
+
+//---------------------------------------------------------
+//   OttavaShiftSegment
+//---------------------------------------------------------
+
+struct OttavaShiftSegment {
+      int stick;
+      int etick;
+      int shift;
+      };
+
+//---------------------------------------------------------
+//   fixPpitch
+//    calculate play pitch and velocity for all notes
+//---------------------------------------------------------
+
+void Score::fixPpitch()
+      {
+      int ns = nstaves();
+      QList<OttavaShiftSegment> osl[ns];
+
+      //
+      //    collect ottavas
+      //
+      foreach(Element* e, _gel) {
+            if (e->type() == OTTAVA) {
+                  Ottava* ottava = static_cast<Ottava*>(e);
+                  OttavaShiftSegment ss;
+                  ss.stick = ottava->tick();
+                  ss.etick = ottava->tick2();
+                  ss.shift = ottava->pitchShift();
+                  osl[e->staffIdx()].append(ss);
+                  }
+            }
+      //
+      //    collect Dynamics
+      //
+      VeloList velo[ns];
+
+      for (int staffIdx = 0; staffIdx < nstaves(); ++staffIdx) {
+            velo[staffIdx].setVelo(0, 80);
+            Part* prt = part(staffIdx);
+            int partStaves = prt->nstaves();
+            int partStaff  = Score::staffIdx(prt);
+
+            for (Measure* m = firstMeasure(); m; m = m->nextMeasure()) {
+                  foreach(const Element* e, *m->el()) {
+                        if (e->type() != DYNAMIC)
+                              continue;
+                        const Dynamic* d = static_cast<const Dynamic*>(e);
+                        int v = d->velocity();
+                        if (v < 1)     //  illegal value
+                              continue;
+                        switch(d->dynType()) {
+                              case DYNAMIC_STAFF:
+                                    velo[staffIdx].setVelo(d->tick(), v);
+                                    break;
+                              case DYNAMIC_PART:
+                                    for (int i = partStaff; i < partStaff+partStaves; ++i)
+                                          velo[i].setVelo(d->tick(), v);
+                                    break;
+                              case DYNAMIC_SYSTEM:
+                                    for (int i = 0; i < nstaves(); ++i)
+                                          velo[i].setVelo(d->tick(), v);
+                                    break;
+                              }
+                        }
+                  }
+            }
+
+      for (int staffIdx = 0; staffIdx < nstaves(); ++staffIdx) {
+            int pitchOffset = styleB(ST_concertPitch) ? 0 : part(staffIdx)->instrument()->pitchOffset;
+
+            for (Segment* seg = firstMeasure()->first(); seg; seg = seg->next1()) {
+                  if (seg->subtype() != Segment::SegChordRest)
+                        continue;
+                  int ottavaShift = 0;
+                  foreach(const OttavaShiftSegment& ss, osl[staffIdx]) {
+                        if (seg->tick() >= ss.stick && seg->tick() < ss.etick) {
+                              ottavaShift = ss.shift;
+                              break;
+                              }
+                        }
+                  int strack = staffIdx * VOICES;
+                  int etrack = strack + VOICES;
+                  for (int track = strack; track < etrack; ++track) {
+                        Element* el = seg->element(track);
+                        if (!el || el->type() != CHORD)
+                              continue;
+                        Chord* chord = static_cast<Chord*>(el);
+                        int velocity = velo[staffIdx].velo(chord->tick());
+
+                        foreach(Articulation* a, *chord->getArticulations()) {
+                              switch(a->subtype()) {
+                                    case SforzatoaccentSym:
+                                          velocity = velocity + (velocity * 100)/50;
+                                          break;
+                                    case UmarcatoSym:
+                                    case DmarcatoSym:
+                                          velocity = velocity + (velocity * 100)/30;
+                                          break;
+                                    default:
+                                          break;
+                                    }
+                              }
+                        if (velocity > 127)
+                              velocity = 127;
+                        else if (velocity < 1)
+                              velocity = 1;
+
+                        NoteList* nl = chord->noteList();
+                        for (iNote in = nl->begin(); in != nl->end(); ++in) {
+                              Note* note = in->second;
+                              note->setPpitch(note->pitch() + pitchOffset + ottavaShift);
+                              note->setVelocity(velocity);
+                              }
+                        }
+                  }
+
             }
       }
 
@@ -189,18 +313,14 @@ void Score::collectChord(EventMap* events, Instrument* instr,
 //   collectMeasureEvents
 //---------------------------------------------------------
 
-void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
-   int tickOffset)
+void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx, int tickOffset)
       {
-      Part* prt         = part(staffIdx);
-      Instrument* instr = prt->instrument();
-      int pitchOffset   = _style->concertPitch ? 0 : instr->pitchOffset;
-
       QList<Chord*> lv;       // appoggiatura
       QList<Chord*> sv;       // acciaccatura
 
       // for the purpose of knowing whether to find the playPos after repeats
       bool playExpandRepeats = getAction("repeat")->isChecked();
+      Instrument* instr = part(staffIdx)->instrument();
 
       for (int voice = 0; voice < VOICES; ++voice) {
             int track = staffIdx * VOICES + voice;
@@ -217,39 +337,35 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
                         continue;
                         }
 
-                  int gateTime    = _style->gateTime;  // 100 - legato (100%)
-                  int tick        = chord->tick();
+                  int gateTime = _style[ST_gateTime].toInt();  // 100 - legato (100%)
+                  int tick     = chord->tick();
                   if (playExpandRepeats && !_foundPlayPosAfterRepeats && tick == playPos()) {
                         setPlayPos(tick + tickOffset);
                         _foundPlayPosAfterRepeats = true;
                         }
-                  int ottavaShift = 0;
                   foreach(Element* e, *m->score()->gel()) {
-                        if (e->type() == OTTAVA) {
-                              Ottava* ottava = (Ottava*)e;
-                              int tick1 = ottava->tick();
-                              int tick2 = ottava->tick2();
-                              if (tick >= tick1 && tick < tick2) {
-                                    ottavaShift = ottava->pitchShift();
-                                    }
-                              }
-                        else if (e->type() == SLUR) {
-                              Slur* slur = (Slur*)e;
+                        if (e->type() == SLUR) {
+                              Slur* slur = static_cast<Slur*>(e);
+                              if (slur->startElement()->staffIdx() != staffIdx)
+                                    continue;
                               int tick1 = slur->tick();
                               int tick2 = slur->tick2();
                               if (tick >= tick1 && tick < tick2 && slur->track() == track) {
-                                    gateTime = _style->slurGateTime;
+                                    gateTime = _style[ST_slurGateTime].toInt();
                                     }
-
                               }
                         }
                   foreach(Articulation* a, *chord->getArticulations()) {
                         switch(a->subtype()) {
                               case TenutoSym:
-                                    gateTime = _style->tenutoGateTime;
+                                    gateTime = _style[ST_tenutoGateTime].toInt();
                                     break;
                               case StaccatoSym:
-                                    gateTime = _style->staccatoGateTime;
+                                    gateTime = _style[ST_staccatoGateTime].toInt();
+                                    break;
+                              case UmarcatoSym:
+                              case DmarcatoSym:
+                                    gateTime = _style[ST_staccatoGateTime].toInt();
                                     break;
                               default:
                                     break;
@@ -259,22 +375,6 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
                   // compute len of chord
 
                   int len = chord->tickLen();
-                  Note* note = chord->noteList()->front();
-                  if (note->tieBack())
-                        continue;
-                  bool tiedNote = false;
-                  int lastNoteLen = len;
-                  if (note->tieFor()) {
-                        tiedNote = true;
-                        while (note->tieFor()) {
-                              if (note->tieFor()->endNote() == 0)
-                                    break;
-                              note = note->tieFor()->endNote();
-                              lastNoteLen = note->chord()->tickLen();
-                              len += lastNoteLen;
-                              }
-                        }
-
                   int apl = 0;
                   if (!lv.isEmpty()) {
                         foreach(Chord* c, lv) {
@@ -293,7 +393,6 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
                         int ssl = sl / sv.size();
                         foreach(Chord* c, sv) {
                               collectChord(events, instr,
-                                 pitchOffset + ottavaShift,
                                  c,
                                  tick + tickOffset - sl,
                                  ssl * gateTime / 100 - 1
@@ -309,7 +408,6 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
                         foreach(Chord* c, lv) {
                               int ssl = c->tickLen();
                               collectChord(events, instr,
-                                 pitchOffset + ottavaShift,
                                  c,
                                  tick + tickOffset + sl,
                                  ssl * gateTime / 100 - 1
@@ -319,16 +417,59 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
                         len -= sl;
                         tick += sl;
                         }
-                  if (tiedNote)
-                        len = len - lastNoteLen + ((lastNoteLen * gateTime) / 100 - 1);
-                  else
-                        len = (len * gateTime) / 100 - 1;
-                  collectChord(events,
-                     instr,
-                     pitchOffset + ottavaShift,
-                     chord, tick + tickOffset,
-                     len
-                     );
+                  {
+                  tick += tickOffset;
+                  NoteList* nl = chord->noteList();
+                  Arpeggio* arpeggio = chord->arpeggio();
+
+                  int arpeggioOffset = 0;
+                  static const int arpeggioNoteDistance = Duration(Duration::V_32ND).ticks();
+                  if (arpeggio && chord->noteList()->size() * arpeggioNoteDistance <= unsigned(len))
+                        arpeggioOffset = arpeggioNoteDistance;
+
+                  int i = 0;
+                  for (iNote in = nl->begin(); in != nl->end(); ++in, ++i) {
+                        Note* note = in->second;
+                        if (note->hidden() || note->tieBack())       // do not play overlapping notes
+                              continue;
+                        int idx = instr->channel[note->subchannel()]->channel;
+
+                        int len = note->chord()->tickLen();
+                        bool tiedNote = false;
+                        int lastNoteLen = len;
+                        if (note->tieFor()) {
+                              Note* n = note;
+                              tiedNote = true;
+                              while (n->tieFor()) {
+                                    if (n->tieFor()->endNote() == 0)
+                                          break;
+                                    n = n->tieFor()->endNote();
+                                    lastNoteLen = n->chord()->tickLen();
+                                    len += lastNoteLen;
+                                    }
+                              }
+                        if (tiedNote)
+                              len = len - lastNoteLen + ((lastNoteLen * gateTime) / 100 - 1);
+                        else
+                              len = (len * gateTime) / 100 - 1;
+
+                        Event* ev = new Event(ME_NOTEON);
+                        int pitch = note->ppitch();
+                        ev->setPitch(pitch);
+                        ev->setTuning(note->tuning());
+                        ev->setVelo(note->velocity());
+                        ev->setNote(note);
+                        ev->setChannel(idx);
+                        events->insertMulti(tick + i * arpeggioOffset, ev);
+
+                        ev = new Event(ME_NOTEON);
+                        ev->setPitch(pitch);
+                        ev->setVelo(0);
+                        ev->setNote(note);
+                        ev->setChannel(idx);
+                        events->insertMulti(tick + len, ev);
+                        }
+                  }
                   lv.clear();
                   sv.clear();
                   }
@@ -342,10 +483,10 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
             const StaffText* st = static_cast<const StaffText*>(e);
             QString ma(st->midiActionName());
             if (!ma.isEmpty()) {
-                  NamedEventList* nel = prt->instrument()->midiAction(ma);
+                  NamedEventList* nel = instr->midiAction(ma);
                   if (nel) {
                         foreach(Event* ev, nel->events) {
-                              Event* event = ev->clone();
+                              Event* event = new Event(*ev);
                               int tick = st->tick() + tickOffset;
                               event->setOntime(tick);
                               events->insertMulti(tick, event);
@@ -355,65 +496,20 @@ void Score::collectMeasureEvents(EventMap* events, Measure* m, int staffIdx,
             }
       }
 
-//---------------------------------------------------------
-//   toEList
-//    export score to event list
-//---------------------------------------------------------
-
-void Score::toEList(EventMap* events, int offset)
-      {
-      bool expandRepeats = getAction("repeat")->isChecked();
-      _foundPlayPosAfterRepeats = false;
-      updateChannel();
-      int staffIdx   = 0;
-      foreach(Part* part, _parts) {
-            for (int i = 0; i < part->staves()->size(); ++i) {
-                  toEList(events, expandRepeats, offset, staffIdx++);
-                  }
-            }
-      }
-
-//---------------------------------------------------------
-//   RepeatLoop
-//---------------------------------------------------------
-
-struct RepeatLoop {
-      enum LoopType { LOOP_REPEAT, LOOP_JUMP };
-
-      LoopType type;
-      MeasureBase* m;   // start measure of LOOP_REPEAT
-      int count;
-      QString stop, cont;
-
-      RepeatLoop() {}
-      RepeatLoop(MeasureBase* _m)  {
-            m     = _m;
-            count = 0;
-            type  = LOOP_REPEAT;
-            }
-      RepeatLoop(const QString s, const QString c)
-         : stop(s), cont(c)
-            {
-            type = LOOP_JUMP;
-            }
-      };
-
 //---------------------------------------------------------
 //   searchLabel
 //---------------------------------------------------------
 
-MeasureBase* Score::searchLabel(const QString& s, MeasureBase* start)
+Measure* Score::searchLabel(const QString& s, Measure* start)
       {
       if (s == "start")
-            return layout()->first();
-//      else if (s == "end")
-//            ;
+            return firstMeasure();
       if (start == 0)
-            start = layout()->first();
-      for (MeasureBase* m = start; m; m = m->next()) {
+            start = firstMeasure();
+      for (Measure* m = start; m; m = m->nextMeasure()) {
             foreach(const Element* e, *m->el()) {
                   if (e->type() == MARKER) {
-                        Marker* marker = (Marker*)e;
+                        const Marker* marker = static_cast<const Marker*>(e);
                         if (marker->label() == s)
                               return m;
                         }
@@ -424,150 +520,60 @@ MeasureBase* Score::searchLabel(const QString& s, MeasureBase* start)
 
 //---------------------------------------------------------
 //   toEList
-//    implements:
-//          - repeats
-//          - volta
-//          - d.c. al fine
-//          - d.s. al fine
-//          - d.s. al coda
 //---------------------------------------------------------
 
-void Score::toEList(EventMap* events, bool expandRepeats, int offset, int staffIdx)
+void Score::toEList(EventMap* events, int staffIdx)
       {
-      if (!expandRepeats) {
-            for (MeasureBase* mb = layout()->first(); mb; mb = mb->next()) {
-                  if (mb->type() != MEASURE)
-                        continue;
-                  collectMeasureEvents(events, (Measure*)mb, staffIdx, offset);
+      foreach(const RepeatSegment* rs, *_repeatList) {
+            int startTick  = rs->tick;
+            int endTick    = startTick + rs->len;
+            int tickOffset = rs->utick - rs->tick;
+            for (Measure* m = tick2measure(startTick); m; m = m->nextMeasure()) {
+                  collectMeasureEvents(events, m, staffIdx, tickOffset);
+                  if (m->tick() + m->tickLen() >= endTick)
+                        break;
                   }
-            return;
-            }
-      QStack<RepeatLoop> rstack;
-      int tickOffset = 0;
-      int overallRepeatCount = 0;
-      int repeatEnd = -1;
-
-      MeasureBase* fm = layout()->first();
-      for (MeasureBase* mb = fm; mb;) {
-            if (mb->type() != MEASURE) {
-                  mb = mb->next();
-                  continue;
-                  }
-            Measure* m = static_cast<Measure*>(mb);
-            if (
-                  (m->repeatFlags() & RepeatStart)
-               && (rstack.isEmpty() || (rstack.top().m != m))
-               && (rstack.isEmpty() || (rstack.top().type != RepeatLoop::LOOP_JUMP))
-               && (repeatEnd == -1)
-               )
-                  rstack.push(RepeatLoop(m));
-
-            if (!rstack.isEmpty() && !isVolta(m->tick(), rstack.top().count + 1))
-                  tickOffset -= m->tickLen();   // skip this measure
-            else
-                  collectMeasureEvents(events, m, staffIdx, tickOffset + offset);
-
-            if (rstack.isEmpty()) {
-                  // Jumps are only accepted outside of other repeats
-                  if (m->repeatFlags() & RepeatJump) {
-                        Jump* s = 0;
-                        foreach(Element* e, *m->el()) {
-                              if (e->type() == JUMP) {
-                                    s = (Jump*)e;
-                                    break;
-                                    }
-                              }
-                        if (s) {
-                              const QString& jumpTo = s->jumpTo();
-                              MeasureBase* nmb = searchLabel(jumpTo);
-                              if (nmb) {
-                                    rstack.push(RepeatLoop(s->playUntil(), s->continueAt()));
-                                    tickOffset += m->tick() + m->tickLen() - nmb->tick();
-                                    mb = nmb;
-                                    continue;
-                                    }
-                              else
-                                    printf("JUMP: label <%s> not found\n",
-                                       qPrintable(jumpTo));
-                              }
-                        else
-                              printf("Jump not found\n");
-                        }
-                  else if (m->repeatFlags() & RepeatEnd) {
-                        // this is a end repeat without start repeat:
-                        //    repeat from beginning
-                        //
-                        // dont repeat inside a repeat
-
-                        if (repeatEnd < 0 || repeatEnd == m->tick()) {
-                              ++overallRepeatCount;
-                              if (overallRepeatCount < m->repeatCount()) {
-                                    repeatEnd = m->tick();
-                                    mb = layout()->first();
-                                    tickOffset += m->tick() + m->tickLen() - mb->tick();
-                                    continue;
-                                    }
-                              else {
-                                    overallRepeatCount = 0;
-                                    repeatEnd = -1;
-                                    }
-                              }
-
-                        }
-                  }
-            else if (rstack.top().type == RepeatLoop::LOOP_REPEAT) {
-                  if (m->repeatFlags() & RepeatEnd) {
-                        //
-                        // increment repeat count
-                        //
-                        // CHECK for nested repeats:
-                        //    repeat a repeat inside a repeat only on first
-                        //    pass of outer reapeat (?!?)
-                        //
-                        bool nestedRepeat = false;
-                        int n = rstack.size();
-                        if (n > 1 && rstack[n-2].type == RepeatLoop::LOOP_REPEAT) {
-                              if (rstack[n-2].count)
-                                    nestedRepeat = true;
-                              }
-                        if (!nestedRepeat && (++rstack.top().count < m->repeatCount())) {
-                              //
-                              // goto start of loop, fix tickOffset
-                              //
-                              mb = rstack.top().m;
-                              tickOffset += m->tick() + m->tickLen() - mb->tick();
-                              continue;
-                              }
-                        rstack.pop();     // end this loop
-                        }
-                  }
-            else if (rstack.top().type == RepeatLoop::LOOP_JUMP) {
-                  MeasureBase* m = searchLabel(rstack.top().stop);
-                  if (m == 0)
-                        printf("LOOP_JUMP: label not found\n");
-                  if (m == mb) {
-                        if (m->next() == 0)
-                              break;
-                        MeasureBase* nmb = searchLabel(rstack.top().cont, m->next());
-                        if (nmb)
-                              tickOffset += m->tick() + m->tickLen() - nmb->tick();
-                        else if (!rstack.top().cont.isEmpty())
-                              printf("Cont label not found: <%s>\n", qPrintable(rstack.top().cont));
-
-                        mb = nmb;
-                        rstack.pop();     // end this loop
-                        continue;
-                        }
-                  }
-            mb = mb->next();
-            }
-      if (!rstack.isEmpty()) {
-            if (rstack.top().type == RepeatLoop::LOOP_JUMP
-               && rstack.top().stop == "end")
-                  ;
-            else
-                  printf("repeat stack not empty!\n");
             }
       }
 
+//---------------------------------------------------------
+//   updateRepeatList
+//---------------------------------------------------------
+
+void Score::updateRepeatList(bool expandRepeats)
+      {
+      if (!expandRepeats) {
+            foreach(RepeatSegment* s, *_repeatList)
+                  delete s;
+            _repeatList->clear();
+            Measure* m = lastMeasure();
+            if (m == 0)
+                  return;
+            RepeatSegment* s = new RepeatSegment;
+            s->tick  = 0;
+            s->len   = m->tick() + m->tickLen();
+            s->utick = 0;
+            s->utime = 0.0;
+            s->timeOffset = 0.0;
+            _repeatList->append(s);
+            }
+      else
+            _repeatList->unwind();
+      if (debugMode)
+            _repeatList->dump();
+      }
+
+//---------------------------------------------------------
+//   toEList
+//    export score to event list
+//---------------------------------------------------------
+
+void Score::toEList(EventMap* events)
+      {
+      updateRepeatList(getAction("repeat")->isChecked());
+      _foundPlayPosAfterRepeats = false;
+      updateChannel();
+      for (int staffIdx = 0; staffIdx < nstaves(); ++staffIdx)
+            toEList(events, staffIdx);
+      }
 
