@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: repeat.cpp 1903 2009-06-30 15:50:49Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -19,12 +19,12 @@
 //=============================================================================
 
 #include "repeat.h"
-#include "layout.h"
 #include "sym.h"
 #include "score.h"
 #include "jumpproperties.h"
 #include "markerproperties.h"
 #include "system.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   JumpProperties
@@ -79,7 +79,7 @@ void MarkerProperties::saveValues()
 //---------------------------------------------------------
 
 RepeatMeasure::RepeatMeasure(Score* score)
-   : Element(score)
+   : Rest(score)
       {
       }
 
@@ -97,9 +97,9 @@ void RepeatMeasure::draw(QPainter& p) const
 //   layout
 //---------------------------------------------------------
 
-void RepeatMeasure::layout(ScoreLayout* layout)
+void RepeatMeasure::layout()
       {
-      double sp  = layout->spatium();
+      double sp  = spatium();
 
       double w   = sp * 2.0;
       double h   = sp * 2.0;
@@ -119,30 +119,7 @@ void RepeatMeasure::layout(ScoreLayout* layout)
       setbbox(path.boundingRect());
       }
 
-//---------------------------------------------------------
-//   write
-//---------------------------------------------------------
-
-void RepeatMeasure::write(Xml& xml) const
-      {
-      xml.stag(name());
-      Element::writeProperties(xml);
-      xml.etag();
-      }
-
-
-//---------------------------------------------------------
-//   read
-//---------------------------------------------------------
-
-void RepeatMeasure::read(QDomElement e)
-      {
-      for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
-            if (!Element::readProperties(e))
-                  domError(e);
-            }
-      }
-
+#if 0
 //---------------------------------------------------------
 //   canvasPos
 //---------------------------------------------------------
@@ -158,6 +135,7 @@ QPointF RepeatMeasure::canvasPos() const
       double yp = y() + system->staff(staffIdx())->y() + system->y();
       return QPointF(xp, yp);
       }
+#endif
 
 //---------------------------------------------------------
 //   Marker
@@ -167,6 +145,7 @@ Marker::Marker(Score* s)
    : Text(s)
       {
       setSubtype(TEXT_REPEAT);
+      setTextStyle(TEXT_STYLE_REPEAT);
       }
 
 //---------------------------------------------------------
@@ -305,6 +284,7 @@ Jump::Jump(Score* s)
    : Text(s)
       {
       setSubtype(TEXT_REPEAT);
+      setTextStyle(TEXT_STYLE_REPEAT);
       }
 
 //---------------------------------------------------------
