@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: note.h,v 1.45 2006/03/03 16:20:42 wschweer Exp $
+//  $Id: repeat.h 1903 2009-06-30 15:50:49Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -22,6 +22,7 @@
 #define __REPEAT_H__
 
 #include "text.h"
+#include "rest.h"
 
 class Score;
 class Segment;
@@ -30,7 +31,7 @@ class Segment;
 //   RepeatMeasure
 //---------------------------------------------------------
 
-class RepeatMeasure : public Element {
+class RepeatMeasure : public Rest {
       QPainterPath path;
 
    public:
@@ -39,11 +40,12 @@ class RepeatMeasure : public Element {
       virtual RepeatMeasure* clone() const  { return new RepeatMeasure(*this); }
       virtual ElementType type() const      { return REPEAT_MEASURE; }
       virtual void draw(QPainter&) const;
-      virtual void layout(ScoreLayout*);
-      virtual void write(Xml& xml) const;
-      virtual void read(QDomElement);
-      virtual QPointF canvasPos() const;
-      Segment* segment() const { return (Segment*)parent(); }
+      virtual void layout();
+//      virtual QPointF canvasPos() const;
+      virtual void read(QDomElement e)   { Element::read(e);    }
+      virtual void write(Xml& xml) const { Element::write(xml); }
+      virtual QRectF bbox() const        { return _bbox;        }
+
       };
 
 enum {
