@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: rest.cpp,v 1.5 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: rest.cpp 1983 2009-08-02 09:44:44Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -32,6 +32,7 @@
 #include "articulation.h"
 #include "chord.h"
 #include "note.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   Rest
@@ -40,10 +41,10 @@
 Rest::Rest(Score* s)
   : ChordRest(s)
       {
+      setOffsetType(OFFSET_SPATIUM);
       _beamMode  = BEAM_NO;
       _staffMove = 0;
       dotline    = -1;
-      setOffsetType(OFFSET_SPATIUM);
       _sym       = quartrestSym;
       }
 
@@ -69,6 +70,8 @@ Rest::Rest(Score* s, int tick, int len)
 
 void Rest::draw(QPainter& p) const
       {
+      double _spatium = spatium();
+
       Measure* m = measure();
       if (m && m->multiMeasure()) {
             int n = m->multiMeasure();
@@ -97,13 +100,13 @@ void Rest::draw(QPainter& p) const
                QString("%1").arg(n));
             }
       else {
-            symbols[_sym].draw(p, mag());
+            symbols[_sym].draw(p, magS());
             if (_dots) {
                   double y = dotline * _spatium * .5;
                   for (int i = 1; i <= _dots; ++i) {
-                        double x = symbols[_sym].width(mag())
-                                   + point(score()->style()->dotNoteDistance) * i;
-                        symbols[dotSym].draw(p, mag(), x, y);
+                        double x = symbols[_sym].width(magS())
+                                   + point(score()->styleS(ST_dotNoteDistance)) * i;
+                        symbols[dotSym].draw(p, magS(), x, y);
                         }
                   }
             }
@@ -118,8 +121,10 @@ void Rest::draw(QPainter& p) const
 
 void Rest::setUserOffset(double x, double y)
       {
-      int line = lrint(y  / _spatium);
-      if (_sym == wholerestSym && (line <= -2 || line >= 4))
+      double _spatium = spatium();
+      int line = lrint(y/_spatium);
+
+      if (_sym == wholerestSym && (line <= -2 || line >= 3))
             _sym = outsidewholerestSym;
       else if (_sym == outsidewholerestSym && (line > -2 && line < 4))
             _sym = wholerestSym;
@@ -127,7 +132,8 @@ void Rest::setUserOffset(double x, double y)
             _sym = outsidehalfrestSym;
       else if (_sym == outsidehalfrestSym && (line > -3 && line < 3))
             _sym = halfrestSym;
-      setUserOff(QPointF(x / _spatium, double(line)));
+
+      setUserOff(QPointF(x, double(line) * _spatium));
       }
 
 //---------------------------------------------------------
@@ -159,13 +165,15 @@ bool Rest::acceptDrop(Viewer* viewer, const QPointF&, int type, int subtype) con
          || (type == ICON && subtype == ICON_NBEAM)
          || (type == ICON && subtype == ICON_BEAM32)
          || (type == ICON && subtype == ICON_AUTOBEAM)
-         || (type == ATTRIBUTE && subtype == UfermataSym)
-         || (type == ATTRIBUTE && subtype == DfermataSym)
+         || (type == ARTICULATION && subtype == UfermataSym)
+         || (type == ARTICULATION && subtype == DfermataSym)
          || (type == CLEF)
          || (type == STAFF_TEXT)
          || (type == BAR_LINE)
          || (type == BREATH)
          || (type == CHORD)
+         || (type == DYNAMIC)
+         || (type == HARMONY)
          ) {
             viewer->setDropTarget(this);
             return true;
@@ -180,7 +188,7 @@ bool Rest::acceptDrop(Viewer* viewer, const QPointF&, int type, int subtype) con
 Element* Rest::drop(const QPointF& p1, const QPointF& p2, Element* e)
       {
       switch (e->type()) {
-            case ATTRIBUTE:
+            case ARTICULATION:
                   if (e->subtype() == UfermataSym || e->subtype() == DfermataSym)
                         score()->addArticulation(this, (Articulation*)e);
                   return 0;
@@ -225,15 +233,22 @@ Element* Rest::drop(const QPointF& p1, const QPointF& p2, Element* e)
                   Chord* c      = static_cast<Chord*>(e);
                   Note* n       = c->upNote();
                   int headGroup = n->headGroup();
-                  int len       = score()->padState()->tickLen;
+                  int len       = score()->inputState().tickLen();
                   Direction dir = c->stemDirection();
                   int t         = track() + n->voice();
                   score()->select(0, SELECT_SINGLE, 0);
-                  score()->setNote(tick(), t, n->pitch(), len, headGroup, dir);
-                  score()->setPos(tick() + len);
+                  n             = score()->setNote(tick(), t, n->pitch(), len, headGroup, dir);
+                  score()->nextInputPos(n->chord());
+                  delete e;
                   }
                   break;
-
+            case HARMONY:
+                  e->setParent(measure());
+                  e->setTick(tick());
+                  e->setTrack((track() / VOICES) * VOICES);
+                  score()->select(e, SELECT_SINGLE, 0);
+                  score()->undoAddElement(e);
+                  return e;
             default:
                   return ChordRest::drop(p1, p2, e);
             }
@@ -259,6 +274,7 @@ void Rest::write(Xml& xml) const
 
 void Rest::read(QDomElement e, const QList<Tuplet*>& tuplets, const QList<Beam*>& beams)
       {
+      setTickLen(0);
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
             QString val(e.text());
@@ -275,7 +291,8 @@ void Rest::read(QDomElement e, const QList<Tuplet*>& tuplets, const QList<Beam*>
             headType(tickLen(), &dt, &_dots);
             setDuration(dt);
             }
-      setUserOffset(userOff().x() * _spatium, userOff().y() * _spatium);
+      QPointF off(userOff());
+      setUserOffset(off.x(), off.y());
       }
 
 //---------------------------------------------------------
@@ -284,7 +301,7 @@ void Rest::read(QDomElement e, const QList<Tuplet*>& tuplets, const QList<Beam*>
 
 void Rest::add(Element* e)
       {
-      if (e->type() != ATTRIBUTE)
+      if (e->type() != ARTICULATION)
             return;
       e->setParent(this);
       e->setTrack(track());
@@ -297,7 +314,7 @@ void Rest::add(Element* e)
 
 void Rest::remove(Element* e)
       {
-      if (e->type() != ATTRIBUTE)
+      if (e->type() != ARTICULATION)
             return;
       int idx = articulations.indexOf((Articulation*)e);
       if (idx == -1)
@@ -307,17 +324,16 @@ void Rest::remove(Element* e)
       }
 
 //---------------------------------------------------------
-//   setDuration
+//   layout
 //---------------------------------------------------------
 
-void Rest::setDuration(Duration dt)
+void Rest::layout()
       {
-      // symbols[outsidewholerestSym]        = Sym("outside whole rest",       0xe102, 0);
-      // symbols[outsidehalfrestSym]         = Sym("outside half rest",        0xe103, 0);
+      double _spatium = spatium();
+      int line = lrint(userOff().y() / _spatium);
 
-      DurationElement::setDuration(dt);
-      setYoff(2.0 * mag());
-      switch(dt.val()) {
+      setYoff(2.0);
+      switch(duration().type()) {
             case Duration::V_LONG:
                   _sym = longarestSym;
                   break;
@@ -326,11 +342,11 @@ void Rest::setDuration(Duration dt)
                   break;
             case Duration::V_MEASURE:
             case Duration::V_WHOLE:
-                  _sym = wholerestSym;
-                  setYoff(1.0 * mag());
+                  _sym = (line <= -2 || line >= 4) ? outsidewholerestSym : wholerestSym;
+                  setYoff(1.0);
                   break;
             case Duration::V_HALF:
-                  _sym = halfrestSym;
+                  _sym = (line <= -3 || line >= 3) ? outsidehalfrestSym : halfrestSym;
                   break;
             case Duration::V_INVALID:
             case Duration::V_QUARTER:
@@ -355,16 +371,8 @@ void Rest::setDuration(Duration dt)
                   _sym = quartrestSym;    // TODO
                   break;
             }
-      }
-
-//---------------------------------------------------------
-//   layout
-//---------------------------------------------------------
-
-void Rest::layout(ScoreLayout* l)
-      {
-      layoutAttributes(l);
-      Element::layout(l);
+      layoutArticulations();
+      Element::layout();
       }
 
 //---------------------------------------------------------
@@ -375,9 +383,9 @@ QRectF Rest::bbox() const
       {
       Measure* m = measure();
       if (m && m->multiMeasure()) {
-            double h = _spatium * 6.5;
-            double w = score()->style()->minMMRestWidth.point();
-            return QRectF(-w * .5, -h + 2 * _spatium, w, h);
+            double h = spatium() * 6.5;
+            double w = point(score()->styleS(ST_minMMRestWidth));
+            return QRectF(-w * .5, -h + 2 * spatium(), w, h);
             }
       else {
             QRectF b = symbols[_sym].bbox(mag());
@@ -391,7 +399,7 @@ QRectF Rest::bbox() const
 
 qreal Rest::centerX() const
       {
-      return symbols[_sym].width(mag())*.5;
+      return symbols[_sym].width(magS())*.5;
       }
 
 //---------------------------------------------------------
@@ -400,7 +408,7 @@ qreal Rest::centerX() const
 
 qreal Rest::upPos() const
       {
-      return symbols[_sym].bbox().y();
+      return symbols[_sym].bbox(magS()).y();
       }
 
 //---------------------------------------------------------
@@ -409,7 +417,7 @@ qreal Rest::upPos() const
 
 qreal Rest::downPos() const
       {
-      return symbols[_sym].bbox().y() + symbols[_sym].height();
+      return symbols[_sym].bbox(magS()).y() + symbols[_sym].height(magS());
       }
 
 //---------------------------------------------------------
@@ -433,16 +441,30 @@ bool Rest::genPropertyMenu(QMenu* popup) const
 void Rest::propertyAction(const QString& s)
       {
       if (s == "props") {
-            RestProperties vp;
-            vp.setSmall(small());
+            Rest r(*this);
+            RestProperties vp(&r);
             int rv = vp.exec();
             if (rv) {
-                  bool val = vp.small();
-                  if (val != small())
-                        score()->undoChangeChordRestSize(this, val);
+                  if (r.small() != small())
+                        score()->undoChangeChordRestSize(this, r.small());
+                  if (r.extraLeadingSpace() != extraLeadingSpace()
+                     || r.extraTrailingSpace() != extraTrailingSpace()) {
+                        score()->undoChangeChordRestSpace(this, r.extraLeadingSpace(),
+                           r.extraTrailingSpace());
+                        }
                   }
             }
       else
             Element::propertyAction(s);
       }
 
+//---------------------------------------------------------
+//   space
+//---------------------------------------------------------
+
+void Rest::space(double& min, double& extra) const
+      {
+      min   = width() + point(_extraTrailingSpace);
+      extra = point(_extraLeadingSpace);
+      }
+
