@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: rest.h,v 1.3 2006/03/02 17:08:41 wschweer Exp $
+//  $Id: rest.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -42,11 +42,12 @@ class Rest : public ChordRest {
       virtual qreal downPos() const;
       virtual qreal centerX() const;
       void setUserOffset(double x, double y);
+      virtual void space(double& min, double& extra) const;
 
    public:
       Rest(Score*);
       Rest(Score*, int tick, int len);
-      virtual Rest* clone() const { return new Rest(*this); }
+      virtual Rest* clone() const      { return new Rest(*this); }
       virtual ElementType type() const { return REST; }
 
       virtual void draw(QPainter&) const;
@@ -57,7 +58,7 @@ class Rest : public ChordRest {
 
       virtual bool acceptDrop(Viewer*, const QPointF&, int, int) const;
       virtual Element* drop(const QPointF&, const QPointF&, Element*);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       virtual QRectF bbox() const;
       virtual bool genPropertyMenu(QMenu*) const;
       virtual void propertyAction(const QString&);
@@ -65,7 +66,6 @@ class Rest : public ChordRest {
       virtual int staffMove() const { return _staffMove; }
       void setStaffMove(int val)    { _staffMove = val; }
 
-      virtual void setDuration(Duration t);
       void setMMWidth(double val)   { _mmWidth = val; }
       double mmWidth() const        { return _mmWidth; }
       };
