@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: score.h,v 1.21 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: score.h 1985 2009-08-02 10:33:47Z wschweer $
 //
 //  Copyright (C) 2002-2009 Werner Schweer and others
 //
@@ -26,10 +26,14 @@
  Definition of Score class.
 */
 
-#include "undo.h"
 #include "input.h"
-#include "padstate.h"
 #include "globals.h"
+#include "style.h"
+#include "durationtype.h"
+#include "select.h"
+#include "config.h"
+#include "element.h"
+#include "bsp.h"
 
 class System;
 class TextStyle;
@@ -54,7 +58,6 @@ class Undo;
 class EditTempo;
 class Part;
 class MidiFile;
-class ScoreLayout;
 class MidiTrack;
 class BSymbol;
 class KeySig;
@@ -69,9 +72,19 @@ struct Channel;
 class Tuplet;
 class Capella;
 class CapVoice;
+class TextC;
+class Dynamic;
+class Measure;
+class MeasureBase;
+class Staff;
+class Part;
+class Instrument;
+class UndoStack;
+class RepeatList;
+class MusicXmlCreator;
 
-struct Style;
 struct SigEvent;
+struct TEvent;
 
 extern bool showRubberBand;
 
@@ -84,7 +97,8 @@ enum {
       STATE_NORMAL     = 1,
       STATE_NOTE_ENTRY = 2,
       STATE_EDIT       = 4,
-      STATE_PLAY       = 8
+      STATE_PLAY       = 8,
+      STATE_SEARCH     = 16
       };
 
 //---------------------------------------------------------
@@ -103,10 +117,11 @@ class MeasureBaseList {
       MeasureBaseList();
       MeasureBase* first() const { return _first; }
       MeasureBase* last()  const { return _last; }
-      void clear()               { _first = _last = 0; }
+      void clear()               { _first = _last = 0; _size = 0; }
       void add(MeasureBase*);
       void remove(MeasureBase*);
       void change(MeasureBase* o, MeasureBase* n);
+      int size() const { return _size; }
       };
 
 //---------------------------------------------------------
@@ -169,9 +184,28 @@ class ImagePath {
 
 class Score : public QObject {
       Q_OBJECT
-      Q_PROPERTY (int nstaves READ nstaves)
-      Q_PROPERTY (QString name READ name)
 
+      double _spatium;
+      PageFormat* _pageFormat;
+      QPaintDevice* _paintDevice;
+      BspTree bspTree;
+
+      //
+      // generated objects during layout:
+      //
+      QList<Page*> _pages;          // pages are build from systems
+      QList<System*> _systems;      // measures are akkumulated to systems
+
+      bool _needLayout;
+      Measure* startLayout;
+
+      // values used during doLayout:
+      int curPage;
+      int curSystem;
+      bool firstSystem;
+      MeasureBase* curMeasure;
+
+      UndoStack* _undo;
       QList<ImagePath*> imagePathList;
 
       int _magIdx;
@@ -182,13 +216,13 @@ class Score : public QObject {
       QList<MidiMapping> _midiMapping;
       MeasureBaseList _measures;          // here are the notes
       QList<Element*> _gel;               // global elements: Slur, SLine
+      RepeatList* _repeatList;
 
-      PadState   _padState;
       InputState _is;
 
       QList<Excerpt*> _excerpts;
 
-      Style* _style;
+      Style _style;
       QVector<TextStyle*> _textStyles;
 
       QFileInfo info;
@@ -207,8 +241,6 @@ class Score : public QObject {
 
       QList<Viewer*> viewer;
 
-      ScoreLayout* _layout;
-
       bool _showInvisible;
       bool _showFrames;
 
@@ -219,9 +251,6 @@ class Score : public QObject {
       QList<Part*> _parts;
       QList<Staff*> _staves;
 
-      UndoList undoList;
-      UndoList redoList;
-
       bool _printing;   ///< True if we are drawing to a printer
       bool _playlistDirty;
       bool _dirty;      ///< Score data was modified.
@@ -242,11 +271,19 @@ class Score : public QObject {
       QString _workNumber;
       QString _workTitle;
       QString _source;
+      QString _rights;
+      QList<MusicXmlCreator*> _creators;
+      bool _creditsRead;             ///< credits were read at MusicXML import
+
+      int textUndoLevel;
+      Selection* _selection;
+
+      //------------------
 
       ChordRest* nextMeasure(ChordRest* element, bool selectBehavior = false);
       ChordRest* prevMeasure(ChordRest* element);
       void cmdSetBeamMode(int);
-      void cmdFlipStemDirection();
+      void cmdFlip();
       Note* getSelectedNote();
       void pageNext();
       void pagePrev();
@@ -267,12 +304,10 @@ class Score : public QObject {
       void convertTrack(MidiTrack*);
       void convertTrack(BBTrack*, int);
 
-      void checkUndoOp();
       void move(const QString& cmd);
       void selectMove(const QString& cmd);
 
-      void collectChord(EventMap*, Instrument*,
-         int pitchOffset, Chord*, int tick, int gateTime);
+      void collectChord(EventMap*, Instrument*, Chord*, int tick, int gateTime);
       void collectMeasureEvents(EventMap*, Measure*, int staffIdx, int tickOffset);
 
       void padToggle(int n);
@@ -287,7 +322,6 @@ class Score : public QObject {
       void addTempo();
       void addMetronome();
 
-      MeasureBase* searchLabel(const QString& s, MeasureBase* start = 0);
       void undoSigInsertTime(int, int);
       void undoFixTicks();
 
@@ -296,7 +330,6 @@ class Score : public QObject {
       void cmdAddChordName2();
       int processPendingNotes(QList<MNote*>* notes, int, int);
       void writeExcerpt(Excerpt*, Xml&);
-      void renumberMeasures();
       void cmdResetBeamMode();
       void connectSlurs();
       void checkSlurs();
@@ -312,13 +345,24 @@ class Score : public QObject {
       void removeChordRest(ChordRest* cr, bool clearSegment);
       void cmdMove(Element* e, QPointF delta);
 
-   public slots:
-      void doUndo();
-      void doRedo();
-      void processUndoOp(UndoOp*, bool);
-
-      void resetUserOffsets();
       void resetUserStretch();
+      void toDefault();
+      void expandVoice();
+
+      Page* addPage();
+      bool layoutPage();
+      bool layoutSystem1(double& minWidth, double w, bool);
+      QList<System*> layoutSystemRow(qreal x, qreal y, qreal w, bool, double*);
+      void processSystemHeader(Measure* m, bool);
+      System* getNextSystem(bool, bool);
+      void getCurPage();
+      bool doReLayout();
+      void rebuildBspTree();
+      Measure* skipEmptyMeasures(Measure*, System*);
+
+
+   private slots:
+      void textUndoLevelAdded();
 
    signals:
       void selectionChanged(int);
@@ -327,11 +371,10 @@ class Score : public QObject {
       void posChanged(int);
 
    public:
-      bool cmdActive;
-      int curTick;            // for read optimizations
+      int curTick;                  // for read optimizations
       int curTrack;
 
-      QTextDocument* rights;
+      TextC* rights;                ///< Copyright printed at bottom of page
 
       int _pageOffset;              ///< Offset for page numbers.
 
@@ -345,10 +388,8 @@ class Score : public QObject {
       int _state;
       int _prevState;               ///< state before playback
 
-      Selection* sel;
       Element* origEditObject;
       Element* editObject;          ///< Valid in edit mode
-      QString oldInstrumentName;    ///< valid while editing instrument names
 
       System* dragSystem;           ///< Valid if DRAG_STAFF.
       int dragStaff;
@@ -371,26 +412,30 @@ class Score : public QObject {
       void cmdRemovePart(Part*);
       void cmdReplaceElements(Measure* sm, Measure* dm, int srcStaff, int dstStaff);
       void cmdAddSlur();
-      void cmdAddStaccato();
+      void cmdAddSlur(Note* note);
       void cmdAddTie();
       void cmdAddHairpin(bool);
       void cmdAddStretch(double);
       void transpose();
+      void transpose(Note* n, int diff);
 
       void cmdEnterRest();
-      void updateEntryMode();
+      void cmdEnterRest(Duration::DurationType d);
 
       Rest* addRest(int tick, int len, int track);
       void lyricsEndEdit();
       void harmonyEndEdit();
 
+   public slots:
+      void setClean(bool val);
+      void setDirty(bool val = true) { setClean(!val); }
+
    public:
-      Score();
+      Score(const Style&);
       ~Score();
 
       Score* clone();
 
-      void clear();
       void write(Xml&, bool autosave);
       bool read(QDomElement);
 
@@ -410,18 +455,8 @@ class Score : public QObject {
 
       void addViewer(Viewer* v);
 
-      void undoOp(QList<int>&);
-      void undoOp(UndoOp::UndoType type, MeasureBase* m);
-      void undoOp(UndoOp::UndoType type, Measure*, int, int);
-      void undoOp(UndoOp::UndoType type, Measure* m, MStaff* s, int staff);
-      void undoOp(UndoOp::UndoType type, Staff* staff, int tick, int oval, int nval);
-      void undoOp(UndoOp::UndoType type, Segment* seg, int staff);
-      void undoOp(UndoOp::UndoType type, Element* object);
-      void undoOp(UndoOp::UndoType type, Element*, const QColor&);
-      void undoOp(UndoOp::UndoType type, Element*, int idx);
-      void undoOp(UndoOp::UndoType type, int a, int b);
-
       void undoChangeSig(int tick, const SigEvent& o, const SigEvent& n);
+      void undoChangeKeySig(Staff* staff, int tick, int o, int n);
       void undoChangeTempo(int tick, const TEvent& o, const TEvent& n);
       void undoChangeKey(Staff* staff, int tick, int o, int n);
       void undoChangeClef(Staff* staff, int tick, int o, int n);
@@ -434,15 +469,18 @@ class Score : public QObject {
       void undoChangeVoltaEnding(Volta* volta, const QList<int>& l);
       void undoChangeVoltaText(Volta* volta, const QString& s);
       void undoChangeChordRestSize(ChordRest* cr, bool small);
+      void undoChangeChordNoStem(Chord* cr, bool noStem);
+      void undoChangeChordRestSpace(ChordRest* cr, Spatium l, Spatium t);
       void undoChangeSubtype(Element* element, int st);
-      void undoChangeNoteHead(Note*, int group);
       void undoChangePitch(Note* note, int pitch, int tpc, int userAccidental);
       void spellNotelist(QList<Note*>& notes);
       void undoChangeTpc(Note* note, int tpc);
-      void undoChangeBeamMode(ChordRest* cr, int mode);
+      void undoChangeBeamMode(ChordRest* cr, BeamMode mode);
       void undoChangeChordRestLen(ChordRest* cr, int len);
       void undoChangeEndBarLineType(Measure*, int);
       void undoChangeBarLineSpan(Staff*, int);
+      void undoChangeUserOffset(Element* e, const QPointF& offset);
+      void undoChangeDynamic(Dynamic* e, int velocity, int type);
       void undoChangeCopyright(const QString&);
       void undoTransposeHarmony(Harmony*, int);
       void undoExchangeVoice(Measure* measure, int val1, int val2, int staff1, int staff2);
@@ -450,8 +488,13 @@ class Score : public QObject {
       void undoInsertPart(Part* part, int idx);
       void undoRemoveStaff(Staff* staff, int idx);
       void undoInsertStaff(Staff* staff, int idx);
+      void undoInsertMeasure(MeasureBase*);
+      void undoToggleInvisible(Element*);
       void undoMove(Element* e, const QPointF& pt);
       void undoChangeBracketSpan(Staff* staff, int column, int span);
+      void undoChangeTuning(Note*, double);
+      void undoChangePageFormat(PageFormat*, double spatium);
+      void undoChangeUserMirror(Note*, DirectionH);
 
       Note* setNote(int tick, int track, int pitch, int len, int headGroup = 0,
          Direction stemDirection = AUTO);
@@ -473,8 +516,6 @@ class Score : public QObject {
 
       void searchSelectedElements();
 
-      bool needLayout() const;
-
       void upDown(bool up, bool octave);
       Element* searchNote(int tick, int track) const;
 
@@ -495,6 +536,7 @@ class Score : public QObject {
       Note* addNote(Chord*, int pitch);
 
       void deleteItem(Element*);
+      void cmdDeleteSelectedMeasures();
       void cmdDeleteSelection();
       void toggleInvisible(Element* obj);
 
@@ -560,34 +602,37 @@ class Score : public QObject {
       bool importLilypond(const QString& name);
       bool importBB(const QString& name);
       bool importCapella(const QString& name);
-      bool saveFile(QFileInfo& info, bool autosave);
-      bool saveFile(QIODevice* f, bool autosave);
-      bool saveCompressedFile(QFileInfo&, bool autosave);
-      bool saveCompressedFile(QIODevice*, QFileInfo&, bool autosave);
+      void saveFile(QFileInfo& info, bool autosave);
+      void saveFile(QIODevice* f, bool autosave);
+      void saveCompressedFile(QFileInfo&, bool autosave);
+      void saveCompressedFile(QIODevice*, QFileInfo&, bool autosave);
       bool saveAs(bool saveCopy = false);
 
       void print(QPrinter* printer);
-      Q_INVOKABLE bool saveXml(const QString& name);
-      Q_INVOKABLE bool saveMxl(const QString& name);
-      Q_INVOKABLE bool saveMidi(const QString& name);
+      bool saveXml(const QString& name);
+      bool saveMxl(const QString& name);
+      bool saveMidi(const QString& name);
       bool savePsPdf(const QString& saveName, QPrinter::OutputFormat format);
-      Q_INVOKABLE bool savePng(const QString& name);
-      Q_INVOKABLE bool saveSvg(const QString& name);
-      Q_INVOKABLE bool saveLilypond(const QString& name);
-
+      bool savePng(const QString& name);
+      bool savePng(const QString& name, bool screenshot, bool transparent, double convDpi, QImage::Format format);
+      bool saveSvg(const QString& name);
+      bool saveLilypond(const QString& name);
+#ifdef HAS_AUDIOFILE
+      bool saveOgg(const QString& name);
+      bool saveFlac(const QString& name);
+      bool saveWav(const QString& name);
+      bool saveAudio(const QString& name, int format);
+#endif
       ChordRest* getSelectedChordRest() const;
-      Element* getSelectedElement() const { return sel->element(); }
-      Selection* selection() const        { return sel; }
+      Element* getSelectedElement() const { return _selection->element(); }
+      Selection* selection() const        { return _selection; }
+      void setSelection(Selection* s);
+
       int pos();
       Measure* tick2measure(int tick) const;
       MeasureBase* tick2measureBase(int tick) const;
       Segment* tick2segment(int tick) const;
       void fixTicks();
-      bool undoEmpty() const;
-      bool redoEmpty() const;
-      PageFormat* pageFormat() const;
-      ScoreLayout* layout() const { return _layout; }
-
       void addArticulation(Element*, Articulation* atr);
 
       bool playlistDirty();
@@ -598,13 +643,13 @@ class Score : public QObject {
       bool saveFile(bool autosave);
       void adjustTime(int tick, MeasureBase*);
 
-      QString filePath() const    { return info.filePath(); }
-      QFileInfo* fileInfo()       { return &info; }
+      QString filePath() const       { return info.filePath(); }
+      QFileInfo* fileInfo()          { return &info; }
 
-      Q_INVOKABLE QString name() const { return info.completeBaseName(); }
+      QString name() const           { return info.completeBaseName(); }
+      void setName(const QString& s) { info.setFile(s); }
 
       bool isSavable() const;
-      void setDirty(bool val = true);
       bool dirty() const        { return _dirty;    }
       void setCreated(bool val) { _created = val;   }
       bool created() const      { return _created;  }
@@ -614,45 +659,54 @@ class Score : public QObject {
 
       bool pos2TickAnchor(const QPointF&, int staffIdx, int* tick, QPointF* anchor) const;
       void spell();
+      void spell(int startStaff, int endStaff, Segment* startSegment, Segment* endSegment);
       void spell(Note*);
       int nextSeg(int tick, int track);
       int nextSeg1(int tick, int& track);
       int prevSeg1(int tick, int& track);
 
-      Style* style() const          { return _style; }
-      void setStyle(const Style& s);
+      Style& style()                           { return _style;                   }
+      const Style& style() const               { return _style;                   }
+      void setStyle(const Style& s)            { _style = s;                      }
+      StyleVal style(StyleIdx idx) const;
+      Spatium styleS(StyleIdx idx) const;
+      bool styleB(StyleIdx idx) const;
+      double styleD(StyleIdx idx) const;
+      int styleI(StyleIdx idx) const;
+      void setStyle(StyleIdx idx, const StyleVal& v);
 
       void insertTime(int tick, int len);
       void cmdRemoveTime(int tick, int len);
-      QList<Viewer*> getViewer()    { return viewer;    }
-      int playPos() const           { return _playPos;  }
-      void setPlayPos(int val)      { _playPos = val;   }
+      QList<Viewer*> getViewer()               { return viewer;      }
+      int playPos() const                      { return _playPos;    }
+      void setPlayPos(int val)                 { _playPos = val;     }
 
-      bool noteEntryMode() const    { return _is.noteEntryMode; }
-      int inputPos() const          { return _is.pos();   }
-      int inputTrack() const        { return _is.track; }
+      bool noteEntryMode() const               { return _is.noteEntryMode; }
+      int inputPos() const                     { return _is.pos();   }
+      int inputTrack() const                   { return _is.track;   }
+      InputState& inputState()                 { return _is;        }
+      void setInputState(const InputState& st) { _is = st;          }
       void setInputTrack(int);
 
-      PadState* padState()          { return &_padState; }
-      TextStyle* textStyle(int idx) { return _textStyles[idx]; }
+      TextStyle* textStyle(int idx) { return idx < 0 ? 0 : _textStyles[idx]; }
       const QVector<TextStyle*>& textStyles() const { return _textStyles; }
-      void setTextStyles(QVector<TextStyle*>&s);
+      void setTextStyles(const QVector<TextStyle*>&s);
+      QVector<TextStyle*> swapTextStyles(QVector<TextStyle*> s);
       bool loadStyle(QFile* qf);
       void loadStyle();
       void saveStyle();
+      void textStyleChanged(const QVector<TextStyle*>&s);
+      void spatiumChanged(double oldValue, double newValue);
 
-      void setCopyright(QTextDocument*);
-      void setCopyright(const QString& s);
-      void setCopyrightHtml(const QString& s);
       void pasteStaff(QDomElement, int dstTick, int staffIdx);
       bool isVolta(int tick, int repeat) const;
-      void toEList(EventMap* events, int tickOffset);
-      void toEList(EventMap* events, bool expandRepeats, int tickOffset, int staffIdx);
-      int mscVersion() const { return _mscVersion; }
+      void toEList(EventMap* events);
+      void toEList(EventMap* events, int staffIdx);
+      int mscVersion() const    { return _mscVersion; }
+      void setMscVersion(int v) { _mscVersion = v; }
 
       SigList*   getSigmap()  { return sigmap; }
       MeasureBase* appendMeasure(int type);
-      UndoList* getUndoList() { return &undoList; }
       void addLyrics(int tick, int staffIdx, const QString&);
 
       QList<Excerpt*>* excerpts() { return &_excerpts; }
@@ -673,22 +727,29 @@ class Score : public QObject {
       void cmdConcertPitchChanged(bool);
       TempoList* getTempomap() const { return tempomap; }
 
-      QString movementNumber() const           { return _movementNumber; }
-      QString movementTitle() const            { return _movementTitle;  }
-      QString workNumber() const               { return _workNumber;     }
-      QString workTitle() const                { return _workTitle;      }
-      QString source() const                   { return _source;         }
-      void setMovementNumber(const QString& s) { _movementNumber = s;    }
-      void setMovementTitle(const QString& s)  { _movementTitle = s;     }
-      void setWorkNumber(const QString& s)     { _workNumber = s;        }
-      void setWorkTitle(const QString& s)      { _workTitle = s;         }
-      void setSource(const QString& s)         { _source = s;            }
+      QString movementNumber() const                 { return _movementNumber;  }
+      QString movementTitle() const                  { return _movementTitle;   }
+      QString workNumber() const                     { return _workNumber;      }
+      QString workTitle() const                      { return _workTitle;       }
+      QString source() const                         { return _source;          }
+      QString mxmlRights() const                     { return _rights;          }
+      bool creditsRead() const                       { return _creditsRead;     }
+      void setMovementNumber(const QString& s)       { _movementNumber = s;     }
+      void setMovementTitle(const QString& s)        { _movementTitle = s;      }
+      void setWorkNumber(const QString& s)           { _workNumber = s;         }
+      void setWorkTitle(const QString& s)            { _workTitle = s;          }
+      void setSource(const QString& s)               { _source = s;             }
+      void setmxmlRights(const QString& s)           { _rights = s;             }
+      void setCreditsRead(bool b)                    { _creditsRead = b;        }
+      void addCreator(MusicXmlCreator* c)            { _creators.append(c);     }
+      const MusicXmlCreator* getCreator(int i) const { return _creators.at(i);  }
+      int numberOfCreators() const                   { return _creators.size(); }
 
       void lassoSelect(const QRectF&);
       void lassoSelectEnd(const QRectF&);
 
       Page* searchPage(const QPointF&) const;
-      bool getPosition(Position* pos, const QPointF&, bool divideSegment) const;
+      bool getPosition(Position* pos, const QPointF&, int voice) const;
 
       void setState(int s);
       int state() const        { return _state; }
@@ -706,15 +767,59 @@ class Score : public QObject {
       void setXoff(double v)  { _xoff = v;    }
       void setYoff(double v)  { _yoff = v;    }
 
-      void setPos(int tick);
-
       ImagePath* addImage(const QString&);      // add image to imagePathList
       void moveBracket(int staffIdx, int srcCol, int dstCol);
-      void textStyleChanged();
       Measure* getCreateMeasure(int tick);
 
       void adjustBracketsDel(int sidx, int eidx);
       void adjustBracketsIns(int sidx, int eidx);
+      void renumberMeasures();
+      UndoStack* undo() const { return _undo; }
+      TextC* copyright() { return rights; }
+      void setCopyright(const QString& s);
+      void setCopyrightHtml(const QString& s);
+      void endUndoRedo();
+      void search(const QString& s);
+      Measure* firstMeasure() const;
+      Measure* lastMeasure() const;
+      Measure* searchLabel(const QString& s, Measure* start = 0);
+      RepeatList* repeatList() { return _repeatList; }
+      double utick2utime(int tick) const;
+      int utime2utick(double utime);
+      void updateRepeatList(bool expandRepeats);
+      void fixPpitch();
+
+      void nextInputPos(ChordRest* cr);
+      void setInputPos(ChordRest* cr);
+      void cmdMirrorNoteHead();
+
+      void layout()                           { _needLayout = true; }
+      void doLayout();
+      void reLayout(Measure*);
+      double spatium() const                  { return _spatium; }
+      PageFormat* pageFormat() const          { return _pageFormat; }
+      void setPageFormat(const PageFormat& pf);
+      const QList<Page*>& pages() const       { return _pages; }
+      QList<System*>* systems()               { return &_systems; }
+      bool needLayout() const                 { return _needLayout; }
+
+      MeasureBase* first() const;
+      MeasureBase* last()  const;
+
+      void setPaintDevice(QPaintDevice* d)          { _paintDevice = d; }
+      QPaintDevice* paintDevice() const             { return _paintDevice; }
+      QList<const Element*> items(const QRectF& r)  { return bspTree.items(r); }
+      QList<const Element*> items(const QPointF& p) { return bspTree.items(p); }
+
+      void insertBsp(Element* e)                    { bspTree.insert(e); }
+      void removeBsp(Element* e)                    { bspTree.remove(e); }
+
+      void setInstrumentNames();
+      void connectTies();
+
+      virtual void add(Element*);
+      virtual void remove(Element*);
+      double point(const Spatium sp) const { return sp.val() * _spatium; }
       };
 
 extern Score* gscore;
