@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: script.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2008 Werner Schweer and others
+//  Copyright (C) 2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -21,5 +21,28 @@
 #ifndef __SCRIPT_H__
 #define __SCRIPT_H__
 
+#include "score.h"
+
+class ScScore;
+class ScSCursor;
+class ByteArrayClass;
+
+//---------------------------------------------------------
+//   ScriptEngine
+//---------------------------------------------------------
+
+class ScriptEngine : public QScriptEngine {
+      Q_OBJECT
+
+      ByteArrayClass* baClass;
+      ScScore* scoreClass;
+      ScSCursor* cursorClass;
+
+   public:
+      ScriptEngine();
+      ScScore* getScoreClass() const { return scoreClass; }
+      ScSCursor* getCursorClass() const { return cursorClass; }
+      };
+
 #endif
 
