@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: segment.h,v 1.18 2006/03/30 07:32:34 wschweer Exp $
+//  $Id: segment.h 1851 2009-05-25 18:03:41Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -27,7 +27,6 @@
 #define __SEGMENT_H__
 
 #include "element.h"
-#include "spatium.h"
 
 class Measure;
 class Segment;
@@ -88,6 +87,8 @@ class Segment : public Element {
    public:
       Segment(Measure*);
       Segment(Measure*, int t);
+      ~Segment();
+
       virtual Segment* clone() const    { return new Segment(*this); }
       virtual ElementType type() const  { return SEGMENT; }
 
