@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: select.cpp,v 1.47 2006/03/22 12:04:14 wschweer Exp $
+//  $Id: select.cpp 1986 2009-08-03 09:03:44Z wschweer $
 //
 //  Copyright (C) 2002-2008 Werner Schweer and others
 //
@@ -30,7 +30,6 @@
 #include "rest.h"
 #include "score.h"
 #include "slur.h"
-#include "padstate.h"
 #include "system.h"
 #include "select.h"
 #include "sig.h"
@@ -39,12 +38,11 @@
 #include "segment.h"
 #include "input.h"
 #include "measure.h"
-#include "layout.h"
 #include "page.h"
 #include "barline.h"
 #include "xml.h"
 #include "lyrics.h"
-#include "values.h"
+#include "limits.h"
 #include "tuplet.h"
 
 //---------------------------------------------------------
@@ -53,13 +51,14 @@
 
 Selection::Selection(Score* s)
       {
-      _score = s;
-      _state = SEL_NONE;
-      _startSegment = 0;
-      _endSegment = 0;
+      _score         = s;
+      _state         = SEL_NONE;
+      _startSegment  = 0;
+      _endSegment    = 0;
       _activeSegment = 0;
-      staffStart = 0;
-      staffEnd   = 0;
+      staffStart     = 0;
+      staffEnd       = 0;
+      activeTrack    = 0;
       }
 
 //---------------------------------------------------------
@@ -231,7 +230,7 @@ void Selection::add(Element* el)
 void Score::deselect(Element* obj)
       {
       refresh |= obj->abbox();
-      sel->remove(obj);
+      selection()->remove(obj);
       obj->setSelected(false);
       }
 
@@ -242,7 +241,7 @@ void Score::deselect(Element* obj)
 void Score::updateSelectedElements(SelState state)
       {
       setUpdateAll();
-      QList<Element*>* el = sel->elements();
+      QList<Element*>* el = selection()->elements();
       foreach(Element* e, *el)
             e->setSelected(false);
       el->clear();
@@ -251,26 +250,26 @@ void Score::updateSelectedElements(SelState state)
       //  select all elements in range
       //
       if (state == SEL_SYSTEM) {
-            sel->staffStart = 0;
-            sel->staffEnd = nstaves();
+            _selection->staffStart = 0;
+            _selection->staffEnd = nstaves();
             }
       // assert:
-      if (sel->staffStart < 0 || sel->staffStart >= nstaves() || sel->staffEnd < 0 || sel->staffEnd > nstaves()
-         || sel->staffStart >= sel->staffEnd) {
-            printf("updateSelectedElements: bad staff selection %d - %d\n", sel->staffStart, sel->staffEnd);
-            sel->staffStart = 0;
-            sel->staffEnd   = 1;
+      if (_selection->staffStart < 0 || _selection->staffStart >= nstaves() || _selection->staffEnd < 0 || _selection->staffEnd > nstaves()
+         || _selection->staffStart >= _selection->staffEnd) {
+            printf("updateSelectedElements: bad staff selection %d - %d\n", _selection->staffStart, _selection->staffEnd);
+            _selection->staffStart = 0;
+            _selection->staffEnd   = 1;
             }
-      int startTrack = sel->staffStart * VOICES;
-      int endTrack   = sel->staffEnd * VOICES;
+      int startTrack = _selection->staffStart * VOICES;
+      int endTrack   = _selection->staffEnd * VOICES;
 
       for (int st = startTrack; st < endTrack; ++st) {
-            for (Segment* s = sel->startSegment(); s && (s != sel->endSegment()); s = s->next1()) {
+            for (Segment* s = _selection->startSegment(); s && (s != _selection->endSegment()); s = s->next1()) {
                   Element* e = s->element(st);
                   if (!e)
                         continue;
                   e->setSelected(true);
-                  sel->elements()->append(e);
+                  _selection->elements()->append(e);
                   }
             }
       }
@@ -282,40 +281,36 @@ void Score::updateSelectedElements(SelState state)
 
 void Score::select(Element* e, SelectType type, int staffIdx)
       {
-//printf("select element <%s> type %d(state %d) staff %d\n",
-//   e ? e->name() : "", type, sel->state(), e ? e->staffIdx() : -1);
+// printf("select element <%s> type %d(state %d) staff %d\n",
+//   e ? e->name() : "", type, selection()->state(), e ? e->staffIdx() : -1);
 
-      SelState selState = sel->state();
+      SelState selState = _selection->state();
 
       if (type == SELECT_SINGLE) {
-            refresh |= sel->deselectAll(this);
-            if (e && (e->type() == MEASURE)) {
-                  select(e, SELECT_RANGE, staffIdx);
-                  return;
-                  }
+            refresh |= _selection->deselectAll(this);
             if (e == 0) {
                   selState = SEL_NONE;
                   if (!noteEntryMode())
-                        _padState.len = 0;
+                        //_is.duration.setVal(Duration::V_INVALID);
                   updateAll = true;
                   }
             else {
+                  if (e->type() == MEASURE) {
+                        select(e, SELECT_RANGE, staffIdx);
+                        return;
+                        }
                   refresh |= e->abbox();
                   if (e->selected()) {
-                        sel->remove(e);
+                        _selection->remove(e);
                         selState = SEL_NONE;
                         }
                   else {
-                        sel->add(e);
+                        _selection->add(e);
                         _is.track = e->track();
                         selState = SEL_SINGLE;
                         }
                   if (e->type() == NOTE || e->type() == REST) {
-                        ChordRest* cr = static_cast<ChordRest*>(e);
-                        if (e->type() == NOTE)
-                              cr = static_cast<ChordRest*>(e->parent());
-                        _is.setPos(cr->tick());
-                        _is.cr = cr;
+                        _is.cr = static_cast<ChordRest*>(e->type() == NOTE ? e->parent() : e);
                         emit posChanged(_is.pos());
                         }
                   }
@@ -325,16 +320,16 @@ void Score::select(Element* e, SelectType type, int staffIdx)
                   Measure* m = static_cast<Measure*>(e);
                   int tick  = m->tick();
                   int etick = tick + m->tickLen();
-                  if (sel->state() == SEL_NONE) {
-                        sel->setStartSegment(m->tick2segment(tick, true));
-                        sel->setEndSegment(tick2segment(etick));
+                  if (_selection->state() == SEL_NONE) {
+                        _selection->setStartSegment(m->tick2segment(tick, true));
+                        _selection->setEndSegment(tick2segment(etick));
                         }
-                  else if (sel->state() == SEL_SYSTEM) {
-                        if (tick < sel->tickStart()) {
-                              sel->setStartSegment(m->tick2segment(tick, true));
+                  else if (_selection->state() == SEL_SYSTEM) {
+                        if (tick < _selection->tickStart()) {
+                              _selection->setStartSegment(m->tick2segment(tick, true));
                               }
-                        else if (etick >= sel->tickEnd())
-                              sel->setEndSegment(tick2segment(etick));
+                        else if (etick >= _selection->tickEnd())
+                              _selection->setEndSegment(tick2segment(etick));
                         }
                   else {
                         select(0, SELECT_SINGLE, 0);
@@ -343,17 +338,17 @@ void Score::select(Element* e, SelectType type, int staffIdx)
                   updateAll = true;
                   selState = SEL_SYSTEM;
                   updateSelectedElements(selState);
-                  _padState.len = 0;
+                  //_is.duration.setVal(Duration::V_INVALID);
                   }
             else {
-                  if (sel->state() == SEL_STAFF || sel->state() == SEL_SYSTEM) {
+                  if (_selection->state() == SEL_STAFF || _selection->state() == SEL_SYSTEM) {
                         select(0, SELECT_SINGLE, 0);
                         return;
                         }
                   else {
                         refresh |= e->abbox();
-                        sel->add(e);
-                        _padState.len = 0;
+                        _selection->add(e);
+                        //_is.duration.setVal(Duration::V_INVALID);
                         selState = SEL_MULT;
                         }
                   }
@@ -366,35 +361,35 @@ void Score::select(Element* e, SelectType type, int staffIdx)
                   int tick  = m->tick();
                   int etick = tick + m->tickLen();
                   activeTrack = staffIdx * VOICES;
-                  if (sel->state() == SEL_NONE) {
-                        sel->staffStart = staffIdx;
-                        sel->staffEnd = staffIdx + 1;
-                        sel->setStartSegment(m->tick2segment(tick, true));
-                        sel->setEndSegment(tick2segment(etick));
+                  if (_selection->state() == SEL_NONE) {
+                        _selection->staffStart = staffIdx;
+                        _selection->staffEnd = staffIdx + 1;
+                        _selection->setStartSegment(m->tick2segment(tick, true));
+                        _selection->setEndSegment(tick2segment(etick));
                         }
-                  else if (sel->state() == SEL_STAFF) {
-                        if (staffIdx < sel->staffStart)
-                              sel->staffStart = staffIdx;
-                        else if (staffIdx >= sel->staffEnd)
-                              sel->staffEnd = staffIdx + 1;
-                        if (tick < sel->tickStart()) {
-                              sel->setStartSegment(m->tick2segment(tick, true));
+                  else if (_selection->state() == SEL_STAFF) {
+                        if (staffIdx < _selection->staffStart)
+                              _selection->staffStart = staffIdx;
+                        else if (staffIdx >= _selection->staffEnd)
+                              _selection->staffEnd = staffIdx + 1;
+                        if (tick < _selection->tickStart()) {
+                              _selection->setStartSegment(m->tick2segment(tick, true));
                               activeIsFirst = true;
                               }
-                        else if (etick >= sel->tickEnd())
-                              sel->setEndSegment(tick2segment(etick));
+                        else if (etick >= _selection->tickEnd())
+                              _selection->setEndSegment(tick2segment(etick));
                         else {
-                              if (sel->activeSegment() == sel->startSegment()) {
-                                    sel->setStartSegment(m->tick2segment(tick, true));
+                              if (_selection->activeSegment() == _selection->startSegment()) {
+                                    _selection->setStartSegment(m->tick2segment(tick, true));
                                     activeIsFirst = true;
                                     }
                               else
-                                    sel->setEndSegment(tick2segment(etick));
+                                    _selection->setEndSegment(tick2segment(etick));
                               }
                         }
-                  else if (sel->state() == SEL_SINGLE) {
+                  else if (_selection->state() == SEL_SINGLE) {
                         Segment* seg = 0;
-                        Element* oe = sel->element();
+                        Element* oe = _selection->element();
                         bool reverse = false;
                         if (tick < oe->tick())
                               seg = m->first();
@@ -421,7 +416,7 @@ void Score::select(Element* e, SelectType type, int staffIdx)
                         return;
                         }
                   else {
-                        printf("SELECT_RANGE: measure: sel state %d\n", sel->state());
+                        printf("SELECT_RANGE: measure: sel state %d\n", _selection->state());
                         }
                   }
             else if (e->type() == NOTE || e->type() == REST || e->type() == CHORD) {
@@ -429,106 +424,105 @@ void Score::select(Element* e, SelectType type, int staffIdx)
                         e = static_cast<Note*>(e)->chord();
                   ChordRest* cr = static_cast<ChordRest*>(e);
 
-                  if (sel->state() == SEL_NONE) {
-                        sel->staffStart = e->staffIdx();
-                        sel->staffEnd   = sel->staffStart + 1;
-                        sel->setStartSegment(cr->segment());
-                        sel->setEndSegment(cr->segment()->nextCR(cr->track()));
+                  if (_selection->state() == SEL_NONE) {
+                        _selection->staffStart = e->staffIdx();
+                        _selection->staffEnd   = _selection->staffStart + 1;
+                        _selection->setStartSegment(cr->segment());
+                        _selection->setEndSegment(cr->segment()->nextCR(cr->track()));
                         }
-                  else if (sel->state() == SEL_SINGLE) {
-                        Element* oe = sel->element();
+                  else if (_selection->state() == SEL_SINGLE) {
+                        Element* oe = _selection->element();
                         if (oe->type() == NOTE || oe->type() == REST || oe->type() == CHORD) {
                               if (oe->type() == NOTE)
                                     oe = oe->parent();
                               ChordRest* ocr = static_cast<ChordRest*>(oe);
-                              sel->staffStart = oe->staffIdx();
-                              sel->staffEnd   = sel->staffStart + 1;
-                              sel->setStartSegment(ocr->segment());
-                              sel->setEndSegment(ocr->segment()->nextCR(ocr->track()));
-                              if (!sel->endSegment())
-                                    sel->setEndSegment(ocr->segment()->next());
+                              _selection->staffStart = oe->staffIdx();
+                              _selection->staffEnd   = _selection->staffStart + 1;
+                              _selection->setStartSegment(ocr->segment());
+                              _selection->setEndSegment(ocr->segment()->nextCR(ocr->track()));
+                              if (!_selection->endSegment())
+                                    _selection->setEndSegment(ocr->segment()->next());
 
                               staffIdx = cr->staffIdx();
                               int tick = cr->tick();
-                              if (staffIdx < sel->staffStart)
-                                    sel->staffStart = staffIdx;
-                              else if (staffIdx >= sel->staffEnd)
-                                    sel->staffEnd = staffIdx + 1;
-                              if (tick < sel->tickStart()) {
-                                    sel->setStartSegment(cr->segment());
+                              if (staffIdx < _selection->staffStart)
+                                    _selection->staffStart = staffIdx;
+                              else if (staffIdx >= _selection->staffEnd)
+                                    _selection->staffEnd = staffIdx + 1;
+                              if (tick < _selection->tickStart()) {
+                                    _selection->setStartSegment(cr->segment());
                                     activeIsFirst = true;
                                     }
-                              else if (tick >= sel->tickEnd())
-                                    sel->setEndSegment(cr->segment()->nextCR(cr->track()));
+                              else if (tick >= _selection->tickEnd())
+                                    _selection->setEndSegment(cr->segment()->nextCR(cr->track()));
                               else {
-                                    if (sel->activeSegment() == sel->startSegment()) {
-                                          sel->setStartSegment(cr->segment());
+                                    if (_selection->activeSegment() == _selection->startSegment()) {
+                                          _selection->setStartSegment(cr->segment());
                                           activeIsFirst = true;
                                           }
                                     else
-                                          sel->setEndSegment(cr->segment()->nextCR(cr->track()));
+                                          _selection->setEndSegment(cr->segment()->nextCR(cr->track()));
                                     }
                               }
                         else {
-printf("select: TODO\n");
-                              select(0, SELECT_SINGLE, 0);
+// printf("select: TODO\n");
+                              select(e, SELECT_SINGLE, 0);
                               return;
                               }
                         }
-                  else if (sel->state() == SEL_STAFF) {
+                  else if (_selection->state() == SEL_STAFF) {
                         staffIdx = cr->staffIdx();
                         int tick = cr->tick();
-                        if (staffIdx < sel->staffStart)
-                              sel->staffStart = staffIdx;
-                        else if (staffIdx >= sel->staffEnd)
-                              sel->staffEnd = staffIdx + 1;
-                        if (tick < sel->tickStart()) {
-                              if (sel->activeSegment() == sel->endSegment())
-                                    sel->setEndSegment(sel->startSegment());
-                              sel->setStartSegment(cr->segment());
+                        if (staffIdx < _selection->staffStart)
+                              _selection->staffStart = staffIdx;
+                        else if (staffIdx >= _selection->staffEnd)
+                              _selection->staffEnd = staffIdx + 1;
+                        if (tick < _selection->tickStart()) {
+                              if (_selection->activeSegment() == _selection->endSegment())
+                                    _selection->setEndSegment(_selection->startSegment());
+                              _selection->setStartSegment(cr->segment());
                               activeIsFirst = true;
                               }
-                        else if (tick >= sel->tickEnd()) {
-                              if (sel->activeSegment() == sel->startSegment())
-                                    sel->setStartSegment(sel->endSegment());
-                              sel->setEndSegment(cr->segment()->nextCR(cr->track()));
+                        else if (tick >= _selection->tickEnd()) {
+                              if (_selection->activeSegment() == _selection->startSegment())
+                                    _selection->setStartSegment(_selection->endSegment());
+                              _selection->setEndSegment(cr->segment()->nextCR(cr->track()));
                               }
                         else {
-                              if (sel->activeSegment() == sel->startSegment()) {
-                                    sel->setStartSegment(cr->segment());
+                              if (_selection->activeSegment() == _selection->startSegment()) {
+                                    _selection->setStartSegment(cr->segment());
                                     activeIsFirst = true;
                                     }
                               else
-                                    sel->setEndSegment(cr->segment()->nextCR(cr->track()));
+                                    _selection->setEndSegment(cr->segment()->nextCR(cr->track()));
                               }
                         }
                   else {
-                        printf("sel state %d\n", sel->state());
+                        printf("sel state %d\n", _selection->state());
                         }
                   selState = SEL_STAFF;
-                  if (!sel->endSegment())
-                        sel->setEndSegment(cr->segment()->next());
+                  if (!_selection->endSegment())
+                        _selection->setEndSegment(cr->segment()->next());
                   }
             else {
                   select(e, SELECT_SINGLE, staffIdx);
                   return;
                   }
 
-// printf("range %d-%d %d-%d\n", sel->staffStart, sel->staffEnd, sel->tickStart(), sel->tickEnd());
+// printf("range %d-%d %d-%d\n", _selection->staffStart, _selection->staffEnd, _selection->tickStart(), _selection->tickEnd());
             if (activeIsFirst)
-                  sel->setActiveSegment(sel->startSegment());
+                  _selection->setActiveSegment(_selection->startSegment());
             else
-                  sel->setActiveSegment(sel->endSegment());
+                  _selection->setActiveSegment(_selection->endSegment());
 
-            sel->activeTrack = activeTrack;
+            _selection->activeTrack = activeTrack;
 
             selState = SEL_STAFF;
             updateSelectedElements(selState);
-            _padState.len = 0;
+            //_is.duration.setVal(Duration::V_INVALID);
             }
-
-      sel->setState(selState);
-      emit selectionChanged(int(sel->state()));
+      _selection->setState(selState);
+      emit selectionChanged(int(_selection->state()));
       }
 
 //---------------------------------------------------------
@@ -539,7 +533,7 @@ void Score::lassoSelect(const QRectF& bbox)
       {
       select(0, SELECT_SINGLE, 0);
       QRectF lr(bbox.normalized());
-      QList<const Element*> el = _layout->items(lr);
+      QList<const Element*> el = items(lr);
       for (int i = 0; i < el.size(); ++i) {
             const Element* e = el.at(i);
             e->itemDiscovered = 0;
@@ -576,7 +570,7 @@ void Score::lassoSelectEnd(const QRectF& /*bbox*/)
       int endStaff          = 0;
       int endTrack          = 0;
 
-      foreach(const Element* e, *(sel->elements())) {
+      foreach(const Element* e, *(_selection->elements())) {
             if (e->type() == NOTE || e->type() == REST) {
                   ++noteRestCount;
                   if (e->type() == NOTE)
@@ -597,10 +591,10 @@ void Score::lassoSelectEnd(const QRectF& /*bbox*/)
             }
       if (noteRestCount > 0) {
             endSegment = endSegment->nextCR(endTrack);
-            sel->setRange(startSegment, endSegment, startStaff, endStaff+1);
-            if (sel->state() != SEL_STAFF) {
-                  sel->setState(SEL_STAFF);
-                  emit selectionChanged(int(sel->state()));
+            _selection->setRange(startSegment, endSegment, startStaff, endStaff+1);
+            if (_selection->state() != SEL_STAFF) {
+                  _selection->setState(SEL_STAFF);
+                  emit selectionChanged(int(_selection->state()));
                   }
             }
       updateAll = true;
@@ -618,18 +612,17 @@ void Score::lassoSelectEnd(const QRectF& /*bbox*/)
 void Score::searchSelectedElements()
       {
       QList<const Element*> l;
-      for(MeasureBase* m = _layout->first(); m; m = m->next())
+      for(MeasureBase* m = first(); m; m = m->next())
             m->collectElements(l);
-      foreach(const Page* page, _layout->pages())
+      foreach(const Page* page, pages())
             page->collectElements(l);
-      sel->clear();
+      _selection->clear();
       foreach(const Element* e, l) {
-            if (e->selected()) {
-                  sel->append(const_cast<Element*>(e));
-                  }
+            if (e->selected())
+                  _selection->append(const_cast<Element*>(e));
             }
-      sel->updateState();
-      emit selectionChanged(int(sel->state()));
+      _selection->updateState();
+      emit selectionChanged(int(_selection->state()));
       }
 
 //---------------------------------------------------------
@@ -678,18 +671,18 @@ void Selection::updateState()
       {
       int n = _el.size();
       if (_state == SEL_NONE || _state == SEL_SINGLE || _state == SEL_MULT) {
-            int n = _el.size();
             if (n == 0)
                   setState(SEL_NONE);
             else if (n == 1) {
                   setState(SEL_SINGLE);
                   Element* e = element();
                   if (e->type() == NOTE || e->type() == REST) {
-                        if (!e->score()->noteEntryMode())
-                              e->score()->setPadState(e);
+                        if (!_score->noteEntryMode())
+                              _score->setPadState(e);
+                        e->setSelected(true);
                         if (e->type() == NOTE)
                               e = e->parent();
-                        e->score()->setInputTrack(e->track());
+                        _score->setInputTrack(e->track());
                         }
                   }
             else
@@ -797,8 +790,10 @@ QByteArray Selection::staffMimeData() const
                         if ((track % VOICES) == 0) {
                               int staffIdx = track / VOICES;
                               LyricsList* ll = seg->lyricsList(staffIdx);
-                              foreach(Lyrics* l, *ll)
-                                    l->write(xml);
+                              foreach(Lyrics* l, *ll) {
+                                    if (l)
+                                          l->write(xml);
+                                    }
                               }
                         }
                   }
@@ -810,7 +805,7 @@ QByteArray Selection::staffMimeData() const
                         break;
                   foreach(Element* e, *m->el()) {
                         if (e->type() == HARMONY) {
-                              if (e->tick() < seg1->tick())
+                              if ((e->staffIdx() != staffIdx) || (e->tick() < seg1->tick()))
                                     continue;
                               if (seg2 && (e->tick() >= seg2->tick()))
                                     continue;
