@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: seq.cpp,v 1.46 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: seq.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -18,9 +18,10 @@
 //  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //=============================================================================
 
+#include "config.h"
 #include "seq.h"
 #include "mscore.h"
-#include "fluid.h"
+
 #ifdef USE_JACK
 #include "jackaudio.h"
 #endif
@@ -33,6 +34,7 @@
 #include "pa.h"
 #endif
 
+#include "synti.h"
 #include "slur.h"
 #include "score.h"
 #include "segment.h"
@@ -43,11 +45,11 @@
 #include "playpanel.h"
 #include "staff.h"
 #include "measure.h"
-#include "layout.h"
 #include "preferences.h"
 #include "part.h"
 #include "ottava.h"
 #include "utils.h"
+#include "repeatlist.h"
 
 Seq* seq;
 
@@ -69,9 +71,20 @@ Seq::Seq()
       _volume  = 1.0;
       playPos  = events.constBegin();
 
+      playTime = 0.0;
+      startTime = 0.0;
+      curTick   = 0;
+      curUtick  = 0;
+
       heartBeatTimer = new QTimer(this);
       connect(heartBeatTimer, SIGNAL(timeout()), this, SLOT(heartBeat()));
       heartBeatTimer->stop();
+
+      noteTimer = new QTimer(this);
+      noteTimer->setSingleShot(true);
+      connect(noteTimer, SIGNAL(timeout()), this, SLOT(stopNotes()));
+      noteTimer->stop();
+
       connect(this, SIGNAL(toGui(int)), this, SLOT(seqMessage(int)), Qt::QueuedConnection);
       }
 
@@ -81,8 +94,7 @@ Seq::Seq()
 
 Seq::~Seq()
       {
-      if (driver)
-            delete driver;
+      delete driver;
       }
 
 //---------------------------------------------------------
@@ -93,6 +105,7 @@ void Seq::setScore(Score* s)
       {
       if (cs) {
             disconnect(cs, SIGNAL(selectionChanged(int)), this, SLOT(selectionChanged(int)));
+            cs = s;
             stop();
 #ifndef __MINGW32__
             while (state != STOP)
@@ -101,7 +114,10 @@ void Seq::setScore(Score* s)
             }
       cs = s;
       playlistChanged = true;
-      connect(cs, SIGNAL(selectionChanged(int)), SLOT(selectionChanged(int)));
+      if (cs) {
+            connect(cs, SIGNAL(selectionChanged(int)), SLOT(selectionChanged(int)));
+            initInstruments();
+            }
       }
 
 //---------------------------------------------------------
@@ -196,6 +212,34 @@ bool Seq::init()
             printf("init audio driver failed\n");
             return false;
             }
+      Synth* synth = driver->getSynth();
+      if (synth) {
+            QString p;
+            if (!preferences.soundFont.isEmpty())
+                  p = preferences.soundFont;
+            else
+                  p = QString(getenv("DEFAULT_SOUNDFONT"));
+            if (p.isEmpty()) {
+                  //
+                  // fallback to integrated soundfont
+                  //
+                  p = ":/data/piano1.sf2";
+                  }
+            if (debugMode)
+                  printf("load soundfont <%s>\n", qPrintable(p));
+            bool rv = synth->loadSoundFont(p);
+            if (!rv) {
+                  QString s = QString("Loading Soundfont\n"
+                                      "\"%1\"\n"
+                                      "failed.\n"
+                                      "Sequencer will be disabled.").arg(p);
+                  QMessageBox::critical(0, "MuseScore: Load SoundFont", s);
+                  delete driver;
+                  driver = 0;
+                  return false;
+                  }
+            }
+
       if (!driver->start()) {
             printf("Cannot start I/O\n");
             return false;
@@ -228,24 +272,6 @@ int Seq::sampleRate() const
       return 44100;
       }
 
-//---------------------------------------------------------
-//   time2tick
-//---------------------------------------------------------
-
-int Seq::time2tick(double time) const
-      {
-      return cs->tempomap->time2tick(time);
-      }
-
-//---------------------------------------------------------
-//   tick2time
-//---------------------------------------------------------
-
-double Seq::tick2time(int tick) const
-      {
-      return cs->tempomap->tick2time(tick);
-      }
-
 //---------------------------------------------------------
 //   inputPorts
 //---------------------------------------------------------
@@ -285,6 +311,8 @@ void Seq::start()
       {
       if (!driver)
             return;
+      if (cs->noteEntryMode())
+            cs->setNoteEntry(false);
       QAction* a = getAction("play");
       if (!a->isChecked()) {
             if (pauseState) {
@@ -387,6 +415,8 @@ void MuseScore::seqStopped()
 
 void Seq::guiStop()
       {
+      if (!cs)
+            return;
       if (!pauseState) {
             QAction* a = getAction("play");
             a->setChecked(false);
@@ -395,12 +425,12 @@ void Seq::guiStop()
       //
       // deselect all selected notes
       //
-      foreach(const NoteOn* n, markedNotes) {
-            n->note()->setSelected(false);
-            cs->addRefresh(n->note()->abbox());
+      foreach(Note* n, markedNotes) {
+            n->setSelected(false);
+            cs->addRefresh(n->abbox());
             }
       markedNotes.clear();
-      cs->setPlayPos(time2tick(playTime));
+      cs->setPlayPos(cs->utime2utick(playTime));
       cs->end();
       if (!pauseState)
             emit stopped();
@@ -443,13 +473,9 @@ void Seq::stopTransport()
       foreach(const Event* e, activeNotes) {
             if (e->type() != ME_NOTEON)
                   continue;
-            const NoteOn* no = static_cast<const NoteOn*>(e);
-            MidiOutEvent e;
-            e.port = cs->midiPort(no->channel());
-            e.type = ME_NOTEON | cs->midiChannel(no->channel());
-            e.a    = no->pitch();
-            e.b    = 0;
-            driver->putEvent(e);
+            Event ee(*e);
+            ee.setVelo(0);
+            driver->putEvent(ee);
             }
       activeNotes.clear();
       emit toGui('0');
@@ -479,54 +505,38 @@ void Seq::playEvent(const Event* event)
       {
       int type = event->type();
       if (type == ME_NOTEON) {
-            NoteOn* n = (NoteOn*) event;
-
-            int channel = n->channel();
-            MidiOutEvent e;
-            e.port = cs->midiPort(channel);
-            e.type = ME_NOTEON | cs->midiChannel(channel);
-            e.a    = n->pitch();
-            e.b    = n->velo();
-
             bool mute;
-            Note* note = n->note();
+            Note* note = event->note();
+
             if (note) {
                   Instrument* instr = note->staff()->part()->instrument();
                   Channel* a = instr->channel[note->subchannel()];
                   mute = a->mute || a->soloMute;
                   }
-            else {
+            else
                   mute = false;
-                  }
 
-            if (n->velo()) {
+            if (event->velo()) {
                   if (!mute) {
-                        driver->putEvent(e);
-                        activeNotes.append(n);
+                        driver->putEvent(*event);
+                        activeNotes.append(event);
                         }
                   }
             else {
-                  for (QList<NoteOn*>::iterator k = activeNotes.begin(); k != activeNotes.end(); ++k) {
-                        NoteOn* l = *k;
-                        if (l->channel() == channel && l->pitch() == n->pitch()) {
+                  for (QList<const Event*>::iterator k = activeNotes.begin(); k != activeNotes.end(); ++k) {
+                        const Event* l = *k;
+                        if (l->channel() == event->channel() && l->pitch() == event->pitch()) {
+                              Event ee(*l);
+                              ee.setVelo(0);
                               activeNotes.erase(k);
-                              driver->putEvent(e);
+                              driver->putEvent(ee);
                               break;
                               }
                         }
                   }
             }
-      else if (type == ME_CONTROLLER)  {
-            const ControllerEvent* c = static_cast<const ControllerEvent*>(event);
-            QList<MidiOutEvent> ol;
-            if (c->midiOutEvent(&ol, cs)) {
-                  foreach(const MidiOutEvent& e, ol)
-                        driver->putEvent(e);
-                  }
-            }
-      else {
-            printf("bad event type %x\n", type);
-            }
+      else if (type == ME_CONTROLLER)
+            driver->putEvent(*event);
       }
 
 //---------------------------------------------------------
@@ -542,14 +552,19 @@ void Seq::processMessages()
             switch(msg.id) {
                   case SEQ_TEMPO_CHANGE:
                         {
-                        int tick = time2tick(playTime);
-                        cs->tempomap->setRelTempo(msg.data);
-                        playTime = tick2time(tick);
-                        startTime = curTime() - playTime;
+                        if (playTime != 0) {
+                              int tick = cs->utime2utick(playTime);
+                              cs->getTempomap()->setRelTempo(msg.data);
+                              cs->repeatList()->update();
+                              playTime = cs->utick2utime(tick);
+                              startTime = curTime() - playTime;
+                              }
+                        else
+                              cs->getTempomap()->setRelTempo(msg.data);
                         }
                         break;
                   case SEQ_PLAY:
-                        driver->putEvent(msg.midiOutEvent);
+                        driver->putEvent(msg.event);
                         break;
                   case SEQ_SEEK:
                         setPos(msg.data);
@@ -584,7 +599,7 @@ void Seq::processMidi()
       if (state == PLAY) {
             double endTime = curTime();
             for (; playPos != events.constEnd(); ++playPos) {
-                  playTime = tick2time(playPos.key());
+                  playTime = cs->utick2utime(playPos.key());
                   double t = startTime + playTime;
                   if (t >= endTime)
                         break;
@@ -631,14 +646,20 @@ void Seq::process(unsigned n, float* lbuffer, float* rbuffer, int stride)
             //
             double endTime = playTime + double(frames)/double(sampleRate());
             for (; playPos != events.constEnd(); ++playPos) {
-                  double f = tick2time(playPos.key());
+                  double f = cs->utick2utime(playPos.key());
                   if (f >= endTime)
                         break;
                   int n = lrint((f - playTime) * sampleRate());
+
+                  if (n < 0) {
+                        printf("%d:  %f - %f\n", playPos.key(), f, playTime);
+                        abort();
+                        }
                   driver->process(n, l, r, stride);
                   l         += n * stride;
                   r         += n * stride;
                   playTime += double(n)/double(sampleRate());
+
                   frames    -= n;
                   playEvent(playPos.value());
                   }
@@ -669,21 +690,16 @@ void Seq::process(unsigned n, float* lbuffer, float* rbuffer, int stride)
 
 void Seq::initInstruments()
       {
-      foreach(Part* part, *cs->parts()) {
+      foreach(const Part* part, *cs->parts()) {
             const Instrument* instr = part->instrument();
 
             foreach(const Channel* a, instr->channel) {
                   foreach(Event* e, a->init) {
                         if (e == 0)
                               continue;
-                        e->setChannel(a->channel);
-                        QList<MidiOutEvent> el;
-                        if (e->midiOutEvent(&el, cs)) {
-                              foreach(const MidiOutEvent& event, el)
-                                    sendEvent(event);
-                              }
-                        else
-                              printf("unknown event\n");
+                        Event ee(*e);
+                        ee.setChannel(a->channel);
+                        sendEvent(ee);
                         }
                   }
             }
@@ -697,10 +713,9 @@ void Seq::collectEvents()
       {
       foreach(Event* e, events)
             delete e;
-
       events.clear();
 
-      cs->toEList(&events, 0);
+      cs->toEList(&events);
       endTick = 0;
       if (!events.empty()) {
             EventMap::const_iterator e = events.constEnd();
@@ -713,6 +728,25 @@ void Seq::collectEvents()
             pp->setEndpos(endTick);
       }
 
+//---------------------------------------------------------
+//   getCurTime
+//---------------------------------------------------------
+
+int Seq::getCurTime()
+      {
+      return lrint(curTime() - startTime);
+      }
+
+//---------------------------------------------------------
+//   getCurTick
+//---------------------------------------------------------
+
+void Seq::getCurTick(int* tick, int* utick)
+      {
+      *tick  = curTick;
+      *utick = curUtick;
+      }
+
 //---------------------------------------------------------
 //   heartBeat
 //    paint currently sounding notes
@@ -729,29 +763,41 @@ void Seq::heartBeat()
             pp->heartBeat2(lrint(endTime));
       Note* note = 0;
       for (; guiPos != events.constEnd(); ++guiPos) {
-            double f = tick2time(guiPos.key());
+            double f = cs->utick2utime(guiPos.key());
             if (f >= endTime)
                   break;
             if (guiPos.value()->type() == ME_NOTEON) {
-                  NoteOn* n = (NoteOn*)guiPos.value();
-                  n->note()->setSelected(n->velo());
-                  cs->addRefresh(n->note()->abbox());
+                  Event* n = guiPos.value();
+                  Note* note1 = n->note();
                   if (n->velo()) {
-                        markedNotes.append(n);
-                        note = n->note();
+                        note = note1;
+                        while (note1) {
+                              note1->setSelected(true);
+                              markedNotes.append(note1);
+                              cs->addRefresh(note1->abbox());
+                              note1 = note1->tieFor() ? note1->tieFor()->endNote() : 0;
+                              }
+
                         }
                   else {
-                        markedNotes.removeAll(n);
+                        while (note1) {
+                              note1->setSelected(false);
+                              cs->addRefresh(note1->abbox());
+                              markedNotes.removeOne(note1);
+                              note1 = note1->tieFor() ? note1->tieFor()->endNote() : 0;
+                              }
                         }
                   }
             }
       if (note) {
             foreach(Viewer* v, cs->getViewer())
-                  v->moveCursor(note->chord()->segment());
+                  v->moveCursor(note->chord()->segment(), -1);
             cs->adjustCanvasPosition(note, true);
+            curTick  = note->chord()->tick();
+            curUtick = guiPos.key();
             if (pp)
-                  pp->heartBeat(note->chord()->tick(), guiPos.key());
-            mscore->setPos(note->chord()->tick());
+                  pp->heartBeat(curTick, curUtick);
+            mscore->setPos(curTick);
             }
       cs->end();
       }
@@ -776,11 +822,11 @@ void Seq::setRelTempo(int relTempo)
       msg.id   = SEQ_TEMPO_CHANGE;
       guiToSeq(msg);
 
-      double tempo = cs->tempomap->tempo(playPos.key()) * relTempo * 0.01;
+      double t = cs->getTempomap()->tempo(playPos.key()) * relTempo * 0.01;
 
       PlayPanel* pp = mscore->getPlayPanel();
       if (pp) {
-            pp->setTempo(tempo);
+            pp->setTempo(t);
             pp->setRelTempo(relTempo);
             }
       }
@@ -791,21 +837,18 @@ void Seq::setRelTempo(int relTempo)
 //    realtime environment
 //---------------------------------------------------------
 
-void Seq::setPos(int tick)
+void Seq::setPos(int utick)
       {
       // send note off events
-      foreach(const NoteOn* n, activeNotes) {
-            MidiOutEvent e;
-            e.port = 0;
-            e.type = ME_NOTEON | n->channel();
-            e.a    = n->pitch();
-            e.b    = 0;
+      foreach(const Event* n, activeNotes) {
+            Event e(*n);
+            e.setVelo(0);
             driver->putEvent(e);
             }
       activeNotes.clear();
-      playTime  = tick2time(tick);
+      playTime  = cs->utick2utime(utick);
       startTime = curTime() - playTime;
-      playPos   = events.lowerBound(tick);
+      playPos   = events.lowerBound(utick);
       guiPos    = playPos;
       }
 
@@ -819,7 +862,7 @@ void Seq::seek(int tick)
       Segment* seg = cs->tick2segment(tick);
       if (seg) {
             foreach(Viewer* v, cs->getViewer())
-                  v->moveCursor(seg);
+                  v->moveCursor(seg, -1);
             }
       cs->setPlayPos(tick);
       cs->end();
@@ -835,49 +878,50 @@ void Seq::seek(int tick)
 //   startNote
 //---------------------------------------------------------
 
-void Seq::startNote(Channel* a, int pitch, int velo)
+void Seq::startNote(Channel* a, int pitch, int velo, double nt)
       {
       if (state != STOP)
             return;
 
       bool active = false;
-      int port    = cs->midiPort(a->channel);
-      int channel = cs->midiChannel(a->channel);
+//      int port    = cs->midiPort(a->channel);
+//      int channel = cs->midiChannel(a->channel);
 
+      //
+      // Check if there is already a note sounding
+      // for channel/pitch. If found, stop note by
+      // sending a note off event
+      //
       foreach(const Event* event, eventList) {
-            NoteOn* n = (NoteOn*)event;
-            if (n->channel() == a->channel && n->pitch() == pitch) {
-                  MidiOutEvent ev;
-                  ev.port = port;
-                  ev.type = ME_NOTEON | channel;
-                  ev.a    = n->pitch();
-                  ev.b    = 0;
-                  sendEvent(ev);
+            if (event->channel() == a->channel && event->pitch() == pitch) {
+                  sendEvent(*event);
                   active = true;
                   break;
                   }
             }
 
-      MidiOutEvent ev;
-      ev.port = port;
-      ev.type = ME_NOTEON | channel;
-      ev.a    = pitch;
-      ev.b    = velo;
+      Event ev(ME_NOTEON);
+      ev.setChannel(a->channel);
+      ev.setPitch(pitch);
+      ev.setTuning(nt);
+      ev.setVelo(velo);
       sendEvent(ev);
 
       if (!active) {
-            NoteOn* e = new NoteOn;
+            Event* e = new Event(ME_NOTEON);
             e->setChannel(a->channel);
             e->setPitch(pitch);
-            e->setVelo(velo);
+            e->setVelo(0);
             eventList.append(e);
             }
       }
 
-void Seq::startNote(Channel* a, int pitch, int velo, int duration)
+void Seq::startNote(Channel* a, int pitch, int velo, int duration, double nt)
       {
-      startNote(a, pitch, velo);
-      QTimer::singleShot(duration, this, SLOT(stopNotes()));
+      stopNotes();
+      startNote(a, pitch, velo, nt);
+      noteTimer->setInterval(duration);
+      noteTimer->start();
       }
 
 //---------------------------------------------------------
@@ -888,15 +932,7 @@ void Seq::startNote(Channel* a, int pitch, int velo, int duration)
 void Seq::stopNotes()
       {
       foreach(const Event* event, eventList) {
-            NoteOn* n = (NoteOn*)event;
-            int port    = cs->midiPort(n->channel());
-            int channel = cs->midiChannel(n->channel());
-            MidiOutEvent ev;
-            ev.port = port;
-            ev.type = ME_NOTEON | channel;
-            ev.a    = n->pitch();
-            ev.b    = 0;
-            sendEvent(ev);
+            sendEvent(*event);
             delete event;
             }
       eventList.clear();
@@ -908,11 +944,10 @@ void Seq::stopNotes()
 
 void Seq::setController(int idx, int ctrl, int data)
       {
-      MidiOutEvent event;
-      event.port = cs->midiPort(idx);
-      event.type = ME_CONTROLLER | cs->midiChannel(idx);
-      event.a    = ctrl;
-      event.b    = data;
+      Event event(ME_CONTROLLER);
+      event.setChannel(idx);
+      event.setController(ctrl);
+      event.setValue(data);
       sendEvent(event);
       }
 
@@ -922,11 +957,11 @@ void Seq::setController(int idx, int ctrl, int data)
 //    midi out or synthesizer
 //---------------------------------------------------------
 
-void Seq::sendEvent(const MidiOutEvent& ev)
+void Seq::sendEvent(const Event& ev)
       {
       SeqMsg msg;
-      msg.id = SEQ_PLAY;
-      msg.midiOutEvent = ev;
+      msg.id    = SEQ_PLAY;
+      msg.event = ev;
       guiToSeq(msg);
       }
 
@@ -940,7 +975,7 @@ void Seq::nextMeasure()
       Note* note = 0;
       for (;;) {
             if (i.value()->type() == ME_NOTEON) {
-                  NoteOn* n = (NoteOn*)i.value();
+                  Event* n = i.value();
                   note = n->note();
                   break;
                   }
@@ -970,7 +1005,7 @@ void Seq::nextChord()
       for (EventMap::const_iterator i = playPos; i != events.constEnd(); ++i) {
             if (i.value()->type() != ME_NOTEON)
                   continue;
-            NoteOn* n = (NoteOn*)i.value();
+            Event* n = i.value();
             if (i.key() > tick && n->velo()) {
                   seek(i.key());
                   break;
@@ -988,7 +1023,7 @@ void Seq::prevMeasure()
       Note* note = 0;
       for (;;) {
             if (i.value()->type() == ME_NOTEON) {
-                  note = ((NoteOn*)i.value())->note();
+                  note = i.value()->note();
                   break;
                   }
             if (i == events.begin())
@@ -1020,7 +1055,7 @@ void Seq::prevChord()
       EventMap::const_iterator i = playPos;
       for (;;) {
             if (i.value()->type() == ME_NOTEON) {
-                  NoteOn* n = (NoteOn*)i.value();
+                  Event* n = i.value();
                   if (i.key() < tick && n->velo()) {
                         seek(i.key());
                         break;
@@ -1069,9 +1104,8 @@ const MidiPatch* Seq::getPatchInfo(bool onlyDrums, const MidiPatch* p)
 
 void Seq::midiInputReady()
       {
-      if (driver && cs) {
+      if (driver && cs)
             driver->midiRead();
-            }
       }
 
 //---------------------------------------------------------
@@ -1108,3 +1142,4 @@ SeqMsg SeqMsgFifo::dequeue()
       pop();
       return msg;
       }
+
