@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: seq.h,v 1.20 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: seq.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -24,8 +24,8 @@
 #include "event.h"
 #include "driver.h"
 #include "fifo.h"
+#include "tempo.h"
 
-class Synth;
 class Note;
 class QTimer;
 class Score;
@@ -35,16 +35,6 @@ class Driver;
 class Part;
 struct Channel;
 
-//---------------------------------------------------------
-//   MidiPatch
-//---------------------------------------------------------
-
-struct MidiPatch {
-      bool drum;
-      signed char hbank, lbank, prog;
-      const char* name;
-      };
-
 //---------------------------------------------------------
 //   SeqMsg
 //    message format for gui -> sequencer messages
@@ -55,7 +45,7 @@ enum { SEQ_NO_MESSAGE, SEQ_TEMPO_CHANGE, SEQ_PLAY, SEQ_SEEK };
 struct SeqMsg {
       int id;
       int data;
-      MidiOutEvent midiOutEvent;
+      Event event;
       };
 
 //---------------------------------------------------------
@@ -93,19 +83,23 @@ class Seq : public QObject {
       Driver* driver;
 
       EventMap events;                    // playlist
-      QList<NoteOn*> activeNotes;         // notes sounding
+
+      QList<const Event*> activeNotes;   // notes sounding
       double playTime;
       double startTime;
 
       EventMap::const_iterator playPos;   // moved in real time thread
       EventMap::const_iterator guiPos;    // moved in gui thread
-      QList<NoteOn*> markedNotes;         // notes marked as sounding
+      QList<Note*> markedNotes;           // notes marked as sounding
 
       int endTick;
+      int curTick;
+      int curUtick;
 
       float _volume;
 
       QTimer* heartBeatTimer;
+      QTimer* noteTimer;
 
       QList<Event*> eventList;
 
@@ -114,14 +108,12 @@ class Seq : public QObject {
 
       void stopTransport();
       void startTransport();
-      int time2tick(double) const;
-      double tick2time(int tick) const;
       void setPos(int);
       void playEvent(const Event*);
-      void playEvent(const MidiOutEvent&);
+      void playEvent(const Event&);
       void guiStop();
       void guiToSeq(const SeqMsg& msg);
-      void startNote(Channel*, int, int);
+      void startNote(Channel*, int, int, double nt);
 
    private slots:
       void seqMessage(int msg);
@@ -171,14 +163,17 @@ class Seq : public QObject {
       float volume() const      { return _volume;  }
       bool isRealtime() const   { return true;     }
       void sendMessage(SeqMsg&) const;
-      void startNote(Channel*, int, int, int);
+      void startNote(Channel*, int, int, int, double nt);
       void setController(int, int, int);
-      void sendEvent(const MidiOutEvent&);
+      void sendEvent(const Event&);
       void setScore(Score* s);
+      Score* score() const { return cs; }
       void initInstruments();
 
       const MidiPatch* getPatchInfo(bool onlyDrums, const MidiPatch* p);
       Driver* getDriver()  { return driver; }
+      int getCurTime();
+      void getCurTick(int*, int*);
       };
 
 extern Seq* seq;
