@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: sig.cpp,v 1.11 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: sig.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -80,6 +80,15 @@ bool SigEvent::operator==(const SigEvent& e) const
          && e.nominator2 == nominator2 && e.denominator2 == denominator2;
       }
 
+//---------------------------------------------------------
+//   SigList
+//---------------------------------------------------------
+
+SigList::SigList()
+      {
+      _serial = 0;
+      }
+
 //---------------------------------------------------------
 //   add
 //---------------------------------------------------------
@@ -169,6 +178,7 @@ void SigList::normalize()
                   e.ticks = tm;
                   }
             }
+      ++_serial;
       }
 
 //---------------------------------------------------------
