@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: slur.cpp,v 1.53 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: slur.cpp 1854 2009-05-28 08:16:55Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -29,7 +29,6 @@
 #include "system.h"
 #include "segment.h"
 #include "staff.h"
-#include "layout.h"
 #include "viewer.h"
 #include "navigate.h"
 #include "canvas.h"
@@ -63,11 +62,14 @@ SlurSegment::SlurSegment(const SlurSegment& b)
 
 void SlurSegment::updatePath()
       {
+      double _spatium = spatium();
+
       QPointF pp[4];
       for (int i = 0; i < 4; ++i)
-            pp[i] = ups[i].pos();
+            pp[i] = ups[i].p + ups[i].off * _spatium;
       path = QPainterPath();
-      QPointF t(0.0, _spatium * .08);    // thickness of slur
+      QPointF t(0.0, spatium() * .08);    // thickness of slur
+
       path.moveTo(pp[0]);
       path.cubicTo(pp[1]-t, pp[2]-t, pp[3]);
       path.cubicTo(pp[2]+t, pp[1]+t, pp[0]);
@@ -103,7 +105,7 @@ void SlurSegment::updateGrips(int* n, QRectF* r) const
       *n = 4;
       QPointF p(canvasPos());
       for (int i = 0; i < 4; ++i)
-            r[i].translate(ups[i].pos() + p);
+            r[i].translate(ups[i].p + ups[i].off * spatium() + p);
       }
 
 //---------------------------------------------------------
@@ -120,7 +122,7 @@ bool SlurSegment::startEdit(Viewer*, const QPointF&)
 //    return true if event is accepted
 //---------------------------------------------------------
 
-bool SlurSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modifiers, const QString& s)
+bool SlurSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modifiers, const QString&)
       {
       if (slurTie()->type() != SLUR)
             return false;
@@ -128,7 +130,7 @@ bool SlurSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modi
 
       if (key == Qt::Key_X) {
             sl->setSlurDirection(sl->isUp() ? DOWN : UP);
-            sl->layout(score()->layout());
+            sl->layout();
             return true;
             }
       if (!((modifiers & Qt::ShiftModifier)
@@ -162,7 +164,7 @@ bool SlurSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modi
             }
 
       ups[curGrip].off = QPointF();
-      sl->layout(score()->layout());
+      sl->layout();
       if (sl->slurSegments()->size() != segments) {
             QList<SlurSegment*>* ss = sl->slurSegments();
             SlurSegment* newSegment = curGrip == 3 ? ss->back() : ss->front();
@@ -220,14 +222,16 @@ QPointF SlurSegment::gripAnchor(int grip) const
 
 void SlurSegment::editDrag(int curGrip, const QPointF& delta)
       {
-      ups[curGrip].off += (delta / _spatium);
+      double _spatium = spatium();
+
+      ups[curGrip].off += (delta / spatium());
 
       if (curGrip == 0 || curGrip == 3) {
             //
             //  compute bezier help points
             //
-            QPointF p0 = ups[0].pos();
-            QPointF p3 = ups[3].pos();
+            QPointF p0 = ups[0].p + ups[0].off * _spatium;
+            QPointF p3 = ups[3].p + ups[3].off * _spatium;
 
             qreal xdelta = p3.x() - p0.x();
             if (xdelta == 0.0) {
@@ -254,8 +258,7 @@ void SlurSegment::editDrag(int curGrip, const QPointF& delta)
 
 QRectF SlurSegment::bbox() const
       {
-      QRectF r(path.boundingRect());
-      return r;
+      return path.boundingRect();
       }
 
 //---------------------------------------------------------
@@ -280,14 +283,14 @@ void SlurSegment::write(Xml& xml, int no) const
                   break;
                   }
             }
-      if (!_userOff.isNull())
+      if (!userOff().isNull())
             empty = false;
       if (empty)
             return;
 
       xml.stag(QString("SlurSegment no=\"%1\"").arg(no));
-      if (!_userOff.isNull())
-            xml.tag("offset", _userOff);
+      if (!userOff().isNull())
+            xml.tag("offset", userOff() / spatium());
       if (!(ups[0].off.isNull()))
             xml.tag("o1", ups[0].off);
       if (!(ups[1].off.isNull()))
@@ -324,7 +327,7 @@ void SlurSegment::read(QDomElement e)
 //   layout
 //---------------------------------------------------------
 
-void SlurSegment::layout(ScoreLayout*, const QPointF& p1, const QPointF& p2, qreal b)
+void SlurSegment::layout(const QPointF& p1, const QPointF& p2, qreal b)
       {
 // printf("SlurSegment %p %p layout\n", slur, this);
       bow      = b;
@@ -334,10 +337,12 @@ void SlurSegment::layout(ScoreLayout*, const QPointF& p1, const QPointF& p2, qre
       //
       //  compute bezier help points
       //
-      qreal x0 = ups[0].pos().x();
-      qreal x3 = ups[3].pos().x();
-      qreal y0 = ups[0].pos().y();
-      qreal y3 = ups[3].pos().y();
+      double _spatium = spatium();
+      qreal x0 = ups[0].p.x() + ups[0].off.x() * _spatium;
+      qreal x3 = ups[3].p.x() + ups[3].off.x() * _spatium;
+
+      qreal y0 = ups[0].p.y() + ups[0].off.y() * _spatium;
+      qreal y3 = ups[3].p.y() + ups[3].off.y() * _spatium;
 
       qreal xdelta = x3 - x0;
       if (xdelta == 0.0) {
@@ -345,6 +350,10 @@ void SlurSegment::layout(ScoreLayout*, const QPointF& p1, const QPointF& p2, qre
                slurTie()->startElement()->tick(), slurTie()->endElement()->tick());
             return;
             }
+      if (x0 > x3) {
+            printf("illegal slurSegment\n");
+            return;
+            }
 
       qreal d  = xdelta / 4.0;
       qreal x1 = x0 + d;
@@ -403,6 +412,7 @@ SlurTie::SlurTie(Score* s)
       up             = true;
       _startElement  = 0;
       _endElement    = 0;
+      _len           = 0;
       }
 
 SlurTie::SlurTie(const SlurTie& t)
@@ -412,6 +422,8 @@ SlurTie::SlurTie(const SlurTie& t)
       _slurDirection = t._slurDirection;
       _startElement  = t._startElement;
       _endElement    = t._endElement;
+      _len           = t._len;
+
       delSegments    = t.delSegments;
 
       //
@@ -478,6 +490,8 @@ void SlurTie::change(Element* o, Element* n)
 
 QPointF SlurTie::slurPos(Element* e, System*& s)
       {
+      double _spatium = spatium();
+
       ChordRest* cr;
       if (e->type() == NOTE)
             cr = static_cast<Note*>(e)->chord();
@@ -643,20 +657,20 @@ void SlurTie::collectElements(QList<const Element*>& el) const
       }
 
 //---------------------------------------------------------
-//   resetUserOffsets
+//   toDefault
 //---------------------------------------------------------
 
-void SlurTie::resetUserOffsets()
+void SlurTie::toDefault()
       {
       setUserOff(QPointF());
       }
 
-void SlurSegment::resetUserOffsets()
+void SlurSegment::toDefault()
       {
       setUserOff(QPointF());
       for (int i = 0; i < 4; ++i)
             ups[i].off = QPointF();
-      parent()->resetUserOffsets();
+      parent()->toDefault();
       }
 
 //---------------------------------------------------------
@@ -754,6 +768,7 @@ void Slur::read(QDomElement e)
 //---------------------------------------------------------
 //   chordsHaveTie
 //---------------------------------------------------------
+
 static bool chordsHaveTie (Chord* c1, Chord* c2)
       {
       NoteList* nl1 = c1->noteList();
@@ -799,9 +814,31 @@ static bool isDirectionMixture (Chord* c1, Chord* c2)
 //   layout
 //---------------------------------------------------------
 
-void Slur::layout(ScoreLayout* layout)
+void Slur::layout()
       {
-      double _spatium = layout->spatium();
+      double _spatium = spatium();
+
+      if (score() == gscore) {      // HACK
+            //
+            // when used in a palette, slur has no parent and
+            // tick and tick2 has no meaning so no layout is
+            // possible and needed
+            //
+            setLen(_spatium * 7);
+            SlurSegment* s;
+            if (segments.isEmpty()) {
+                  s = new SlurSegment(score());
+                  s->setTrack(track());
+                  add(s);
+                  }
+            else {
+                  s = segments.front();
+                  }
+            s->setLineSegmentType(SEGMENT_SINGLE);
+            qreal bow = up ? 1.5 * -_spatium : 1.5 * _spatium;
+            s->layout(QPointF(0, 0), QPointF(_len, 0), bow);
+            return;
+            }
       switch (_slurDirection) {
             case UP:    up = true; break;
             case DOWN:  up = false; break;
@@ -851,7 +888,7 @@ void Slur::layout(ScoreLayout* layout)
       QPointF p1 = slurPos(startElement(), s1);
       QPointF p2 = slurPos(endElement(), s2);
 
-      QList<System*>* sl = layout->systems();
+      QList<System*>* sl = score()->systems();
       iSystem is = sl->begin();
       while (is != sl->end()) {
             if (*is == s1)
@@ -901,27 +938,27 @@ void Slur::layout(ScoreLayout* layout)
 
             // case 1: one segment
             if (s1 == s2) {
-                  segment->layout(layout, p1, p2, bow);
+                  segment->layout(p1, p2, bow);
                   segment->setLineSegmentType(SEGMENT_SINGLE);
                   }
             // case 2: start segment
             else if (i == 0) {
                   qreal x = sp.x() + system->bbox().width();
-                  segment->layout(layout, p1, QPointF(x, p1.y()), bow);
+                  segment->layout(p1, QPointF(x, p1.y()), bow);
                   segment->setLineSegmentType(SEGMENT_BEGIN);
                   }
             // case 3: middle segment
             else if (i != 0 && system != s2) {
                   qreal x1 = firstNoteRestSegmentX(system) - _spatium;
                   qreal x2 = sp.x() + system->bbox().width();
-                  segment->layout(layout, QPointF(x1, sp.y()), QPointF(x2, sp.y()), bow);
+                  segment->layout(QPointF(x1, sp.y()), QPointF(x2, sp.y()), bow);
                   segment->setLineSegmentType(SEGMENT_MIDDLE);
                   }
             // case 4: end segment
             else {
                   //qreal x = sp.x();
                   qreal x = firstNoteRestSegmentX(system) - _spatium;
-                  segment->layout(layout, QPointF(x, p2.y()), p2, bow);
+                  segment->layout(QPointF(x, p2.y()), p2, bow);
                   segment->setLineSegmentType(SEGMENT_END);
                   }
             if (system == s2)
@@ -956,10 +993,10 @@ double SlurTie::firstNoteRestSegmentX(System* system)
 
 QRectF Slur::bbox() const
       {
-      QRectF r;
-      foreach(SlurSegment* ss, segments)
-            r |= ss->abbox().translated(canvasPos());
-      return r;
+      if (segments.isEmpty())
+            return QRectF();
+      else
+            return segments[0]->bbox();
       }
 
 //---------------------------------------------------------
@@ -1025,7 +1062,7 @@ void Tie::read(QDomElement e)
 //   layout
 //---------------------------------------------------------
 
-void Tie::layout(ScoreLayout* layout)
+void Tie::layout()
       {
       //
       // TODO: if there is a startNote but no endNote
@@ -1033,7 +1070,7 @@ void Tie::layout(ScoreLayout* layout)
       if (startElement() == 0 || endElement() == 0)
             return;
 
-      double _spatium = layout->spatium();
+      double _spatium = spatium();
 
       Chord* c1   = startNote()->chord();
       Measure* m1 = c1->measure();
@@ -1063,7 +1100,7 @@ void Tie::layout(ScoreLayout* layout)
       QPointF p1 = startNote()->canvasPos() + off1;
       QPointF p2 = endNote()->canvasPos()   + off2;
 
-      QList<System*>* systems = layout->systems();
+      QList<System*>* systems = score()->systems();
       setPos(0, 0);
 
       //---------------------------------------------------------
@@ -1107,13 +1144,13 @@ void Tie::layout(ScoreLayout* layout)
 
             // case 1: one segment
             if (s1 == s2) {
-                  segment->layout(layout, p1, p2, bow);
+                  segment->layout(p1, p2, bow);
                   segment->setLineSegmentType(SEGMENT_SINGLE);
                   }
             // case 2: start segment
             else if (i == 0) {
                   qreal x = sp.x() + system->bbox().width();
-                  segment->layout(layout, p1, QPointF(x, p1.y()), bow);
+                  segment->layout(p1, QPointF(x, p1.y()), bow);
                   segment->setLineSegmentType(SEGMENT_BEGIN);
                   }
             // case 3: middle segment
@@ -1130,7 +1167,7 @@ void Tie::layout(ScoreLayout* layout)
                   // qreal x = sp.x();
                   qreal x = firstNoteRestSegmentX(system) - 2 * _spatium - canvasPos().x();
 
-                  segment->layout(layout, QPointF(x, p2.y()), p2, bow);
+                  segment->layout(QPointF(x, p2.y()), p2, bow);
                   segment->setLineSegmentType(SEGMENT_END);
                   }
             }
