@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: slur.h,v 1.27 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: slur.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -33,7 +33,7 @@ class Viewer;
 struct UP {
       QPointF p;            // layout position relative to pos()
       QPointF off;          // user offset in spatium units
-      QPointF pos() const { return p + off * _spatium; }
+//      QPointF pos() const { return p + off * _spatium; }
       };
 
 //---------------------------------------------------------
@@ -49,7 +49,6 @@ class SlurSegment : public Element {
       LineSegmentType _segmentType;
       System* _system;
 
-      virtual QRectF bbox() const;
       void updatePath();
 
    public:
@@ -59,7 +58,9 @@ class SlurSegment : public Element {
       virtual ElementType type() const   { return SLUR_SEGMENT; }
       virtual bool isMovable() const     { return true; }
 
-      void layout(ScoreLayout*, const QPointF& p1, const QPointF& p2, qreal bow);
+      virtual QRectF bbox() const;
+
+      void layout(const QPointF& p1, const QPointF& p2, qreal bow);
       virtual QPainterPath shape() const;
       virtual void draw(QPainter&) const;
 
@@ -81,7 +82,7 @@ class SlurSegment : public Element {
       void dump() const;
       void setLineSegmentType(LineSegmentType s)  { _segmentType = s;  }
       void setSystem(System* s)                   { _system = s;       }
-      virtual void resetUserOffsets();
+      virtual void toDefault();
       };
 
 //---------------------------------------------------------
@@ -93,6 +94,7 @@ class SlurTie : public Element {
       Element* _endElement;
 
    protected:
+      double _len;
       bool up;
       QList<SlurSegment*> segments;
       QQueue<SlurSegment*> delSegments;   // "deleted" segments
@@ -112,7 +114,7 @@ class SlurTie : public Element {
       Direction slurDirection() const    { return _slurDirection; }
       void setSlurDirection(Direction d) { _slurDirection = d; }
 
-      virtual void layout2(ScoreLayout*, const QPointF, int, struct UP&)  {}
+      virtual void layout2(const QPointF, int, struct UP&)  {}
       virtual void setSelected(bool f);
       virtual bool contains(const QPointF&) const { return false; }  // not selectable
 
@@ -130,7 +132,8 @@ class SlurTie : public Element {
       void setEndElement(Element* e)      { _endElement = e;      }
       Element* startElement() const       { return _startElement; }
       Element* endElement() const         { return _endElement;   }
-      virtual void resetUserOffsets();
+      virtual void toDefault();
+      void setLen(double v)               { _len = v; }
       };
 
 //---------------------------------------------------------
@@ -149,7 +152,7 @@ class Slur : public SlurTie {
       virtual ElementType type() const { return SLUR; }
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       virtual QRectF bbox() const;
       virtual void setTrack(int val);
 
@@ -182,7 +185,7 @@ class Tie : public SlurTie {
       Note* endNote() const               { return (Note*)endElement();   }
       virtual void write(Xml& xml) const;
       virtual void read(QDomElement);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       };
 
 #endif
