@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: spatium.h,v 1.4 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: spatium.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -21,9 +21,6 @@
 #ifndef __SPATIUM_H__
 #define __SPATIUM_H__
 
-extern double _spatium; // spatium value in points
-extern double _spatiumMag;
-
 //---------------------------------------------------------
 //   Spatium
 //    - a unit of measure
@@ -35,30 +32,22 @@ class Spatium {
       double _val;
 
    public:
-      Spatium()                       { _val = 0.0; }
-      explicit Spatium(double v)      { _val = v; }
-      double point() const            { return _val * _spatium; }
-      double val() const              { return _val; }
-      void set(double v)              { _val = v/_spatium; }
-      bool operator>(Spatium a) const { return _val > a._val; }
-      bool operator<(Spatium a) const { return _val < a._val; }
+      Spatium()                        { _val = 0.0; }
+      explicit Spatium(double v)       { _val = v; }
+      double val() const               { return _val; }
+      bool operator>(Spatium a) const  { return _val > a._val; }
+      bool operator<(Spatium a) const  { return _val < a._val; }
+      bool operator==(Spatium a) const { return _val == a._val; }
+      bool operator!=(Spatium a) const { return _val != a._val; }
 
       Spatium& operator+=(Spatium a) {
             _val += a._val;
             return *this;
             }
-      Spatium& operator+=(double a) {
-            _val += (a / _spatium);
-            return *this;
-            }
       Spatium& operator-=(Spatium a) {
             _val -= a._val;
             return *this;
             }
-      Spatium& operator-=(double a) {
-            _val -= (a / _spatium);
-            return *this;
-            }
       Spatium& operator/=(double d) {
             _val /= d;
             return *this;
@@ -74,8 +63,7 @@ class Spatium {
             _val *= d;
             return *this;
             }
-      friend Spatium spatium(double v);
-      friend double point(Spatium s);
+      Spatium operator-() const { return Spatium(-_val); }
       };
 
 inline Spatium operator+(Spatium a, Spatium b)
@@ -121,8 +109,5 @@ inline Spatium operator*(double a, Spatium b)
       return r;
       }
 
-inline Spatium spatium(double v) { return Spatium(v / _spatium); }
-inline double point(Spatium s)   { return s._val * _spatium; }
-
 #endif
 
