@@ -1,7 +1,7 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: staff.cpp,v 1.11 2006/03/28 14:58:58 wschweer Exp $
+//  $Id: staff.cpp 1920 2009-07-08 18:41:59Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -31,6 +31,7 @@
 #include "keysig.h"
 #include "segment.h"
 #include "style.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   idx
@@ -139,6 +140,7 @@ Staff::Staff(Score* s, Part* p, int rs)
       _small        = false;
       _slashStyle   = false;
       _barLineSpan  = 1;
+      _invisible    = false;
       }
 
 //---------------------------------------------------------
@@ -149,7 +151,7 @@ Staff::~Staff()
       {
       delete _clefList;
       delete _keymap;
-      _keymap = 0;      // DEBUG
+      _keymap   = 0;      // DEBUG
       _clefList = 0;
       }
 
@@ -182,6 +184,8 @@ void Staff::write(Xml& xml) const
             xml.tag("lines", lines());
       if (small())
             xml.tag("small", small());
+      if (invisible())
+            xml.tag("invisible", invisible());
       if (slashStyle())
             xml.tag("slashStyle", slashStyle());
       _clefList->write(xml, "cleflist");
@@ -204,12 +208,15 @@ void Staff::read(QDomElement e)
       setSmall(false);
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
+            int v = e.text().toInt();
             if (tag == "lines")
-                  setLines(e.text().toInt());
+                  setLines(v);
             else if (tag == "small")
-                  setSmall(e.text().toInt());
+                  setSmall(v);
+            else if (tag == "invisible")
+                  setInvisible(v);
             else if (tag == "slashStyle")
-                  setSlashStyle(e.text().toInt());
+                  setSlashStyle(v);
             else if (tag == "cleflist")
                   _clefList->read(e, _score);
             else if (tag == "keylist")
@@ -221,7 +228,7 @@ void Staff::read(QDomElement e)
                   _brackets.append(b);
                   }
             else if (tag == "barLineSpan")
-                  _barLineSpan = e.text().toInt();
+                  _barLineSpan = v;
             else
                   domError(e);
             }
@@ -240,36 +247,28 @@ void Staff::changeKeySig(int tick, int st)
       if (ot == st)
             return;                 // no change
 
-      int oval = -1000;
-      iKeyEvent ki = _keymap->find(tick);
-      if (ki != _keymap->end()) {
-            oval = ki->second;
-            _keymap->erase(ki);
-            }
+      iKeyEvent ki    = _keymap->find(tick);
+      int oval        = ki != _keymap->end() ? ki->second : NO_KEY;
+      bool removeFlag = st == _keymap->key(tick-1);
+      int nval        = removeFlag ? NO_KEY : st;
 
-      bool removeFlag = st == _keymap->key(tick);
-      int nval = -1000;
-      if (!removeFlag) {
-            setKey(tick, st);
-            nval = st;
-            }
-      _score->undoOp(UndoOp::ChangeKeySig, this, tick, oval, nval);
-
-      Measure* measure = _score->tick2measure(tick);
-      if (!measure) {
-            printf("measure for tick %d not found!\n", tick);
-            return;
-            }
+      _score->undoChangeKeySig(this, tick, oval, nval);
 
       //---------------------------------------------
       //    if the next keysig has the same subtype
       //    then its unnecessary and must be removed
       //---------------------------------------------
 
-      for (MeasureBase* mb = measure; mb; mb = mb->next()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            Measure* m = (Measure*)mb;
+      Measure* measure = _score->tick2measure(tick);
+      if (!measure) {
+            printf("measure for tick %d not found!\n", tick);
+            return;
+            }
+      Measure* m = measure;
+      if (m->prevMeasure())
+            m = m->prevMeasure();
+      int track = idx() * VOICES;
+      for (Measure* m = measure; m; m = m->nextMeasure()) {
             bool found = false;
             for (Segment* segment = m->first(); segment; segment = segment->next()) {
                   if (segment->subtype() != Segment::SegKeySig)
@@ -277,8 +276,7 @@ void Staff::changeKeySig(int tick, int st)
                   //
                   // we assume keySigs are only in first track (voice 0)
                   //
-                  int track = idx() * VOICES;
-                  KeySig* e = (KeySig*)segment->element(track);
+                  KeySig* e = static_cast<KeySig*>(segment->element(track));
                   int etick = segment->tick();
                   if (!e || (etick < tick))
                         continue;
@@ -337,46 +335,37 @@ void Staff::changeClef(int tick, int st)
       if (tick == 0 && st == CLEF_PERC)
             part()->setUseDrumset(true);
 
-      int oval = NO_CLEF;
-      iClefEvent ki = _clefList->find(tick);
-      if (ki != _clefList->end()) {
-            oval = ki->second;
-            _clefList->erase(ki);
-            }
+      iClefEvent ki   = _clefList->find(tick);
+      int oval        = ki != _clefList->end() ? ki->second : NO_CLEF;
+      bool removeFlag = st == _clefList->clef(tick-1);
+      int nval        = removeFlag ? NO_CLEF : st;
 
-      bool removeFlag = st == _clefList->clef(tick);
-      int nval = NO_CLEF;
-      if (!removeFlag) {
-            (*_clefList)[tick] = st;
-            nval = st;
-            }
-      _score->undoOp(UndoOp::ChangeClef, this, tick, oval, nval);
-
-      Measure* measure = _score->tick2measure(tick);
-      if (!measure) {
-            printf("measure for tick %d not found!\n", tick);
-            return;
-            }
+      _score->undoChangeClef(this, tick, oval, nval);
 
       //---------------------------------------------
       //    if the next clef has the same subtype
       //    then its unnecessary and must be removed
       //---------------------------------------------
 
-      for (MeasureBase* mb = measure; mb; mb = mb->next()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            Measure* m = (Measure*)mb;
+      Measure* measure = _score->tick2measure(tick);
+      if (!measure) {
+            printf("measure for tick %d not found!\n", tick);
+            return;
+            }
+      Measure* m = measure;
+      if (m->prevMeasure())
+            m = m->prevMeasure();
+      for (; m; m = m->nextMeasure()) {
             bool found = false;
             //
-            // we assume Clefs are only in first track (voice 0)
+            // we assume Clefs are only in first track of staff (voice 0)
             //
             int track = idx() * VOICES;
             for (Segment* segment = m->first(); segment; segment = segment->next()) {
                   if (segment->subtype() != Segment::SegClef)
                         continue;
-                  Clef* e = (Clef*)segment->element(track);
                   int etick = segment->tick();
+                  Clef* e = static_cast<Clef*>(segment->element(track));
                   if (!e || (etick < tick))
                         continue;
                   int est = e->subtype();
@@ -411,13 +400,8 @@ void Staff::changeClef(int tick, int st)
             // if clef is at measure beginning, move to end of
             // previous measure
             //
-            if (measure->tick() == tick) {
-                  MeasureBase* mb = measure->prev();
-                  while (mb && mb->type() != MEASURE)
-                        mb = mb->prev();
-                  if (mb)
-                        measure = static_cast<Measure*>(mb);
-                  }
+            if (measure->tick() == tick && (tick != 0))
+                  measure = measure->prevMeasure();
             Segment::SegmentType stype = Segment::segmentType(CLEF);
             Segment* s = measure->findSegment(stype, tick);
             if (!s) {
@@ -436,7 +420,16 @@ void Staff::changeClef(int tick, int st)
 
 double Staff::height() const
       {
-      return 4 * _spatium * mag();
+      return 4 * _score->spatium() * mag();
+      }
+
+//---------------------------------------------------------
+//   spatium
+//---------------------------------------------------------
+
+double Staff::spatium() const
+      {
+      return _score->spatium() * mag();
       }
 
 //---------------------------------------------------------
@@ -445,7 +438,7 @@ double Staff::height() const
 
 double Staff::mag() const
       {
-      return _small ? score()->style()->smallStaffMag : 1.0;
+      return _small ? score()->styleD(ST_smallStaffMag) : 1.0;
       }
 
 //---------------------------------------------------------
