@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: sym.h,v 1.16 2006/04/06 13:03:11 wschweer Exp $
+//  $Id: sym.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -24,7 +24,6 @@
 #include "style.h"
 
 extern void initSymbols();
-extern double _spatiumMag;
 
 enum SymbolType {
       SYMBOL_UNKNOWN,
@@ -67,15 +66,15 @@ class Sym {
       Sym(const char* name, const QChar& c, int fid);
 
       const char* name() const             { return _name; }
-      const QRectF bbox(double mag = 1.0) const;
-      double height(double mag = 1.0) const;
-      double width(double mag) const;
+      const QRectF bbox(double mag) const;
+      double height(double mag) const      { return _bbox.height() * mag; }
+
+      double width(double mag) const       { return w * mag;  }
       QChar code() const                   { return _code;    }
       int getFontId() const                { return fontId;   }
       const QFont& font() const            { return *_font;   }
       void setCode(const QChar& c)         { _code = c;       }
       void draw(QPainter& painter, double mag, qreal x, qreal y, int n) const;
-      void draw(QPainter& painter, qreal x, qreal y) const;
       void draw(QPainter& painter, double mag, qreal x, qreal y) const;
       void draw(QPainter& painter) const;
       void draw(QPainter& painter, double mag) const;
