@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: symbol.cpp 1847 2009-05-24 13:05:54Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -24,7 +24,6 @@
 #include "system.h"
 #include "staff.h"
 #include "measure.h"
-#include "layout.h"
 #include "page.h"
 #include "score.h"
 #include "viewer.h"
@@ -117,10 +116,10 @@ Element* BSymbol::drop(const QPointF&, const QPointF&, Element* el)
 //   layout
 //---------------------------------------------------------
 
-void BSymbol::layout(ScoreLayout* l)
+void BSymbol::layout()
       {
       foreach(Element* e, _leafs)
-            e->layout(l);
+            e->layout();
       if (parent() && parent()->type() == MEASURE) {
             Measure* m = static_cast<Measure*>(parent());
             double y = track() != -1 ? m->system()->staff(track() / VOICES)->y() : 0.0;
@@ -138,7 +137,7 @@ QRectF BSymbol::drag(const QPointF& pos)
       QRectF r(abbox());
       foreach(const Element* e, _leafs)
             r |= e->abbox();
-      setUserOff(pos / _spatium);
+      setUserOff(pos);
       r |= abbox();
       foreach(const Element* e, _leafs)
             r |= e->abbox();
@@ -171,10 +170,10 @@ void Symbol::setAbove(bool val)
 //    values when calling this method
 //---------------------------------------------------------
 
-void Symbol::layout(ScoreLayout* layout)
+void Symbol::layout()
       {
-      Element::layout(layout);
-      BSymbol::layout(layout);
+      Element::layout();
+      BSymbol::layout();
       }
 
 //---------------------------------------------------------
@@ -183,7 +182,7 @@ void Symbol::layout(ScoreLayout* layout)
 
 void Symbol::draw(QPainter& p) const
       {
-      symbols[_sym].draw(p, mag());
+      symbols[_sym].draw(p, magS());
       }
 
 //---------------------------------------------------------
@@ -192,7 +191,7 @@ void Symbol::draw(QPainter& p) const
 
 QRectF Symbol::bbox() const
       {
-      return symbols[_sym].bbox(mag());
+      return symbols[_sym].bbox(magS());
       }
 
 //---------------------------------------------------------
@@ -246,11 +245,12 @@ void Symbol::read(QDomElement e)
                   if (!ee.isNull())
                         path = ee.text();
                   Image* image = 0;
-                  if (path.endsWith(".svg"))
+                  QString s(path.toLower());
+                  if (s.endsWith(".svg"))
                         image = new SvgImage(score());
-                  else if (path.endsWith(".jpg")
-                     || path.endsWith(".png")
-                     || path.endsWith(".xpm")
+                  else if (s.endsWith(".jpg")
+                     || s.endsWith(".png")
+                     || s.endsWith(".xpm")
                         ) {
                         image = new RasterImage(score());
                         }
