@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: symboldialog.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -26,9 +26,6 @@
 class Palette;
 class Element;
 
-extern Palette* symbolPalette;
-extern void createSymbolPalette();
-
 //---------------------------------------------------------
 //   SymbolDialog
 //---------------------------------------------------------
@@ -37,9 +34,10 @@ class SymbolDialog : public QWidget, Ui::SymbolDialogBase {
       Q_OBJECT
 
       Palette* sp;
+      void createSymbolPalette();
 
    private slots:
-      void elementDropped(Element*);
+      void setDirty();
       void startDragElement(Element*);
       void deleteElement();
 
