@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: synti.h,v 1.12 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: synti.h 1863 2009-06-04 16:13:37Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -22,7 +22,17 @@
 #define __SYNTI_H__
 
 struct MidiPatch;
-struct MidiOutEvent;
+class Event;
+
+//---------------------------------------------------------
+//   MidiPatch
+//---------------------------------------------------------
+
+struct MidiPatch {
+      bool drum;
+      int bank, prog;
+      QString name;
+      };
 
 //---------------------------------------------------------
 //   Synth
@@ -33,10 +43,10 @@ class Synth {
    public:
       Synth() {}
       virtual ~Synth() {}
-      virtual bool init(int sampleRate) = 0;
+      virtual void init(int sampleRate) = 0;
       virtual bool loadSoundFont(const QString&) = 0;
       virtual void process(unsigned, float*, float*, int) = 0;
-      virtual void play(const MidiOutEvent&) = 0;
+      virtual void play(const Event&) = 0;
       virtual const MidiPatch* getPatchInfo(bool onlyDrums, const MidiPatch* p) const = 0;
       };
 
