@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: system.cpp,v 1.41 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: system.cpp 1988 2009-08-04 12:09:12Z lasconic $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -42,7 +42,6 @@
 #include "globals.h"
 #include "barline.h"
 #include "lyrics.h"
-#include "layout.h"
 #include "system.h"
 
 //---------------------------------------------------------
@@ -62,15 +61,11 @@ SysStaff::SysStaff()
 
 SysStaff::~SysStaff()
       {
-      foreach(Bracket* b, brackets) {
-            if (b)
-                  delete b;
-            }
+      foreach(Bracket* b, brackets)
+            delete b;
       brackets.clear();
-      if (instrumentName) {
-            delete instrumentName;
-            instrumentName = 0;
-            }
+      delete instrumentName;
+      instrumentName = 0;
       }
 
 //---------------------------------------------------------
@@ -93,8 +88,7 @@ System::System(Score* s)
 
 System::~System()
       {
-      if (barLine)
-            delete barLine;
+      delete barLine;
       }
 
 //---------------------------------------------------------
@@ -142,16 +136,13 @@ SysStaff* System::removeStaff(int idx)
  Layout the System.
 */
 
-void System::layout(ScoreLayout* layout, double xo1)
+void System::layout(double xo1)
       {
       if (isVbox())                 // ignore vbox
             return;
       static const Spatium instrumentNameOffset(1.0);
 
-      Style* style = score()->style();
-
       int nstaves  = _staves.size();
-
       if (nstaves != score()->nstaves())
             printf("System::layout: nstaves %d != %d\n", nstaves, score()->nstaves());
 
@@ -181,8 +172,7 @@ void System::layout(ScoreLayout* layout, double xo1)
             if (bracketLevels < ss->brackets.size()) {
                   for (int i = bracketLevels; i < ss->brackets.size(); ++i) {
                         Bracket* b = ss->brackets.takeLast();
-                        if (b)
-                              delete b;
+                        delete b;
                         }
                   }
             else if (bracketLevels > ss->brackets.size()) {
@@ -192,10 +182,8 @@ void System::layout(ScoreLayout* layout, double xo1)
             for (int i = 0; i < bracketLevels; ++i) {
                   Bracket* b = ss->brackets[i];
                   if (s->bracket(i) == NO_BRACKET) {
-                        if (b) {
-                              delete b;
-                              ss->brackets[i] = 0;
-                              }
+                        delete b;
+                        ss->brackets[i] = 0;
                         }
                   else {
                         if (b == 0) {
@@ -213,8 +201,8 @@ void System::layout(ScoreLayout* layout, double xo1)
                         }
                   }
             if (ss->instrumentName && !ss->instrumentName->isEmpty()) {
-                  ss->instrumentName->layout(layout);
-                  double w = ss->instrumentName->width() + instrumentNameOffset.point();
+                  ss->instrumentName->layout();
+                  double w = ss->instrumentName->width() + point(instrumentNameOffset);
                   if (w > xoff2)
                         xoff2 = w;
                   }
@@ -228,7 +216,7 @@ void System::layout(ScoreLayout* layout, double xo1)
       _leftMargin = xoff2;
 
       for (int i = 0; i < bracketLevels; ++i)
-            _leftMargin += bracketWidth[i] + style->bracketDistance.point();
+            _leftMargin += bracketWidth[i] + point(score()->styleS(ST_bracketDistance));
 
       for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
             SysStaff* s  = _staves[staffIdx];
@@ -238,12 +226,12 @@ void System::layout(ScoreLayout* layout, double xo1)
                   continue;
                   }
             double staffMag = staff->mag();
-            s->setbbox(QRectF(_leftMargin + xo1, 0.0, 0.0, 4 * _spatium * staffMag));
+            s->setbbox(QRectF(_leftMargin + xo1, 0.0, 0.0, 4 * spatium() * staffMag));
             }
 
       if (nstaves > 1 && barLine == 0) {
             barLine = new Line(score(), true);
-            barLine->setLineWidth(style->barWidth);
+            barLine->setLineWidth(score()->styleS(ST_barWidth));
             barLine->setParent(this);
             }
       else if (nstaves <= 1 && barLine) {
@@ -251,7 +239,7 @@ void System::layout(ScoreLayout* layout, double xo1)
             barLine = 0;
             }
       if (barLine) {
-            barLine->setPos(_leftMargin + xo1, style->barWidth.point() * .25);
+            barLine->setPos(_leftMargin + xo1, point(score()->styleS(ST_barWidth)) * .25);
             }
 
       //---------------------------------------------------
@@ -265,7 +253,7 @@ void System::layout(ScoreLayout* layout, double xo1)
 
             double xo = -xo1;
             for (int i = 0; i < bracketLevels; ++i) {
-                  xo += bracketWidth[i] + style->bracketDistance.point();
+                  xo += bracketWidth[i] + point(score()->styleS(ST_bracketDistance));
                   Bracket* b = ss->brackets[i];
                   if (b == 0)
                         continue;
@@ -303,7 +291,7 @@ void System::layout(ScoreLayout* layout, double xo1)
             SysStaff* s = staff(idx);
             int nstaves = p->nstaves();
             if (s->instrumentName && !s->instrumentName->isEmpty()) {
-                  double d  = instrumentNameOffset.point() + s->instrumentName->bbox().width();
+                  double d  = point(instrumentNameOffset) + s->instrumentName->bbox().width();
                   s->instrumentName->setXpos(xoff2 - d + xo1);
                   }
             idx += nstaves;
@@ -316,33 +304,34 @@ void System::layout(ScoreLayout* layout, double xo1)
 //    adjusts staff distance
 //---------------------------------------------------------
 
-void System::layout2(ScoreLayout* layout)
+void System::layout2()
       {
       if (isVbox())                 // ignore vbox
             return;
 
       int nstaves = _staves.size();
+      double _spatium = spatium();
 
       qreal y = 0.0;
       int lastStaffIdx = 0;   // last visible staff
       for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
             Staff* staff = score()->staff(staffIdx);
             if ((staffIdx + 1) == nstaves) {
-                  setDistance(staffIdx, score()->style()->systemDistance);
+                  setDistance(staffIdx, score()->styleS(ST_systemDistance));
                   }
             else if (staff->rstaff() < (staff->part()->staves()->size()-1)) {
-                  setDistance(staffIdx, score()->style()->accoladeDistance);
+                  setDistance(staffIdx, score()->styleS(ST_akkoladeDistance));
                   }
             else {
-                  setDistance(staffIdx, score()->style()->staffDistance);
+                  setDistance(staffIdx, score()->styleS(ST_staffDistance));
                   }
             double dist = 0.0;
             foreach(MeasureBase* m, ml) {
                   dist = std::max(dist, m->distance(staffIdx));
-                  dist = std::max(dist, m->userDistance(staffIdx).point());
+                  dist = std::max(dist, point(m->userDistance(staffIdx)));
                   }
-            if (dist > distance(staffIdx))
-                  setDistance(staffIdx, dist);
+            if (dist > (distance(staffIdx).val() * _spatium))
+                  setDistance(staffIdx, Spatium(dist/_spatium));
             SysStaff* s = _staves[staffIdx];
             if (!s->show()) {
                   s->setbbox(QRectF());
@@ -350,7 +339,7 @@ void System::layout2(ScoreLayout* layout)
                   }
             double sHeight = staff->height();   // (staff->lines() - 1) * _spatium * staffMag;
             s->setbbox(QRectF(_leftMargin, y, width() - _leftMargin, sHeight));
-            y += sHeight + s->distance();
+            y += sHeight + (s->distance().val() * _spatium);
             lastStaffIdx = staffIdx;
             }
       qreal systemHeight = staff(lastStaffIdx)->bbox().bottom();
@@ -365,9 +354,9 @@ void System::layout2(ScoreLayout* layout)
             staffY[i] = staff(i)->bbox().y();
 
       if (barLine) {
-            Spatium lw = score()->style()->barWidth;
+            Spatium lw = score()->styleS(ST_barWidth);
             barLine->setLen(Spatium(systemHeight / _spatium) - lw * .5);
-            barLine->layout(layout);
+            barLine->layout();
             }
 
       //---------------------------------------------------
@@ -394,7 +383,7 @@ void System::layout2(ScoreLayout* layout)
                   qreal ey = _staves[staffIdx + b->span() - 1]->bbox().bottom();
                   b->setYpos(sy);
                   b->setHeight(ey - sy);
-                  b->layout(layout);
+                  b->layout();
                   }
             }
 
@@ -467,20 +456,18 @@ void System::setInstrumentName(int staffIdx)
       if (isVbox())                 // ignore vbox
             return;
 
-      Score* cs = score();
-      Staff* s  = cs->staff(staffIdx);
+      Staff* s   = score()->staff(staffIdx);
+      Part* part = s->part();
+
       if (!s->isTop())
             return;
       SysStaff* staff = _staves[staffIdx];
-      if (staff->instrumentName == 0)
-            staff->instrumentName = new TextC(s->part()->longNameBase(), cs);
-      if (_firstSystem) {
-            staff->instrumentName->setSubtype(TEXT_INSTRUMENT_LONG);
-            staff->instrumentName->setTextBase(s->part()->longNameBase());
-            }
-      else {
-            staff->instrumentName->setSubtype(TEXT_INSTRUMENT_SHORT);
-            staff->instrumentName->setTextBase(s->part()->shortNameBase());
+
+      if (staff->instrumentName == 0) {
+            if (_firstSystem)
+                  staff->instrumentName = new TextC(*part->longName());
+            else
+                  staff->instrumentName = new TextC(*part->shortName());
             }
       staff->instrumentName->setParent(this);
       staff->instrumentName->setTrack(staffIdx * VOICES);
@@ -574,8 +561,10 @@ void System::remove(Element* el)
 
 void System::change(Element* o, Element* n)
       {
-      if (o->type() == VBOX || o->type() == HBOX)
-            score()->layout()->change((MeasureBase*)o, (MeasureBase*)n);
+      if (o->type() == VBOX || o->type() == HBOX) {
+            score()->remove((MeasureBase*)o);
+            score()->add((MeasureBase*)n);
+            }
       else {
             remove(o);
             add(n);
@@ -660,12 +649,17 @@ MeasureBase* System::nextMeasure(const MeasureBase* m) const
 //   layoutLyrics
 //---------------------------------------------------------
 
-void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffIdx)
+void System::layoutLyrics(Lyrics* l, Segment* s, int staffIdx)
       {
       if ((l->syllabic() == Lyrics::SINGLE || l->syllabic() == Lyrics::END) && (l->endTick() == 0)) {
             l->clearSeparator();
             return;
             }
+      double _spatium = spatium();
+
+      TextStyle* ts = score()->textStyle(l->textStyle());
+      double lmag = double(ts->size) / 11.0;
+
       if (l->endTick()) {
             Segment* seg = score()->tick2segment(l->endTick());
             if (seg == 0) {
@@ -674,7 +668,7 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
                   }
 
             QList<Line*>* sl = l->separatorList();
-            QList<System*>* systems = layout->systems();
+            QList<System*>* systems = score()->systems();
             System* s1 = this;
             System* s2 = seg->measure()->system();
             int sysIdx1  = systems->indexOf(s1);
@@ -689,9 +683,9 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
                   Line* line = sl->value(segIdx);
                   if (line == 0) {
                         line = new Line(l->score(), false);
-                        line->setLineWidth(Spatium(0.1));
                         l->add(line);
                         }
+                  line->setLineWidth(Spatium(0.1 * lmag));
                   if (sysIdx1 == sysIdx2) {
                         // single segment
                         line->setPos(p1);
@@ -712,7 +706,7 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
                   else if (i == sysIdx2) {
                         // end segment
                         }
-                  line->layout(layout);
+                  line->layout();
                   }
             return;
             }
@@ -724,8 +718,8 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
       Segment* ns = s;
 
       // TODO: then next two values should be style parameters
-      const double maxl = 0.8 * _spatium;   // lyrics hyphen length
-      const Spatium hlw(0.1);               // hyphen line width
+      const double maxl = 0.5 * _spatium * lmag;   // lyrics hyphen length
+      const Spatium hlw(0.14 * lmag);              // hyphen line width
 
       while ((ns = ns->next1())) {
             LyricsList* nll = ns->lyricsList(staffIdx);
@@ -741,7 +735,6 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
             Line* line;
             if (sl->isEmpty()) {
                   line = new Line(l->score(), false);
-                  line->setLineWidth(hlw);
                   l->add(line);
                   }
             else {
@@ -751,13 +744,13 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
             qreal w  = b.width();
             qreal h  = b.height();
             qreal x  = b.x() + w;
-            qreal y  = b.y() + h * .5;
+            qreal y  = b.y() + h * .58;
             line->setPos(QPointF(x, y));
 
             qreal x1 = l->canvasPos().x();
             qreal x2 = nl->canvasPos().x();
             qreal len;
-            if (x2 < x1) {
+            if (x2 < x1 || s->measure()->system()->page() != ns->measure()->system()->page()) {
                   System* system = s->measure()->system();
                   x2 = system->canvasPos().x() + system->bbox().width();
                   }
@@ -768,11 +761,10 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
                   len = maxl;
             double xo = (gap - len) * .5;
 
+            line->setLineWidth(hlw);
             line->setPos(QPointF(x + xo, y));
-            Spatium sp;
-            sp.set(len);
-            line->setLen(sp);
-            line->layout(layout);
+            line->setLen(Spatium(len / _spatium));
+            line->layout();
             return;
             }
       l->clearSeparator();
