@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: system.h,v 1.23 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: system.h 1969 2009-07-29 17:43:16Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -37,7 +37,6 @@ class TextC;
 class Bracket;
 class Lyrics;
 class Segment;
-class ScoreLayout;
 class MeasureBase;
 
 //---------------------------------------------------------
@@ -58,14 +57,13 @@ class SysStaff {
       QList<Bracket*> brackets;
       TextC* instrumentName;
 
-      const QRectF& bbox() const      { return _bbox; }
-      double y() const                { return _bbox.y(); }
-      double right() const            { return _bbox.right(); }
-      void setbbox(const QRectF& r)   { _bbox = r; }
+      const QRectF& bbox() const     { return _bbox; }
+      double y() const               { return _bbox.y(); }
+      double right() const           { return _bbox.right(); }
+      void setbbox(const QRectF& r)  { _bbox = r; }
       void move(double x, double y);
 
-      double distance() const        { return point(_distance); }
-      void setDistance(double v)     { _distance.set(v); }
+      Spatium distance() const       { return _distance; }
       void setDistance(Spatium v)    { _distance = v; }
 
       bool show() const              { return _show; }
@@ -112,8 +110,8 @@ class System : public Element {
 
       Page* page() const                 { return (Page*)parent(); }
 
-      virtual void layout(ScoreLayout*, double xoffset);
-      void layout2(ScoreLayout*);         ///< Called after Measure layout.
+      virtual void layout(double xoffset);
+      void layout2();         ///< Called after Measure layout.
       void clear();                       ///< Clear measure list.
 
       QList<MeasureBase*>& measures()      { return ml; }
@@ -123,8 +121,8 @@ class System : public Element {
       const QList<SysStaff*>* staves() const { return &_staves;   }
       SysStaff* staff(int n) const         { return _staves[n]; }
 
-      double distance(int n) const         { return _staves[n]->distance(); }
-      void setDistance(int n, double v)    { _staves[n]->setDistance(v); }
+      Spatium distance(int n) const        { return _staves[n]->distance(); }
+//      void setDistance(int n, Spatium v)   { _staves[n]->setDistance(v); }
       bool pageBreak() const               { return _pageBreak; }
       void setPageBreak(bool val)          { _pageBreak = val; }
 
@@ -147,7 +145,7 @@ class System : public Element {
       void setFirstSystem(bool v) { _firstSystem = v;   }
       bool isVbox() const         { return _vbox;       }
       void setVbox(bool v)        { _vbox = v;          }
-      void layoutLyrics(ScoreLayout*, Lyrics*, Segment*, int staffIdx);
+      void layoutLyrics(Lyrics*, Segment*, int staffIdx);
       };
 
 typedef QList<System*>::iterator iSystem;
