@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: tempotext.h 1992 2009-08-04 16:30:54Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -29,6 +29,7 @@
 //---------------------------------------------------------
 
 class TempoText : public Text  {
+      Q_DECLARE_TR_FUNCTIONS(TempoText)
       double _tempo;     // beats per second
 
    public:
@@ -39,8 +40,6 @@ class TempoText : public Text  {
       virtual void read(QDomElement);
       virtual bool genPropertyMenu(QMenu*) const;
       virtual void propertyAction(const QString&);
-//      virtual void layout(ScoreLayout*);
-//      QLineF dragAnchor() const;
       Measure* measure() const { return (Measure*)parent(); }
       double tempo() const     { return _tempo; }
       void setTempo(double v)  { _tempo = v; }
