@@ -1,9 +1,9 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: text.cpp 1992 2009-08-04 16:30:54Z wschweer $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -30,12 +30,13 @@
 #include "textpalette.h"
 #include "sym.h"
 #include "symbol.h"
-#include "layout.h"
 #include "textline.h"
 #include "preferences.h"
 #include "system.h"
+#include "measure.h"
+#include "textproperties.h"
 
-TextPalette* palette;
+TextPalette* textPalette;
 
 //---------------------------------------------------------
 //   TextBase
@@ -43,10 +44,13 @@ TextPalette* palette;
 
 TextBase::TextBase()
       {
+      _refCount     = 1;
       _doc          = new QTextDocument(0);
       _doc->setUseDesignMetrics(true);
+      _doc->setUndoRedoEnabled(true);
 
-      _frameWidth   = 0.0;
+      _hasFrame     = false;
+      _frameWidth   = 0.35;         // default line width
       _paddingWidth = 0.0;
       _frameColor   = preferences.defaultColor;
       _frameRound   = 25;
@@ -71,6 +75,7 @@ TextBase::TextBase(const TextBase& t)
       _frameColor   = t._frameColor;
       _frameRound   = t._frameRound;
       _circle       = t._circle;
+      _hasFrame     = t._hasFrame;
       frame         = t.frame;
       _bbox         = t._bbox;
       _layoutWidth  = t._layoutWidth;
@@ -78,17 +83,95 @@ TextBase::TextBase(const TextBase& t)
       layout(_layoutWidth);
       }
 
+//---------------------------------------------------------
+//   isSimpleText
+//    Return true if this is simple text and conforms to
+//    the text style.
+//    This kind of text is reponsive to style changes and
+//    can be saved as simple text string.
+//---------------------------------------------------------
+
+bool TextBase::isSimpleText(TextStyle* style, double spatium) const
+      {
+      if (_doc->blockCount() > 1) {
+            //printf("blocks > 1: %s\n", qPrintable(getText()));
+            return false;
+            }
+
+      int fragments = 0;
+      QTextBlock b = _doc->begin();
+      QTextCharFormat cf;
+      for (QTextBlock::iterator i = b.begin(); !i.atEnd(); ++i) {
+            if (i.fragment().isValid())
+                  ++fragments;
+            if (fragments > 1) {
+                  return false;
+                  }
+            cf = i.fragment().charFormat();
+            }
+      if (!style) {
+            // printf("no style: %s\n", qPrintable(getText()));
+            return false;
+            }
+      //
+      // the style font has another size than the actual
+      // font used in cf (due to rounding errors?), but
+      // QFontInfo gives the right size
+      //
+
+      QFontInfo fi(style->font(spatium));
+
+      QFont f(cf.font());
+      if (fi.family() == f.family()
+         && fi.pointSize() == f.pointSize()
+         && fi.bold() == f.bold()
+         && style->font(spatium).underline() == f.underline()
+         && fi.italic() == f.italic())
+            return true;
+
+//      printf("bad font: %s %f <%s><%s> %d %d\n", qPrintable(getText()), spatium,
+//         qPrintable(fi.family()), qPrintable(f.family()), fi.pointSize(), f.pointSize());
+//      printf("%s\n", qPrintable(style->font(spatium).toString()));
+//      printf("%s\n", qPrintable(f.toString()));
+
+      return false;
+      }
+
 //---------------------------------------------------------
 //   setDoc
 //---------------------------------------------------------
 
 void TextBase::setDoc(const QTextDocument& d)
       {
-      if (_doc)
-            delete _doc;
+      delete _doc;
       _doc = d.clone(0);
       }
 
+QTextDocument* TextBase::swapDoc(QTextDocument* d)
+      {
+      QTextDocument* od = _doc;
+      _doc = d;
+      return od;
+      }
+
+QFont TextBase::defaultFont() const
+      {
+      QTextCursor cursor(_doc);
+      cursor.movePosition(QTextCursor::Start);
+      return cursor.charFormat().font();
+      }
+
+void TextBase::setDefaultFont(QFont f)
+      {
+      _doc->setDefaultFont(f);
+      QTextCursor cursor(_doc);
+      cursor.movePosition(QTextCursor::Start);
+      cursor.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+      QTextCharFormat cf = cursor.charFormat();
+      cf.setFont(f);
+      cursor.setCharFormat(cf);
+      }
+
 //---------------------------------------------------------
 //   setText
 //---------------------------------------------------------
@@ -120,6 +203,7 @@ void TextBase::setText(const QString& s, Align align)
 
 void TextBase::setHtml(const QString& s)
       {
+      _doc->clear();
       _doc->setHtml(s);
       }
 
@@ -145,23 +229,31 @@ QString TextBase::getHtml() const
 //   writeProperties
 //---------------------------------------------------------
 
-void TextBase::writeProperties(Xml& xml) const
+void TextBase::writeProperties(Xml& xml, TextStyle* ts, double spatium, bool writeText) const
       {
       // write all properties which are different from style
 
-      if (_frameWidth != 0.0)
-            xml.tag("frameWidth", _frameWidth);
-      if (_paddingWidth != 0.0)
-            xml.tag("paddingWidth", _paddingWidth);
-      if (_frameColor != preferences.defaultColor)
-            xml.tag("frameColor", _frameColor);
-      if (_frameRound != 25)
-            xml.tag("frameRound", _frameRound);
-      if (_circle)
-            xml.tag("circle", _circle);
-      xml.stag("html-data");
-      xml.writeHtml(_doc->toHtml("utf-8"));
-      xml.etag();
+      if (_hasFrame) {
+            if (ts == 0 || _frameWidth != ts->frameWidth)
+                  xml.tag("frameWidth", _frameWidth);
+            if (ts == 0 || _paddingWidth != ts->paddingWidth)
+                  xml.tag("paddingWidth", _paddingWidth);
+            if (ts == 0 || _frameColor != ts->frameColor)
+                  xml.tag("frameColor", _frameColor);
+            if (ts == 0 || _frameRound != ts->frameRound)
+                  xml.tag("frameRound", _frameRound);
+            if (ts == 0 || _circle != ts->circle)
+                  xml.tag("circle", _circle);
+            }
+      if (writeText) {
+            if (isSimpleText(ts, spatium))
+                  xml.tag("text", _doc->toPlainText());
+            else {
+                  xml.stag("html-data");
+                  xml.writeHtml(_doc->toHtml("utf-8"));
+                  xml.etag();
+                  }
+            }
       }
 
 //---------------------------------------------------------
@@ -179,12 +271,16 @@ bool TextBase::readProperties(QDomElement e)
             QString s = Xml::htmlToString(e);
             _doc->setHtml(s);
             }
+      else if (tag == "text")
+            _doc->setPlainText(val);
       else if (tag == "html-data") {
             QString s = Xml::htmlToString(e.firstChildElement());
             _doc->setHtml(s);
             }
-      else if (tag == "frameWidth")
+      else if (tag == "frameWidth") {
             _frameWidth = val.toDouble();
+            _hasFrame   = true;
+            }
       else if (tag == "paddingWidth")
             _paddingWidth = val.toDouble();
       else if (tag == "frameColor")
@@ -218,7 +314,7 @@ void TextBase::layout(double w)
             }
       _doc->setTextWidth(w);   // to make alignment work
 
-      if (_frameWidth > 0.0) {
+      if (_hasFrame) {
             frame = QRectF();
             for (QTextBlock tb = _doc->begin(); tb.isValid(); tb = tb.next()) {
                   QTextLayout* tl = tb.layout();
@@ -269,8 +365,8 @@ void TextBase::draw(QPainter& p, QTextCursor* cursor) const
       _doc->documentLayout()->setProperty("cursorWidth", QVariant(int(lrint(2.0*DPI/PDPI))));
       _doc->documentLayout()->draw(&p, c);
 
-      // draw border
-      if (_frameWidth > 0.0) {
+      // draw frame
+      if (_hasFrame) {
             p.setPen(QPen(QBrush(_frameColor), _frameWidth * DPMM));
             p.setBrush(QBrush(Qt::NoBrush));
             if (_circle)
@@ -298,52 +394,53 @@ TextB::TextB(Score* s)
       cursorPos  = 0;
       cursor     = 0;
       _movable   = true;
+      _textStyle = -1;
       }
 
 TextB::TextB(const TextB& e)
    : Element(e)
       {
-      _movable  = e._movable;
+      _movable                = e._movable;
       _sizeIsSpatiumDependent = e._sizeIsSpatiumDependent;
-
-      editMode  = e.editMode;
-      cursorPos = e.cursorPos;
-      }
-
-//---------------------------------------------------------
-//   styleChanged
-//---------------------------------------------------------
-
-void TextB::styleChanged()
-      {
-      setSubtype(subtype());
+      editMode                = e.editMode;
+      cursorPos               = e.cursorPos;
+      _textStyle              = e._textStyle;
+      cursor                  = 0;
       }
 
 //---------------------------------------------------------
 //   TextC
 //---------------------------------------------------------
 
-TextC::TextC(TextBase** txtb, Score* s)
+TextC::TextC(Score* s)
    : TextB(s)
       {
-      _tbb = txtb;
-      _otb = 0;
+      _tb = new TextBase();
       setSubtype(TEXT_STAFF);
       }
 
 TextC::TextC(const TextC& e)
    : TextB(e)
       {
-      _tbb   = e._tbb;
-      _otb   = 0;
+      _tb    = e._tb;
+      _tb->incRefCount();
       cursor = 0;
       baseChanged();
       }
 
+TextC* TextC::clone() const
+      {
+      TextC* t = new TextC(*this);
+      _tb->decRefCount();
+      t->_tb = new TextBase(*_tb);
+      return t;
+      }
+
 TextC::~TextC()
       {
-      if (_otb)
-            delete _otb;
+      _tb->decRefCount();
+      if (_tb->refCount() <= 0)
+            delete _tb;
       }
 
 //---------------------------------------------------------
@@ -353,37 +450,14 @@ TextC::~TextC()
 void TextC::baseChanged()
       {
       if (editMode) {
-            if (cursor)
-                  delete cursor;
+            delete cursor;
             cursor = new QTextCursor(textBase()->doc());
             cursor->setPosition(cursorPos);
-            // cursor->setBlockFormat(e.cursor->blockFormat());
-            // cursor->setCharFormat(e.cursor->charFormat());
             }
-      else
+      else {
+            delete cursor;
             cursor = 0;
-      }
-
-//---------------------------------------------------------
-//   setStyle
-//---------------------------------------------------------
-
-void TextC::setStyle(const TextStyle* s)
-      {
-      if (s == 0)
-            return;
-      doc()->setDefaultFont(s->font());
-      _align         = s->align;
-      _xoff          = s->xoff;
-      _yoff          = s->yoff;
-      _rxoff         = s->rxoff;
-      _ryoff         = s->ryoff;
-      _offsetType    = s->offsetType;
-      _sizeIsSpatiumDependent = s->sizeIsSpatiumDependent;
-
-      setSystemFlag(s->systemFlag);
-      if (s->systemFlag)
-            setTrack(-1);
+            }
       }
 
 //---------------------------------------------------------
@@ -427,13 +501,12 @@ void TextB::setDoc(const QTextDocument& d)
       }
 
 //---------------------------------------------------------
-//   setSubtype
+//   swapDoc
 //---------------------------------------------------------
 
-void TextB::setSubtype(int val)
+QTextDocument* TextB::swapDoc(QTextDocument* d)
       {
-      Element::setSubtype(val);
-      setStyle(style());
+      return textBase()->swapDoc(d);
       }
 
 //---------------------------------------------------------
@@ -445,50 +518,6 @@ void TextB::setAbove(bool /*val*/)
       // setYoff(val ? -2.0 : 7.0);
       }
 
-//---------------------------------------------------------
-//   style
-//---------------------------------------------------------
-
-TextStyle* TextB::style() const
-      {
-      int st = -1;
-      switch (subtype()) {
-            case TEXT_TITLE:            st = TEXT_STYLE_TITLE; break;
-            case TEXT_SUBTITLE:         st = TEXT_STYLE_SUBTITLE; break;
-            case TEXT_COMPOSER:         st = TEXT_STYLE_COMPOSER; break;
-            case TEXT_POET:             st = TEXT_STYLE_POET; break;
-            case TEXT_TRANSLATOR:       st = TEXT_STYLE_TRANSLATOR; break;
-            case TEXT_MEASURE_NUMBER:   st = TEXT_STYLE_MEASURE_NUMBER; break;
-            case TEXT_PAGE_NUMBER_ODD:  st = TEXT_STYLE_PAGE_NUMBER_ODD; break;
-            case TEXT_PAGE_NUMBER_EVEN: st = TEXT_STYLE_PAGE_NUMBER_EVEN; break;
-            case TEXT_COPYRIGHT:        st = TEXT_STYLE_COPYRIGHT; break;
-            case TEXT_FINGERING:        st = TEXT_STYLE_FINGERING; break;
-            case TEXT_INSTRUMENT_LONG:  st = TEXT_STYLE_INSTRUMENT_LONG; break;
-            case TEXT_INSTRUMENT_SHORT: st = TEXT_STYLE_INSTRUMENT_SHORT; break;
-            case TEXT_INSTRUMENT_EXCERPT: st = TEXT_STYLE_INSTRUMENT_EXCERPT; break;
-            case TEXT_TEMPO:            st = TEXT_STYLE_TEMPO; break;
-            case TEXT_LYRIC:            st = TEXT_STYLE_LYRIC; break;
-            case TEXT_TUPLET:           st = TEXT_STYLE_TUPLET; break;
-            case TEXT_SYSTEM:           st = TEXT_STYLE_SYSTEM; break;
-            case TEXT_STAFF:            st = TEXT_STYLE_STAFF; break;
-            case TEXT_CHORD:            st = TEXT_STYLE_CHORD; break;
-            case TEXT_REHEARSAL_MARK:   st = TEXT_STYLE_REHEARSAL_MARK; break;
-            case TEXT_REPEAT:           st = TEXT_STYLE_REPEAT; break;
-            case TEXT_VOLTA:            st = TEXT_STYLE_VOLTA; break;
-            case TEXT_FRAME:            st = TEXT_STYLE_FRAME; break;
-            case TEXT_TEXTLINE:         st = TEXT_STYLE_TEXTLINE; break;
-            case TEXT_STRING_NUMBER:    st = TEXT_STYLE_STRING_NUMBER; break;
-            case TEXT_UNKNOWN:          break;
-            default:
-                  printf("unknown text subtype %d <%s>\n",
-                     subtype(), qPrintable(doc()->toPlainText()));
-                  break;
-            }
-      if (st == -1)
-            st = TEXT_STYLE_STAFF;
-      return score()->textStyle(st);
-      }
-
 //---------------------------------------------------------
 //   subtypeName
 //---------------------------------------------------------
@@ -597,10 +626,8 @@ void TextB::setSubtype(const QString& s)
 void TextB::resetMode()
       {
       editMode = 0;
-      if (cursor) {
-            delete cursor;
-            cursor = 0;
-            }
+      delete cursor;
+      cursor = 0;
       }
 
 //---------------------------------------------------------
@@ -616,33 +643,31 @@ bool TextB::isEmpty() const
 //   layout
 //---------------------------------------------------------
 
-void TextB::layout(ScoreLayout* layout)
+void TextB::layout()
       {
-      if (parent() && parent()->type() == HBOX) {
+      if (parent() && parent()->type() == HBOX)
             textBase()->layout(parent()->width());
-            }
       else
             textBase()->layout(-1.0);
       setbbox(textBase()->bbox());
 
-      if (parent() == 0)
-            return;
-
-      Element::layout(layout);      // process alignment
+      Element::layout();      // process alignment
 
       if (_align & ALIGN_VCENTER && subtype() == TEXT_TEXTLINE) {
             // special case: vertically centered text with TextLine needs to
             // take into account the line width
             TextLineSegment* tls = (TextLineSegment*)parent();
             TextLine* tl = (TextLine*)(tls->line());
-            qreal textlineLineWidth = tl->lineWidth().point();
+            qreal textlineLineWidth = point(tl->lineWidth());
             setYpos(pos().y() - textlineLineWidth * .5);
             }
 
+      if (parent() == 0)
+            return;
       if (parent()->type() == MEASURE) {
             Measure* m = static_cast<Measure*>(parent());
-            double y = track() != -1 ? m->system()->staff(track() / VOICES)->y() : 0.0;
-            double x = (tick() != -1) ? m->tick2pos(tick()) : 0.0;
+            double y = track() < 0 ? 0.0 : m->system()->staff(track() / VOICES)->y();
+            double x = (tick() < 0) ? 0.0 : m->tick2pos(tick());
             setPos(ipos() + QPointF(x, y));
             }
       }
@@ -657,14 +682,15 @@ void TextB::draw(QPainter& p) const
       }
 
 //---------------------------------------------------------
-//   setStyle
+//   setTextStyle
 //---------------------------------------------------------
 
-void TextB::setStyle(const TextStyle* s)
+void TextB::setTextStyle(int idx)
       {
-      if (s == 0)
-            return;
-      doc()->setDefaultFont(s->font());
+      _textStyle   = idx;
+      TextStyle* s = score()->textStyle(idx);
+      doc()->setDefaultFont(s->font(spatium()));
+
       _align         = s->align;
       _xoff          = s->xoff;
       _yoff          = s->yoff;
@@ -675,6 +701,7 @@ void TextB::setStyle(const TextStyle* s)
       setSystemFlag(s->systemFlag);
 
       textBase()->setFrameWidth(s->frameWidth);
+      textBase()->setHasFrame(s->hasFrame);
       textBase()->setPaddingWidth(s->paddingWidth);
       textBase()->setFrameColor(s->frameColor);
       textBase()->setFrameRound(s->frameRound);
@@ -699,7 +726,7 @@ void TextB::write(Xml& xml, const char* name) const
       if (doc()->isEmpty())
             return;
       xml.stag(name);
-      writeProperties(xml);
+      writeProperties(xml, true);
       xml.etag();
       }
 
@@ -721,15 +748,17 @@ void TextB::read(QDomElement e)
 //   writeProperties
 //---------------------------------------------------------
 
-void TextB::writeProperties(Xml& xml) const
+void TextB::writeProperties(Xml& xml, bool writeText) const
       {
       Element::writeProperties(xml);
 
       // write all properties which are different from style
 
-      TextStyle* st = style();
+      if (_textStyle >= 0)
+            xml.tag("style", _textStyle);
 
-      if (_align != st->align) {
+      TextStyle* st = score()->textStyle(_textStyle);
+      if (st == 0 || _align != st->align) {
             if (_align & (ALIGN_RIGHT | ALIGN_HCENTER)) {          // default is ALIGN_LEFT
                   xml.tag("halign", (_align & ALIGN_RIGHT) ? "right" : "center");
                   }
@@ -742,16 +771,16 @@ void TextB::writeProperties(Xml& xml) const
                         xml.tag("valign", "baseline");
                   }
             }
-      if (_xoff != st->xoff)
+      if (st == 0 || _xoff != st->xoff)
             xml.tag("xoffset", _xoff);
-      if (_yoff != st->yoff)
+      if (st == 0 || _yoff != st->yoff)
             xml.tag("yoffset", _yoff);
-      if (_rxoff != st->rxoff)
+      if (st == 0 || _rxoff != st->rxoff)
             xml.tag("rxoffset", _rxoff);
-      if (_ryoff != st->ryoff)
+      if (st == 0 || _ryoff != st->ryoff)
             xml.tag("ryoffset", _ryoff);
 
-      if (_offsetType != st->offsetType) {
+      if (st == 0 || _offsetType != st->offsetType) {
             const char* p = 0;
             switch(_offsetType) {
                   case OFFSET_SPATIUM:                    break;
@@ -761,11 +790,71 @@ void TextB::writeProperties(Xml& xml) const
                   xml.tag("offsetType", p);
             }
 
-      if (!_sizeIsSpatiumDependent && _sizeIsSpatiumDependent != st->sizeIsSpatiumDependent)
+      if (st == 0 || (!_sizeIsSpatiumDependent && _sizeIsSpatiumDependent != st->sizeIsSpatiumDependent))
             xml.tag("spatiumSizeDependent", _sizeIsSpatiumDependent);
       if (subtype() == TEXT_MEASURE_NUMBER)
             return;
-      textBase()->writeProperties(xml);
+      textBase()->writeProperties(xml, st, score()->spatium(), writeText);
+      }
+
+//---------------------------------------------------------
+//   textStyleChanged
+//    do not touch values which are modified by user
+//    (== different from old style)
+//---------------------------------------------------------
+
+void TextB::textStyleChanged(const QVector<TextStyle*>& styles)
+      {
+      if (_textStyle < 0)
+            return;
+      TextStyle* ns = styles[_textStyle];
+      TextStyle* os = score()->textStyle(_textStyle);
+
+      if (_align == os->align)
+            _align = ns->align;
+      if (_xoff == os->xoff)
+            _xoff = ns->xoff;
+      if (_yoff == os->yoff)
+            _yoff = ns->yoff;
+      if (_rxoff == os->rxoff)
+            _rxoff = ns->rxoff;
+      if (_ryoff == os->ryoff)
+            _ryoff = ns->ryoff;
+      if (_offsetType == os->offsetType)
+            _offsetType = ns->offsetType;
+      if (_sizeIsSpatiumDependent == os->sizeIsSpatiumDependent)
+            _sizeIsSpatiumDependent = ns->sizeIsSpatiumDependent;
+      if (frameWidth() == os->frameWidth)
+            setFrameWidth(ns->frameWidth);
+      if (paddingWidth() == os->paddingWidth)
+            setPaddingWidth(ns->paddingWidth);
+      if (frameRound() == os->frameRound)
+            setFrameRound(ns->frameRound);
+      if (frameColor() == os->frameColor)
+            setFrameColor(ns->frameColor);
+      if (circle() == os->circle)
+            setCircle(ns->circle);
+      if (systemFlag() == os->systemFlag)
+            setSystemFlag(ns->systemFlag);
+      double _spatium = spatium();
+      if (textBase()->isSimpleText(os, _spatium))
+            setDefaultFont(ns->font(_spatium));
+      if (textBase()->hasFrame() == os->hasFrame)
+            textBase()->setHasFrame(ns->hasFrame);
+      }
+
+//---------------------------------------------------------
+//   spatiumChanged
+//---------------------------------------------------------
+
+void TextB::spatiumChanged(double oldValue, double newValue)
+      {
+      Element::spatiumChanged(oldValue, newValue);
+      if (!_sizeIsSpatiumDependent)
+            return;
+      TextStyle* style = score()->textStyle(_textStyle);
+      if (textBase()->isSimpleText(style, oldValue))
+            setDefaultFont(style->font(newValue));
       }
 
 //---------------------------------------------------------
@@ -777,7 +866,12 @@ bool TextB::readProperties(QDomElement e)
       QString tag(e.tagName());
       QString val(e.text());
 
-      if (tag == "align")            // obsolete
+      if (tag == "style") {
+            int i = val.toInt();
+            if (i >= 0)
+                  setTextStyle(i);
+            }
+      else if (tag == "align")            // obsolete
             _align = Align(val.toInt());
       else if (tag == "halign") {
             if (val == "center")
@@ -830,6 +924,7 @@ bool TextB::startEdit(Viewer* view, const QPointF& p)
       {
       if ((type() == TEXT) && (subtype() == TEXT_MEASURE_NUMBER))
             return false;
+      mscore->textTools()->show();
       cursor = new QTextCursor(doc());
       cursor->setPosition(cursorPos);
       editMode = true;
@@ -848,11 +943,6 @@ bool TextB::startEdit(Viewer* view, const QPointF& p)
             bf.setAlignment(alignment);
             setBlockFormat(bf);
             }
-      //never true?
-      if (palette) {
-            palette->setCharFormat(cursor->charFormat());
-            palette->setBlockFormat(cursor->blockFormat());
-            }
       qreal w = 8.0 / view->matrix().m11();
       score()->addRefresh(abbox().adjusted(-w, -w, w, w));
       return true;
@@ -867,32 +957,20 @@ bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modi
       {
       if (debugMode)
             printf("TextB::edit\n");
+      if (!editMode || !cursor)
+            return false;
       bool lo = (subtype() == TEXT_INSTRUMENT_SHORT) || (subtype() == TEXT_INSTRUMENT_LONG);
       score()->setLayoutAll(lo);
       qreal w = 8.0 / view->matrix().m11();
       score()->addRefresh(abbox().adjusted(-w, -w, w, w));
 
-      if (key == Qt::Key_F2) {
-            if (palette == 0)
-                  palette = new TextPalette(score()->canvas());
-            if (palette->isVisible())
-                  palette->hide();
-            else {
-                  palette->setText(this);
-                  palette->show();
-                  palette->setCharFormat(cursor->charFormat());
-                  palette->setBlockFormat(cursor->blockFormat());
-                  }
-            return true;
-            }
       if (modifiers == Qt::ControlModifier) {
             switch (key) {
                   case Qt::Key_B:   // toggle bold face
                         {
                         QTextCharFormat f = cursor->charFormat();
                         f.setFontWeight(f.fontWeight() == QFont::Bold ? QFont::Normal : QFont::Bold);
-                        if (palette)
-                              palette->setCharFormat(f);
+                        mscore->textTools()->setCharFormat(f);
                         cursor->setCharFormat(f);
                         }
                         break;
@@ -900,8 +978,7 @@ bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modi
                         {
                         QTextCharFormat f = cursor->charFormat();
                         f.setFontItalic(!f.fontItalic());
-                        if (palette)
-                              palette->setCharFormat(f);
+                        mscore->textTools()->setCharFormat(f);
                         cursor->setCharFormat(f);
                         }
                         break;
@@ -909,8 +986,7 @@ bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modi
                         {
                         QTextCharFormat f = cursor->charFormat();
                         f.setFontUnderline(!f.fontUnderline());
-                        if (palette)
-                              palette->setCharFormat(f);
+                        mscore->textTools()->setCharFormat(f);
                         cursor->setCharFormat(f);
                         }
                         break;
@@ -936,9 +1012,18 @@ bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modi
                         }
                         break;
                   }
-            if (key != Qt::Key_Space && key != Qt::Key_Minus)
+            #ifndef Q_WS_MAC
+			if (key != Qt::Key_Space && key != Qt::Key_Minus)
                   return true;
-            }
+			#endif
+
+			}
+	  #ifdef Q_WS_MAC
+		else if(modifiers == Qt::AltModifier){
+			if (key != Qt::Key_Space && key != Qt::Key_Minus)
+                  return true;
+		}
+	  #endif
       QTextCursor::MoveMode mm = (modifiers & Qt::ShiftModifier)
          ? QTextCursor::KeepAnchor : QTextCursor::MoveAnchor;
       switch (key) {
@@ -995,11 +1080,9 @@ bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modi
       if (key == Qt::Key_Return || key == Qt::Key_Space || key == Qt::Key_Tab) {
             replaceSpecialChars();
             }
-      if (palette) {
-            palette->setCharFormat(cursor->charFormat());
-            palette->setBlockFormat(cursor->blockFormat());
-            }
-      layout(0);
+      mscore->textTools()->setCharFormat(cursor->charFormat());
+      mscore->textTools()->setBlockFormat(cursor->blockFormat());
+      layout();
       score()->addRefresh(abbox().adjusted(-w, -w, w, w));
       return true;
       }
@@ -1008,7 +1091,8 @@ bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modi
 //   replaceSpecialChars
 //---------------------------------------------------------
 
-bool TextB::replaceSpecialChars() {
+bool TextB::replaceSpecialChars()
+      {
       QTextCursor startCur = *cursor;
       foreach (const char* s, charReplaceMap.keys()) {
             SymCode sym = *charReplaceMap.value(s);
@@ -1063,9 +1147,14 @@ void TextB::moveCursor(int col)
 
 void TextB::endEdit()
       {
+      if (!editMode || !cursor) {
+            printf("endEdit: not in edit mode or no cursor: %d %p\n", editMode, cursor);
+            return;
+            }
       cursorPos = cursor->position();
-      if (palette)
-            palette->hide();
+      if (textPalette)
+            textPalette->hide();
+      mscore->textTools()->hide();
       delete cursor;
       cursor = 0;
       editMode = false;
@@ -1218,13 +1307,50 @@ void TextB::paste()
       if (debugMode)
             printf("TextB::paste() <%s>\n", qPrintable(txt));
       cursor->insertText(txt);
-      layout(0);
+      layout();
       bool lo = (subtype() == TEXT_INSTRUMENT_SHORT) || (subtype() == TEXT_INSTRUMENT_LONG);
       score()->setLayoutAll(lo);
       score()->setUpdateAll();
       score()->end();
       }
 
+//---------------------------------------------------------
+//   genPropertyMenu
+//---------------------------------------------------------
+
+bool Text::genPropertyMenu(QMenu* popup) const
+      {
+      QAction* a;
+      if (visible())
+            a = popup->addAction(tr("Set Invisible"));
+      else
+            a = popup->addAction(tr("Set Visible"));
+      a->setData("invisible");
+      a = popup->addAction(tr("Text Properties..."));
+      a->setData("props");
+      return true;
+      }
+
+//---------------------------------------------------------
+//   propertyAction
+//---------------------------------------------------------
+
+void Text::propertyAction(const QString& s)
+      {
+      if (s == "props") {
+            Text* nText = new Text(*this);
+            TextProperties tp(nText, 0);
+            int rv = tp.exec();
+            if (rv) {
+                  score()->undoChangeElement(this, nText);
+                  }
+            else
+                  delete nText;
+            }
+      else
+            Element::propertyAction(s);
+      }
+
 //---------------------------------------------------------
 //   dragAnchor
 //---------------------------------------------------------
@@ -1260,3 +1386,39 @@ QLineF TextB::dragAnchor() const
       return QLineF(p1, abbox().topLeft() + QPointF(x, y));
       }
 
+//---------------------------------------------------------
+//   TextProperties
+//---------------------------------------------------------
+
+TextProperties::TextProperties(TextB* t, QWidget* parent)
+   : QDialog(parent)
+      {
+      setWindowTitle(tr("MuseScore: Text Properties"));
+      QGridLayout* layout = new QGridLayout;
+      tp = new TextProp;
+      layout->addWidget(tp, 0, 1);
+      QLabel* l = new QLabel;
+      l->setPixmap(QPixmap(":/data/bg1.jpg"));
+      layout->addWidget(l, 0, 0, 2, 1);
+      QDialogButtonBox* bb = new QDialogButtonBox(
+         QDialogButtonBox::Ok | QDialogButtonBox::Cancel
+         );
+      layout->addWidget(bb, 1, 0, 1, 2);
+      setLayout(layout);
+
+      tb = t;
+      tp->set(tb);
+      connect(bb, SIGNAL(accepted()), SLOT(accept()));
+      connect(bb, SIGNAL(rejected()), SLOT(reject()));
+      }
+
+//---------------------------------------------------------
+//   accept
+//---------------------------------------------------------
+
+void TextProperties::accept()
+      {
+      tp->get(tb);
+      QDialog::accept();
+      }
+
