@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: text.h 1992 2009-08-04 16:30:54Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -26,6 +26,7 @@
 
 class TextPalette;
 class Viewer;
+class TextProp;
 
 struct SymCode;
 
@@ -50,12 +51,14 @@ enum {
 //---------------------------------------------------------
 
 class TextBase {
+      int _refCount;
       QTextDocument* _doc;
       double _frameWidth;                 // unit: mm
       double _paddingWidth;               // unit: mm
       QColor _frameColor;
       int _frameRound;
       bool _circle;
+      bool _hasFrame;
       double _layoutWidth;
 
       QRectF frame;
@@ -65,8 +68,16 @@ class TextBase {
       TextBase();
       TextBase(const TextBase&);
 
+      int refCount() const                  { return _refCount; }
+      void incRefCount()                    { ++_refCount; }
+      void decRefCount()                    { --_refCount; }
+
       void setDoc(const QTextDocument& d);
+      QTextDocument* swapDoc(QTextDocument* d);
+
       QTextDocument* doc() const            { return _doc;          }
+      bool hasFrame() const                 { return _hasFrame;     }
+      void setHasFrame(bool val)            { _hasFrame = val;      }
       double frameWidth() const             { return _frameWidth;   }
       double paddingWidth() const           { return _paddingWidth; }
       QColor frameColor() const             { return _frameColor;   }
@@ -78,10 +89,10 @@ class TextBase {
       void setFrameRound(int v)             { _frameRound = v;      }
       void setCircle(bool v)                { _circle = v;          }
 
-      void writeProperties(Xml& xml) const;
+      void writeProperties(Xml&, TextStyle*, double spatium, bool writeText) const;
       bool readProperties(QDomElement e);
-      QFont defaultFont() const             { return _doc->defaultFont(); }
-      void setDefaultFont(QFont f)          { _doc->setDefaultFont(f);    }
+      QFont defaultFont() const;
+      void setDefaultFont(QFont f);
       void clear()                          { _doc->clear();              }
       void layout(double w);
       QRectF bbox() const                   { return _bbox; }
@@ -92,6 +103,7 @@ class TextBase {
       void setHtml(const QString& s);
       bool isEmpty() const                  { return _doc->isEmpty(); }
       void setModified(bool v)              { _doc->setModified(v);   }
+      bool isSimpleText(TextStyle*, double) const;
       };
 
 //---------------------------------------------------------
@@ -107,7 +119,7 @@ class TextB : public Element {
       QTextCursor* cursor;
       bool setCursor(const QPointF& p);
       int cursorPos;
-      TextStyle* style() const;
+      int _textStyle;
 
    public:
       TextB(Score*);
@@ -118,8 +130,8 @@ class TextB : public Element {
       virtual ElementType type() const        { return TEXT; }
 
       virtual const QString subtypeName() const;
-      virtual void setSubtype(int val);
       virtual void setSubtype(const QString& s);
+      virtual void setSubtype(int val)      { Element::setSubtype(val);    }
 
       virtual bool isMovable() const        { return _movable;             }
       void setMovable(bool val)             { _movable = val;              }
@@ -131,11 +143,11 @@ class TextB : public Element {
       QString getHtml() const               { return textBase()->getHtml(); }
       void setHtml(const QString& s)        { textBase()->setHtml(s);       }
       void setDoc(const QTextDocument&);
+      QTextDocument* swapDoc(QTextDocument* d);
       QTextDocument* doc() const            { return textBase()->doc(); }
 
       virtual void resetMode();
-      bool isEmpty() const;
-      virtual void setStyle(const TextStyle*);
+      virtual bool isEmpty() const;
 
       double frameWidth() const             { return textBase()->frameWidth();   }
       double paddingWidth() const           { return textBase()->paddingWidth(); }
@@ -163,9 +175,9 @@ class TextB : public Element {
       virtual void write(Xml& xml) const;
       virtual void write(Xml& xml, const char*) const;
       virtual void read(QDomElement);
-      void writeProperties(Xml& xml) const;
+      void writeProperties(Xml& xml, bool writeText = true) const;
       bool readProperties(QDomElement node);
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       virtual QPainterPath shape() const;
       virtual bool mousePress(const QPointF&, QMouseEvent* ev);
       double lineSpacing() const;
@@ -183,7 +195,12 @@ class TextB : public Element {
       virtual void paste();
 
       bool replaceSpecialChars();
-      void styleChanged();
+      QTextCursor* getCursor() const { return cursor; }
+
+      virtual void textStyleChanged(const QVector<TextStyle*>&s);
+      virtual void spatiumChanged(double oldValue, double newValue);
+      virtual void setTextStyle(int);
+      int textStyle() const                 { return _textStyle; }
       };
 
 //---------------------------------------------------------
@@ -191,36 +208,54 @@ class TextB : public Element {
 //---------------------------------------------------------
 
 class Text : public TextB {
+      Q_DECLARE_TR_FUNCTIONS(Text)
+
       TextBase* _tb;
 
    public:
       Text(Score*);
       Text(const Text&);
       ~Text();
-      virtual Text* clone() const           { return new Text(*this); }
-      virtual TextBase* textBase() const    { return _tb; }
-      void setModified(bool v)              { _tb->setModified(v); }
+      virtual Text* clone() const        { return new Text(*this); }
+      virtual TextBase* textBase() const { return _tb; }
+
+      virtual bool genPropertyMenu(QMenu* popup) const;
+      virtual void propertyAction(const QString& s);
+
+      void setModified(bool v)           { _tb->setModified(v); }
       };
 
 //---------------------------------------------------------
 //   class Text
+//    shared text
 //---------------------------------------------------------
 
 class TextC : public TextB {
-      TextBase** _tbb;
-      TextBase*  _otb;
+      TextBase*  _tb;
 
    public:
-      TextC(TextBase**, Score*);
+      TextC(Score*);
       TextC(const TextC&);
       ~TextC();
-      virtual TextC* clone() const          { return new TextC(*this); }
-      virtual TextBase* textBase() const    { return *_tbb;            }
-      void setTextBase(TextBase** tb)       { _tbb = tb;               }
-      TextBase* otb()                       { return _otb;             }
-      void setOtb(TextBase* b)              { _otb = b;                }
-      virtual void setStyle(const TextStyle*);
+      virtual TextC* clone() const;
+      virtual TextBase* textBase() const { return _tb; }
       void baseChanged();
       };
 
+//---------------------------------------------------------
+//   TextProperties
+//---------------------------------------------------------
+
+class TextProperties : public QDialog {
+      Q_OBJECT
+      TextProp* tp;
+      TextB* tb;
+
+   private slots:
+      virtual void accept();
+
+   public:
+      TextProperties(TextB*, QWidget* parent = 0);
+      };
+
 #endif
