@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: timesig.cpp,v 1.4 2006/04/12 14:58:10 wschweer Exp $
+//  $Id: timesig.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -25,9 +25,10 @@
 #include "style.h"
 #include "sym.h"
 #include "symbol.h"
-#include "layout.h"
 #include "viewer.h"
 #include "system.h"
+#include "segment.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   TimeSig
@@ -222,16 +223,17 @@ void TimeSig::read(QDomElement e)
 //   layout
 //---------------------------------------------------------
 
-void TimeSig::layout(ScoreLayout*)
+void TimeSig::layout()
       {
+      double _spatium = spatium();
       QRectF bb;
       int st = subtype();
       if (st == 0)
             bb = QRectF(0, 0,0, 0);
       else if (st ==  TSIG_FOUR_FOUR)
-            bb = symbols[fourfourmeterSym].bbox().translated(0.0, 2.0 * _spatium);
+            bb = symbols[fourfourmeterSym].bbox(magS()).translated(0.0, 2.0 * _spatium);
       else if (st == TSIG_ALLA_BREVE)
-            bb = symbols[allabreveSym].bbox().translated(0.0, 2.0 * _spatium);
+            bb = symbols[allabreveSym].bbox(magS()).translated(0.0, 2.0 * _spatium);
       else {
             int n, z1, z2, z3, z4;
             getSig(&n, &z1, &z2, &z3, &z4);
@@ -251,8 +253,6 @@ void TimeSig::layout(ScoreLayout*)
 
             double m = _spatium / (DPI * SPATIUM20);
             double im = 1.0 / m;
-//            pz = QPointF(0.0, 2.0 * _spatium * mag()) * im;
-//            pn = QPointF((rz.width() - rn.width())*.5, 4.0 * _spatium * mag()) * im;
             pz = QPointF(0.0, 2.0 * _spatium) * im;
             pn = QPointF((rz.width() - rn.width())*.5, 4.0 * _spatium) * im;
 
@@ -271,13 +271,14 @@ void TimeSig::draw(QPainter& p) const
       int st = subtype();
       if (st == 0)
             return;
+      double _spatium = spatium();
       if (st ==  TSIG_FOUR_FOUR)
-            symbols[fourfourmeterSym].draw(p, mag(), 0.0, 2.0 * _spatium * mag());
+            symbols[fourfourmeterSym].draw(p, magS(), 0.0, 2.0 * _spatium);
       else if (st == TSIG_ALLA_BREVE)
-            symbols[allabreveSym].draw(p, mag(), 0.0, 2.0 * _spatium * mag());
+            symbols[allabreveSym].draw(p, magS(), 0.0, 2.0 * _spatium);
       else {
             p.setFont(symbols[allabreveSym].font());
-            double m  = _spatium * mag() / (DPI * SPATIUM20);
+            double m  = _spatium / (DPI * SPATIUM20);
             double im = 1.0 / m;
             p.scale(m, m);
             p.drawText(pz, sz);
