@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id:$
+//  $Id: transposedialog.cpp 1851 2009-05-25 18:03:41Z wschweer $
 //
 //  Copyright (C) 2008 Werner Schweer and others
 //
@@ -21,7 +21,6 @@
 #include "transposedialog.h"
 #include "score.h"
 #include "mscore.h"
-#include "layout.h"
 #include "chord.h"
 #include "note.h"
 #include "key.h"
@@ -29,6 +28,8 @@
 #include "harmony.h"
 #include "part.h"
 #include "pitchspelling.h"
+#include "measure.h"
+#include "undo.h"
 
 //---------------------------------------------------------
 //   TransposeDialog
@@ -44,11 +45,24 @@ TransposeDialog::TransposeDialog(QWidget* parent)
 //   transpose
 //---------------------------------------------------------
 
+void Score::transpose(Note* n, int diff)
+      {
+      int npitch = n->pitch() + diff;
+      int d      = diff < 0 ? -diff : diff;
+      int tpc    = (d % 12) == 0 ? n->tpc() : pitch2tpc(npitch, 0);
+      int acc    = (d % 12) == 0 ? n->userAccidental() : 0;
+      undoChangePitch(n, npitch, tpc, acc);
+      }
+
+//---------------------------------------------------------
+//   transpose
+//---------------------------------------------------------
+
 void Score::transpose()
       {
-      if (_layout->last() == 0)     // empty score?
+      if (last() == 0)     // empty score?
             return;
-      if (sel->state() == SEL_NONE) {
+      if (selection()->state() == SEL_NONE) {
             QMessageBox::StandardButton sb = QMessageBox::question(mscore,
                tr("MuseScore: transpose"),
                tr("There is nothing selected. Transpose whole score?"),
@@ -60,44 +74,45 @@ void Score::transpose()
             //
             // select all
             //
-            sel->setState(SEL_SYSTEM);
-            sel->setStartSegment(tick2segment(0));
-            sel->setEndSegment(
-               tick2segment(_layout->last()->tick() + _layout->last()->tickLen())
+            _selection->setState(SEL_SYSTEM);
+            _selection->setStartSegment(tick2segment(0));
+            _selection->setEndSegment(
+               tick2segment(last()->tick() + last()->tickLen())
                );
-            sel->staffStart = 0;
-            sel->staffEnd   = nstaves();
+            _selection->staffStart = 0;
+            _selection->staffEnd   = nstaves();
             }
       TransposeDialog td;
-      td.enableTransposeKeys(sel->state() == SEL_SYSTEM);
+      td.enableTransposeKeys(_selection->state() == SEL_SYSTEM);
       if (!td.exec())
             return;
       int diff           = td.getSemitones();
       bool transposeKeys = td.getTransposeKeys();
-      if (sel->state() != SEL_SYSTEM)
+      if (_selection->state() != SEL_SYSTEM)
+            transposeKeys = false;
+      int d = diff < 0 ? -diff : diff;
+      bool fullOctave = (d % 12) == 0;
+      if (fullOctave)
             transposeKeys = false;
 
-      if (sel->state() == SEL_SINGLE || sel->state() == SEL_MULT) {
-            QList<Element*>* el = sel->elements();
+      if (_selection->state() == SEL_SINGLE || _selection->state() == SEL_MULT) {
+            QList<Element*>* el = _selection->elements();
             foreach(Element* e, *el) {
                   if (e->type() != NOTE)
                         continue;
-                  Note* n = static_cast<Note*>(e);
-                  int npitch = n->pitch() + diff;
-                  int tpc    = pitch2tpc(npitch, 0);
-                  undoChangePitch(n, npitch, tpc, 0);
+                  transpose(static_cast<Note*>(e), diff);
                   }
             return;
             }
 
-      int startTrack = sel->staffStart * VOICES;
-      int endTrack   = sel->staffEnd * VOICES;
-      if (sel->state() == SEL_SYSTEM) {
+      int startTrack = _selection->staffStart * VOICES;
+      int endTrack   = _selection->staffEnd * VOICES;
+      if (_selection->state() == SEL_SYSTEM) {
             startTrack = 0;
             endTrack   = nstaves() * VOICES;
             }
       for (int st = startTrack; st < endTrack; ++st) {
-            for (Segment* segment = sel->startSegment(); segment && segment != sel->endSegment(); segment = segment->next1()) {
+            for (Segment* segment = _selection->startSegment(); segment && segment != _selection->endSegment(); segment = segment->next1()) {
                   Element* e = segment->element(st);
                   if (!e || e->type() != CHORD)
                         continue;
@@ -109,19 +124,16 @@ void Score::transpose()
                   QList<Note*> nl;
                   for (iNote i = notes->begin(); i != notes->end(); ++i)
                         nl.append(i->second);
-                  foreach(Note* note, nl) {
-                        int npitch = note->pitch() + diff;
-                        int tpc    = pitch2tpc(npitch, 0);
-                        undoChangePitch(note, npitch, tpc, 0);
-                        }
+                  foreach(Note* n, nl)
+                        transpose(n, diff);
                   }
             }
 
-      if (td.getTransposeChordNames()) {
-            Measure* sm = sel->startSegment()->measure();
-            Measure* em = sel->endSegment()->measure();
-            int stick = sel->startSegment()->tick();
-            int etick = sel->endSegment()->tick();
+      if (!fullOctave && td.getTransposeChordNames()) {
+            Measure* sm = _selection->startSegment()->measure();
+            Measure* em = _selection->endSegment()->measure();
+            int stick   = _selection->startSegment()->tick();
+            int etick   = _selection->endSegment()->tick();
 
             for (Measure* m = sm;;) {
                   foreach (Element* e, *m->el()) {
@@ -144,10 +156,10 @@ void Score::transpose()
             }
 
       if (transposeKeys) {
-            for (int staffIdx = sel->staffStart; staffIdx < sel->staffEnd; ++staffIdx) {
+            for (int staffIdx = _selection->staffStart; staffIdx < _selection->staffEnd; ++staffIdx) {
                   KeyList* km = staff(staffIdx)->keymap();
-                  for (iKeyEvent ke = km->lower_bound(sel->tickStart());
-                     ke != km->lower_bound(sel->tickEnd()); ++ke) {
+                  for (iKeyEvent ke = km->lower_bound(_selection->tickStart());
+                     ke != km->lower_bound(_selection->tickEnd()); ++ke) {
                         int oKey  = ke->second;
                         int tick  = ke->first;
                         int nKey  = transposeKey(oKey, diff);
@@ -155,7 +167,12 @@ void Score::transpose()
                         }
                   }
             }
-      spell();
+      //
+      // do not respell if transposing a full octave
+      //
+      if (!fullOctave) {
+            spell(_selection->staffStart, _selection->staffEnd, _selection->startSegment(), _selection->endSegment());
+            }
       }
 
 //---------------------------------------------------------
@@ -167,7 +184,7 @@ void Score::cmdTransposeStaff(int staffIdx, int diff)
       int startTrack = staffIdx * VOICES;
       int endTrack   = startTrack + VOICES;
 
-      for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
+      for (MeasureBase* mb = first(); mb; mb = mb->next()) {
             if (mb->type() != MEASURE)
                   continue;
             Measure* m = (Measure*)mb;
@@ -184,11 +201,8 @@ void Score::cmdTransposeStaff(int staffIdx, int diff)
                         QList<Note*> nl;
                         for (iNote i = notes->begin(); i != notes->end(); ++i)
                               nl.append(i->second);
-                        foreach(Note* n, nl) {
-                              int npitch = n->pitch() + diff;
-                              int tpc    = pitch2tpc(npitch, 0);
-                              undoChangePitch(n, npitch, tpc, 0);
-                              }
+                        foreach(Note* n, nl)
+                              transpose(n, diff);
                         }
                   }
 #if 0
@@ -207,6 +221,7 @@ void Score::cmdTransposeStaff(int staffIdx, int diff)
             int nKey  = transposeKey(oKey, diff);
             undoChangeKey(staff(staffIdx), tick, oKey, nKey);
             }
+      // spell(staffIdx, staffIdx+1, _selection->startSegment(), _selection->endSegment());
       spell();
       }
 
@@ -216,12 +231,7 @@ void Score::cmdTransposeStaff(int staffIdx, int diff)
 
 void Score::cmdConcertPitchChanged(bool flag)
       {
-      checkUndoOp();
-      UndoOp i;
-      i.type   = UndoOp::ChangeConcertPitch;
-      i.val1   = flag;
-      undoList.back()->push_back(i);
-      processUndoOp(&undoList.back()->back(), false);
+      _undo->push(new ChangeConcertPitch(this, flag));
 
       foreach(Staff* staff, _staves) {
             Instrument* instr = staff->part()->instrument();
