@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: tuplet.cpp,v 1.12 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: tuplet.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -27,8 +27,8 @@
 #include "style.h"
 #include "text.h"
 #include "element.h"
-#include "layout.h"
 #include "utils.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   Tuplet
@@ -74,13 +74,14 @@ void Tuplet::setSelected(bool f)
 //   layout
 //---------------------------------------------------------
 
-void Tuplet::layout(ScoreLayout* layout)
+void Tuplet::layout()
       {
-      double _spatium = layout->spatium();
+      double _spatium = spatium();
       if (_numberType != NO_TEXT) {
             if (_number == 0) {
                   _number = new Text(score());
                   _number->setSubtype(TEXT_TUPLET);
+                  _number->setTextStyle(TEXT_STYLE_TUPLET);
                   _number->setParent(this);
                   }
             if (_numberType == SHOW_NUMBER)
@@ -223,7 +224,7 @@ void Tuplet::layout(ScoreLayout* layout)
       qreal x3 = 0.0, y3 = 0.0;
       qreal numberWidth = 0.0;
       if (_number) {
-            _number->layout(layout);
+            _number->layout();
             x3 = p1.x() + (p2.x() - p1.x()) * .5;
 
             y3 = p1.y() + (p2.y() - p1.y()) * .5
@@ -320,7 +321,7 @@ void Tuplet::draw(QPainter& p) const
             }
       if (_hasBracket) {
             QPen pen(p.pen());
-            pen.setWidthF(_spatium * 0.1);
+            pen.setWidthF(spatium() * 0.1);
             p.setPen(pen);
             if (!_number)
                   p.drawPolyline(bracketL, 4);
@@ -383,6 +384,7 @@ void Tuplet::read(QDomElement e)
                   _number->setParent(this);
                   _number->read(e);
                   _number->setSubtype(TEXT_TUPLET);   // override read
+                  _number->setTextStyle(TEXT_STYLE_TUPLET);
                   }
             else if (tag == "p1") {
                   _userModified = true;
@@ -638,10 +640,10 @@ void Tuplet::updateGrips(int* grips, QRectF*grip) const
       }
 
 //---------------------------------------------------------
-//   resetUserOffsets
+//   toDefault
 //---------------------------------------------------------
 
-void Tuplet::resetUserOffsets()
+void Tuplet::toDefault()
       {
       _userModified = false;
       _p1           = QPointF();
