@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: tuplet.h,v 1.9 2006/03/13 21:35:59 wschweer Exp $
+//  $Id: tuplet.h 1936 2009-07-17 16:12:47Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -51,14 +51,14 @@ class Tuplet : public DurationElement {
       int _bracketType;
       bool _hasBracket;
 
-      int _baseLen;     // tick len of a "normal note"
+      int _baseLen;           // tick len of a "normal note"
       int _normalNotes;
       int _actualNotes;
 
       bool _userModified;
       QPointF p1, p2;
       QPointF _p1, _p2;
-      mutable int _id;          // used during read/write
+      mutable int _id;        // used during read/write
 
       Text* _number;
       QPointF bracketL[4];
@@ -83,7 +83,7 @@ class Tuplet : public DurationElement {
       virtual void editDrag(int, const QPointF&);
       virtual void updateGrips(int*, QRectF*) const;
 
-      Measure* measure() const      { return (Measure*)parent(); }
+      virtual Measure* measure() const { return (Measure*)parent(); }
 
       int numberType() const        { return _numberType;   }
       int bracketType() const       { return _bracketType;  }
@@ -97,22 +97,23 @@ class Tuplet : public DurationElement {
       int baseLen() const           { return _baseLen;      }
       int normalNotes() const       { return _normalNotes;  }
       int actualNotes() const       { return _actualNotes;  }
-      int normalLen() const         { return _baseLen / _normalNotes; }
+//      int normalLen() const         { return _baseLen / _normalNotes; }
       int noteLen() const           { return _baseLen * _normalNotes / _actualNotes; }
       const QList<DurationElement*>& elements() const   { return _elements; }
       void clear()                  { _elements.clear(); }
-      virtual void layout(ScoreLayout*);
+      virtual void layout();
       Text* number() const { return _number; }
 
       virtual void read(QDomElement);
       void write(Xml&) const;
 
-      virtual void resetUserOffsets();
+      virtual void toDefault();
 
       virtual void draw(QPainter&) const;
       int id() const                  { return _id; }
-      void setId(int i) const         { _id = i; }
-      virtual int tickLen() const     { return _baseLen * _normalNotes; }
+      void setId(int i) const         { _id = i;                                }
+      virtual int tickLen() const     { return _baseLen * _normalNotes;         }
+      int actualTickLen(int l) const  { return l * _normalNotes / _actualNotes; }
       };
 
 
