@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: utils.cpp,v 1.24 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: utils.cpp 1889 2009-06-21 17:00:53Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -18,21 +18,17 @@
 //  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //=============================================================================
 
-#if defined(__MINGW32__) || defined(__APPLE__)
 #include <sys/time.h>
-#else
-#include <time.h>
-#endif
 
 #include "score.h"
 #include "page.h"
 #include "segment.h"
-#include "layout.h"
 #include "recordbutton.h"
 #include "greendotbutton.h"
 #include "clef.h"
 #include "utils.h"
 #include "system.h"
+#include "measure.h"
 
 //---------------------------------------------------------
 //   RecordButton
@@ -93,7 +89,7 @@ QRectF handleRect(const QPointF& pos)
 
 Measure* Score::tick2measure(int tick) const
       {
-      for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
+      for (MeasureBase* mb = first(); mb; mb = mb->next()) {
             if (mb->type() != MEASURE)
                   continue;
             Measure* m = static_cast<Measure*>(mb);
@@ -110,16 +106,15 @@ Measure* Score::tick2measure(int tick) const
             if (nmb == 0)
                   return m;
             }
-      printf("tick2measure %d not found\n", tick);
-      if (debugMode) {
-            int idx = 0;
-            for (MeasureBase* m = _layout->first(); m; m = m->next()) {
+      printf("-tick2measure %d not found\n", tick);
+//      if (debugMode) {
+        printf("first %p\n", first());
+            for (MeasureBase* m = first(); m; m = m->next()) {
                   int st = m->tick();
                   int l  = m->tickLen();
-                  printf("(%d)   %d - %d\n", idx, st, st+l);
-                  ++idx;
+                  printf("%d - %d\n", st, st+l);
                   }
-            }
+//            }
       return 0;
       }
 
@@ -129,13 +124,13 @@ Measure* Score::tick2measure(int tick) const
 
 MeasureBase* Score::tick2measureBase(int tick) const
       {
-      for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
+      for (MeasureBase* mb = first(); mb; mb = mb->next()) {
             int st = mb->tick();
             int l  = mb->tickLen();
             if (tick >= st && tick < (st+l))
                   return mb;
             }
-      printf("tick2measureBase %d not found\n", tick);
+//      printf("tick2measureBase %d not found\n", tick);
       return 0;
       }
 
@@ -302,7 +297,7 @@ int pitchKeyAdjust(int step, int key)
 //   y2pitch
 //---------------------------------------------------------
 
-int y2pitch(double y, int clef)
+int y2pitch(double y, int clef, double _spatium)
       {
       int l = lrint(y / _spatium * 2.0);
       return line2pitch(l, clef, 0);
@@ -355,7 +350,7 @@ int quantizeLen(int, int len, int raster)
 
 double curTime()
       {
-#ifdef __MINGW32__
+#if defined (__MINGW32__) || defined (__APPLE__)
       struct timeval t;
       gettimeofday(&t, 0);
       return (double)((double)t.tv_sec + (t.tv_usec / 1000000.0));
@@ -425,4 +420,3 @@ QString pitch2string(int v)
       return QString("%1%2").arg(octave < 0 ? valu[i] : vall[i]).arg(octave);
       }
 
-
