@@ -1,7 +1,7 @@
 //=============================================================================
 //  MuseScore
 //  Linux Music Score Editor
-//  $Id: canvas.cpp,v 1.80 2006/09/15 09:34:57 wschweer Exp $
+//  $Id: viewer.cpp 1840 2009-05-20 11:57:51Z wschweer $
 //
 //  Copyright (C) 2002-2007 Werner Schweer and others
 //
@@ -25,6 +25,7 @@
 
 #include "viewer.h"
 #include "score.h"
+#include "element.h"
 
 //---------------------------------------------------------
 //   Viewer
@@ -130,12 +131,3 @@ void Viewer::setDropAnchor(const QLineF& l)
             }
       }
 
-//---------------------------------------------------------
-//   layout
-//---------------------------------------------------------
-
-ScoreLayout* Viewer::layout()
-      {
-      return _score->layout();
-      }
-
