@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE Score
 //  Linux Music Score Editor
-//  $Id: viewer.h,v 1.6 2006/03/02 17:08:43 wschweer Exp $
+//  $Id: viewer.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2002-2008 Werner Schweer and others
+//  Copyright (C) 2002-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -23,7 +23,6 @@
 
 class Score;
 class Element;
-class ScoreLayout;
 class Segment;
 class TextB;
 
@@ -62,7 +61,6 @@ class Viewer {
       virtual ~Viewer() {}
       void setScore(Score* s) { _score = s; }
       Score* score() const    { return _score; }
-      ScoreLayout* layout();
       virtual void dataChanged(const QRectF&) = 0;
 
       virtual void updateAll(Score*) = 0;
@@ -71,7 +69,7 @@ class Viewer {
       void setDropAnchor(const QLineF&);
       const QMatrix& matrix() const { return _matrix; }
       virtual void moveCursor() {}
-      virtual void moveCursor(Segment*) {}
+      virtual void moveCursor(Segment*, int) {}
       virtual void setCursorOn(bool) {}
       void setEditText(TextB* t)              { _editText = t;      }
       TextB* editText() const                 { return _editText;   }
