@@ -1,9 +1,9 @@
 //=============================================================================
 //  MusE
 //  Linux Music Score Editor
-//  $Id: xml.h,v 1.15 2006/04/05 08:15:12 wschweer Exp $
+//  $Id: xml.h 1840 2009-05-20 11:57:51Z wschweer $
 //
-//  Copyright (C) 2004-2008 Werner Schweer and others
+//  Copyright (C) 2004-2009 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -21,6 +21,9 @@
 #ifndef __XML_H__
 #define __XML_H__
 
+#include "globals.h"
+#include "spatium.h"
+
 //---------------------------------------------------------
 //   Property
 //---------------------------------------------------------
@@ -72,7 +75,9 @@ class Xml : public QTextStream {
       void tag(const QString& name, QVariant data);
       void tag(const char* name, const char* s)    { tag(name, QVariant(s)); }
       void tag(const char* name, const QString& s) { tag(name, QVariant(s)); }
+      void tag(const char* name, Spatium sp)       { tag(name, QVariant(sp.val())); }
       void tag(const char* name, const QWidget*);
+      void tag(const char* name, Placement);
 
       void writeHtml(const QString& s);
 
@@ -89,5 +94,6 @@ extern QSizeF readSize(QDomElement);
 extern QColor readColor(QDomElement e);
 extern void domError(QDomElement node);
 extern void domNotImplemented(QDomElement node);
+extern Placement readPlacement(QDomElement e);
 #endif
 
