@@ -90,7 +90,7 @@ static QScriptValue qtscript_QLineF_throw_ambiguity_error_helper(
     QStringList fullSignatures;
     for (int i = 0; i < lines.size(); ++i)
         fullSignatures.append(QString::fromLatin1("%0(%1)").arg(functionName).arg(lines.at(i)));
-    return context->throwError(QString::fromLatin1("QFile::%0(): could not find a function match; candidates are:\n%1")
+    return context->throwError(QString::fromLatin1("QLineF::%0(): could not find a function match; candidates are:\n%1")
         .arg(functionName).arg(fullSignatures.join(QLatin1String("\n"))));
 }
 
@@ -132,7 +132,7 @@ static const char * const qtscript_QLineF_IntersectType_keys[] = {
 static QString qtscript_QLineF_IntersectType_toStringHelper(QLineF::IntersectType value)
 {
     if ((value >= QLineF::NoIntersection) && (value <= QLineF::UnboundedIntersection))
-        return qtscript_QLineF_IntersectType_keys[static_cast<int>(value)];
+        return qtscript_QLineF_IntersectType_keys[static_cast<int>(value)-static_cast<int>(QLineF::NoIntersection)];
     return QString();
 }
 
@@ -204,7 +204,7 @@ static QScriptValue qtscript_QLineF_prototype_call(QScriptContext *context, QScr
     if (!_q_self) {
         return context->throwError(QScriptContext::TypeError,
             QString::fromLatin1("QLineF.%0(): this object is not a QLineF")
-            .arg(qtscript_QLineF_function_names[_id+1]));
+            .arg(qtscript_QLineF_function_names[_id+2]));
     }
 
     switch (_id) {
